#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_Cxx_Attribute
#include "ASE/Compiler/Cxx/Attribute_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

#ifndef included_ASE_BaseException
#include "ASE/BaseException_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

static const struct ASE_Compiler_Cxx_Attribute__external *_epv = NULL;

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_Cxx_Attribute_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_Cxx_Attribute__object *ior;
  enum ReferenceType refType;
};

static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_Cxx_Attribute_PythonObject *self = (struct ASE_Compiler_Cxx_Attribute_PythonObject *) type->tp_alloc(type, 0);
    
    if (self)
    {
      struct ASE_BaseClass__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseClass", NULL);
      
      self->ior = (*(ior->d_epv->castObject))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      /* Allow objects without an attached implementation (used in the skeleton constructor) */
      if (baseIor->d_data)
      {
        if (refType == NEW_REF)
        {
          (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
        }
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_BaseInterface__wrap_ior(struct ASE_BaseInterface__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseInterface");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseInterface");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseInterface");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseInterface");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_BaseException__wrap_ior(struct ASE_BaseException__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseException");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseException");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseException");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseException");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Cxx_Attribute__wrap_ior(struct ASE_Compiler_Cxx_Attribute__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Cxx.Attribute");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Attribute");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Cxx.Attribute");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Cxx.Attribute");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_Compiler_Cxx_Attribute__extract_ior(struct ASE_Compiler_Cxx_Attribute_PythonObject *obj, struct ASE_Compiler_Cxx_Attribute__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Cxx.Attribute", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Cxx.Attribute");
    return 0;
  }
  return 1;
}

static long ASE_BaseInterface__extract_ior(struct ASE_Compiler_Cxx_Attribute_PythonObject *obj, struct ASE_BaseInterface__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.BaseInterface", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.BaseInterface");
    return 0;
  }
  return 1;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getReferenceCount))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addReference))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->deleteReference))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  int _okay = 0;
  struct ASE_BaseInterface__object *obj = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_BaseInterface__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isSame))(_self_ior, obj, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getClassName))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getName))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  int _okay = 0;
  const char *name = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setName))(_self_ior, name, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  int _okay = 0;
  const char *typeName = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isInstanceOf))(_self_ior, typeName, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getSuper(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Cxx_Attribute__object *_self_ior = ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_BaseInterface__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSuper");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getSuper");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getSuper))(_self_ior, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_BaseInterface__wrap_ior, _return_value);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Cxx_Attribute_Attribute(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Cxx_Attribute__object *obj = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_Compiler_Cxx_Attribute__object *_return_value = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) ASE_Compiler_Cxx_Attribute__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    (*(obj->d_epv->addReference))(obj->d_object, NULL);
  }
  else
  {
    obj = (*(_epv->createObject))(NULL);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Cxx_Attribute__wrap_ior, _return_value);
  return _pyReturn;
}

static struct PyMethodDef _AttributeObjectMethods[] = {
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "isSame", (PyCFunction) ASE_BaseInterface_isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in ASE.BaseInterface obj)\n\
RETURNS (return bool _return_value)\n\
 Return true if and only if <code>obj</code> refers to the same underlying object as this object"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "getSuper", (PyCFunction) ASE_BaseInterface_getSuper, (METH_VARARGS | METH_KEYWORDS), (char *) "getSuper()\n\
RETURNS (return ASE.BaseInterface _return_value)\n\
 Retrieve an object with the parent interface IMPL: Obviously, this must be implemented in the IOR of each interface/class"},
  {NULL, NULL}};

static struct PyMethodDef _AttributeModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct ASE_Compiler_Cxx_Attribute_PythonObject *self)
{
  if ((self->refType != WEAK_REF) && (self->ior->d_data))
  {
    (*(self->ior->d_epv->deleteReference))(self->ior->d_object, NULL);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct ASE_Compiler_Cxx_Attribute_PythonObject *self, PyObject *arg)
{
  struct ASE_Compiler_Cxx_Attribute__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct ASE_Compiler_Cxx_Attribute_PythonObject *self)
{
  struct ASE_Compiler_Cxx_Attribute__object *ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior->d_object);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct ASE_Compiler_Cxx_Attribute_PythonObject *self)
{
  struct ASE_Compiler_Cxx_Attribute__object *ior = self->ior;
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  
  _str = (*(ior->d_epv->strObject))(ior, NULL);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct ASE_Compiler_Cxx_Attribute_PythonObject *self, PyObject *arg, int op)
{
  struct ASE_Compiler_Cxx_Attribute__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_Cxx_Attribute_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "Could not parse arguments for sidl_richcmp");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "ASE.Compiler.Cxx.Attribute",
  sizeof(struct ASE_Compiler_Cxx_Attribute_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
  (char *) "This is a Python wrapper for a SIDL object of type ASE.Compiler.Cxx.Attribute",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  0,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _AttributeObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) ASE_Compiler_Cxx_Attribute_Attribute,
  (freefunc) 0,
  (inquiry) 0};

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

void initAttribute(void)
{
  PyObject *module = NULL;
  const struct ASE_Compiler_Cxx_Attribute__external *(*_extFunc)(void);
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "Attribute", _AttributeModuleMethods, (char *) " A Cxx AST attribute");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module ASE.Compiler.Cxx.Attribute");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "Attribute", (PyObject *) &SIDLPythonObjectType);
  /* Set externals function */
  _extFunc = (const struct ASE_Compiler_Cxx_Attribute__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(&_exception)->getClassMethodTable("ASE.Compiler.Cxx.Attribute", &_exception);
  /* Check exception */
  if (_exception)
  {
    _exception = _exception->d_epv->castObject(_exception, "ASE.BaseException", NULL);
    PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    return;
  }
  /* Set epv pointer */
  if (_extFunc)
  {
    _epv = (*_extFunc)();
  }
  if (!_epv)
  {
    PyErr_SetString(PyExc_ImportError, "Cannot load SIDL implementation for ASE.Compiler.Cxx.Attribute");
    return;
  }
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
