#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_IORGenerator
#include "ASE/Compiler/IORGenerator_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

#ifndef included_ASE_BaseException
#include "ASE/BaseException_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

static const struct ASE_Compiler_IORGenerator__external *_epv = NULL;

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_IORGenerator_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_IORGenerator__object *ior;
  enum ReferenceType refType;
};

static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_IORGenerator_PythonObject *self = (struct ASE_Compiler_IORGenerator_PythonObject *) type->tp_alloc(type, 0);
    
    if (self)
    {
      struct ASE_BaseInterface__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseInterface", NULL);
      
      self->ior = (*(ior->d_epv->castObject))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      if (refType == NEW_REF)
      {
        (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_BaseInterface__wrap_ior(struct ASE_BaseInterface__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseInterface");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseInterface");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseInterface");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseInterface");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_IORGenerator__wrap_ior(struct ASE_Compiler_IORGenerator__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.IORGenerator");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "IORGenerator");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.IORGenerator");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.IORGenerator");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_BaseException__wrap_ior(struct ASE_BaseException__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseException");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseException");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseException");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseException");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_CompilerException__wrap_ior(struct ASE_Compiler_CompilerException__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "CompilerException");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.CompilerException");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.CompilerException");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Vertex__wrap_ior(struct ASE_Compiler_Vertex__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Vertex");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vertex");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Vertex");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Vertex");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_Compiler_Vertex__extract_ior(struct ASE_Compiler_IORGenerator_PythonObject *obj, struct ASE_Compiler_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Vertex");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_IORGenerator__extract_ior(struct ASE_Compiler_IORGenerator_PythonObject *obj, struct ASE_Compiler_IORGenerator__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.IORGenerator", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.IORGenerator");
    return 0;
  }
  return 1;
}

static long ASE_BaseInterface__extract_ior(struct ASE_Compiler_IORGenerator_PythonObject *obj, struct ASE_BaseInterface__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.BaseInterface", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.BaseInterface");
    return 0;
  }
  return 1;
}

#ifndef included_string
#include "string_IOR.h"
#endif

static PyObject *string__wrap_array(struct string__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct string__external *_extFunc = string__externals();
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    long dimen = (*_extFunc->getDimen)(array, NULL);
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (array)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        const char *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        if (elem)
        {
          *(PyObject **) data = PyString_FromString(elem);
          free((char *) elem);
        }
        else
        {
          Py_INCREF(Py_None);
          *(PyObject **) data = Py_None;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static PyObject *ASE_Compiler_Vertex__wrap_array(struct ASE_Compiler_Vertex__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    long dimen = (*_extFunc->getDimen)(array, NULL);
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (array)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Vertex__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Vertex__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_string
#include "string_IOR.h"
#endif

static long string__extract_array(PyObject *obj, struct string__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct string__external *_extFunc = string__externals();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      const char *elem;
      
      if (!(elem = PyString_AsString(*(PyObject **) data)))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      /* Copying the Python string to IOR controlled memory */
      elem = elem ? strcpy(malloc(strlen(elem) + 1), elem) : NULL;
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static long ASE_Compiler_Vertex__extract_array(PyObject *obj, struct ASE_Compiler_Vertex__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Vertex__object *elem;
      
      if (!ASE_Compiler_Vertex__extract_ior((struct ASE_Compiler_IORGenerator_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getReferenceCount))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addReference))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->deleteReference))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  int _okay = 0;
  struct ASE_BaseInterface__object *obj = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_BaseInterface__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isSame))(_self_ior->d_object, obj, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getClassName))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getName))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  int _okay = 0;
  const char *name = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setName))(_self_ior->d_object, name, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  int _okay = 0;
  const char *typeName = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isInstanceOf))(_self_ior->d_object, typeName, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getSuper(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_BaseInterface__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSuper");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getSuper");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getSuper))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_BaseInterface__wrap_ior, _return_value);
  (*(((struct ASE_BaseInterface__object *) _return_value)->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Visitor_getRepository(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_Compiler_Vertex__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getRepository");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getRepository");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getRepository))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Vertex__wrap_array, _return_value);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Visitor_setRepository(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "repository",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Vertex__array *repository = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setRepository");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setRepository");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_array, &repository);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setRepository))(_self_ior->d_object, repository, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(repository, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Visitor_visitVertex(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "vertex",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Vertex__object *vertex = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitVertex");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitVertex");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_ior, &vertex);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitVertex))(_self_ior->d_object, vertex, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Generator_getSourceLanguages(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  struct string__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSourceLanguages");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getSourceLanguages");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getSourceLanguages))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) string__wrap_array, _return_value);
  (*(string__externals()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Generator_setSourceLanguages(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "outputLanguages",
    NULL};
  int _okay = 0;
  struct string__array *outputLanguages = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setSourceLanguages");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setSourceLanguages");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) string__extract_array, &outputLanguages);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setSourceLanguages))(_self_ior->d_object, outputLanguages, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Generator_getSource(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "language",
    NULL};
  int _okay = 0;
  const char *language = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_Compiler_Vertex__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSource");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getSource");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &language);
  /* Copying the Python string to IOR controlled memory */
  language = language ? strcpy(malloc(strlen(language) + 1), language) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getSource))(_self_ior->d_object, language, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Vertex__wrap_array, _return_value);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Generator_setSource(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "language",
    (char *) "vertices",
    NULL};
  int _okay = 0;
  const char *language = NULL;
  struct ASE_Compiler_Vertex__array *vertices = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setSource");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setSource");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zO&", _kwlist, &language, (void *) ASE_Compiler_Vertex__extract_array, &vertices);
  /* Copying the Python string to IOR controlled memory */
  language = language ? strcpy(malloc(strlen(language) + 1), language) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setSource))(_self_ior->d_object, language, vertices, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(vertices, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Generator_addSource(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "language",
    (char *) "vertices",
    NULL};
  int _okay = 0;
  const char *language = NULL;
  struct ASE_Compiler_Vertex__array *vertices = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addSource");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addSource");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zO&", _kwlist, &language, (void *) ASE_Compiler_Vertex__extract_array, &vertices);
  /* Copying the Python string to IOR controlled memory */
  language = language ? strcpy(malloc(strlen(language) + 1), language) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addSource))(_self_ior->d_object, language, vertices, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(vertices, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Generator_removeSource(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_IORGenerator__object *_self_ior = ((struct ASE_Compiler_IORGenerator_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "language",
    (char *) "vertices",
    NULL};
  int _okay = 0;
  const char *language = NULL;
  struct ASE_Compiler_Vertex__array *vertices = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for removeSource");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in removeSource");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "zO&", _kwlist, &language, (void *) ASE_Compiler_Vertex__extract_array, &vertices);
  /* Copying the Python string to IOR controlled memory */
  language = language ? strcpy(malloc(strlen(language) + 1), language) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->removeSource))(_self_ior->d_object, language, vertices, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(vertices, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_IORGenerator_IORGenerator(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_IORGenerator__object *obj = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_Compiler_IORGenerator__object *_return_value = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) ASE_Compiler_IORGenerator__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_IORGenerator__wrap_ior, _return_value);
  (*(((struct ASE_BaseInterface__object *) _return_value)->d_epv->addReference))(((struct ASE_BaseInterface__object *) _return_value)->d_object, NULL);
  return _pyReturn;
}

static struct PyMethodDef _IORGeneratorObjectMethods[] = {
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "isSame", (PyCFunction) ASE_BaseInterface_isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in ASE.BaseInterface obj)\n\
RETURNS (return bool _return_value)\n\
 Return true if and only if <code>obj</code> refers to the same underlying object as this object"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "getSuper", (PyCFunction) ASE_BaseInterface_getSuper, (METH_VARARGS | METH_KEYWORDS), (char *) "getSuper()\n\
RETURNS (return ASE.BaseInterface _return_value)\n\
 Retrieve an object with the parent interface IMPL: Obviously, this must be implemented in the IOR of each interface/class"},
  {(char *) "getRepository", (PyCFunction) ASE_Compiler_Visitor_getRepository, (METH_VARARGS | METH_KEYWORDS), (char *) "getRepository()\n\
RETURNS (return array _return_value)\n\
 Retrieve auxilliary ASTs"},
  {(char *) "setRepository", (PyCFunction) ASE_Compiler_Visitor_setRepository, (METH_VARARGS | METH_KEYWORDS), (char *) "setRepository(in array repository)\n\
RETURNS None\n\
\n\
 Set auxilliary ASTs"},
  {(char *) "visitVertex", (PyCFunction) ASE_Compiler_Visitor_visitVertex, (METH_VARARGS | METH_KEYWORDS), (char *) "visitVertex(in ASE.Compiler.Vertex vertex)\n\
RETURNS None\n\
\n\
"},
  {(char *) "getSourceLanguages", (PyCFunction) ASE_Compiler_Generator_getSourceLanguages, (METH_VARARGS | METH_KEYWORDS), (char *) "getSourceLanguages()\n\
RETURNS (return array _return_value)\n\
 Retrieve the list of the languages in which ASTs are produced"},
  {(char *) "setSourceLanguages", (PyCFunction) ASE_Compiler_Generator_setSourceLanguages, (METH_VARARGS | METH_KEYWORDS), (char *) "setSourceLanguages(in array outputLanguages)\n\
RETURNS None\n\
\n\
 Set the list of the languages in which ASTs are produced"},
  {(char *) "getSource", (PyCFunction) ASE_Compiler_Generator_getSource, (METH_VARARGS | METH_KEYWORDS), (char *) "getSource(in string language)\n\
RETURNS (return array _return_value)\n\
 Retrieve the list of source ASTs for the given language   Throw an exception if the language does not exist"},
  {(char *) "setSource", (PyCFunction) ASE_Compiler_Generator_setSource, (METH_VARARGS | METH_KEYWORDS), (char *) "setSource(in string language, in array vertices)\n\
RETURNS None\n\
\n\
 Set the list of source ASTs for the given language   Throw an exception if the language does not exist"},
  {(char *) "addSource", (PyCFunction) ASE_Compiler_Generator_addSource, (METH_VARARGS | METH_KEYWORDS), (char *) "addSource(in string language, in array vertices)\n\
RETURNS None\n\
\n\
 Add to the list of source ASTs for the given language   Throw an exception if the language does not exist"},
  {(char *) "removeSource", (PyCFunction) ASE_Compiler_Generator_removeSource, (METH_VARARGS | METH_KEYWORDS), (char *) "removeSource(in string language, in array vertices)\n\
RETURNS None\n\
\n\
 Remove to the list of source ASTs for the given language   Throw an exception if a vertex or language does not exist"},
  {NULL, NULL}};

static struct PyMethodDef _IORGeneratorModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct ASE_Compiler_IORGenerator_PythonObject *self)
{
  if (self->refType != WEAK_REF)
  {
    (*(((struct ASE_BaseInterface__object *) self->ior)->d_epv->deleteReference))(self->ior->d_object, NULL);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct ASE_Compiler_IORGenerator_PythonObject *self, PyObject *arg)
{
  struct ASE_Compiler_IORGenerator__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_IORGenerator_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct ASE_Compiler_IORGenerator_PythonObject *self)
{
  struct ASE_Compiler_IORGenerator__object *ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior->d_object);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct ASE_Compiler_IORGenerator_PythonObject *self)
{
  struct ASE_Compiler_IORGenerator__object *ior = self->ior;
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  
  _str = (*(ior->d_epv->strObject))(ior, NULL);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct ASE_Compiler_IORGenerator_PythonObject *self, PyObject *arg, int op)
{
  struct ASE_Compiler_IORGenerator__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_IORGenerator_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "Could not parse arguments for sidl_richcmp");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "ASE.Compiler.IORGenerator",
  sizeof(struct ASE_Compiler_IORGenerator_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
  (char *) "This is a Python wrapper for a SIDL object of type ASE.Compiler.IORGenerator",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  0,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _IORGeneratorObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) ASE_Compiler_IORGenerator_IORGenerator,
  (freefunc) 0,
  (inquiry) 0};

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

void initIORGenerator(void)
{
  PyObject *module = NULL;
  const struct ASE_Compiler_IORGenerator__external *(*_extFunc)(void);
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "IORGenerator", _IORGeneratorModuleMethods, (char *) " This is merely a tagging interface to indicate that IOR code is generated");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module ASE.Compiler.IORGenerator");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "IORGenerator", (PyObject *) &SIDLPythonObjectType);
  /* Set externals function */
  _extFunc = (const struct ASE_Compiler_IORGenerator__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(&_exception)->getClassMethodTable("ASE.Compiler.IORGenerator", &_exception);
  /* Check exception */
  if (_exception)
  {
    _exception = _exception->d_epv->castObject(_exception, "ASE.BaseException", NULL);
    PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    return;
  }
  /* Set epv pointer */
  if (_extFunc)
  {
    _epv = (*_extFunc)();
  }
  if (!_epv)
  {
    PyErr_SetString(PyExc_ImportError, "Cannot load SIDL implementation for ASE.Compiler.IORGenerator");
    return;
  }
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
