#!/usr/bin/env python
import user
import importer
import script

import os

def checkMemory(printClass = 0):
  import ASE.BaseInterface
  import ASE.BaseClass_impl
  printClass = printClass and hasattr(ASE.BaseInterface.BaseInterface, 'getIORClassName')
  for addr in ASE.BaseClass_impl.objects:
    if ASE.BaseClass_impl.objects[addr] == 1:
      if printClass:
        if hasattr(ASE.BaseInterface.BaseInterface, 'getIORClassName') and not ASE.BaseInterface.BaseInterface.getIORClassName(addr) == 'ASE.DLL':
          print 'Failed to destroy object',hex(addr),ASE.BaseInterface.BaseInterface.getIORClassName(addr)
      else:
        print 'Failed to destroy object',hex(addr),'(probably ASE.DLL)'
##    elif ASE.BaseClass_impl.objects[addr] == 2:
##      print 'Object',hex(addr),'successfully destroyed'
  return

class Scandal(script.Script):
  def __init__(self, clArgs = None, argDB = None):
    script.Script.__init__(self, clArgs, argDB)
    self.clear()
    return

  def setupHelp(self, help):
    import nargs

    defaultVisitors = {'ASE.Compiler.StubGenerator':           ['ASE.Compiler.Cxx.StubGenerator', 'ASE.Compiler.Python.StubGenerator'],
                       'ASE.Compiler.CartilageGenerator':      [],
                       'ASE.Compiler.IORGenerator':            ['ASE.Compiler.Cxx.IORGenerator'],
                       'ASE.Compiler.SkeletonGenerator':       ['ASE.Compiler.Python.SkeletonGenerator'],
                       'ASE.Compiler.ImplementationGenerator': ['ASE.Compiler.Cxx.ImplementationGenerator', 'ASE.Compiler.Python.ImplementationGenerator'],
                       'ASE.Compiler.Output':                  ['ASE.Compiler.Cxx.Output', 'ASE.Compiler.Python.Output']}

    help = script.Script.setupHelp(self, help)
    if '__file__' in globals():
      baseDir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))
    else:
      baseDir = os.path.dirname(os.path.dirname(os.path.abspath(os.getcwd())))
    help.addArgument('Scandal', 'baseDirectory', nargs.ArgDir(None, baseDir, 'The base directory for the ASE installation', isTemporary = 1))
    help.addArgument('Scandal', 'includes',     nargs.Arg(None, [],    'List of include files',                        isTemporary = 1)) # TODO: Should be a list
    help.addArgument('Scandal', 'debug',        nargs.ArgBool(None, 0, 'Flag to spawn the debugger',                   isTemporary = 1))
    help.addArgument('Scandal', 'noStackTrace', nargs.ArgBool(None, 1, 'Suppress the stack trace on error'))
    help.addArgument('Scandal', 'client',       nargs.Arg(None, [],    'Client language or list of languages',         isTemporary = 1))
    help.addArgument('Scandal', 'clientDirs',   nargs.Arg(None, {},    'Dictionary of client directories by language', isTemporary = 1)) # TODO: Should be a dict
    help.addArgument('Scandal', 'server',       nargs.Arg(None, [],    'Server language or list of languages',         isTemporary = 1))
    help.addArgument('Scandal', 'serverDirs',   nargs.Arg(None, {},    'Dictionary of server directories by language', isTemporary = 1)) # TODO: Should be a dict
    help.addArgument('Scandal', 'ior=<no,yes,client,server>', nargs.ArgFuzzyBool(None, 1, 'IOR generation flag', isTemporary = 1))
    help.addArgument('Scandal', 'outputSIDLFiles',  nargs.ArgBool(None, 1, 'Flag to output files generated from SIDL', isTemporary = 1))
    help.addArgument('Scandal', 'outputFiles',  nargs.ArgBool(None, 0, 'Flag to output dictionary of generated files', isTemporary = 1))
    help.addArgument('Scandal', 'defaultVisitors', nargs.Arg(None, defaultVisitors, 'A dictionary of visitor classes', isTemporary = 1)) # TODO: Should be a dict
    help.addArgument('Scandal', 'includeDirs',       nargs.Arg(None, {}, 'The dictionary of server include directories by language'))
    help.addArgument('Scandal', 'systemIncludeDirs', nargs.Arg(None, {'Cxx':['/usr/include', '/usr/include/linux', '/usr/local/include', '/usr/lib/gcc-lib/i486-linux/3.3.4/include']}, 'The dictionary of server system include directories by language'))
    return help

  def setupPaths(self):
    '''If Scandal was called as a script, setup both the python include path and the loader search path'''
    if not __name__ == '__main__':
      return
    import sys
    dirs = [os.path.join(self.baseDirectory, 'ase', 'Runtime'),
            os.path.join(self.baseDirectory, 'ase', 'Compiler')]
    serverDirs = [os.path.join(dirs[0], 'server-python-ase'),
                  os.path.join(dirs[1], 'server-python-compiler'),
                  os.path.join(dirs[1], 'server-python-sidl'),
                  os.path.join(dirs[1], 'server-python-sidlVisitor'),
                  os.path.join(dirs[1], 'server-python-pythonI'),
                  os.path.join(dirs[1], 'server-python-pythonII'),
                  os.path.join(dirs[1], 'server-python-pythonIII'),
                  os.path.join(dirs[1], 'server-python-pythonVisitor'),
                  os.path.join(dirs[1], 'server-python-bootstrap'),
                  os.path.join(dirs[1], 'server-python-cxxI'),
                  os.path.join(dirs[1], 'server-python-cxxII'),
                  os.path.join(dirs[1], 'server-python-cxxIII'),
                  os.path.join(dirs[1], 'server-python-cxxIV'),
                  os.path.join(dirs[1], 'server-python-cxxV'),
                  os.path.join(dirs[1], 'server-python-cxxVisitor')]
    sys.path.extend([os.path.join(self.baseDirectory, 'ply', 'ply-dev')])
    sys.path.extend([os.path.join(d, 'client-bootstrap') for d in dirs])
    sys.path.extend([os.path.join(d, 'client-python') for d in dirs])
    sys.path.extend([d for d in serverDirs])
    import ASE.Loader
    ASE.Loader.Loader.setSearchPath([os.path.join(d, 'lib') for d in dirs])
    return

  def getBaseDirectory(self):
    if not hasattr(self, '_baseDirectory'):
      return self.argDB['baseDirectory']
    return self._baseDirectory
  def setBaseDirectory(self, baseDirectory):
    self._baseDirectory = baseDirectory
    return
  baseDirectory = property(getBaseDirectory, setBaseDirectory, doc = 'The resolver for SIDL references')

  def clear(self):
    '''Reset the compiler, discarding all cached results'''
    import sets

    self.visitors           = {}
    self.generationTypes    = sets.Set()
    self.outputFiles        = {}
    return

  def getParser(self):
    if not hasattr(self, '_parser'):
      import ASE.Loader
      import ASE.Compiler.Parser

      cls = 'ASE.Compiler.SIDL.Parser'
      p = ASE.Compiler.Parser.Parser(ASE.Loader.Loader.createClass(cls))
      if not p:
        raise RuntimeError('Unable to instantiate the SIDL parser: '+cls)
      self._parser = p
    return self._parser
  def setParser(self, p):
    self._parser = p
    return
  parser = property(getParser, setParser, doc = 'The SIDL parser')

  def loadVisitor(self, className, defaultClass = None, castToVisitor = 1):
    '''Create a visitor of the given class
       - If defaultClass is specified, one will be created on failure to load class'''
    import ASE.Loader
    import ASE.Compiler.SIDL.Visitor

    cls     = className
    visitor = ASE.Loader.Loader.createClass(cls)
    if not visitor:
      if defaultClass:
        cls     = defaultClass
        visitor = ASE.Loader.Loader.createClass(cls)
        if not visitor:
          raise RuntimeError('Unable to instantiate visitor: '+className+' or '+defaultClass)
        else:
          self.logPrint('Loaded visitor: '+cls, debugSection = 'scandal')
      else:
        raise RuntimeError('Unable to instantiate visitor: '+cls)
    else:
      self.logPrint('Loaded visitor: '+cls, debugSection = 'scandal')
    if castToVisitor:
      try:
        visitor = ASE.Compiler.SIDL.Visitor.Visitor(visitor)
      except TypeError:
        raise RuntimeError('Class does not implement ASE.Compiler.SIDL.Visitor: '+cls)
    return visitor

  def getResolver(self):
    if not hasattr(self, '_resolver'):
      self._resolver = self.loadVisitor('ASE.Compiler.SIDL.Resolver')
    return self._resolver
  def setResolver(self, resolver):
    self._resolver = resolver
    return
  resolver = property(getResolver, setResolver, doc = 'The resolver for SIDL references')

  def getValidator(self):
    if not hasattr(self, '_validator'):
      self._validator = self.loadVisitor('ASE.Compiler.SIDL.Validator')
    return self._validator
  def setValidator(self, validator):
    self._validator = validator
    return
  validator = property(getValidator, setValidator, doc = 'The validator for SIDL objects')

  def getMethodProcessor(self):
    if not hasattr(self, '_methodProcessor'):
      self._methodProcessor = self.loadVisitor('ASE.Compiler.SIDL.MethodProcessor')
    return self._methodProcessor
  def setMethodProcessor(self, methodProcessor):
    self._methodProcessor = methodProcessor
    return
  methodProcessor = property(getMethodProcessor, setMethodProcessor, doc = 'The SIDL method generator')

  def getDependencyAnalyzer(self):
    if not hasattr(self, '_dependencyAnalyzer'):
      self._dependencyAnalyzer = self.loadVisitor('ASE.Compiler.SIDL.DependencyAnalyzer')
    return self._dependencyAnalyzer
  def setDependencyAnalyzer(self, dependencyAnalyzer):
    self._dependencyAnalyzer = dependencyAnalyzer
    return
  dependencyAnalyzer = property(getDependencyAnalyzer, setDependencyAnalyzer, doc = 'The dependencyAnalyzer for SIDL references')

  def getMerger(self):
    if not hasattr(self, '_merger'):
      self._merger = self.loadVisitor('ANL.SIDL.PackageMerge')
    return self._merger
  def setMerger(self, merger):
    self._merger = merger
    return
  merger = property(getMerger, setMerger, doc = 'The merger for SIDL references')

  def getClients(self):
    if not hasattr(self, '_clients'):
      clients = self.argDB['client']
      if not isinstance(clients, list): clients = [clients]
      self._clients = clients
    return self._clients
  def setClients(self, clients):
    self._clients = clients
    return
  clients = property(getClients, setClients, doc = 'The list of client languages')

  def getClientDirs(self):
    if not hasattr(self, '_clientDirs'):
      self._clientDirs = self.argDB['clientDirs']
    return self._clientDirs
  def setClientDirs(self, clientDirs):
    self._clientDirs = clientDirs
    return
  clientDirs = property(getClientDirs, setClientDirs, doc = 'The mapping of client language to output directory')

  def getServers(self):
    if not hasattr(self, '_servers'):
      servers = self.argDB['server']
      if not isinstance(servers, list): servers = [servers]
      self._servers = servers
    return self._servers
  def setServers(self, servers):
    self._servers = servers
    return
  servers = property(getServers, setServers, doc = 'The list of server languages')

  def getServerDirs(self):
    if not hasattr(self, '_serverDirs'):
      self._serverDirs = self.argDB['serverDirs']
    return self._serverDirs
  def setServerDirs(self, serverDirs):
    self._serverDirs = serverDirs
    return
  serverDirs = property(getServerDirs, setServerDirs, doc = 'The mapping of server language to output directory')

  def getIncludes(self):
    if not hasattr(self, '_includes'):
      return self.argDB['includes']
    return self._includes
  def setIncludes(self, includes):
    if not isinstance(includes, list):
      includes = [includes]
    self._includes = includes
    return
  includes = property(getIncludes, setIncludes, doc = 'The list of include filenames')

  def getIncludeDirectories(self):
    if not hasattr(self, '_includeDirectories'):
      return self.argDB['includeDirs']
    return self._includeDirectories
  def setIncludeDirectories(self, includeDirectories):
    self._includeDirectories = includeDirectories
    return
  includeDirectories = property(getIncludeDirectories, setIncludeDirectories, doc = 'The list of include directories')

  def getSystemIncludeDirectories(self):
    if not hasattr(self, '_systemIncludeDirectories'):
      return self.argDB['systemIncludeDirs']
    return self._systemIncludeDirectories
  def setSystemIncludeDirectories(self, systemIncludeDirectories):
    self._systemIncludeDirectories = systemIncludeDirectories
    return
  systemIncludeDirectories = property(getSystemIncludeDirectories, setSystemIncludeDirectories, doc = 'The list of system include directories')

  def getOutputSIDLFiles(self):
    if not hasattr(self, '_outputSIDLFiles'):
      return self.argDB['outputSIDLFiles']
    return self._outputSIDLFiles
  def setOutputSIDLFiles(self, outputSIDLFiles):
    self._outputSIDLFiles = outputSIDLFiles
    return
  outputSIDLFiles = property(getOutputSIDLFiles, setOutputSIDLFiles, doc = 'The flag for writing generated files to disk')

  def parseIncludes(self):
    '''Parse the includes and return a list of AST objects'''
    # Could check that they are files with os.path.isfile(include)
    return [self.parser.parseFile(include) for include in self.includes]

  def getTargets(self):
    if not hasattr(self, '_targets'):
      return self.argDB.target
    return self._targets
  def setTargets(self, targets):
    if not isinstance(targets, list):
      targets = [targets]
    self._targets = targets
    return
  targets = property(getTargets, setTargets, doc = 'The list of target filenames')

  def parseTargets(self):
    '''Read the target filenames from the argument database'''
    import sys

    if self.targets == ['default']:
      print 'Usage: '+sys.argv[0]+' [options] <sidl files>'
      sys.exit(1)
    # Could check that they are files with os.path.isfile(include)
    return [self.parser.parseFile(target) for target in self.targets]

  def createVisitor(self, phase, cls, interfaceCheck = None, subType = None):
    '''Load a visitor, if necessary
       - If interfaceCheck is provided, check that the cast succeeds'''
    if phase in self.visitors:
      if cls in self.visitors[phase]:
        self.logPrint('Duplicate class '+cls+' in phase '+phase, debugSection = 'screen')
        if not phase.startswith('output'):
          return
    else:
      self.visitors[phase] = {}
    v = self.loadVisitor(cls, castToVisitor = not interfaceCheck)
    if interfaceCheck:
      if not isinstance(interfaceCheck, list): interfaceCheck = [interfaceCheck]
      cast = None
      for interface in interfaceCheck:
        mod  = self.importModule(interface)
        try:
          cast = getattr(mod, interface.split('.')[-1])(v)
        except TypeError:
          pass
        if not cast is None:
          break
      if cast is None:
        raise RuntimeError('Visitor '+cls+' failed to cast to '+str(interfaceCheck))
      else:
        v = cast
    if not cls in self.visitors[phase]:
      self.visitors[phase][cls] = []
    self.visitors[phase][cls].append((subType, v))
    return v

  def createVisitors(self, phase, classes, interfaceCheck = None, setupFunc = None, ignoreFailure = 0, subType = None):
    '''Load a set of visitors for the given phase
       - If interfaceCheck is provided, check that the cast succeeds
       - If setupFunc is provided, call it with each visitor object'''
    if not isinstance(classes, list):
      classes = [classes]
    for cls in classes:
      try:
        v = self.createVisitor(phase, cls, interfaceCheck, subType)
        if not setupFunc is None:
          setupFunc(v)
      except Exception, e:
        if not ignoreFailure:
          raise e
        else:
          self.logPrint('Failed to create visitor '+cls+': '+str(e), debugSection = 'scandal')
    return self.visitors

  def getModuleClasses(self, name):
    import sets

    mod  = self.importModule(name)
    mods = []
    for f in os.listdir(mod.__file__):
      (modName, ext) = os.path.splitext(f)

      if modName == '__init__': continue
      if ext == '.py':
        mods.append(name+'.'+modName)
      if ext == '.so':
        mods.append(name+'.'+modName)
    return list(sets.Set(mods))

  def createGenerators(self, language, setupFunc, interfaceCheck = 'ASE.Compiler.Generator', type = None):
    try:
      if type is None:
        type = language
      if not isinstance(interfaceCheck, list): interfaceCheck = [interfaceCheck]
      classes = []
      for interface in interfaceCheck:
        classes.extend(filter(lambda klass: klass.startswith('ASE.Compiler.'+language), self.argDB['defaultVisitors'][interface]))
      if not len(classes):
        classes = self.getModuleClasses('ASE.Compiler.'+language)
      visitors = self.createVisitors('generation '+type, classes, interfaceCheck, setupFunc, ignoreFailure = 1)
      self.generationTypes.add(type)
      return visitors
    except ImportError, e:
      raise RuntimeError('Could not load any code generators for '+language+': '+str(e))

  def setupClientOutput(self, output, dir):
    import ASE.Compiler.CodePurpose

    output.setRoot(ASE.Compiler.CodePurpose.STUB, dir)
    output.setRoot(ASE.Compiler.CodePurpose.CARTILAGE, dir)
    output.setDisableOutput(not self.outputSIDLFiles)
    return

  def setupServerOutput(self, output, dir, includes = [], systemIncludes = []):
    import ASE.Compiler.CodePurpose

    output.setRoot(ASE.Compiler.CodePurpose.SKELETON, dir)
    output.setRoot(ASE.Compiler.CodePurpose.IMPLEMENTATION, dir)
    output.setIncludeDirectories(includes)
    output.setSystemIncludeDirectories(systemIncludes)
    output.setDisableOutput(not self.outputSIDLFiles)
    return

  def setupIOROutput(self, output, dir, headerOnly = 0):
    import ASE.Compiler.CodePurpose

    output.setRoot(ASE.Compiler.CodePurpose.IOR, dir)
    output.setDisableOutput(not self.outputSIDLFiles)
    if headerOnly:
      output.setDisableSource(1)
    return

  def createOutputs(self, language, outputLanguage, setupFunc, interfaceCheck = 'ASE.Compiler.Output', subType = None):
    try:
      if not isinstance(interfaceCheck, list): interfaceCheck = [interfaceCheck]
      classes = []
      for interface in interfaceCheck:
        classes.extend(filter(lambda klass: klass.startswith('ASE.Compiler.'+outputLanguage), self.argDB['defaultVisitors'][interface]))
      if not len(classes):
        classes = self.getModuleClasses('ASE.Compiler.'+outputLanguage)
      return self.createVisitors('output '+language+' '+outputLanguage, classes, interfaceCheck, setupFunc, ignoreFailure = 1, subType = subType)
    except ImportError, e:
      raise RuntimeError('Could not load any outputter for '+outputLanguage+': '+str(e))

  def setupClients(self, clients):
    self.logPrint('Creating clients', debugSection = 'scandal')
    for client in clients:
      try:
        clientDir = self.clientDirs[client]
      except KeyError:
        raise RuntimeError('Could not find a client directory')
        clientDir = 'tmp'
        self.clientDirs[client] = clientDir
        self.logPrint('Using default root \''+clientDir+'\' for client '+client, debugSection = 'scandal')
      self.createGenerators(client, None, ['ASE.Compiler.StubGenerator', 'ASE.Compiler.CartilageGenerator'])
      for dummy, visitor in reduce(lambda x,y: x+y, self.visitors['generation '+client].values(), []):
        for language in visitor.getSourceLanguages():
          self.createOutputs(client, language, lambda o: self.setupClientOutput(o, clientDir))
    return

  def setupServers(self, servers):
    self.logPrint('Creating servers', debugSection = 'scandal')
    for server in servers:
      try:
        serverDir = self.serverDirs[server]
      except:
        raise RuntimeError('Could not find a server directory')
        serverDir = 'tmp'
        self.serverDirs[server] = serverDir
        self.logPrint('Using default root \''+clientDir+'\' for server '+server, debugSection = 'scandal')
      self.createGenerators(server, None, ['ASE.Compiler.SkeletonGenerator', 'ASE.Compiler.ImplementationGenerator'])
      for dummy, visitor in reduce(lambda x,y: x+y, self.visitors['generation '+server].values(), []):
        for language in visitor.getSourceLanguages():
          if not language in self.includeDirectories:
            self.includeDirectories[language] = []
          if not language in self.systemIncludeDirectories:
            self.systemIncludeDirectories[language] = []
          self.createOutputs(server, language, lambda o: self.setupServerOutput(o, serverDir, self.includeDirectories[language], self.systemIncludeDirectories[language]))
    return

  def setupIOR(self, clients, servers):
    if self.argDB['ior'] == 0:
      return
    self.logPrint('Creating IOR', debugSection = 'scandal')
    if len(clients) or len(servers) or self.argDB['ior'] == 2 or self.argDB['ior'] == 3:
      self.createGenerators('Cxx', None, 'ASE.Compiler.IORGenerator', type = 'IOR')
    if self.argDB['ior'] == 1 or self.argDB['ior'] == 2:
      if len(clients):
        for client in clients:
          for dummy, visitor in reduce(lambda x,y: x+y, self.visitors['generation IOR'].values(), []):
            for language in visitor.getSourceLanguages():
              self.createOutputs('IOR', language, lambda o: self.setupIOROutput(o, self.clientDirs[client], headerOnly = 1), subType = client)
      elif self.argDB['ior'] == 2:
        for client in self.clientDirs:
          for dummy, visitor in reduce(lambda x,y: x+y, self.visitors['generation IOR'].values(), []):
            for language in visitor.getSourceLanguages():
              self.createOutputs('IOR', language, lambda o: self.setupIOROutput(o, self.clientDirs[client], headerOnly = 1), subType = client)
    if self.argDB['ior'] == 1 or self.argDB['ior'] == 3:
      if len(servers):
        for server in servers:
          for dummy, visitor in reduce(lambda x,y: x+y, self.visitors['generation IOR'].values(), []):
            for language in visitor.getSourceLanguages():
              self.createOutputs('IOR', language, lambda o: self.setupIOROutput(o, self.serverDirs[server]), subType = server)
      elif self.argDB['ior'] == 3:
        for server in self.serverDirs:
          for dummy, visitor in reduce(lambda x,y: x+y, self.visitors['generation IOR'].values(), []):
            for language in visitor.getSourceLanguages():
              self.createOutputs('IOR', language, lambda o: self.setupIOROutput(o, self.serverDirs[server]), subType = server)
    return

  def setupVisitors(self):
    self.logPrint('Creating Visitors', debugSection = 'scandal')
    self.setupClients(self.clients)
    self.setupServers(self.servers)
    self.setupIOR(self.clients, self.servers)
    return

  def addOutputFiles(self, type, subType, language, files):
    import sets

    if not subType is None:
      type = type+' '+subType
    if not type in self.outputFiles:
      self.outputFiles[type] = {}
    if not language in self.outputFiles[type]:
      self.outputFiles[type][language] = sets.Set()
    self.outputFiles[type][language].union_update(sets.Set(files))
    return

  def processIncludes(self, includes):
    '''Process SIDL AST with all visitors which must process targets and includes'''
    import ASE.Compiler.Search
    self.logPrint('Resolving and validating sidl', debugSection = 'scandal')
    for visitor in [self.resolver, self.validator]:
      visitor.setRepository(includes)
      for tree in includes:
        tree.accept(visitor)
      visitor.setRepository([])
      ASE.Compiler.Search.Search(visitor).setVerticesSeen([])
    return

  def processTargets(self, targets, includes):
    '''Process SIDL AST with all visitors which must process just the targets'''
    import ASE.Compiler.Search
    self.logPrint('Analyzing sidl dependencies', debugSection = 'scandal')
    for visitor in [self.methodProcessor, self.dependencyAnalyzer]:
      visitor.setRepository(includes)
      for tree in targets:
        tree.accept(visitor)
      visitor.setRepository([])
      ASE.Compiler.Search.Search(visitor).setVerticesSeen([])
    return

  def generateTarget(self, target, visitors):
    '''Process SIDL AST with code generation visitors'''
    import ASE.Compiler.Search
    import ASE.Compiler.SIDL.Vertex
    import gc
    import types
    if self.argDB['debug']:
      originalObjects = filter(lambda c: c.__module__.endswith('_impl'), filter(lambda o: type(o) is types.InstanceType, gc.get_objects()))
      print len(originalObjects),'original objects'
    for genType in self.generationTypes:
      self.logPrint('Generating '+genType+'('+str(ASE.Compiler.SIDL.Vertex.Vertex(target).getFullIdentifier())+')', debugSection = 'scandal')
      for dummy, generator in reduce(lambda x,y: x+y, visitors['generation '+genType].values(), []):
        target.accept(generator)
        if self.argDB['debug']:
          print len(filter(lambda c: c.__module__.endswith('_impl'), filter(lambda o: type(o) is types.InstanceType, gc.get_objects()))),'objects after',genType,'generation'
        ASE.Compiler.Search.Search(generator).setVerticesSeen([])
        for lang in generator.getSourceLanguages():
          self.logPrint('  Outputing '+genType+' in '+lang, debugSection = 'scandal')
          for subType, output in reduce(lambda x,y: x+y, visitors['output '+genType+' '+lang].values(), []):
            output.setFiles([])
            trees = list(generator.getSource(lang))
            for tree in trees:
              tree.accept(output)
              ASE.Compiler.Search.Search(output).setVerticesSeen([])
              del tree
            self.addOutputFiles(genType, subType, lang, output.getFiles())
            del trees
          generator.setSource(lang, [])
          gc.collect()
    if self.argDB['debug']:
      newObjects = filter(lambda c: c.__module__.endswith('_impl'), filter(lambda o: type(o) is types.InstanceType, gc.get_objects()))
      if not len(originalObjects) == len(newObjects):
        for obj in originalObjects:
          newObjects.remove(obj)
        print len(newObjects),'leaked objects'
        print len(filter(lambda c: c.__module__.startswith('ASE.BaseClass'), newObjects)),'Basic leaked objects'
        print len(filter(lambda c: c.__module__.startswith('ASE.Compiler'), newObjects)),'Compiler leaked objects'
        print len(filter(lambda c: c.__module__.startswith('ASE.Compiler.Cxx'), newObjects)),'Cxx leaked objects'
        print len(filter(lambda c: c.__module__.startswith('ASE.Compiler.Python'), newObjects)),'Python leaked objects'
        print len(filter(lambda c: c.__module__.startswith('ASE.Compiler.SIDL'), newObjects)),'SIDL leaked objects'
        print filter(lambda c: c.__module__.startswith('ASE.Compiler.SIDL'), newObjects)
        if len(newObjects) < 100:
          print newObjects
        else:
          raise RuntimeError('FUCK, this shit does not work')
    return

  def walkTree(self, tree, genFunc, visitors):
    if tree.isInstanceOf('ASE.Compiler.SIDL.Enumeration') or tree.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      return genFunc(tree, visitors)
    for child in tree.getChildren():
      self.walkTree(child, genFunc, visitors)
    tree.setChildren([])
    return genFunc(tree, visitors)

  def processAST(self, includes, targets, visitors):
    '''Run over all trees with each visitor in turn
       - All visitors recieve all trees as a repository'''
    includes += targets
    self.processIncludes(includes)
    self.processTargets(targets, includes)
    for target in targets:
      self.walkTree(target, self.generateTarget, visitors)
    return

  def cleanup(self):
    '''It is necessary to call gc.collect() to handle the circular references in the ASTs'''
    import ASE.Loader
    import atexit
    import gc
    self.log.flush()
    self.clear()
    self.parser = None
    self.resolver = None
    self.validator = None
    self.methodProcessor = None
    self.dependencyAnalyzer = None
    self.merger = None
    import ASE.Compiler.SIDL.sidlparse as sidlparse
    sidlparse.baseTypes.clear()
    gc.collect()
    atexit.register(checkMemory)
    atexit.register(ASE.Loader.Loader.removeLibraries, ASE.Loader.Loader.getLibraries())
    return

  def run(self):
    self.clear()
    self.setup()
    self.setupPaths()
    self.setupVisitors()

    import ASE.BaseException
    import ASE.ASEException
    import ASE.Compiler.CompilerException
    try:
      self.processAST(self.parseIncludes(), self.parseTargets(), self.visitors)
      if self.argDB['outputFiles']:
        print repr(self.outputFiles)
    except (ASE.BaseException.Exception, ASE.ASEException.Exception, ASE.Compiler.CompilerException.Exception), e:
      import sys
      import traceback

      if not hasattr(e, 'getMessage'):
        e = e.args[0]
      print dir(e)
      print 'ERROR',e.getMessage()
      if not self.argDB['noStackTrace']:
        import traceback
        sys.stderr.write(e.getStackTrace())
        print traceback.print_tb(sys.exc_info()[2])
      self.cleanup()
      sys.exit(1)
    self.cleanup()
    return

if __name__ == '__main__':
  scandal = Scandal()
  scandal.run()
  del scandal
  checkMemory(1)
