#!/usr/bin/env python
import user
import importer
import script

import os
import sys

def checkMemory(printClass = 0):
  import ASE.BaseInterface
  import ASE.BaseClass_impl
  printClass = printClass and hasattr(ASE.BaseInterface.BaseInterface, 'getIORClassName')
  for addr in ASE.BaseClass_impl.objects:
    if ASE.BaseClass_impl.objects[addr] == 1:
      if printClass:
        if not ASE.BaseInterface.BaseInterface.getIORClassName(addr) == 'ASE.DLL':
          print 'Failed to destroy object',hex(addr),ASE.BaseInterface.BaseInterface.getIORClassName(addr)
      else:
        print 'Failed to destroy object',hex(addr),'(probably ASE.DLL)'
##    elif ASE.BaseClass_impl.objects[addr] == 2:
##      print 'Object',hex(addr),'successfully destroyed'
  return

class ParseTester(script.Script):
  def __init__(self):
    import RDict

    script.Script.__init__(self, argDB = RDict.RDict())
    return

  def setupHelp(self, help):
    import nargs

    supportedLanguages = [method[5:] for method in dir(self) if method.startswith('parse')]

    help = script.Script.setupHelp(self, help)
    if '__file__' in globals():
      baseDir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))))
    else:
      baseDir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(os.getcwd()))))))
    help.addArgument('Parser', '-baseDirectory',     nargs.ArgDir(None, baseDir, 'The base directory for the ASE installation', isTemporary = 1))
    help.addArgument('Parser', '-language',          nargs.Arg(None, 'Python', 'The language to parse and print, e.g. '+str(supportedLanguages)[1:-1]))
    help.addArgument('Parser', '-includeDirs',       nargs.Arg(None, {'Cxx': ['/home/knepley/PETSc-dev/petsc-dev/include', '/home/knepley/PETSc-dev/petsc-dev/bmake/linux-gnu', '/PETSc3/mpich/include']}, 'The list of include directories'))
    help.addArgument('Parser', '-systemIncludeDirs', nargs.Arg(None, {'Cxx': ['/usr/include', '/usr/include/linux', '/usr/local/include', '/usr/lib/gcc-lib/i486-linux/3.3.4/include']}, 'The list of system include directories'))
    help.addArgument('Parser', '-includes',          nargs.Arg(None, [], 'The list of include files'))
    help.addArgument('Parser', '-resolve',           nargs.ArgBool(None, 1, 'Flag indicating whether we should resolve references'))
    help.addArgument('Parser', '-outputFile',        nargs.Arg(None, None, 'The output filename'))
    return help

  def setupPaths(self):
    '''If Scandal was called as a script, setup both the python include path and the loader search path'''
    if not __name__ == '__main__':
      return
    import sys
    dirs = [os.path.join(self.baseDirectory, 'ase', 'Runtime'),
            os.path.join(self.baseDirectory, 'ase', 'Compiler')]
    serverDirs = [os.path.join(dirs[0], 'server-python-ase'),
                  os.path.join(dirs[1], 'server-python-compiler'),
                  os.path.join(dirs[1], 'server-python-sidl'),
                  os.path.join(dirs[1], 'server-python-sidlVisitor'),
                  os.path.join(dirs[1], 'server-python-pythonI'),
                  os.path.join(dirs[1], 'server-python-pythonII'),
                  os.path.join(dirs[1], 'server-python-pythonIII'),
                  os.path.join(dirs[1], 'server-python-pythonVisitor'),
                  os.path.join(dirs[1], 'server-python-bootstrap'),
                  os.path.join(dirs[1], 'server-python-cxxI'),
                  os.path.join(dirs[1], 'server-python-cxxII'),
                  os.path.join(dirs[1], 'server-python-cxxIII'),
                  os.path.join(dirs[1], 'server-python-cxxIV'),
                  os.path.join(dirs[1], 'server-python-cxxV'),
                  os.path.join(dirs[1], 'server-python-cxxVisitor')]
    sys.path.extend([os.path.join(self.baseDirectory, 'ply', 'ply-dev')])
    sys.path.extend([os.path.join(d, 'client-bootstrap') for d in dirs])
    sys.path.extend([os.path.join(d, 'client-python') for d in dirs])
    sys.path.extend([d for d in serverDirs])
    import ASE.Loader
    ASE.Loader.Loader.setSearchPath([os.path.join(d, 'lib') for d in dirs])
    return

  def cleanup(self):
    '''It is necessary to call gc.collect() to handle the circular references in the ASTs'''
    import ASE.Loader
    import atexit
    import gc
    self.log.flush()
    gc.collect()
    atexit.register(checkMemory)
    atexit.register(ASE.Loader.Loader.removeLibraries, ASE.Loader.Loader.getLibraries())
    return

  def getBaseDirectory(self):
    if not hasattr(self, '_baseDirectory'):
      return self.argDB['baseDirectory']
    return self._baseDirectory
  def setBaseDirectory(self, baseDirectory):
    self._baseDirectory = baseDirectory
    return
  baseDirectory = property(getBaseDirectory, setBaseDirectory, doc = 'The resolver for SIDL references')

  def parseSIDL(self, source):
    import ASE.Compiler.SIDL.Parser

    parser = ASE.Compiler.SIDL.Parser.Parser()
    ast    = parser.parseFile(source)
    if self.argDB['resolve']:
      import ASE.Compiler.SIDL.Resolver

      includes = [parser.parseFile(inc) for inc in self.argDB['includes']]
      resolver = ASE.Compiler.SIDL.Resolver.Resolver()
      resolver.setRepository(includes)
      ast.accept(resolver)
      del resolver
    if 'outputFile' in self.argDB:
      raise RuntimeError('Output cannot be sent to a file yet.')
    else:
      import ASE.Compiler.SIDL.Printer

      printer = ASE.Compiler.SIDL.Printer.Printer()
      ast.accept(printer)
      del printer
    del parser
    del ast
    import ASE.Compiler.SIDL.sidlparse as sidlparse
    sidlparse.baseTypes.clear()
    return

  def parseCpp(self, source):
    import ASE.Compiler.Cxx_impl.cIncludeExpansion

    expander = ASE.Compiler.Cxx_impl.cIncludeExpansion.IncludeExpander()
    expander.includeDirs       = self.argDB['includeDirs']
    expander.systemIncludeDirs = self.argDB['systemIncludeDirs']
    f = file(source)
    s = f.read()
    f.close()
    expander.input(s)
    for token in expander.handleFile():
      print token
    del expander
    return

  def parseC(self, source):
    return self.parseCxx(source)

  def parseCxx(self, source):
    import ASE.Compiler.Cxx.Parser

    parser = ASE.Compiler.Cxx.Parser.Parser()
    parser.setIncludeDirectories(self.argDB['includeDirs']['Cxx'])
    parser.setSystemIncludeDirectories(self.argDB['systemIncludeDirs']['Cxx'])
    ast    = parser.parseFile(source)
    if 'outputFile' in self.argDB:
      import ASE.Compiler.CodePurpose
      import ASE.Compiler.Cxx.Output

      output = ASE.Compiler.Cxx.Output.Output()
      output.setRoot(ASE.Compiler.CodePurpose.STUB, 'tmp-parser')
      ast.accept(output)
      del output
    else:
      import ASE.Compiler.Cxx.Printer

      printer = ASE.Compiler.Cxx.Printer.Printer()
      ast.accept(printer)
      del printer
    del parser
    del ast
    return

  def parsePython(self, source):
    import ASE.Compiler.Python.Parser

    parser = ASE.Compiler.Python.Parser.Parser()
    ast    = parser.parseFile(source)
    if 'outputFile' in self.argDB:
      import ASE.Compiler.CodePurpose
      import ASE.Compiler.Python.Output

      output = ASE.Compiler.Python.Output.Output()
      output.setRoot(ASE.Compiler.CodePurpose.STUB, 'tmp-parser')
      ast.accept(output)
      del output
    else:
      import ASE.Compiler.Python.Printer

      printer = ASE.Compiler.Python.Printer.Printer()
      ast.accept(printer)
      del printer
    del parser
    del ast
    return

  def run(self):
    self.setup()
    self.setupPaths()

    import ASE.BaseException
    import ASE.ASEException
    try:
      for source in self.argDB.target:
        if not os.path.isfile(source):
          self.logPrint('Error: Source file '+source+' does not exist', debugSection = 'screen')
          continue
        parser = 'parse'+self.argDB['language']
        if not hasattr(self, parser):
          self.logPrint('Error: Invalid language '+self.argDB['language']+': No parser', debugSection = 'screen')
        self.logPrint('Parsing '+source, debugSection = 'screen')
        getattr(self, parser)(source)
    except (ASE.BaseException.Exception, ASE.ASEException.Exception), e:
      import traceback

      print dir(e)
      exc = e.args[0]
      print exc.getMessage()
      sys.stderr.write(exc.getStackTrace())
      print traceback.print_tb(sys.exc_info()[2])
    self.cleanup()
    checkMemory(1)
    return

if __name__ == '__main__':
  ParseTester().run()
