#include <stdlib.h>

#include <string.h>

#ifndef included_ASE_Compiler_Parser
#include "ASE/Compiler/Parser_IOR.h"
#endif

#ifndef included_ASE_ASEException
#include "ASE/ASEException_IOR.h"
#endif

static struct ASE_Compiler_Parser__array *createArray(/*in*/ int32_t dimen, /*in*/ struct int__array *lower, /*in*/ struct int__array *upper, /*out*/ struct ASE_BaseException__object **_exception)
{
  static int32_t arraySize = sizeof(struct ASE_Compiler_Parser__array) + (sizeof(int32_t) - (sizeof(struct ASE_Compiler_Parser__array)) % sizeof(int32_t)) % sizeof(int32_t);
  struct ASE_Compiler_Parser__array *_return_value = (struct ASE_Compiler_Parser__array *) malloc(arraySize + 3 * sizeof(int32_t) * (dimen ? dimen : 1));
  int32_t size = 1;
  int32_t d = 0;
  
  /* Initialize array structure */
  _return_value->d_dimen = dimen;
  _return_value->d_borrowed = 0;
  _return_value->d_lower = (int32_t *) ((char *) _return_value + arraySize);
  _return_value->d_upper = _return_value->d_lower + (dimen ? dimen : 1);
  _return_value->d_stride = _return_value->d_upper + (dimen ? dimen : 1);
  _return_value->d_lower[0] = 0;
  _return_value->d_upper[0] = 0;
  _return_value->d_stride[0] = 1;
  /* Determine array size and set bounds */
  for(d = 0; d < dimen; d++)
  {
    _return_value->d_lower[d] = lower->d_firstElement[d];
    _return_value->d_upper[d] = upper->d_firstElement[d];
    _return_value->d_stride[d] = size;
    size *= upper->d_firstElement[d] - lower->d_firstElement[d] + 1;
  }
  size *= sizeof(struct ASE_Compiler_Parser__object *);
  /* Allocate and initialize data */
  _return_value->d_firstElement = (struct ASE_Compiler_Parser__object **) malloc(size);
  memset(_return_value->d_firstElement, 0, size);
  return _return_value;
}

static struct ASE_Compiler_Parser__array *borrowArray(/*in*/ int32_t dimen, /*in*/ struct int__array *lower, /*in*/ struct int__array *upper, /*in*/ struct int__array *stride, /*in*/ void *first, /*out*/ struct ASE_BaseException__object **_exception)
{
  static int32_t arraySize = sizeof(struct ASE_Compiler_Parser__array) + (sizeof(int32_t) - (sizeof(struct ASE_Compiler_Parser__array)) % sizeof(int32_t)) % sizeof(int32_t);
  struct ASE_Compiler_Parser__array *_return_value = (struct ASE_Compiler_Parser__array *) malloc(arraySize + 3 * sizeof(int32_t) * (dimen ? dimen : 1));
  int32_t size = 1;
  int32_t d = 0;
  
  /* Initialize array structure */
  _return_value->d_dimen = dimen;
  _return_value->d_borrowed = 1;
  _return_value->d_lower = (int32_t *) ((char *) _return_value + arraySize);
  _return_value->d_upper = _return_value->d_lower + (dimen ? dimen : 1);
  _return_value->d_stride = _return_value->d_upper + (dimen ? dimen : 1);
  _return_value->d_lower[0] = 0;
  _return_value->d_upper[0] = 0;
  _return_value->d_stride[0] = 1;
  /* Determine array size and set bounds */
  for(d = 0; d < dimen; d++)
  {
    _return_value->d_lower[d] = lower->d_firstElement[d];
    _return_value->d_upper[d] = upper->d_firstElement[d];
    _return_value->d_stride[d] = stride->d_firstElement[d];
    size *= upper->d_firstElement[d] - lower->d_firstElement[d] + 1;
  }
  size *= sizeof(struct ASE_Compiler_Parser__object *);
  _return_value->d_firstElement = (struct ASE_Compiler_Parser__object **) first;
  return _return_value;
}

static void destroyArray(/*in*/ struct ASE_Compiler_Parser__array *array, /*out*/ struct ASE_BaseException__object **_exception)
{
}

static int32_t getDimen(/*in*/ struct ASE_Compiler_Parser__array *array, /*out*/ struct ASE_BaseException__object **_exception)
{
  return array->d_dimen;
}

static int32_t getLower(/*in*/ struct ASE_Compiler_Parser__array *array, /*in*/ int32_t dimen, /*out*/ struct ASE_BaseException__object **_exception)
{
  return array->d_lower[dimen];
}

static int32_t getUpper(/*in*/ struct ASE_Compiler_Parser__array *array, /*in*/ int32_t dimen, /*out*/ struct ASE_BaseException__object **_exception)
{
  return array->d_upper[dimen];
}

static struct ASE_Compiler_Parser__object *getElement(/*in*/ struct ASE_Compiler_Parser__array *array, /*in*/ struct int__array *indices, /*out*/ struct ASE_BaseException__object **_exception)
{
  struct ASE_Compiler_Parser__object *_result = NULL;
  struct ASE_Compiler_Parser__object **_element = NULL;
  int32_t d = 0;
  
  _element = array->d_firstElement;
  for(d = 0; d < array->d_dimen; d++)
  {
    if (indices->d_firstElement[d] < array->d_lower[d] || indices->d_firstElement[d] > array->d_upper[d])
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Invalid indices", _exception);
      if (!(*_exception))
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return _result;
    }
    _element += array->d_stride[d] * (indices->d_firstElement[d] - array->d_lower[d]);
  }
  if (*_element)
  {
    (*(((struct ASE_BaseInterface__object *) (*_element)->d_object)->d_epv->deleteReference))((*_element)->d_object, (void **) _exception);
  }
  return *_element;
}

static struct ASE_Compiler_Parser__object *getElement4(/*in*/ struct ASE_Compiler_Parser__array *array, /*in*/ int32_t i1, /*in*/ int32_t i2, /*in*/ int32_t i3, /*in*/ int32_t i4, /*out*/ struct ASE_BaseException__object **_exception)
{
}

static void setElement(/*in*/ struct ASE_Compiler_Parser__array *array, /*in*/ struct int__array *indices, /*in*/ struct ASE_Compiler_Parser__object *value, /*out*/ struct ASE_BaseException__object **_exception)
{
  struct ASE_Compiler_Parser__object **_element = NULL;
  int32_t d = 0;
  
  _element = array->d_firstElement;
  for(d = 0; d < array->d_dimen; d++)
  {
    if (indices->d_firstElement[d] < array->d_lower[d] || indices->d_firstElement[d] > array->d_upper[d])
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Invalid indices", _exception);
      if (!(*_exception))
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    _element += array->d_stride[d] * (indices->d_firstElement[d] - array->d_lower[d]);
  }
  if (value)
  {
    (*(((struct ASE_BaseInterface__object *) (value)->d_object)->d_epv->addReference))((value)->d_object, (void **) _exception);
  }
  if (*_element)
  {
    (*(((struct ASE_BaseInterface__object *) (*_element)->d_object)->d_epv->deleteReference))((*_element)->d_object, (void **) _exception);
  }
  *_element = value;
}

static void setElement4(/*in*/ struct ASE_Compiler_Parser__array *array, /*in*/ int32_t i1, /*in*/ int32_t i2, /*in*/ int32_t i3, /*in*/ int32_t i4, /*in*/ struct ASE_Compiler_Parser__object *value, /*out*/ struct ASE_BaseException__object **_exception)
{
}

/* The entry points into the DLL containing this type */
static struct ASE_Compiler_Parser__external externalEntryPoints = {
  createArray,
  borrowArray,
  destroyArray,
  getDimen,
  getLower,
  getUpper,
  getElement,
  getElement4,
  setElement,
  setElement4};

const struct ASE_Compiler_Parser__external *ASE_Compiler_Parser__externals(void)
{
  return &externalEntryPoints;
}
