#!/usr/bin/env python
'''
Include expansion for ANSI C
'''
import user
import importer
import lex
import ASE.Compiler.Cxx.clex

import os
import sys

class ExpressionLexer:
  def input(self, tokens):
    self.tokens = []
    for token in tokens:
      if token.type == 'ID':
        self.tokens.append(lex.LexToken('ICONST','0',token.lineno))
      elif token.type == 'LINE_CONT':
        pass
      else:
        self.tokens.append(token)
    #self.tokens = filter(lambda token: not token.type == 'LINE_CONT', tokens)
    self.tokensCopy = self.tokens[:]
    self.tokens.append(None)
    self.tokens.reverse()
    return

  def token(self):
    return self.tokens.pop()

class IncludeExpander(lex.Lexer):
  def __init__(self):
    lex.Lexer.__init__(self)
    #self.lexer             = None
    self.setLexer(None)
    self.systemIncludeDirs = ['/usr/include', '/usr/include/linux', '/usr/local/include', '/usr/lib/gcc-lib/i686-pld-linux/3.0.4/include']
    self.includeDirs       = ['/usr/local/python-new/include/python2.3', '/PETSc3/sidl/Runtime/server-cxx-sidl', '/PETSc3/ase/Compiler/client-python']
    self.includeFiles      = []
    self.defines           = self.setupDefaultDefines({})
    self.currentFile       = 'Unknown'
    self.debug             = 0
    self.typeDefs          = {}
    self.setupExpressions()
    return

  def getLexer(self):
    if self._lexer is None:
      self._lexer = ASE.Compiler.Cxx.clex.lexer
    return self._lexer

  def setLexer(self, lexer):
    self._lexer = lexer
    return
  lexer = property(getLexer, setLexer, doc = 'The underlying lexer')

  def setupExpressions(self):
    import ASE.Compiler.Cxx.cPreprocessorParse

    self.expLexer  = ExpressionLexer()
    self.expParser = ASE.Compiler.Cxx.cPreprocessorParse.getDriver()
    return

  def setupDefaultDefines(self, defines):
    defines['__GNUC__'] = ([], [lex.LexToken('ICONST','3',-1)])
    return defines

  def isNameToken(self, token):
    '''Names are either identifiers or reserved words'''
    if token.type == 'ID' or token.type in ASE.Compiler.Cxx.clex.reserved:
      return 1
    return 0

  def astToTokens(self, ast, isAbstract = 0):
    if ast.isInstanceOf('ASE.Compiler.Cxx.Type'):
      import ASE.Compiler.Cxx.Type

      type = ASE.Compiler.Cxx.Type.Type(ast)
      if type.getBaseType():
        return [lex.LexToken(type.getIdentifier().upper(),type.getIdentifier(),-1)]
      elif type.isInstanceOf('ASE.Compiler.Cxx.Pointer'):
        return self.astToTokens(type.getChildren()[0])+[lex.LexToken('TIMES','*',-1)]
      elif type.isInstanceOf('ASE.Compiler.Cxx.Struct'):
        import ASE.Compiler.Cxx.Struct

        struct = ASE.Compiler.Cxx.Struct.Struct(type)
        tokens = []
        if struct.getUnion():
          tokens.append(lex.LexToken('UNION','union',-1))
        else:
          tokens.append(lex.LexToken('STRUCT','struct',-1))
        if struct.getIdentifier():
          tokens.append(lex.LexToken('ID',struct.getIdentifier(),-1))
        tokens.append(lex.LexToken('LBRACE','{',-1))
        [tokens.extend(self.astToTokens(child)+[lex.LexToken('SEMI',';',-1)]) for child in struct.getChildren()]
        tokens.append(lex.LexToken('RBRACE','}',-1))
        return tokens
    elif ast.isInstanceOf('ASE.Compiler.Cxx.Declarator'):
      import ASE.Compiler.Cxx.Declarator

      decl = ASE.Compiler.Cxx.Declarator.Declarator(ast)
      if decl.isInstanceOf('ASE.Compiler.Cxx.Function'):
        import ASE.Compiler.Cxx.Function

        func   = ASE.Compiler.Cxx.Function.Function(decl)
        tokens = self.astToTokens(func.getType())
        tokens.extend([lex.LexToken('LPAREN','(',-1), lex.LexToken('RPAREN',')',-1)])
        tokens.append(lex.LexToken('LPAREN','(',-1))
        [tokens.extend(self.astToTokens(parameter)) for parameter in func.getParameters()]
        if func.getOptionalArguments():
          tokens.append(lex.LexToken('ELLIPSIS','...',-1))
        tokens.append(lex.LexToken('RPAREN',')',-1))
        return tokens
      elif decl.isInstanceOf('ASE.Compiler.Cxx.Parameter'):
        return self.astToTokens(decl.getType())
      else:
        if isAbstract:
          return self.astToTokens(decl.getType())
        return self.astToTokens(decl.getType())+[lex.LexToken('ID',decl.getIdentifier(),-1)]
    raise RuntimeError('Cannot convert ast to tokens')

  def addTypedef(self, name, type):
    #self.typeDefs[name] = self.astToTokens(type, isAbstract = 1)
    self.typeDefs[name] = [lex.LexToken('DEFINED_TYPE',name,-1)]
    if self.debug: print 'cIncludeExpansion: Adding typedef for',name
    return

  def expandToken(self, token):
    '''Do crazy define expansion
       - We also handle the DEFINED token here directly'''
    name   = token.value
    lineno = token.lineno
    if token.type == 'ID' and token.value == 'defined':
      token = lex.LexToken('DEFINED','defined',lineno)
    if self.isNameToken(token) and token.value in self.defines:
      if not len(self.defines[token.value][0]):
        tokens = self.defines[token.value][1]
        macroText = ''
      else:
        # Get argument values
        args   = self.defines[token.value][0]
        body   = self.defines[token.value][1]
        values = []
        if not self.lexer.token().type == 'LPAREN':
          raise RuntimeError('Missing LPAREN in macro expansion: '+str(token))
        nesting = 1
        for i in range(len(args)):
          value = []
          token = self.lexer.token()
          # Note that commas in nested parentheses are ignored
          while (not token.type == 'COMMA' or nesting > 1) and (not token.type == 'RPAREN' or nesting > 1):
            if token.type == 'LPAREN':
              nesting += 1
            elif token.type == 'RPAREN':
              nesting -= 1
            value.append(token)
            token = self.lexer.token()
          values.append(value)
          if token.type == 'RPAREN':
            nesting -= 1
            if nesting == 0 and not i == len(args)-1:
              raise RuntimeError('Premature RPAREN in macro expansion: '+str(token))
        if not token.type == 'RPAREN':
          raise RuntimeError('Missing RPAREN in macro expansion: '+str(token))
        elif nesting > 0:
          raise RuntimeError('Invalid nesting of parentheses in macro expansion: '+str(token))
        # Replace argument values
        tokens = []
        for token in body:
          if self.isNameToken(token) and token in args:
            tokens.extend(values[args.index(token)])
          else:
            tokens.append(token)
        macroText = '('+', '.join([''.join([t.value for t in v]) for v in values])+')'
      #print 'Replacing',token.value,'with',tokens
      newTokens = []
      for t in tokens:
        newTokens.append(lex.LexToken(t.type, t.value, lineno))
        if hasattr(t, 'define'): newTokens[-1].define = t.define+macroText
      return newTokens
    elif token.type == 'DEFINED':
      token = self.lexer.token()
      if not self.isNameToken(token):
        if not token.type == 'LPAREN':
          raise RuntimeError('Invalid argument to defined() directive: '+str(token))
        token = self.lexer.token()
        if not self.isNameToken(token):
          raise RuntimeError('Invalid argument to defined() directive: '+str(token))
        if not self.lexer.token().type == 'RPAREN':
          raise RuntimeError('Missing closing parenthesis in defined() directive after '+str(token))
      if token.value in self.defines:
        return [lex.LexToken('ICONST','1',lineno)]
      else:
        return [lex.LexToken('ICONST','0',lineno)]
    elif token.type == 'ID' and token.value in self.typeDefs:
      tokens = self.typeDefs[token.value]
      for t in tokens:
        t.lineno = token.lineno
      return tokens
    return [token]

  def tokenLine(self, initialTokens = None, exclude = []):
    startPos = self.lexer.lexpos
    endPos   = self.lexer.lexdata.find('\n', startPos)
    tokens   = []
    if not initialTokens is None:
      for token in initialTokens:
        if self.isNameToken(token) and token.value in exclude:
          tokens.append(token)
        else:
          tokens.extend(self.expandToken(token))
    while self.lexer.lexpos < endPos:
      token = self.lexer.token()
      if token is None:
        break
      elif token.type == 'LINE_CONT':
        # Careful here since line continuations have an incremented line number
        endPos = self.lexer.lexdata.find('\n', endPos+1)
      elif len(tokens) and token.lineno > tokens[-1].lineno:
        self.lexer.pushToken(token)
        break
      if self.isNameToken(token) and token.value in exclude:
        tokens.append(token)
      else:
        tokens.extend(self.expandToken(token))
    return tokens

  def evaluateTokens(self, tokens):
    if self.debug: print 'Evaluating',tokens
    if not len(tokens):
      raise RuntimeError('Empty token list')
    try:
      test = self.expParser.parse(tokens, lexer = self.expLexer)
      if self.debug: print '  Result',test
      return test
    except RuntimeError, e:
      print 'Expression error for',self.expLexer.tokensCopy,'in',self.currentFile
    return 0

  def handleInclude(self):
    # Get include filename
    token = self.lexer.token()
    if token.type == 'SCONST':
      yield token
      filename = token.value.strip('"\'')
    elif token.type == 'LT':
      token = self.lexer.token()
      filename = ''
      while self.isNameToken(token) or token.type == 'PERIOD' or token.type == 'DIVIDE' or token.type == 'MINUS':
        filename += token.value 
        token = self.lexer.token()
      if not token.type == 'GT':
        raise RuntimeError('Invalid token in include statement: '+str(token))
      yield lex.LexToken('SCONST', filename, token.lineno)
    else:
      raise RuntimeError('Invalid token in include statement: '+str(token))
    yield lex.LexToken('SEMI',';',token.lineno)
    # Locate include file
    if not os.path.isfile(filename):
      for dir in self.includeDirs+self.systemIncludeDirs:
        path = os.path.join(dir, filename)
        if os.path.isfile(path):
          filename = path
          break
    # Process the include file
    if not filename in self.includeFiles:
      if self.debug: print 'Expanding include',filename
      self.includeFiles.append(filename)
      oldFile = self.currentFile
      self.currentFile = filename
      f = file(filename)
      s = f.read()
      f.close()
      self.pushInput(s)
      for token in self.handleFile(): pass
      self.popInput()
      self.currentFile = oldFile
      if self.debug: print 'Done expanding include',filename
    return

  def handleDefine(self):
    token = self.lexer.token()
    yield token
    if not self.isNameToken(token):
      raise RuntimeError('Invalid token for define name: '+str(token)+' in '+str(self.currentFile))
    name = token
    # Handle any arguments
    args  = []
    pos   = self.lexer.lexpos
    token = self.lexer.token()
    cont  = 0
    # Check for empty define
    if token is None or (token.lineno > name.lineno and not token.type == 'LINE_CONT'):
      self.lexer.pushToken(token)
      token = lex.LexToken('ICONST','1',name.lineno)
      yield token
      yield lex.LexToken('SEMI',';',name.lineno)
      self.defines[name.value] = ([], [token])
      if self.debug: print 'Added define',name.value,'-->',self.defines[name.value][1]
      return
    if token.type == 'LPAREN' and self.lexer.lexpos == pos+1:
      yield token
      arg   = []
      token = self.lexer.token()
      arg.append(token)
      yield token
      while not token.type == 'RPAREN':
        if token.type == 'LINE_CONT':
          cont += 1
          token = self.lexer.token()
        if not self.isNameToken(token):
          raise RuntimeError('Invalid define argument: '+str(token))
        token = self.lexer.token()
        yield token
        if token.type == 'COMMA':
          args.append(arg[0])
          token = self.lexer.token()
          if token.type == 'LINE_CONT':
            cont += 1
            token = self.lexer.token()
          arg   = [token]
          yield token
        elif token.type == 'RPAREN':
          args.append(arg[0])
          break
        elif token.type == 'ELLIPSIS':
          args.append(arg[0])
          args.append(token)
          # I guess this is legal
          token = self.lexer.token()
          yield token
          if not token.type == 'RPAREN': raise RuntimeError('Invalid define argument: '+str(token))
          break
        else:
          raise RuntimeError('Invalid define argument separator: '+str(token))
      token = self.lexer.token()
    # Handle replacement text
    #   WARNING: The argument are unexpanded in the replacement, even if they are macros themselves
    if token is None:
      tokens = []
    else:
      lineno = name.lineno + cont
      while token.type == 'LINE_CONT':
        token   = self.lexer.token()
        lineno += 1
      if token.lineno <= lineno:
        tokens = self.tokenLine([token], exclude = [arg.value for arg in args])
      else:
        self.lexer.pushToken(token)
        tokens = [lex.LexToken('ICONST','1',name.lineno)]
    for t in tokens:
      t.define = name.value
    self.defines[name.value] = (args, tokens)
    if self.debug: print 'Adding define',name.value,'-->',tokens
    if len(tokens) == 1:
      t = tokens[0]
      if t.type in ['ID', 'ICONST', 'SCONST']:
        yield t
      else:
        yield lex.LexToken('SCONST',t.value,t.lineno)
    elif len(tokens) > 1:
      # Should pretty print here
      yield lex.LexToken('SCONST',''.join(map(lambda t: t.value, tokens)),tokens[0].lineno)
    yield lex.LexToken('SEMI',';',tokens[0].lineno)
    return

  def handleUndef(self):
    token = self.lexer.token()
    yield token
    yield lex.LexToken('SEMI',';',token.lineno)
    if not self.isNameToken(token):
      raise RuntimeError('Invalid token for undef name: '+str(token)+' in '+str(self.currentFile))
    name = token.value
    next = self.lexer.token()
    if not next is None and next.lineno == token.lineno:
      raise RuntimeError('Invalid arguments to #undef statement '+str(token)+' '+str(next)+' in '+str(self.currentFile))
    self.lexer.pushToken(next)
    if name in self.defines: del self.defines[name]
    if self.debug: print 'Removed define',name
    return

  def handleTokens(self, directives, ignore = 0, nestedDirectives = [('if', 'endif'), ('ifdef', 'endif'), ('ifndef', 'endif')]):
    '''Handle all tokens until we find one of the given preprocessor directives
      - Returns the directive which caused termination'''
    nestStart = [t[0] for t in nestedDirectives]
    nestEnd   = []
    while 1:
      token = self.lexer.token()
      if token is None:
        raise RuntimeError('Null token')
      if token.type == 'POUND':
        name = self.lexer.token()
        if self.isNameToken(name):
          if name.value in directives:
            if len(nestEnd):
              #if not ignore: raise RuntimeError('Invalid nesting of processor conditionals')
              if name.value == nestEnd[-1]:
                nestEnd.pop()
            else:
              if ignore:
                if self.debug: print '  Ignoring',token
                if self.debug: print '  Ignoring',name,'break'
              else:
                if self.debug: print '  Handling',token
                if self.debug: print '  Handling',name,'break'
              break
          elif ignore and name.value in nestStart:
            yield token
            yield name
            nestEnd.append(nestedDirectives[nestStart.index(name.value)][1])
        self.lexer.pushToken(name)
      if ignore:
        if self.debug: print '  Ignoring',token
        yield token
      else:
        for t in self.handleToken(token):
          yield t
    raise StopIteration(name.value)

  def handleIf(self, negated = 0, onlyDefine = 0):
    if onlyDefine:
      name   = self.lexer.token()
      yield lex.LexToken('SCONST',name.value,name.lineno)
      self.lexer.pushToken(name)
      tokens = self.tokenLine([lex.LexToken('DEFINED','defined',-1)])
    else:
      tokens = self.tokenLine()
      yield lex.LexToken('SCONST',' '.join(map(lambda t: t.value, tokens)),tokens[0].lineno)
    if self.debug: print '--> Got if',tokens[0].lineno
    test   = self.evaluateTokens(tokens)
    lineno = -1
    if negated:
      test = not test
    if test:
      # Handling tokens, so close off opening conditional
      yield lex.LexToken('SEMI',';',tokens[0].lineno)
      g = self.handleTokens(['else', 'endif'])
      try:
        while 1:
          t      = g.next()
          lineno = t.lineno
          yield t
      except StopIteration, e:
        if str(e) == 'else':
          if self.debug: print '--> Got else',lineno+1
          yield lex.LexToken('POUND','#',lineno+1)
          yield lex.LexToken('ID',str(e),lineno+1)
          s = []
          for t in self.handleTokens(['endif'], ignore = 1):
            s.append(t.value)
          yield lex.LexToken('SCONST',' '.join(s),lineno+1)
          yield lex.LexToken('POUND','#',lineno+1)
          yield lex.LexToken('ID','endif',lineno+1)
          yield lex.LexToken('SEMI',';',lineno+1)
        else:
          if self.debug: print '--> Got endif',lineno+1
          yield lex.LexToken('POUND','#',lineno+1)
          yield lex.LexToken('ID',str(e),lineno+1)
          yield lex.LexToken('SEMI',';',lineno+1)
    else:
      g = self.handleTokens(['else', 'endif'], ignore = 1)
      s = []
      try:
        while 1:
          t      = g.next()
          lineno = t.lineno
          s.append(t.value)
      except StopIteration, e:
        yield lex.LexToken('SCONST',' '.join(s),lineno)
        yield lex.LexToken('POUND','#',lineno+1)
        yield lex.LexToken('ID',str(e),lineno+1)
        yield lex.LexToken('SEMI',';',lineno+1)
        if str(e) == 'else':
          if self.debug: print '--> Got else',lineno+1
          for token in self.handleTokens(['endif']):
            yield token
          if self.debug: print '--> Got endif',lineno+1
          yield lex.LexToken('POUND','#',token.lineno+1)
          yield lex.LexToken('ID','endif',token.lineno+1)
          yield lex.LexToken('SEMI',';',token.lineno+1)
        else:
          if self.debug: print '--> Got endif',lineno+1
    return

  def handleTypedef(self, token):
    '''This should really be fed to a separate parser'''
    yield token
    tokens = []
    [tokens.extend(self.expandToken(self.lexer.token())) for i in range(3)]
    if tokens[-1].type == 'SEMI':
      if self.debug: print 'Simple typedef',tokens[0],'to',tokens[1]
      self.addTypedef(tokens[1].value, tokens[0])
    elif tokens[0].type == 'STRUCT' or tokens[0].type == 'ENUM':
      self.lexer.pushToken(tokens[2])
      tokens = tokens[0:2]
      while 1:
        if tokens[-1].type == 'LBRACE':
          braceCount = 1
          break
        elif tokens[-1].type == 'SEMI':
          braceCount = 0
          break
        tokens.extend(self.expandToken(self.lexer.token()))
      if braceCount:
        while 1:
          tokens.extend(self.expandToken(self.lexer.token()))
          if tokens[-1].type == 'LBRACE':
            braceCount += 1
          elif tokens[-1].type == 'RBRACE':
            braceCount -= 1
          if braceCount == 0:
            break
        tokens.append(self.lexer.token())
        tokens.append(self.lexer.token())
      while tokens[-2].type == 'TIMES':
        tokens.append(self.lexer.token())
      if not tokens[-1].type == 'SEMI':
        raise RuntimeError('Invalid structure or enumeration typedef: '+str(tokens))
      if self.debug: print 'Structure typedef',tokens[:-2],'to',tokens[-2]
      self.addTypedef(tokens[-2].value, tokens[:-2])
    elif tokens[0].type in ['LONG', 'SHORT', 'UNSIGNED']:
      i = 0
      while not tokens[-1].type == 'SEMI':
        tokens.append(self.lexer.token())
        i += 1
      if not tokens[-1].type == 'SEMI':
        raise RuntimeError('Invalid qualified typedef: '+str(tokens))
      if self.debug: print 'Qualified typedef',tokens[:-2],'to',tokens[-2]
      self.addTypedef(tokens[-2].value, tokens[:-2])
    else:
      if self.debug: print 'Missed typedef',tokens
    for token in tokens:
      yield token
    return

  def handleToken(self, token):
    if self.debug: print 'Handling',token
    if token is None:
      raise StopIteration('final')
    elif token.type == 'POUND':
      name = self.lexer.token()
      if self.debug: print 'Handling',name
      if self.isNameToken(name):
        yield token
        if name.value == 'if':
          yield lex.LexToken('ID',name.value,name.lineno)
        else:
          yield name
        if name.value == 'include':
          for token in self.handleInclude():
            yield token
        elif name.value == 'define':
          for token in self.handleDefine():
            yield token
        elif name.value == 'undef':
          for token in self.handleUndef():
            yield token
        elif name.value == 'if':
          for token in self.handleIf():
            yield token
        elif name.value == 'ifdef':
          for token in self.handleIf(onlyDefine = 1):
            yield token
        elif name.value == 'ifndef':
          for token in self.handleIf(onlyDefine = 1, negated = 1):
            yield token
    elif token.type == 'TYPEDEF':
      for token in self.handleTypedef(token):
        yield token
    else:
      for t in self.expandToken(token):
        yield t
    return

  def handleFile(self):
    while 1:
      g = self.handleToken(self.lexer.token())
      try:
        while 1:
          yield g.next()
      except StopIteration, e:
        if str(e) == 'final': break
    return

  def input(self, s, pos = 0):
    return self.pushInput(s)

  def pushInput(self, s):
    self.lexer.pushInput(s)
    self.token = self.handleFile().next
    return

  def popInput(self):
    return self.lexer.popInput()

  def process(self, s):
    self.input(s)
    return [t for t in self.handleFile()]

  def clearDefines(self):
    self.defines.clear()
    return

if __name__ == '__main__':
  expander = IncludeExpander()
  try:
    f = file(sys.argv[1])
    s = f.read()
    f.close()
    expander.input(s)
    for token in expander.handleFile():
      print token
  except RuntimeError, e:
    print 'ERROR in '+expander.currentFile+':', e
