#!/usr/bin/env python
# cparse.py
#
# Simple parser for ANSI C.  Based on the grammar in K&R, 2nd Ed.
# -----------------------------------------------------------------------------
import user
import importer
import clex
import yacc

import ASE.Compiler.Cxx.Addition
import ASE.Compiler.Cxx.AdditionAssignment
import ASE.Compiler.Cxx.Address
import ASE.Compiler.Cxx.And
import ASE.Compiler.Cxx.AndAssignment
import ASE.Compiler.Cxx.Array
import ASE.Compiler.Cxx.ArrayReference
import ASE.Compiler.Cxx.Assignment
import ASE.Compiler.Cxx.BitField
import ASE.Compiler.Cxx.Break
import ASE.Compiler.Cxx.Cast
import ASE.Compiler.Cxx.Complement
import ASE.Compiler.Cxx.CompoundStatement
import ASE.Compiler.Cxx.Constant
import ASE.Compiler.Cxx.Continue
import ASE.Compiler.Cxx.Declaration
import ASE.Compiler.Cxx.Declarator
import ASE.Compiler.Cxx.DeclaratorGroup
import ASE.Compiler.Cxx.Define
import ASE.Compiler.Cxx.Division
import ASE.Compiler.Cxx.DivisionAssignment
import ASE.Compiler.Cxx.Enumeration
import ASE.Compiler.Cxx.Enumerator
import ASE.Compiler.Cxx.Equality
import ASE.Compiler.Cxx.ExclusiveOr
import ASE.Compiler.Cxx.ExclusiveOrAssignment
import ASE.Compiler.Cxx.ExpressionStatement
import ASE.Compiler.Cxx.For
import ASE.Compiler.Cxx.Function
import ASE.Compiler.Cxx.FunctionCall
import ASE.Compiler.Cxx.Group
import ASE.Compiler.Cxx.Goto
import ASE.Compiler.Cxx.GreaterThan
import ASE.Compiler.Cxx.GreaterThanOrEqual
import ASE.Compiler.Cxx.If
import ASE.Compiler.Cxx.Include
import ASE.Compiler.Cxx.InclusiveOr
import ASE.Compiler.Cxx.InclusiveOrAssignment
import ASE.Compiler.Cxx.Increment
import ASE.Compiler.Cxx.Indirection
import ASE.Compiler.Cxx.Inequality
import ASE.Compiler.Cxx.Initializer
import ASE.Compiler.Cxx.LeftShift
import ASE.Compiler.Cxx.LeftShiftAssignment
import ASE.Compiler.Cxx.LessThan
import ASE.Compiler.Cxx.LessThanOrEqual
import ASE.Compiler.Cxx.LogicalAnd
import ASE.Compiler.Cxx.LogicalOr
import ASE.Compiler.Cxx.Modulo
import ASE.Compiler.Cxx.ModuloAssignment
import ASE.Compiler.Cxx.Multiplication
import ASE.Compiler.Cxx.MultiplicationAssignment
import ASE.Compiler.Cxx.Negation
import ASE.Compiler.Cxx.Parameter
import ASE.Compiler.Cxx.Pointer
import ASE.Compiler.Cxx.PreprocessorConditional
import ASE.Compiler.Cxx.Return
import ASE.Compiler.Cxx.RightShift
import ASE.Compiler.Cxx.RightShiftAssignment
import ASE.Compiler.Cxx.Sizeof
import ASE.Compiler.Cxx.Struct
import ASE.Compiler.Cxx.StructReference
import ASE.Compiler.Cxx.Subtraction
import ASE.Compiler.Cxx.SubtractionAssignment
import ASE.Compiler.Cxx.Switch
import ASE.Compiler.Cxx.Type
import ASE.Compiler.Cxx.Typedef
import ASE.Compiler.Cxx.UnaryMinus
import ASE.Compiler.Cxx.UnaryPlus
import ASE.Compiler.Cxx.Variable
import ASE.Compiler.Cxx.While

# Get the token map
tokens = clex.tokens+('DEFINED_TYPE',)

# Need some way to match up preprocessor conditional sections
activeBranch  = []
branchClosure = 0

def applySpecifiers(decl, storageSpecifiers, typeSpecifiers, typeQualifiers):
  '''Currently, typeSpecifiers is constrained to be a single type'''
  for storage in storageSpecifiers:
    if storage in ['auto', 'register']:
      decl.setStatic(0)
    elif storage == 'static':
      decl.setStatic(1)
      decl.setExtern(0)
    elif storage == 'extern':
      decl.setExtern(1)
    elif storage == 'typedef':
      decl.setTypedef(1)
  if not len(typeSpecifiers) == 1:
    # Should make other types which correspond to these
    typeSpecifiers = [t for t in typeSpecifiers if not t.getIdentifier() in ['short', 'long', 'int']]
    if not len(typeSpecifiers) == 1:
      raise RuntimeError('Must have a single type specifier: '+str([t.getIdentifier() for t in typeSpecifiers]))
  type = typeSpecifiers[0]
  if not decl.getType() is None:
    pointer = decl.getType()
    while len(pointer.getChildren()):
      pointer = pointer.getChildren()[0]
    pointer.setChildren([type])
  else:
    decl.setType(type)
  for qualifier in typeQualifiers:
    if qualifier == 'const':
      type.setConst(1)
    if qualifier == 'volatile':
      type.setVolatile(1)
  return decl

# translation-unit:
def p_translation_unit_1(t):
  'translation_unit : external_declaration'
  t[0] = [t[1]]

def p_translation_unit_2(t):
  'translation_unit : translation_unit external_declaration'
  global branchClosure, activeBranch

  if branchClosure:
    activeNode = activeBranch.pop()
    try:
      ind = t[1].index(activeNode)
    except ValueError:
      print t[1]
      raise RuntimeError('Cannot find active branch')
    activeNode.addChildren(t[1][ind+1:])
    t[0] = t[1][0:ind+1]
    branchClosure = 0
  else:
    t[0] = t[1]+[t[2]]

# external-declaration:
def p_external_declaration_1(t):
  'external_declaration : function_definition'
  t[0] = t[1]

def p_external_declaration_2(t):
  'external_declaration : declaration'
  t[0] = t[1]

def p_external_declaration_3(t):
  'external_declaration : preprocessor_directive'
  t[0] = t[1]

# preprocessor-directive
def p_preprocessor_directive(t):
  '''preprocessor_directive : preprocessor_include_or_if_true
                            | preprocessor_if_false
                            | preprocessor_endif_true
                            | preprocessor_endif_false
                            | preprocessor_define'''
  t[0] = t[1]

def p_preprocessor_include_or_if_true(t):
  'preprocessor_include_or_if_true : POUND ID SCONST SEMI'
  global activeBranch

  if t[2] == 'include':
    include = ASE.Compiler.Cxx.Include.Include()
    include.setIdentifier('<'+t[3].strip('"')+'>')
    t[0] = include
  elif t[2] in ['if', 'ifdef', 'ifndef']:
    conditional = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    conditional.setIdentifier(t[2])
    conditional.setBranchExpression(t[3])
    conditional.setValue(1)
    activeBranch.append(conditional)
    t[0] = conditional
  else:
    raise RuntimeError('Invalid preprocessor directive: '+t[2])

def p_preprocessor_if_false(t):
  'preprocessor_if_false : POUND ID SCONST SCONST POUND ID SEMI'
  global branchClosure, activeBranch

  if t[2] in ['if', 'ifdef', 'ifndef']:
    conditional = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    conditional.setIdentifier(t[2])
    conditional.setBranchExpression(t[3])
    conditional.setValue(0)
    conditional.setInactiveCode(t[4])
    if t[6] == 'else':
      activeBranch.append(conditional)
    elif not t[6] == 'endif':
      raise RuntimeError('Invalid preprocessor directive: '+t[6])
    t[0] = conditional
  else:
    raise RuntimeError('Invalid preprocessor directive: '+t[2])

def p_preprocessor_endif_true(t):
  'preprocessor_endif_true : POUND ID SEMI'
  global branchClosure, activeBranch

  if not t[2] == 'endif':
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  if not len(activeBranch):
    raise RuntimeError('Could not match #endif')
  branchClosure = 1

def p_preprocessor_endif_false(t):
  'preprocessor_endif_false : POUND ID SCONST POUND ID SEMI'
  global branchClosure, activeBranch

  if not t[2] == 'else':
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  if not t[5] == 'endif':
    raise RuntimeError('Invalid preprocessor directive: '+t[5])
  if not len(activeBranch):
    raise RuntimeError('Could not match #endif')
  activeBranch[-1].setInactiveCode(t[3])
  branchClosure = 1

def p_preprocessor_define_1(t):
  'preprocessor_define : POUND ID ID SEMI'
  if not t[2] in ['define', 'undef']:
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  define = ASE.Compiler.Cxx.Define.Define()
  if t[2] == 'undef':
    define.setUndef(1)
  define.setIdentifier(t[3])
  t[0] = define

def p_preprocessor_define_2(t):
  'preprocessor_define : POUND ID ID ICONST SEMI'
  if not t[2] == 'define':
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  define = ASE.Compiler.Cxx.Define.Define()
  define.setIdentifier(t[3])
  define.setReplacementText(str(t[4]))
  t[0] = define

def p_preprocessor_define_3(t):
  'preprocessor_define : POUND ID ID SCONST SEMI'
  if not t[2] == 'define':
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  define = ASE.Compiler.Cxx.Define.Define()
  define.setIdentifier(t[3])
  define.setReplacementText(t[4])
  t[0] = define

def p_preprocessor_define_4(t):
  'preprocessor_define : POUND ID ID ID SEMI'
  if not t[2] == 'define':
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  define = ASE.Compiler.Cxx.Define.Define()
  define.setIdentifier(t[3])
  define.setReplacementText(t[4])
  t[0] = define

def p_preprocessor_define_5(t):
  'preprocessor_define : POUND ID ID LPAREN identifier_list_opt RPAREN SCONST SEMI'
  if not t[2] == 'define':
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  define = ASE.Compiler.Cxx.Define.Define()
  define.setIdentifier(t[3])
  if not t[5] is None:
    define.setArgumentNames(t[5])
  define.setReplacementText(t[7])
  t[0] = define

def p_preprocessor_define_6(t):
  'preprocessor_define : POUND ID ID LPAREN identifier_list_opt RPAREN ICONST SEMI'
  if not t[2] == 'define':
    raise RuntimeError('Invalid preprocessor directive: '+t[2])
  define = ASE.Compiler.Cxx.Define.Define()
  define.setIdentifier(t[3])
  if not t[5] is None:
    define.setArgumentNames(t[5])
  define.setReplacementText(str(t[7]))
  t[0] = define
    
# function-definition:
def p_function_definition_1(t):
  'function_definition : declaration_specifiers declarator declaration_list compound_statement'
  raise RuntimeError('K&R declarations are no longer valid')

def p_function_definition_2(t):
  'function_definition : declarator declaration_list compound_statement'
  raise RuntimeError('K&R declarations are no longer valid')

def p_function_definition_3(t):
  'function_definition : declarator compound_statement'
  if not t[1].isInstanceOf('ASE.Compiler.Cxx.Function'):
    printError()
    raise RuntimeError('This must be a function definition')
  t[1].setBody(t[2])
  t[0] = t[1]

def p_function_definition_4(t):
  'function_definition : declaration_specifiers declarator compound_statement'
  if not t[2].isInstanceOf('ASE.Compiler.Cxx.Function'):
    raise RuntimeError('This must be a function definition')
  applySpecifiers(t[2], t[1][0], t[1][1], t[1][2])
  t[2].setBody(t[3])
  t[0] = t[2]

# declaration:

def p_declaration_1(t):
  'declaration : declaration_specifiers init_declarator_list SEMI'
  decl = ASE.Compiler.Cxx.Declaration.Declaration()
  if 'typedef' in t[1][0]:
    if not len(t[1][1]) == 1 or not len(t[2]) == 1:
      raise RuntimeError('Invalid typedef form')
    if not t[1][1][0].isInstanceOf('ASE.Compiler.Cxx.Type'):
      print map(lambda t: t.getIdentifier(), t[1][1])
      raise RuntimeError('Invalid typedef source')
    if t[2][0].isInstanceOf('ASE.Compiler.Cxx.Function'):
      group = t[2][0].getChildren()[0]
      if not group.isInstanceOf('ASE.Compiler.Cxx.DeclaratorGroup'):
        raise RuntimeError('Invalid function typedef form')
      typeDefs[group.getChildren()[0].getIdentifier()] = t[2][0]
    else:
      typeDefs[t[2][0].getIdentifier()] = t[1][1][0]
  for declarator in t[2]:
    applySpecifiers(declarator, t[1][0], t[1][1], t[1][2])
  decl.setChildren(t[2])
  t[0] = decl

def p_declaration_2(t):
  'declaration : declaration_specifiers SEMI'
  decl = ASE.Compiler.Cxx.Declaration.Declaration()
  if not len(t[1][1]):
    raise RuntimeError('Empty declaration: '+str(t[1]))
  if 'typedef' in t[1][0]:
    raise RuntimeError('Invalid typedef in declaration')
  else:
    # This form is for structs, unions, and enums
    type = t[1][1][0]
    for qualifier in t[1][2]:
      if qualifier == 'const':
        type.setConst(1)
      if qualifier == 'volatile':
        type.setVolatile(1)
    decl.setChildren([type])
    t[0] = decl

# declaration-list:

def p_declaration_list_1(t):
  'declaration_list : declaration'
  t[0] = [t[1]]

def p_declaration_list_2(t):
  'declaration_list : declaration_list declaration '
  t[0] = t[1]+[t[2]]

# declaration-specifiers
def p_declaration_specifiers_1(t):
    'declaration_specifiers : storage_class_specifier declaration_specifiers'
    t[2][0].insert(0, t[1])
    t[0] = t[2]

def p_declaration_specifiers_2(t):
    'declaration_specifiers : type_specifier declaration_specifiers'
    t[2][1].insert(0, t[1])
    t[0] = t[2]

def p_declaration_specifiers_3(t):
    'declaration_specifiers : type_qualifier declaration_specifiers'
    t[2][2].insert(0, t[1])
    t[0] = t[2]

def p_declaration_specifiers_4(t):
    'declaration_specifiers : storage_class_specifier'
    t[0] = ([t[1]], [], [])

def p_declaration_specifiers_5(t):
    'declaration_specifiers : type_specifier'
    t[0] = ([], [t[1]], [])

def p_declaration_specifiers_6(t):
    'declaration_specifiers : type_qualifier'
    t[0] = ([], [], [t[1]])

# storage-class-specifier
def p_storage_class_specifier(t):
  '''storage_class_specifier : AUTO
                             | REGISTER
                             | STATIC
                             | EXTERN
                             | TYPEDEF'''
  t[0] = t[1]

# type-specifier:
def p_type_specifier_1(t):
  '''type_specifier : VOID
                    | CHAR
                    | SHORT
                    | INT
                    | LONG
                    | FLOAT
                    | DOUBLE
                    | SIGNED
                    | UNSIGNED'''
  type = ASE.Compiler.Cxx.Type.Type()
  type.setIdentifier(t[1])
  type.setBaseType(1)
  t[0] = type  

def p_type_specifier_2(t):
  '''type_specifier : struct_or_union_specifier
                    | enum_specifier'''
  t[0] = t[1]

def p_type_specifier_3(t):
  '''type_specifier : DEFINED_TYPE'''
  type = ASE.Compiler.Cxx.Typedef.Typedef()
  type.setIdentifier(t[1])
  if t[1] in typeDefs:
    type.setChildren([typeDefs[t[1]]])
  t[0] = type  

# type-qualifier:
def p_type_qualifier(t):
  '''type_qualifier : CONST
                    | VOLATILE'''
  t[0] = t[1]

# struct-or-union-specifier
def p_struct_or_union_specifier_1(t):
  'struct_or_union_specifier : struct_or_union ID LBRACE struct_declaration_list RBRACE'
  struct = ASE.Compiler.Cxx.Struct.Struct()
  struct.setIdentifier(t[2])
  struct.setChildren(t[4])
  if t[1] == 'union':
    struct.setUnion(1)
  t[0] = struct

def p_struct_or_union_specifier_2(t):
  'struct_or_union_specifier : struct_or_union LBRACE struct_declaration_list RBRACE'
  struct = ASE.Compiler.Cxx.Struct.Struct()
  struct.setChildren(t[3])
  if t[1] == 'union':
    struct.setUnion(1)
  t[0] = struct

def p_struct_or_union_specifier_3(t):
  'struct_or_union_specifier : struct_or_union ID'
  struct = ASE.Compiler.Cxx.Struct.Struct()
  struct.setIdentifier(t[2])
  t[0] = struct

# struct-or-union:
def p_struct_or_union(t):
  '''struct_or_union : STRUCT
                     | UNION'''
  t[0] = t[1]

# struct-declaration-list:

def p_struct_declaration_list_1(t):
  'struct_declaration_list : struct_declaration'
  t[0] = [t[1]]

def p_struct_declaration_list_2(t):
  'struct_declaration_list : struct_declaration_list struct_declaration'
  t[0] = t[1]+[t[2]]

# init-declarator-list:
def p_init_declarator_list_1(t):
  'init_declarator_list : init_declarator'
  t[0] = [t[1]]

def p_init_declarator_list_2(t):
  'init_declarator_list : init_declarator_list COMMA init_declarator'
  t[0] = t[1]+[t[3]]

# init-declarator
def p_init_declarator_1(t):
  'init_declarator : declarator'
  t[0] = t[1]

def p_init_declarator_2(t):
  'init_declarator : declarator EQUALS initializer'
  t[1].setInitializer(t[3])
  t[0] = t[1]

# struct-declaration:
def p_struct_declaration(t):
  'struct_declaration : specifier_qualifier_list struct_declarator_list SEMI'
  decl = ASE.Compiler.Cxx.Declaration.Declaration()
  for declarator in t[2]:
    applySpecifiers(declarator, [], t[1][0], t[1][1])
  decl.setChildren(t[2])
  t[0] = decl

# specifier-qualifier-list:
def p_specifier_qualifier_list_1(t):
  'specifier_qualifier_list : type_specifier specifier_qualifier_list'
  t[2][0].append(t[1])
  t[0] = t[2]

def p_specifier_qualifier_list_2(t):
  'specifier_qualifier_list : type_specifier'
  t[0] = ([t[1]], [])

def p_specifier_qualifier_list_3(t):
  'specifier_qualifier_list : type_qualifier specifier_qualifier_list'
  t[2][1].append(t[1])
  t[0] = t[2]

def p_specifier_qualifier_list_4(t):
  'specifier_qualifier_list : type_qualifier'
  t[0] = ([], [t[1]])

# struct-declarator-list:
def p_struct_declarator_list_1(t):
  'struct_declarator_list : struct_declarator'
  t[0] = [t[1]]

def p_struct_declarator_list_2(t):
  'struct_declarator_list : struct_declarator_list COMMA struct_declarator'
  t[0] = t[1]+[t[3]]

# struct-declarator:
def p_struct_declarator_1(t):
  'struct_declarator : declarator'
  t[0] = t[1]

def p_struct_declarator_2(t):
  'struct_declarator : declarator COLON constant_expression'
  bitField = ASE.Compiler.Cxx.BitField.BitField()
  bitField.setIdentifier(t[1])
  bitField.setSize(t[3].getValue())
  t[0] = bitField

def p_struct_declarator_3(t):
  'struct_declarator : COLON constant_expression'
  bitField = ASE.Compiler.Cxx.BitField.BitField()
  bitField.setSize(t[3].getValue())
  t[0] = bitField

# enum-specifier:
def p_enum_specifier_1(t):
  'enum_specifier : ENUM ID LBRACE enumerator_list RBRACE'
  enum = ASE.Compiler.Cxx.Enumeration.Enumeration()
  enum.setIdentifier(t[2])
  if t[4]:
    enum.addChildren(t[4])
  t[0] = enum

def p_enum_specifier_2(t):
  'enum_specifier : ENUM LBRACE enumerator_list RBRACE'
  enum = ASE.Compiler.Cxx.Enumeration.Enumeration()
  if t[3]:
    enum.addChildren(t[3])
  t[0] = enum

def p_enum_specifier_3(t):
  'enum_specifier : ENUM ID'
  enum = ASE.Compiler.Cxx.Enumeration.Enumeration()
  enum.setIdentifier(t[2])
  t[0] = enum

# enumerator_list:
def p_enumerator_list_1(t):
  'enumerator_list : enumerator'
  t[0] = [t[1]]

def p_enumerator_list_2(t):
  'enumerator_list : enumerator_list COMMA enumerator'
  t[0] = t[1]+[t[3]]

# enumerator:
def p_enumerator_1(t):
  'enumerator : ID'
  enum = ASE.Compiler.Cxx.Enumerator.Enumerator()
  enum.setIdentifier(t[1])
  t[0] = enum

def p_enumerator_2(t):
  'enumerator : ID EQUALS constant_expression'
  enum = ASE.Compiler.Cxx.Enumerator.Enumerator()
  enum.setIdentifier(t[1])
  enum.setValue(t[3].getValue_int())
  enum.setInitialized(1)
  t[0] = enum

# declarator:

def p_declarator_1(t):
  'declarator : pointer direct_declarator'
  type = t[2].getType()
  if not type is None:
    t[1].setChildren([type])
  t[2].setType(t[1])
  t[0] = t[2]

def p_declarator_2(t):
  'declarator : direct_declarator'
  t[0] = t[1]

# direct-declarator:

def p_direct_declarator_1(t):
  'direct_declarator : ID'
  declarator = ASE.Compiler.Cxx.Declarator.Declarator()
  declarator.setIdentifier(t[1])
  t[0] = declarator

def p_direct_declarator_2(t):
  'direct_declarator : direct_declarator LBRACKET constant_expression_opt RBRACKET'
  array = ASE.Compiler.Cxx.Array.Array()
  array.setChildren([t[1]])
  array.setSize(t[3])
  t[0] = array

def p_direct_declarator_3(t):
  'direct_declarator : LPAREN declarator RPAREN'
  group = ASE.Compiler.Cxx.DeclaratorGroup.DeclaratorGroup()
  group.setChildren([t[2]])
  t[0] = group

def p_direct_declarator_4(t):
  'direct_declarator : direct_declarator LPAREN parameter_type_list RPAREN '
  function = ASE.Compiler.Cxx.Function.Function()
  function.setChildren([t[1]])
  if t[3][-1] == '...':
    function.setOptionalArguments(1)
    function.setParameters(t[3][:-1])
  else:
    function.setParameters(t[3])
  t[0] = function

def p_direct_declarator_5(t):
  'direct_declarator : direct_declarator LPAREN identifier_list RPAREN '
  # Reject old-style declarations
  raise RuntimeError('K&R declarations are invalid')

def p_direct_declarator_6(t):
  'direct_declarator : direct_declarator LPAREN RPAREN '
  # Reject old-style declarations
  raise RuntimeError('Empty parameter lists are invalid')

# pointer:
def p_pointer_1(t):
  'pointer : TIMES type_qualifier_list'
  # Ignoring qualifiers for now
  pointer = ASE.Compiler.Cxx.Pointer.Pointer()
  for qualifier in t[2]:
    if qualifier == 'const':
      pointer.setConst(1)
    elif qualifier == 'volatile':
      pointer.setVolatile(1)
  t[0] = pointer

def p_pointer_2(t):
  'pointer : TIMES'
  t[0] = ASE.Compiler.Cxx.Pointer.Pointer()

def p_pointer_3(t):
  'pointer : TIMES type_qualifier_list pointer'
  pointer = ASE.Compiler.Cxx.Pointer.Pointer()
  pointer.setChildren([t[3]])
  for qualifier in t[2]:
    if qualifier == 'const':
      pointer.setConst(1)
    elif qualifier == 'volatile':
      pointer.setVolatile(1)
  t[0] = pointer

def p_pointer_4(t):
  'pointer : TIMES pointer'
  pointer = ASE.Compiler.Cxx.Pointer.Pointer()
  pointer.setChildren([t[2]])
  t[0] = pointer

# type-qualifier-list:

def p_type_qualifier_list_1(t):
  'type_qualifier_list : type_qualifier'
  t[0] = [t[1]]

def p_type_qualifier_list_2(t):
  'type_qualifier_list : type_qualifier_list type_qualifier'
  t[0] = t[1]+[t[2]]

# parameter-type-list:

def p_parameter_type_list_1(t):
  'parameter_type_list : parameter_list'
  t[0] = t[1]

def p_parameter_type_list_2(t):
  'parameter_type_list : parameter_list COMMA ELLIPSIS'
  t[0] = t[1]+[t[3]]

# parameter-list:

def p_parameter_list_1(t):
  'parameter_list : parameter_declaration'
  t[0] = [t[1]]

def p_parameter_list_2(t):
  'parameter_list : parameter_list COMMA parameter_declaration'
  t[0] = t[1]+[t[3]]

# parameter-declaration:
def p_parameter_declaration_1(t):
  'parameter_declaration : declaration_specifiers declarator'
  # TODO: Not handling array or functions
  parameter = ASE.Compiler.Cxx.Parameter.Parameter()
  parameter.setIdentifier(t[2].getIdentifier())
  if not t[2].getType() is None:
    parameter.setType(t[2].getType())
  t[0] = applySpecifiers(parameter, t[1][0], t[1][1], t[1][2])

def p_parameter_declaration_2(t):
  'parameter_declaration : declaration_specifiers abstract_declarator_opt'
  # TODO: Not handling array or functions
  parameter = ASE.Compiler.Cxx.Parameter.Parameter()
  if not t[2] is None and not t[2].getType() is None:
    parameter.setType(t[2].getType())
  t[0] = applySpecifiers(parameter, t[1][0], t[1][1], t[1][2])

# identifier-list-opt:
def p_identifier_list_opt_1(t):
  'identifier_list_opt : identifier_list'
  t[0] = t[1]

def p_identifier_list_opt_2(t):
  'identifier_list_opt : empty'
  pass

# identifier-list:
def p_identifier_list_1(t):
  'identifier_list : ID'
  t[0] = [t[1]]

def p_identifier_list_2(t):
  'identifier_list : identifier_list COMMA ID'
  t[0] = t[1]+[t[3]]

# initializer:
def p_initializer_1(t):
  'initializer : assignment_expression'
  initializer = ASE.Compiler.Cxx.Initializer.Initializer()
  initializer.setChildren([t[1]])
  t[0] = initializer

def p_initializer_2(t):
  '''initializer : LBRACE initializer_list RBRACE
                 | LBRACE initializer_list COMMA RBRACE'''
  initializer = ASE.Compiler.Cxx.Initializer.Initializer()
  initializer.setChildren(t[2])
  initializer.setList(1)
  t[0] = initializer

# initializer-list:

def p_initializer_list_1(t):
  'initializer_list : initializer'
  t[0] = [t[1]]

def p_initializer_list_2(t):
  'initializer_list : initializer_list COMMA initializer'
  t[0] = t[1]+[t[3]]

# type-name:

def p_type_name(t):
  'type_name : specifier_qualifier_list abstract_declarator_opt'
  if t[2] is None:
    t[2] = ASE.Compiler.Cxx.Declarator.Declarator()
  t[0] = applySpecifiers(t[2], [], t[1][0], t[1][1])

def p_abstract_declarator_opt_1(t):
  'abstract_declarator_opt : empty'
  pass

def p_abstract_declarator_opt_2(t):
  'abstract_declarator_opt : abstract_declarator'
  t[0] = t[1]

# abstract-declarator:

def p_abstract_declarator_1(t):
  'abstract_declarator : pointer '
  decl = ASE.Compiler.Cxx.Declarator.Declarator()
  decl.setType(t[1])
  t[0] = decl

def p_abstract_declarator_2(t):
  'abstract_declarator : pointer direct_abstract_declarator'
  type = t[2].getType()
  if not type is None:
    t[1].setChildren([type])
  t[2].setType(t[1])
  t[0] = t[2]

def p_abstract_declarator_3(t):
  'abstract_declarator : direct_abstract_declarator'
  t[0] = t[1]

# direct-abstract-declarator:

def p_direct_abstract_declarator_1(t):
  'direct_abstract_declarator : LPAREN abstract_declarator RPAREN'
  group = ASE.Compiler.Cxx.DeclaratorGroup.DeclaratorGroup()
  group.setChildren([t[2]])
  t[0] = group

def p_direct_abstract_declarator_2(t):
  'direct_abstract_declarator : direct_abstract_declarator LBRACKET constant_expression_opt RBRACKET'
  array = ASE.Compiler.Cxx.Array.Array()
  array.setChildren([t[1]])
  array.setSize(t[2])
  t[0] = array

def p_direct_abstract_declarator_3(t):
  'direct_abstract_declarator : LBRACKET constant_expression_opt RBRACKET'
  array = ASE.Compiler.Cxx.Array.Array()
  array.setChildren([ASE.Compiler.Cxx.Declarator.Declarator()])
  array.setSize(t[2])
  t[0] = array

def p_direct_abstract_declarator_4(t):
  'direct_abstract_declarator : direct_abstract_declarator LPAREN parameter_type_list_opt RPAREN'
  function = ASE.Compiler.Cxx.Function.Function()
  function.setChildren([t[1]])
  if t[3][-1] == '...':
    function.setOptionalArguments(1)
    function.setParameters(t[3][:-1])
  else:
    function.setParameters(t[3])
  t[0] = function

def p_direct_abstract_declarator_5(t):
  'direct_abstract_declarator : LPAREN parameter_type_list_opt RPAREN'
  function = ASE.Compiler.Cxx.Function.Function()
  function.setChildren([ASE.Compiler.Cxx.Declarator.Declarator()])
  if t[2][-1] == '...':
    function.setOptionalArguments(1)
    function.setParameters(t[2][:-1])
  else:
    function.setParameters(t[2])
  t[0] = function

# Optional fields in abstract declarators

def p_constant_expression_opt_1(t):
  'constant_expression_opt : empty'
  pass

def p_constant_expression_opt_2(t):
  'constant_expression_opt : constant_expression'
  t[0] = t[1]

def p_parameter_type_list_opt_1(t):
  'parameter_type_list_opt : empty'
  pass

def p_parameter_type_list_opt_2(t):
  'parameter_type_list_opt : parameter_type_list'
  t[0] = t[1]

# statement:

def p_statement(t):
    '''
    statement : labeled_statement
              | expression_statement
              | compound_statement
              | if_statement
              | switch_statement
              | iteration_statement
              | jump_statement
              '''
    t[0] = t[1]

# labeled-statement:

def p_labeled_statement_1(t):
  'labeled_statement : ID COLON statement'
  t[3].setIdentifier(t[1])
  t[0] = t[3]

def p_labeled_statement_2(t):
  'labeled_statement : CASE constant_expression COLON statement'
  stmt = ASE.Compiler.Cxx.Statement.Statement()
  stmt.setCaseLabel(t[2])
  stmt.setChildren([t[4]])
  t[0] = stmt

def p_labeled_statement_3(t):
  'labeled_statement : DEFAULT COLON statement'
  t[3].setIdentifier(t[1])
  t[0] = t[3]

# expression-statement:
def p_expression_statement(t):
  'expression_statement : expression_opt SEMI'
  stmt = ASE.Compiler.Cxx.ExpressionStatement.ExpressionStatement()
  stmt.setChildren([t[1]])
  t[0] = stmt

# compound-statement:
def p_compound_statement_1(t):
  'compound_statement : LBRACE declaration_list statement_list RBRACE'
  stmt = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
  stmt.setDeclarations(t[2])
  stmt.setChildren(t[3])
  t[0] = stmt

def p_compound_statement_2(t):
  'compound_statement : LBRACE statement_list RBRACE'
  stmt = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
  stmt.setChildren(t[2])
  t[0] = stmt

def p_compound_statement_3(t):
  'compound_statement : LBRACE declaration_list RBRACE'
  stmt = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
  stmt.setDeclarations(t[2])
  t[0] = stmt

def p_compound_statement_4(t):
  'compound_statement : LBRACE RBRACE'
  stmt = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
  t[0] = stmt

# statement-list:

def p_statement_list_1(t):
  'statement_list : statement'
  t[0] = [t[1]]

def p_statement_list_2(t):
  'statement_list : statement_list statement'
  t[0] = t[1]+[t[2]]  

# if-statement

def p_if_statement_1(t):
  'if_statement : IF LPAREN expression RPAREN statement'
  ifstmt = ASE.Compiler.Cxx.If.If()
  ifstmt.setBranch(t[3])
  ifstmt.setChildren([t[5]])
  t[0] = ifstmt

def p_if_statement_2(t):
  'if_statement : IF LPAREN expression RPAREN statement ELSE statement '
  ifstmt = ASE.Compiler.Cxx.If.If()
  ifstmt.setBranch(t[3])
  ifstmt.setChildren([t[5],t[7]])
  t[0] = ifstmt

# switch-statement

def p_switch_statement(t):
  'switch_statement : SWITCH LPAREN expression RPAREN compound_statement '
  switch = ASE.Compiler.Cxx.Switch.Switch()
  switch.setBranch(t[3])
  switch.setChildren([t[5]])
  t[0] = switch

# iteration_statement:

def p_iteration_statement_1(t):
  'iteration_statement : WHILE LPAREN expression RPAREN statement'
  stmt = ASE.Compiler.Cxx.While.While()
  stmt.setBranch(t[3])
  stmt.setChildren([t[5]])
  t[0] = stmt

def p_iteration_statement_2(t):
  'iteration_statement : FOR LPAREN expression_opt SEMI expression_opt SEMI expression_opt RPAREN statement '
  stmt = ASE.Compiler.Cxx.For.For()
  if not t[3] is None:
    stmt.setInitialization(t[3])
  if not t[5] is None:
    stmt.setBranch(t[5])
  if not t[7] is None:
    stmt.setIncrement(t[7])
  stmt.setChildren([t[9]])
  t[0] = stmt

def p_iteration_statement_3(t):
  'iteration_statement : DO statement WHILE LPAREN expression RPAREN SEMI'
  stmt = ASE.Compiler.Cxx.While.While()
  stmt.setBranch(t[5])
  stmt.setDoWhile(1)
  stmt.setChildren([t[2]])
  t[0] = stmt

# jump_statement:

def p_jump_statement_1(t):
  'jump_statement : GOTO ID SEMI'
  stmt = ASE.Compiler.Cxx.Goto.Goto()
  stmt.setIdentifier(t[2])
  t[0] = stmt

def p_jump_statement_2(t):
  'jump_statement : CONTINUE SEMI'
  stmt = ASE.Compiler.Cxx.Continue.Continue()
  t[0] = stmt

def p_jump_statement_3(t):
  'jump_statement : BREAK SEMI'
  t[0] = ASE.Compiler.Cxx.Break.Break()

def p_jump_statement_4(t):
  'jump_statement : RETURN expression_opt SEMI'
  stmt = ASE.Compiler.Cxx.Return.Return()
  if not t[2] is None:
    stmt.setChildren([t[2]])
  t[0] = stmt

# expression-opt:

def p_expression_opt_1(t):
  'expression_opt : empty'
  pass

def p_expression_opt_2(t):
  'expression_opt : expression'
  t[0] = t[1]

# expression:
def p_expression_1(t):
  'expression : assignment_expression'
  t[0] = t[1]

def p_expression_2(t):
  'expression : expression COMMA assignment_expression'
  # This is the multiple expression form
  #raise RuntimeError('What in the hell is this form for?\n'+str(t[1])+' '+str(t[1].getChildren())+' '+str(t[3].getValue_int()))
  t[0] = t[3]

# assigment_expression:
def p_assignment_expression_1(t):
  'assignment_expression : conditional_expression'
  t[0] = t[1]

def p_assignment_expression_2(t):
  'assignment_expression : unary_expression EQUALS assignment_expression'
  assign = ASE.Compiler.Cxx.Assignment.Assignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_3(t):
  'assignment_expression : unary_expression OREQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.InclusiveOrAssignment.InclusiveOrAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_4(t):
  'assignment_expression : unary_expression XOREQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.ExclusiveOrAssignment.ExclusiveOrAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_5(t):
  'assignment_expression : unary_expression ANDEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.AndAssignment.AndAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_6(t):
  'assignment_expression : unary_expression LSHIFTEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.LeftShiftAssignment.LeftShiftAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_7(t):
  'assignment_expression : unary_expression RSHIFTEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.RightShiftAssignment.RightShiftAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_8(t):
  'assignment_expression : unary_expression PLUSEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_9(t):
  'assignment_expression : unary_expression MINUSEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.SubtractionAssignment.SubtractionAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_10(t):
  'assignment_expression : unary_expression TIMESEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.MultiplicationAssignment.MultiplicationAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_11(t):
  'assignment_expression : unary_expression DIVEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.DivisionAssignment.DivisionAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

def p_assignment_expression_12(t):
  'assignment_expression : unary_expression MODEQUAL assignment_expression'
  assign = ASE.Compiler.Cxx.ModuloAssignment.ModuloAssignment()
  assign.setChildren([t[1], t[3]])
  t[0] = assign

# conditional-expression
def p_conditional_expression_1(t):
  'conditional_expression : logical_or_expression'
  t[0] = t[1]

def p_conditional_expression_2(t):
  'conditional_expression : logical_or_expression CONDOP expression COLON conditional_expression '
  ifstmt = ASE.Compiler.Cxx.If.If()
  ifstmt.setBranch(t[1])
  ifstmt.setChildren([t[3], t[5]])
  ifstmt.setTernary(1)
  t[0] = ifstmt

# constant-expression

def p_constant_expression(t):
  'constant_expression : conditional_expression'
  t[0] = t[1]

# logical-or-expression

def p_logical_or_expression_1(t):
  'logical_or_expression : logical_and_expression'
  t[0] = t[1]

def p_logical_or_expression_2(t):
  'logical_or_expression : logical_or_expression LOR logical_and_expression'
  orExp = ASE.Compiler.Cxx.LogicalOr.LogicalOr()
  orExp.setChildren([t[1], t[3]])
  t[0] = orExp

# logical-and-expression

def p_logical_and_expression_1(t):
  'logical_and_expression : inclusive_or_expression'
  t[0] = t[1]

def p_logical_and_expression_2(t):
  'logical_and_expression : logical_and_expression LAND inclusive_or_expression'
  andExp = ASE.Compiler.Cxx.LogicalAnd.LogicalAnd()
  andExp.setChildren([t[1], t[3]])
  t[0] = andExp

# inclusive-or-expression:
def p_inclusive_or_expression_1(t):
  'inclusive_or_expression : exclusive_or_expression'
  t[0] = t[1]

def p_inclusive_or_expression_2(t):
  'inclusive_or_expression : inclusive_or_expression OR exclusive_or_expression'
  orExp = ASE.Compiler.Cxx.InclusiveOr.InclusiveOr()
  orExp.setChildren([t[1],t[3]])
  t[0] = orExp

# exclusive-or-expression:

def p_exclusive_or_expression_1(t):
  'exclusive_or_expression :  and_expression'
  t[0] = t[1]

def p_exclusive_or_expression_2(t):
  'exclusive_or_expression :  exclusive_or_expression XOR and_expression'
  xorExp = ASE.Compiler.Cxx.ExclusiveOr.ExclusiveOr()
  xorExp.setChildren([t[1],t[3]])
  t[0] = xorExp

# AND-expression
def p_and_expression_1(t):
  'and_expression : equality_expression'
  t[0] = t[1]

def p_and_expression_2(t):
  'and_expression : and_expression AND equality_expression'
  andExp = ASE.Compiler.Cxx.And.And()
  andExp.setChildren([t[1],t[3]])
  t[0] = andExp

# equality-expression:
def p_equality_expression_1(t):
  'equality_expression : relational_expression'
  t[0] = t[1]

def p_equality_expression_2(t):
  'equality_expression : equality_expression EQ relational_expression'
  eqExp = ASE.Compiler.Cxx.Equality.Equality()
  eqExp.setChildren([t[1],t[3]])
  t[0] = eqExp

def p_equality_expression_3(t):
  'equality_expression : equality_expression NE relational_expression'
  neqExp = ASE.Compiler.Cxx.Inequality.Inequality()
  neqExp.setChildren([t[1],t[3]])
  t[0] = neqExp


# relational-expression:
def p_relational_expression_1(t):
  'relational_expression : shift_expression'
  t[0] = t[1]

def p_relational_expression_2(t):
  'relational_expression : relational_expression LT shift_expression'
  ltExp = ASE.Compiler.Cxx.LessThan.LessThan()
  ltExp.setChildren([t[1],t[3]])
  t[0] = ltExp

def p_relational_expression_3(t):
  'relational_expression : relational_expression GT shift_expression'
  gtExp = ASE.Compiler.Cxx.GreaterThan.GreaterThan()
  gtExp.setChildren([t[1],t[3]])
  t[0] = gtExp

def p_relational_expression_4(t):
  'relational_expression : relational_expression LE shift_expression'
  lteqExp = ASE.Compiler.Cxx.LessThanOrEqual.LessThanOrEqual()
  lteqExp.setChildren([t[1],t[3]])
  t[0] = lteqExp

def p_relational_expression_5(t):
  'relational_expression : relational_expression GE shift_expression'
  gteqExp = ASE.Compiler.Cxx.GreaterThanOrEqual.GreaterThanOrEqual()
  gteqExp.setChildren([t[1],t[3]])
  t[0] = gteqExp

# shift-expression

def p_shift_expression_1(t):
  'shift_expression : additive_expression'
  t[0] = t[1]

def p_shift_expression_2(t):
  'shift_expression : shift_expression LSHIFT additive_expression'
  lshift = ASE.Compiler.Cxx.LeftShift.LeftShift()
  lshift.setChildren([t[1],t[3]])
  t[0] = lshift

def p_shift_expression_3(t):
  'shift_expression : shift_expression RSHIFT additive_expression'
  rshift = ASE.Compiler.Cxx.RightShift.RightShift()
  rshift.setChildren([t[1],t[3]])
  t[0] = rshift

# additive-expression

def p_additive_expression_1(t):
  'additive_expression : multiplicative_expression'
  t[0] = t[1]

def p_additive_expression_2(t):
  'additive_expression : additive_expression PLUS multiplicative_expression'
  add = ASE.Compiler.Cxx.Addition.Addition()
  add.setChildren([t[1],t[3]])
  t[0] = add

def p_additive_expression_3(t):
  'additive_expression : additive_expression MINUS multiplicative_expression'
  sub = ASE.Compiler.Cxx.Subtraction.Subtraction()
  sub.setChildren([t[1],t[3]])
  t[0] = sub

# multiplicative-expression

def p_multiplicative_expression_1(t):
  'multiplicative_expression : cast_expression'
  t[0] = t[1]

def p_multiplicative_expression_2(t):
  'multiplicative_expression : multiplicative_expression TIMES cast_expression'
  mult = ASE.Compiler.Cxx.Multiplication.Multiplication()
  mult.setChildren([t[1],t[3]])
  t[0] = mult

def p_multiplicative_expression_3(t):
  'multiplicative_expression : multiplicative_expression DIVIDE cast_expression'
  div = ASE.Compiler.Cxx.Division.Division()
  div.setChildren([t[1],t[3]])
  t[0] = div

def p_multiplicative_expression_4(t):
  'multiplicative_expression : multiplicative_expression MOD cast_expression'
  mod = ASE.Compiler.Cxx.Modulo.Modulo()
  mod.setChildren([t[1],t[3]])
  t[0] = mod

# cast-expression:
def p_cast_expression_1(t):
  'cast_expression : unary_expression'
  t[0] = t[1]

def p_cast_expression_2(t):
  'cast_expression : LPAREN type_name RPAREN cast_expression'
  cast = ASE.Compiler.Cxx.Cast.Cast()
  cast.setChildren([t[4]])
  cast.setTypeName(t[2])
  t[0] = cast

# unary-expression:
def p_unary_expression_1(t):
  'unary_expression : postfix_expression'
  t[0] = t[1]

def p_unary_expression_2(t):
  'unary_expression : PLUSPLUS unary_expression'
  inc = ASE.Compiler.Cxx.Increment.Increment()
  inc.setChildren([t[2]])
  t[0] = inc

def p_unary_expression_3(t):
  'unary_expression : MINUSMINUS unary_expression'
  inc = ASE.Compiler.Cxx.Increment.Increment()
  inc.setChildren([t[2]])
  inc.setDecrement(1)
  t[0] = inc

def p_unary_expression_4(t):
  'unary_expression : SIZEOF unary_expression'
  sizeof = ASE.Compiler.Cxx.Sizeof.Sizeof()
  sizeof.setChildren([t[2]])
  t[0] = sizeof

def p_unary_expression_5(t):
  'unary_expression : SIZEOF LPAREN type_name RPAREN'
  sizeof = ASE.Compiler.Cxx.Sizeof.Sizeof()
  sizeof.setTypeName(t[3])
  t[0] = sizeof

def p_unary_expression_6(t):
  'unary_expression : AND cast_expression'
  address = ASE.Compiler.Cxx.Address.Address()
  address.setChildren([t[2]])
  t[0] = address

def p_unary_expression_7(t):
  'unary_expression : TIMES cast_expression'
  data = ASE.Compiler.Cxx.Indirection.Indirection()
  data.setChildren([t[2]])
  t[0] = data

def p_unary_expression_8(t):
  'unary_expression : PLUS cast_expression'
  plus = ASE.Compiler.Cxx.UnaryPlus.UnaryPlus()
  plus.setChildren([t[2]])
  t[0] = plus

def p_unary_expression_9(t):
  'unary_expression : MINUS cast_expression'
  minus = ASE.Compiler.Cxx.UnaryMinus.UnaryMinus()
  minus.setChildren([t[2]])
  t[0] = minus

def p_unary_expression_10(t):
  'unary_expression : NOT cast_expression'
  comp = ASE.Compiler.Cxx.Complement.Complement()
  comp.setChildren([t[2]])
  t[0] = comp

def p_unary_expression_11(t):
  'unary_expression : LNOT cast_expression'
  neg = ASE.Compiler.Cxx.Negation.Negation()
  neg.setChildren([t[2]])
  t[0] = neg

# postfix-expression:
def p_postfix_expression_1(t):
  'postfix_expression : primary_expression'
  t[0] = t[1]

def p_postfix_expression_2(t):
  'postfix_expression : postfix_expression LBRACKET expression RBRACKET'
  ref = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
  ref.setChildren([t[1]])
  ref.setIndex(t[3])
  t[0] = ref

def p_postfix_expression_3(t):
  'postfix_expression : postfix_expression LPAREN argument_expression_list RPAREN'
  call = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
  call.setChildren([t[1]])
  call.setArguments(t[3])
  t[0] = call

def p_postfix_expression_4(t):
  'postfix_expression : postfix_expression LPAREN RPAREN'
  call = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
  call.setChildren([t[1]])
  t[0] = call

def p_postfix_expression_5(t):
  'postfix_expression : postfix_expression PERIOD ID'
  ref = ASE.Compiler.Cxx.StructReference.StructReference()
  ref.setChildren([t[1]])
  ref.setMember(t[3])
  t[0] = ref

def p_postfix_expression_6(t):
  'postfix_expression : postfix_expression ARROW ID'
  ref = ASE.Compiler.Cxx.StructReference.StructReference()
  ref.setChildren([t[1]])
  ref.setMember(t[3])
  ref.setPointer(1)
  t[0] = ref

def p_postfix_expression_7(t):
  'postfix_expression : postfix_expression PLUSPLUS'
  inc = ASE.Compiler.Cxx.Increment.Increment()
  inc.setChildren([t[1]])
  inc.setPostfix(1)
  t[0] = inc

def p_postfix_expression_8(t):
  'postfix_expression : postfix_expression MINUSMINUS'
  inc = ASE.Compiler.Cxx.Increment.Increment()
  inc.setChildren([t[1]])
  inc.setDecrement(1)
  inc.setPostfix(1)
  t[0] = inc

# primary-expression:
def p_primary_expression_1(t):
  '''primary_expression : LPAREN expression RPAREN'''
  if hasattr(t.slice[1], 'define') and hasattr(t.slice[3], 'define'):
    group = ASE.Compiler.Cxx.Variable.Variable()
    group.setIdentifier(t.slice[1].define)
  else:
    group = ASE.Compiler.Cxx.Group.Group()
    group.setChildren([t[2]])
  t[0] = group

def p_primary_expression_2(t):
  '''primary_expression : ID'''
  variable = ASE.Compiler.Cxx.Variable.Variable()
  variable.setIdentifier(t[1])
  t[0] = variable

def p_primary_expression_3(t):
  '''primary_expression : ICONST'''
  if hasattr(t.slice[1], 'define'):
    constant = ASE.Compiler.Cxx.Variable.Variable()
    constant.setIdentifier(t.slice[1].define)
  else:
    constant = ASE.Compiler.Cxx.Constant.Constant()
    if t[1].endswith('L'):
      constant.setValue_int(int(long(t[1])))
    else:
      constant.setValue_int(int(t[1]))
  t[0] = constant

def p_primary_expression_4(t):
  '''primary_expression : HCONST'''
  constant = ASE.Compiler.Cxx.Constant.Constant()
  constant.setValue_int(int(float(t[1])))
  t[0] = constant

def p_primary_expression_5(t):
  '''primary_expression : FCONST'''
  constant = ASE.Compiler.Cxx.Constant.Constant()
  constant.setValue_double(float(t[1]))
  t[0] = constant

def p_primary_expression_6(t):
  '''primary_expression : CCONST'''
  constant = ASE.Compiler.Cxx.Constant.Constant()
  constant.setValue_char(t[1])
  t[0] = constant

def p_primary_expression_7(t):
  '''primary_expression : SCONST'''
  constant = ASE.Compiler.Cxx.Constant.Constant()
  constant.setValue_string(t[1])
  t[0] = constant

# argument-expression-list:
def p_argument_expression_list_1(t):
  '''argument_expression_list : assignment_expression'''
  t[0] = [t[1]]

def p_argument_expression_list_2(t):
  '''argument_expression_list : argument_expression_list COMMA assignment_expression'''
  t[0] = t[1]+[t[3]]

def p_empty(t):
  'empty : '
  pass

def printError():
  import sys
  if hasattr(lexer, 'currentFile'):
    sys.stderr.write('Current file: ')
    sys.stderr.write(lexer.currentFile)
    sys.stderr.write('\n')
  sys.stderr.write(str(parser.symstack))
  sys.stderr.write('\n')
  sys.stderr.write(str(parser.statestack))
  sys.stderr.write('\n')
  return

def p_error(t):
  #print 'ERROR:',t
  printError()
  raise RuntimeError('Parse error: '+str(t))

# Driver
def setup(debug = 0, forceRebuild = 0):
  global parser, typeDefs

  if forceRebuild or not 'parser' in globals():
    parser = yacc.yacc(debug = debug)
  typeDefs = {}
  return parser

if __name__ == '__main__':
  import ASE.Compiler.Cxx.cIncludeExpansion
  import ASE.Compiler.Cxx.Printer
  import sys

  #import profile
  # Build the grammar
  #profile.run("yacc.yacc()")

  setup(1)
  expander = ASE.Compiler.Cxx.cIncludeExpansion.IncludeExpander()
  printer  = ASE.Compiler.Cxx.Printer.Printer()
  try:
    for i in range(1, len(sys.argv)):
      print "About to parse %s" % sys.argv[i]
      f = file(sys.argv[i])
      s = f.read()
      f.close()
      ast = parser.parse(s, lexer = expander)
      map(lambda root: root.accept(printer), ast)
  except RuntimeError, e:
    print 'In file '+expander.currentFile
    print e
