#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_Cxx_Modulo
#include "ASE/Compiler/Cxx/Modulo_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

#ifndef included_ASE_ASEException
#include "ASE/ASEException_IOR.h"
#endif

#ifndef included_ASE_BaseException
#include "ASE/BaseException_IOR.h"
#endif

#ifndef included_ASE_Compiler_CompilerException
#include "ASE/Compiler/CompilerException_IOR.h"
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_Cxx_Modulo_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_Cxx_Modulo__object *ior;
  enum ReferenceType refType;
};

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_Cxx_Modulo_PythonObject *self = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) type->tp_alloc(type, 0);
    
    if (self)
    {
      struct ASE_BaseClass__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseClass", NULL);
      
      self->ior = (*(ior->d_epv->castObject))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      /* Allow objects without an attached implementation (used in the skeleton constructor) */
      if (baseIor->d_data)
      {
        if (refType == NEW_REF)
        {
          (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
        }
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_Compiler_Vertex__wrap_ior(struct ASE_Compiler_Vertex__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Vertex");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vertex");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Vertex");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Vertex");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Cxx_Visitor__wrap_ior(struct ASE_Compiler_Cxx_Visitor__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Cxx.Visitor");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Visitor");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Cxx.Visitor");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Cxx.Visitor");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Cxx_Modulo__wrap_ior(struct ASE_Compiler_Cxx_Modulo__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Cxx.Modulo");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Modulo");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Cxx.Modulo");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Cxx.Modulo");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_BaseInterface__wrap_ior(struct ASE_BaseInterface__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseInterface");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseInterface");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseInterface");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseInterface");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Comment__wrap_ior(struct ASE_Compiler_Comment__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Comment");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Comment");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Comment");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Comment");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Attribute__wrap_ior(struct ASE_Compiler_Attribute__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Attribute");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Attribute");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Attribute");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Attribute");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_Compiler_Attribute__extract_ior(struct ASE_Compiler_Cxx_Modulo_PythonObject *obj, struct ASE_Compiler_Attribute__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Attribute", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Attribute");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Comment__extract_ior(struct ASE_Compiler_Cxx_Modulo_PythonObject *obj, struct ASE_Compiler_Comment__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Comment", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Comment");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Cxx_Vertex__extract_ior(struct ASE_Compiler_Cxx_Modulo_PythonObject *obj, struct ASE_Compiler_Cxx_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Cxx.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Cxx.Vertex");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Vertex__extract_ior(struct ASE_Compiler_Cxx_Modulo_PythonObject *obj, struct ASE_Compiler_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Vertex");
    return 0;
  }
  return 1;
}

static long ASE_BaseInterface__extract_ior(struct ASE_Compiler_Cxx_Modulo_PythonObject *obj, struct ASE_BaseInterface__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.BaseInterface", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.BaseInterface");
    return 0;
  }
  return 1;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static PyObject *ASE_Compiler_Vertex__wrap_array(struct ASE_Compiler_Vertex__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    long dimen = (*_extFunc->getDimen)(array, NULL);
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (array)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Vertex__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Vertex__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Comment
#include "ASE/Compiler/Comment_IOR.h"
#endif

static PyObject *ASE_Compiler_Comment__wrap_array(struct ASE_Compiler_Comment__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Comment__external *(*_externals)(void) = (const struct ASE_Compiler_Comment__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Comment", NULL);
    const struct ASE_Compiler_Comment__external *_extFunc = (*_externals)();
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    long dimen = (*_extFunc->getDimen)(array, NULL);
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (array)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Comment__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Comment__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Attribute
#include "ASE/Compiler/Attribute_IOR.h"
#endif

static PyObject *ASE_Compiler_Attribute__wrap_array(struct ASE_Compiler_Attribute__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Attribute__external *(*_externals)(void) = (const struct ASE_Compiler_Attribute__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Attribute", NULL);
    const struct ASE_Compiler_Attribute__external *_extFunc = (*_externals)();
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    long dimen = (*_extFunc->getDimen)(array, NULL);
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (array)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Attribute__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Attribute__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static long ASE_Compiler_Vertex__extract_array(PyObject *obj, struct ASE_Compiler_Vertex__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Vertex__object *elem;
      
      if (!ASE_Compiler_Vertex__extract_ior((struct ASE_Compiler_Cxx_Modulo_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

#ifndef included_ASE_Compiler_Comment
#include "ASE/Compiler/Comment_IOR.h"
#endif

static long ASE_Compiler_Comment__extract_array(PyObject *obj, struct ASE_Compiler_Comment__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Comment__external *(*_externals)(void) = (const struct ASE_Compiler_Comment__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Comment", NULL);
    const struct ASE_Compiler_Comment__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Comment__object *elem;
      
      if (!ASE_Compiler_Comment__extract_ior((struct ASE_Compiler_Cxx_Modulo_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

#ifndef included_ASE_Compiler_Attribute
#include "ASE/Compiler/Attribute_IOR.h"
#endif

static long ASE_Compiler_Attribute__extract_array(PyObject *obj, struct ASE_Compiler_Attribute__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Attribute__external *(*_externals)(void) = (const struct ASE_Compiler_Attribute__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Attribute", NULL);
    const struct ASE_Compiler_Attribute__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Attribute__object *elem;
      
      if (!ASE_Compiler_Attribute__extract_ior((struct ASE_Compiler_Cxx_Modulo_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

static void ASE_Compiler_Cxx_Modulo_accept(struct ASE_Compiler_Cxx_Modulo__object *_self, struct ASE_Compiler_Cxx_Visitor__object *visitor, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Cxx.Modulo.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "accept");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Cxx.Modulo.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  (*(((struct ASE_BaseInterface__object *) visitor)->d_epv->addReference))(((struct ASE_BaseInterface__object *) visitor)->d_object, NULL);
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Cxx_Visitor__wrap_ior, visitor);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Cxx.Modulo.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Cxx_Modulo_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Cxx_Modulo_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_DECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Cxx.Modulo.accept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static struct ASE_Compiler_Cxx_Vertex__object *ASE_Compiler_Cxx_Modulo_copy(struct ASE_Compiler_Cxx_Modulo__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  struct ASE_Compiler_Cxx_Vertex__object *_return_value = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Cxx.Modulo.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "copy");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Cxx.Modulo.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Cxx.Modulo.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Cxx_Modulo_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_DECREF(_excTraceback);
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "O&", (void *) ASE_Compiler_Cxx_Vertex__extract_ior, &_return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.Cxx.Modulo.copy", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static void ASE_Compiler_Cxx_Modulo_Modulo(struct ASE_Compiler_Cxx_Modulo__object *obj, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = PyImport_ImportModule((char *) "ASE.Compiler.Cxx.Modulo_impl");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Cxx.Modulo.Modulo", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "Modulo");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Cxx.Modulo.Modulo", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Cxx_Modulo__wrap_ior, obj);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Cxx.Modulo.Modulo", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Create a new instance */
  _return_value = PyInstance_New(_method, _args, NULL);
  _pyReturn = Py_None;
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_return_value)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not create instance of ASE.Compiler.Cxx.Modulo_impl.Modulo", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  obj->d_data = _return_value;
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Cxx_Modulo_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Cxx_Modulo_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_DECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Cxx.Modulo.Modulo", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Cxx_Modulo_Modulo_(struct ASE_Compiler_Cxx_Modulo__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *data = (PyObject *) _self->d_data;
  
  /* If we do not set d_data to NULL, we get a SEGV on the recursive call to deleteReference() coming from self.__IORself */
  _self->d_data = NULL;
  /* Remove only reference to Python object */
  Py_XDECREF(data);
}

void ASE_Compiler_Cxx_Modulo__set_epv(struct ASE_Compiler_Cxx_Modulo__epv *_epv, struct ASE_BaseException__object **_exception);

void ASE_Compiler_Cxx_Modulo__set_epv(struct ASE_Compiler_Cxx_Modulo__epv *_epv, struct ASE_BaseException__object **_exception)
{
  static int initialized = 0;
  
  if (!initialized)
  {
    initialized = 1;
    SIDL_Python_Init();
    if (!PyImport_ImportModule((char *) "user"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import user", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "importer"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import importer", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "ASE.Compiler.Cxx.Modulo"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import ASE.Compiler.Cxx.Modulo", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    import_array();
  }
  _epv->Modulo = ASE_Compiler_Cxx_Modulo_Modulo;
  _epv->Modulo_ = ASE_Compiler_Cxx_Modulo_Modulo_;
  _epv->accept = ASE_Compiler_Cxx_Modulo_accept;
  _epv->copy = ASE_Compiler_Cxx_Modulo_copy;
}
