# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.IORGenerator
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class IORGenerator:
  '''
  This visitor transforms a SIDL AST into a set of C IOR ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.sourceLanguages = ['Cxx']
    self.IOR = IOR()
    self.SIDL = SIDL()
    self.Cxx = Cxx()
    self.templateMethods = {}

  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    if not language in self.source:
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    if not language in self.source:
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    if not language in self.source:
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitInterface(self, node):
    if node.getFullIdentifier() == 'ASE.BaseInterface':
      self.createBaseTypes()
    decls = []
    defns = []
    
    def extend(d):
      decls.extend(d[0])
      defns.extend(d[1])
      return
    if node.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      [self.createTemplateMethod(method) for method in node.getMethods()]
    extend(self.getEPV(node))
    extend(self.getIORImplementations(node))
    extend(self.getSEPV(node))
    extend(self.getObject(node))
    extend(self.getArray(node))
    extend(self.getExternal(node))
    extend(self.getExternals(node))
    self.addSource('Cxx', [self.getHeaderFile(node, decls)])
    self.addSource('Cxx', [self.getSourceFile(node, defns)])
  
  def visitClass(self, node):
    self.visitInterface(node)
  
  def visitEnumeration(self, node):
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Enumerator
    import ASE.Compiler.SIDL.Enumerator
    enumType = ASE.Compiler.Cxx.Enumeration.Enumeration()
    enumType.setIdentifier(node.getFullIdentifier().replace('.', '_') + '__enum')
    enumType.setInitialized(1)
    for child in node.getChildren():
      enumerator = ASE.Compiler.SIDL.Enumerator.Enumerator(child)
      entry = ASE.Compiler.Cxx.Enumerator.Enumerator()
      entry.setIdentifier(enumerator.getIdentifier())
      entry.setInitialized(1)
      entry.setValue(enumerator.getValue())
      enumType.addChildren([entry])
    enumDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    enumDecl.setType(enumType)
    self.addSource('Cxx', [self.getHeaderFile(node, [self.Cxx.getDecl(enumDecl)])])
  # Ending class methods section
  pass
  
  def hasStaticMethod(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      for method in type.getMethods():
        if method.getStatic():
          return 1
    return 0
  
  def getForwardDeclarations(self, type, noExceptions = 0):
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.SIDL.Interface
    import ASE.Compiler.SIDL.Type
    import ASE.Compiler.Cxx.Struct
    if ASE.Compiler.SIDL.Type.Type(type).getBaseType() or type.isInstanceOf('ASE.Compiler.SIDL.Enumeration') or type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
      return ([], [])
    interface = ASE.Compiler.SIDL.Interface.Interface(type)
    decls = []
    objStruct = self.IOR.getType(interface).getChildren()[0]
    decls.append(self.Cxx.getDecl(objStruct, 'These structures are used in this header'))
    if self.hasStaticMethod(interface):
      sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
      sepvStruct.setIdentifier(self.IOR.getStaticExternalType(interface).getChildren()[0].getIdentifier())
      decls.append(self.Cxx.getDecl(sepvStruct))
    objArray = ASE.Compiler.SIDL.Array.Array()
    objArray.setType(interface)
    arrayStruct = self.IOR.getType(objArray).getChildren()[0]
    decls.append(self.Cxx.getDecl(arrayStruct))
    if not noExceptions:
      if len(interface.getMethods()) and not interface.getFullIdentifier() == 'ASE.BaseInterface':
        baseExc = self.SIDL.getBaseException(interface.getMethods()[0].getExceptions()[0])
        excStruct = self.IOR.getType(baseExc).getChildren()[0]
        decls.append(self.Cxx.getDecl(excStruct))
        excArray = ASE.Compiler.SIDL.Array.Array()
        excArray.setType(baseExc)
        arrayStruct = self.IOR.getType(excArray).getChildren()[0]
        decls.append(self.Cxx.getDecl(arrayStruct))
    return (decls, [])
  
  def getEPV(self, interface):
    import ASE.Compiler.Cxx.Comment
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    if interface.getBaseType():
      return ([], [])
    comment = ASE.Compiler.Cxx.Comment.Comment()
    comment.setComment('The entry point vector (EPV) is the virtual function table for this interface')
    epvStruct = ASE.Compiler.Cxx.Struct.Struct()
    epvStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__epv')
    epvStruct.setComments([comment])
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Interface
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    epvStruct.addChildren([self.IOR.getFunctionDeclaration(interface, method, 1) for method in self.getBuiltinMethods(interface)])
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      epvStruct.addChildren([self.IOR.getFunctionDeclaration(interface, method, 1) for method in [self.getImplConstructor(interface), self.getImplDestructor(interface)]])
    epvStruct.addChildren([self.IOR.getFunctionDeclaration(interface, method, 1) for method in self.getTemplateMethods(interface.getMethods()) if not method.getStatic()])
    epvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    epvDecl.setType(epvStruct)
    return ([self.Cxx.getDecl(epvDecl)], [])
  
  def getSEPVFunc(self, klass):
    '''
    This is a hack
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.Type
    sepvType = ASE.Compiler.SIDL.Type.Type()
    sepvType.setIdentifier('struct ' + klass.getFullIdentifier().replace('.', '_') + '__sepv *')
    sepvReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    sepvReturn.setIdentifier('_return_value')
    sepvReturn.setType(sepvType)
    sepvReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    getStaticEPV = ASE.Compiler.SIDL.Method.Method()
    getStaticEPV.setIdentifier('getStaticEPV')
    getStaticEPV.setReturnParameter(sepvReturn)
    getStaticEPV.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    getStaticEPV.setStatic(1)
    return getStaticEPV
  
  def getSEPV(self, klass):
    import ASE.Compiler.SIDL.Interface
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    if not self.hasStaticMethod(klass):
      return ([], [])
    sepvType = self.IOR.getStaticExternalType(klass)
    sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
    sepvStruct.setIdentifier(sepvType.getChildren()[0].getIdentifier())
    sepvStruct.addChildren([self.IOR.getFunctionDeclaration(klass, method, 1) for method in self.getTemplateMethods(klass.getMethods()) if method.getStatic()])
    sepvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    sepvDecl.setType(sepvStruct)
    sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
    sepvStruct.setIdentifier(sepvType.getChildren()[0].getIdentifier())
    sepvVar = self.Cxx.getVar('sepv')
    sepvInit = ASE.Compiler.Cxx.Initializer.Initializer()
    sepvInit.setChildren([self.Cxx.getInteger(0) for method in self.getTemplateMethods(klass.getMethods()) if method.getStatic()])
    initVar = self.Cxx.getVar('initialized')
    setSEPVVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '__set_sepv')
    sepvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    sepvParam.setIdentifier(sepvVar.getIdentifier())
    sepvParam.setType(sepvType)
    setSEPVName = ASE.Compiler.Cxx.Declarator.Declarator()
    setSEPVName.setIdentifier(setSEPVVar.getIdentifier())
    setSEPVDecl = ASE.Compiler.Cxx.Function.Function()
    setSEPVDecl.setChildren([setSEPVName])
    setSEPVDecl.setType(self.Cxx.typeMap['void'])
    setSEPVDecl.setParameters([sepvParam])
    setSEPVDecl.setExtern(1)
    negInitVar = ASE.Compiler.Cxx.Negation.Negation()
    negInitVar.setChildren([initVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    initSEPV = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    initSEPV.setChildren([setSEPVVar])
    initSEPV.setArguments([self.Cxx.getAddress(sepvVar)])
    compound.addChildren([self.Cxx.getExpStmt(initSEPV)])
    for method in [m for m in self.getIORMethods(klass) if m.getStatic()]:
      objMethod = ASE.Compiler.Cxx.StructReference.StructReference()
      objMethod.setChildren([sepvVar])
      objMethod.setMember(method.getIdentifier())
      setMethod = ASE.Compiler.Cxx.Assignment.Assignment()
      setMethod.setChildren([objMethod, self.Cxx.getVar(method.getIdentifier())])
      compound.addChildren([self.Cxx.getExpStmt(setMethod)])
    setInit = ASE.Compiler.Cxx.Assignment.Assignment()
    setInit.setChildren([initVar, self.Cxx.getInteger(1)])
    compound.addChildren([self.Cxx.getExpStmt(setInit)])
    checkInit = ASE.Compiler.Cxx.If.If()
    checkInit.setBranch(negInitVar)
    checkInit.setChildren([compound])
    returnSEPV = ASE.Compiler.Cxx.Return.Return()
    returnSEPV.setChildren([self.Cxx.getAddress(sepvVar)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(sepvVar, sepvStruct, sepvInit, 'Static methods for this class', 1), self.Cxx.getDeclaration(initVar, self.IOR.typeMap['bool'], isStatic = 1)])
    compound.setChildren([checkInit, returnSEPV])
    sepvFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(klass, self.getSEPVFunc(klass)).getChildren()[0])
    sepvFunc.setBody(compound)
    sepvFunc.setStatic(1)
    return ([self.Cxx.getDecl(sepvDecl, 'The static entry point vector (SEPV) is the static function table for this class')], [self.Cxx.getDecl(setSEPVDecl, 'Initialization routine defined in the skeleton'), sepvFunc])
  
  def getConstructor(self, klass):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Builtin method')
    iorReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    iorReturn.setIdentifier('_return_value')
    iorReturn.setType(klass)
    iorReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier('createObject')
    constructor.setReturnParameter(iorReturn)
    constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    constructor.setParent(klass)
    constructor.setStatic(1)
    constructor.setComments([comment])
    return constructor
  
  def getImplConstructor(self, klass):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Builtin method')
    objParam = ASE.Compiler.SIDL.Parameter.Parameter()
    objParam.setIdentifier('obj')
    objParam.setType(klass)
    objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier(klass.getIdentifier())
    constructor.setChildren([objParam])
    constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    constructor.setParent(klass)
    constructor.setStatic(1)
    constructor.setComments([comment])
    return constructor
  
  def getInit(self, klass):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    iorParam = ASE.Compiler.SIDL.Parameter.Parameter()
    iorParam.setIdentifier('self')
    iorParam.setType(klass)
    iorParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    objParam = ASE.Compiler.SIDL.Parameter.Parameter()
    objParam.setIdentifier('obj')
    objParam.setType(self.SIDL.typeMap['opaque'])
    objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    init = ASE.Compiler.SIDL.Method.Method()
    init.setIdentifier('initObject')
    init.setChildren([iorParam, objParam])
    init.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    init.setStatic(1)
    return init
  
  def getFini(self, klass):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    iorParam = ASE.Compiler.SIDL.Parameter.Parameter()
    iorParam.setIdentifier('self')
    iorParam.setType(klass)
    iorParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    fini = ASE.Compiler.SIDL.Method.Method()
    fini.setIdentifier('finiObject')
    fini.setChildren([iorParam])
    fini.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    fini.setStatic(1)
    return fini
  
  def getDestructor(self, interface):
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Builtin method')
    destructor = ASE.Compiler.SIDL.Method.Method()
    destructor.setIdentifier('destroyObject')
    destructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    destructor.setParent(interface)
    destructor.setComments([comment])
    return destructor
  
  def getImplDestructor(self, klass):
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Builtin method')
    destructor = ASE.Compiler.SIDL.Method.Method()
    destructor.setIdentifier(klass.getIdentifier() + '_')
    destructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
    destructor.setParent(klass)
    destructor.setComments([comment])
    return destructor
  
  def getCast(self, interface):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Builtin method')
    nameParam = ASE.Compiler.SIDL.Parameter.Parameter()
    nameParam.setIdentifier('name')
    nameParam.setType(self.SIDL.typeMap['string'])
    nameParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    iorReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    iorReturn.setIdentifier('ior')
    iorReturn.setType(self.SIDL.typeMap['opaque'])
    iorReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    cast = ASE.Compiler.SIDL.Method.Method()
    cast.setIdentifier('castObject')
    cast.setReturnParameter(iorReturn)
    cast.setChildren([nameParam])
    cast.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    cast.setParent(interface)
    cast.setComments([comment])
    return cast
  
  def getStr(self, interface):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Builtin method')
    stringReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    stringReturn.setIdentifier('_return_value')
    stringReturn.setType(self.SIDL.typeMap['string'])
    stringReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    string = ASE.Compiler.SIDL.Method.Method()
    string.setIdentifier('strObject')
    string.setReturnParameter(stringReturn)
    string.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    string.setParent(interface)
    string.setComments([comment])
    return string
  
  def getBuiltinMethods(self, interface):
    return [self.getCast(interface), self.getDestructor(interface), self.getStr(interface)]
  
  def initInterfaceEpv(self, interface, implementor, epvVar, body, seen):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.StructReference
    if interface in seen:
      return
    seen.append(interface)
    comment = 'Initialize interface ' + interface.getFullIdentifier() + ' methods with those of class ' + implementor.getFullIdentifier()
    for methodName in [method.getIdentifier() for method in self.getBuiltinMethods(interface) + list(self.getTemplateMethods(interface.getMethods()))]:
      infEpvVar = self.Cxx.getVar(interface.getFullIdentifier().replace('.', '_') + '_epv')
      infMethod = ASE.Compiler.Cxx.StructReference.StructReference()
      infMethod.setChildren([infEpvVar])
      infMethod.setMember(methodName)
      objMethod = ASE.Compiler.Cxx.StructReference.StructReference()
      objMethod.setChildren([epvVar])
      objMethod.setMember(methodName)
      setMethod = ASE.Compiler.Cxx.Assignment.Assignment()
      setMethod.setChildren([infMethod, objMethod])
      body.addChildren([self.Cxx.getExpStmt(setMethod, comment)])
      comment = None
    for parent in interface.getParents():
      self.initInterfaceEpv(parent, implementor, epvVar, body, seen)
    return
  
  def initClassEpv(self, klass, obj, body, isPointer = 0, interfacesSeen = None, implMethods = None):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.StructReference
    if interfacesSeen is None:
      interfacesSeen = []
    epvVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '_epv')
    for parent in klass.getParents():
      epvVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '_epv')
      parentObj = ASE.Compiler.Cxx.StructReference.StructReference()
      parentObj.setChildren([obj])
      parentObj.setMember(parent.getFullIdentifier().replace('.', '_').lower())
      if isPointer:
        parentObj.setPointer(1)
      parentEpvVar = ASE.Compiler.Cxx.StructReference.StructReference()
      parentEpvVar.setChildren([parentObj])
      parentEpvVar.setMember('d_epv')
      comment = 'Override class ' + klass.getFullIdentifier() + ' methods with those of superclass ' + parent.getFullIdentifier()
      if implMethods is None:
        implMethods = []
      implMethods.extend([m.getIdentifier() for m in self.getTemplateMethods(list(klass.getImplementedMethods())+self.getIORMethods(klass))])
      for method in [m for m in self.getTemplateMethods(parent.getMethods()) if not m.getIdentifier() in implMethods]:
        classMethod = ASE.Compiler.Cxx.StructReference.StructReference()
        classMethod.setChildren([epvVar])
        classMethod.setMember(method.getIdentifier())
        parentMethod = ASE.Compiler.Cxx.StructReference.StructReference()
        parentMethod.setChildren([parentEpvVar])
        parentMethod.setMember(method.getIdentifier())
        parentMethod.setPointer(1)
        setMethod = ASE.Compiler.Cxx.Assignment.Assignment()
        setMethod.setChildren([classMethod, parentMethod])
        body.addChildren([self.Cxx.getExpStmt(setMethod, comment)])
        comment = None
      comment = 'Virtualize superclass ' + parent.getFullIdentifier() + ' methods with those of class ' + klass.getFullIdentifier()
      parentEpvVar = self.Cxx.getVar(parent.getFullIdentifier().replace('.', '_') + '_epv')
      for method in self.getBuiltinMethods(klass) + list(self.getTemplateMethods(parent.getMethods())):
        classMethod = ASE.Compiler.Cxx.StructReference.StructReference()
        classMethod.setChildren([epvVar])
        classMethod.setMember(method.getIdentifier())
        parentMethod = ASE.Compiler.Cxx.StructReference.StructReference()
        parentMethod.setChildren([parentEpvVar])
        parentMethod.setMember(method.getIdentifier())
        setMethod = ASE.Compiler.Cxx.Assignment.Assignment()
        setMethod.setChildren([parentMethod, classMethod])
        body.addChildren([self.Cxx.getExpStmt(setMethod, comment)])
        comment = None
    for interface in klass.getInterfaces():
      self.initInterfaceEpv(interface, klass, epvVar, body, interfacesSeen)
    for parent in klass.getParents():
      import ASE.Compiler.SIDL.Class
      parentObj = ASE.Compiler.Cxx.StructReference.StructReference()
      parentObj.setChildren([obj])
      parentObj.setMember(parent.getFullIdentifier().replace('.', '_').lower())
      if isPointer:
        parentObj.setPointer(1)
      self.initClassEpv(ASE.Compiler.SIDL.Class.Class(parent), parentObj, body, interfacesSeen = interfacesSeen, implMethods = implMethods)
    return
  
  def initEpv(self, klass, body):
    '''
    Initialize the static EPV structures for klass
    '''
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Negation
    epvInitVar = self.Cxx.getVar('epvInitialized')
    negEpvInit = ASE.Compiler.Cxx.Negation.Negation()
    negEpvInit.setChildren([epvInitVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    epvVar = self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '_epv')
    for method in self.getBuiltinMethods(klass) + [m for m in self.getTemplateMethods(self.getIORMethods(klass)) if not m.getStatic()]:
      objMethod = ASE.Compiler.Cxx.StructReference.StructReference()
      objMethod.setChildren([epvVar])
      objMethod.setMember(method.getIdentifier())
      setMethod = ASE.Compiler.Cxx.Assignment.Assignment()
      setMethod.setChildren([objMethod, self.Cxx.getVar(method.getIdentifier())])
      compound.addChildren([self.Cxx.getExpStmt(setMethod)])
    skelInit = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    skelInit.setChildren([self.Cxx.getVar(klass.getFullIdentifier().replace('.', '_') + '__set_epv')])
    skelInit.setArguments([self.Cxx.getAddress(epvVar), self.Cxx.getVar('_exception')])
    compound.addChildren([self.Cxx.getExpStmt(skelInit)])
    self.initClassEpv(klass, self.Cxx.getVar('self'), compound, isPointer = 1)
    setEpvInit = ASE.Compiler.Cxx.Assignment.Assignment()
    setEpvInit.setChildren([epvInitVar, self.Cxx.getInteger(1)])
    compound.addChildren([self.Cxx.getExpStmt(setEpvInit)])
    checkEpvInit = ASE.Compiler.Cxx.If.If()
    checkEpvInit.setBranch(negEpvInit)
    checkEpvInit.setChildren([compound])
    body.addDeclarations([self.Cxx.getDeclaration(epvInitVar, self.IOR.typeMap['bool'])])
    body.addChildren([checkEpvInit])
    return
  
  def setEpv(self, interface, obj, body, seen = None, isPointer = 0):
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Type
    if seen is None:
      seen = []
    epvVar = self.Cxx.getVar(interface.getFullIdentifier().replace('.', '_') + '_epv')
    if not interface in seen:
      seen.append(interface)
      body.addDeclarations([self.Cxx.getDeclaration(epvVar, ASE.Compiler.Cxx.Type.Type(self.IOR.getEpvType(interface).getChildren()[0]), isStatic = 1)])
    for parent in interface.getParents():
      parentObj = ASE.Compiler.Cxx.StructReference.StructReference()
      parentObj.setChildren([obj])
      parentObj.setMember(parent.getFullIdentifier().replace('.', '_').lower())
      parentObj.setPointer(isPointer)
      self.setEpv(parent, parentObj, body, seen)
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      import ASE.Compiler.SIDL.Class
      for inf in ASE.Compiler.SIDL.Class.Class(interface).getInterfaces():
        infObj = ASE.Compiler.Cxx.StructReference.StructReference()
        infObj.setChildren([obj])
        infObj.setMember(inf.getFullIdentifier().replace('.', '_').lower())
        infObj.setPointer(isPointer)
        self.setEpv(inf, infObj, body, seen)
    objEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    objEpv.setChildren([obj])
    objEpv.setMember('d_epv')
    objEpv.setPointer(isPointer)
    setEpv = ASE.Compiler.Cxx.Assignment.Assignment()
    setEpv.setChildren([objEpv, self.Cxx.getAddress(epvVar)])
    body.addChildren([self.Cxx.getExpStmt(setEpv)])
    return
  
  def setInterfaceObject(self, interface, selfVar, objVar, body, useSelfPointer = 0):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.StructReference
    infStruct = ASE.Compiler.Cxx.StructReference.StructReference()
    infStruct.setChildren([selfVar])
    infStruct.setMember(interface.getFullIdentifier().replace('.', '_').lower())
    infStruct.setPointer(useSelfPointer)
    infObj = ASE.Compiler.Cxx.StructReference.StructReference()
    infObj.setChildren([infStruct])
    infObj.setMember('d_object')
    setObj = ASE.Compiler.Cxx.Assignment.Assignment()
    setObj.setChildren([infObj, objVar])
    body.addChildren([self.Cxx.getExpStmt(setObj)])
    for parent in interface.getParents():
      self.setInterfaceObject(parent, infStruct, objVar, body)
    return
  
  def getInterfaceCastCheck(self, interface, obj):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Negation
    setCast = ASE.Compiler.Cxx.Assignment.Assignment()
    setCast.setChildren([self.Cxx.getVar('cast'), obj])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setChildren([self.Cxx.getExpStmt(setCast)])
    strCmp = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strCmp.setChildren([self.Cxx.getVar('strcmp')])
    strCmp.setArguments([self.Cxx.getVar('name'), self.Cxx.getString(interface.getFullIdentifier())])
    negStrCmp = ASE.Compiler.Cxx.Negation.Negation()
    negStrCmp.setChildren([strCmp])
    checkInterface = ASE.Compiler.Cxx.If.If()
    checkInterface.setBranch(negStrCmp)
    checkInterface.setChildren([trueBranch])
    yield (interface, checkInterface)
    for parent in interface.getParents():
      parentObj = ASE.Compiler.Cxx.StructReference.StructReference()
      parentObj.setChildren([obj])
      parentObj.setMember(parent.getFullIdentifier().replace('.', '_').lower())
      parentObj.setPointer(1)
      for check in self.getInterfaceCastCheck(parent, self.Cxx.getGroup(self.Cxx.getAddress(parentObj))):
        yield check
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      for inf in ASE.Compiler.SIDL.Class.Class(interface).getInterfaces():
        infObj = ASE.Compiler.Cxx.StructReference.StructReference()
        infObj.setChildren([obj])
        infObj.setMember(inf.getFullIdentifier().replace('.', '_').lower())
        infObj.setPointer(1)
        for check in self.getInterfaceCastCheck(inf, self.Cxx.getGroup(self.Cxx.getAddress(infObj))):
          yield check
    return 
  
  def getCastCheck(self, interface, obj):
    interfaces = []
    topCheck = None
    currentCheck = None
    for (inf, check) in self.getInterfaceCastCheck(interface, obj):
      if not inf in interfaces:
        interfaces.append(inf)
        if currentCheck is None:
          topCheck = check
        else:
          currentCheck.addChildren([check])
        currentCheck = check
    return topCheck
  
  def getObject(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    defns = []
    if type.getBaseType():
      typeName = type.getIdentifier()
      if typeName in ['dcomplex', 'fcomplex']:
        realDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        realDecl.setIdentifier('real')
        imagDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        imagDecl.setIdentifier('imaginary')
        if typeName == 'dcomplex':
          realDecl.setType(self.IOR.typeMap['double'])
          imagDecl.setType(self.IOR.typeMap['double'])
        if typeName == 'fcomplex':
          realDecl.setType(self.IOR.typeMap['float'])
          imagDecl.setType(self.IOR.typeMap['float'])
        objStruct = ASE.Compiler.Cxx.Struct.Struct()
        objStruct.setIdentifier('SIDL_' + typeName)
        objStruct.setChildren([self.Cxx.getDecl(realDecl), self.Cxx.getDecl(imagDecl)])
        objDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        objDecl.setType(objStruct)
      else:
        return ([], [])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      interface = ASE.Compiler.SIDL.Interface.Interface(type)
      objStruct = ASE.Compiler.Cxx.Struct.Struct()
      objStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__object')
      for parent in interface.getParents():
        parentDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        parentDecl.setIdentifier(parent.getFullIdentifier().replace('.', '_').lower())
        parentDecl.setType(ASE.Compiler.Cxx.Type.Type(self.IOR.getType(parent).getChildren()[0]))
        objStruct.addChildren([self.Cxx.getDecl(parentDecl)])
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        for interf in ASE.Compiler.SIDL.Class.Class(interface).getInterfaces():
          interfaceDecl = ASE.Compiler.Cxx.Declarator.Declarator()
          interfaceDecl.setIdentifier(interf.getFullIdentifier().replace('.', '_').lower())
          interfaceDecl.setType(ASE.Compiler.Cxx.Type.Type(self.IOR.getType(interf).getChildren()[0]))
          objStruct.addChildren([self.Cxx.getDecl(interfaceDecl)])
      epvStruct = ASE.Compiler.Cxx.Struct.Struct()
      epvStruct.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__epv')
      epvType = ASE.Compiler.Cxx.Pointer.Pointer()
      epvType.setChildren([epvStruct])
      epvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      epvDecl.setIdentifier('d_epv')
      epvDecl.setType(epvType)
      oldEpvDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      oldEpvDecl.setIdentifier('d_oldEpv')
      oldEpvDecl.setType(epvType)
      objStruct.addChildren([self.Cxx.getDecl(epvDecl), self.Cxx.getDecl(oldEpvDecl)])
      origDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      origDecl.setIdentifier('d_object')
      origDecl.setType(self.IOR.typeMap['opaque'])
      objStruct.addChildren([self.Cxx.getDecl(origDecl)])
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        implDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        implDecl.setIdentifier('d_data')
        implDecl.setType(self.IOR.typeMap['opaque'])
        objStruct.addChildren([self.Cxx.getDecl(implDecl)])
      objDecl = ASE.Compiler.Cxx.Declarator.Declarator()
      objDecl.setType(objStruct)
      if type.isInstanceOf('ASE.Compiler.SIDL.Class'):
        selfVar = self.Cxx.getVar('self')
        objVar = self.Cxx.getVar('obj')
        selfType = self.IOR.getType(type)
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        for parent in type.getParents():
          parentVar = self.Cxx.getVar('parent')
          parentType = self.IOR.getType(parent)
          parentObj = ASE.Compiler.Cxx.StructReference.StructReference()
          parentObj.setChildren([selfVar])
          parentObj.setMember(parent.getFullIdentifier().replace('.', '_').lower())
          parentObj.setPointer(1)
          parentInit = ASE.Compiler.Cxx.Initializer.Initializer()
          parentInit.setChildren([self.Cxx.getAddress(parentObj)])
          compound.addDeclarations([self.Cxx.getDeclaration(parentVar, parentType, parentInit)])
          extFuncVar = self.Cxx.getVar('extFunc')
          extFuncCastPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncCastPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncCastDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncCastDecl.setChildren([self.Cxx.getDeclGroup(extFuncCastPtr)])
          extFuncCastDecl.setType(self.IOR.getExternalType(parent))
          extFunc = ASE.Compiler.Cxx.Cast.Cast()
          extFunc.setTypeName(extFuncCastDecl)
          extFunc.setChildren([self.IOR.getClassMethodTable(parent)])
          extFuncInit = ASE.Compiler.Cxx.Initializer.Initializer()
          extFuncInit.setChildren([extFunc])
          extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
          extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
          extFuncDecl.setType(self.IOR.getExternalType(parent))
          extFuncDecl.setInitializer(extFuncInit)
          compound.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
          parentInitVar = self.Cxx.getVar('parentInit')
          extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          extFuncCall.setChildren([extFuncVar])
          parentInitFunc = ASE.Compiler.Cxx.StructReference.StructReference()
          parentInitFunc.setChildren([self.Cxx.getGroup(extFuncCall)])
          parentInitFunc.setMember('initObject')
          parentInitFunc.setPointer(1)
          parentInitInit = ASE.Compiler.Cxx.Initializer.Initializer()
          parentInitInit.setChildren([parentInitFunc])
          parentInitDecl = self.IOR.getFunctionDeclaration(parent, self.getInit(parent), 1)
          ASE.Compiler.Cxx.Function.Function(parentInitDecl.getChildren()[0]).setInitializer(parentInitInit)
          parentInitDecl.setComments([])
          parentInitDecl.getChildren()[0].getChildren()[0].getChildren()[0].setIdentifier(parentInitVar.getIdentifier())
          compound.addDeclarations([parentInitDecl])
          compound.addChildren([self.IOR.getErrorCheck(type, None, parentInitVar, 'Unable to find initializer for ' + parent.getFullIdentifier())])
          parentInitInd = ASE.Compiler.Cxx.Indirection.Indirection()
          parentInitInd.setChildren([parentInitVar])
          parentInitCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          parentInitCall.setChildren([self.Cxx.getGroup(parentInitInd)])
          parentInitCall.setArguments([parentVar, objVar, self.Cxx.getVar('_exception')])
          compound.addChildren([self.Cxx.getExpStmt(parentInitCall)])
        self.initEpv(type, compound)
        self.setEpv(type, selfVar, compound, isPointer = 1)
        objEpv = ASE.Compiler.Cxx.StructReference.StructReference()
        objEpv.setChildren([selfVar])
        objEpv.setMember('d_epv')
        objEpv.setPointer(1)
        oldEpv = ASE.Compiler.Cxx.StructReference.StructReference()
        oldEpv.setChildren([selfVar])
        oldEpv.setMember('d_oldEpv')
        oldEpv.setPointer(1)
        setOldEpv = ASE.Compiler.Cxx.Assignment.Assignment()
        setOldEpv.setChildren([oldEpv, objEpv])
        compound.addChildren([self.Cxx.getExpStmt(setOldEpv, 'Save old EPV for getSuper()')])
        selfObj = ASE.Compiler.Cxx.StructReference.StructReference()
        selfObj.setChildren([selfVar])
        selfObj.setMember('d_object')
        selfObj.setPointer(1)
        setObj = ASE.Compiler.Cxx.Assignment.Assignment()
        setObj.setChildren([selfObj, objVar])
        compound.addChildren([self.Cxx.getExpStmt(setObj, 'Initialize pointers to the original object')])
        for interface in type.getInterfaces():
          self.setInterfaceObject(interface, selfVar, objVar, compound, useSelfPointer = 1)
        dataVar = ASE.Compiler.Cxx.StructReference.StructReference()
        dataVar.setChildren([selfVar])
        dataVar.setMember('d_data')
        dataVar.setPointer(1)
        setData = ASE.Compiler.Cxx.Assignment.Assignment()
        setData.setChildren([dataVar, self.Cxx.getNull()])
        selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
        selfEpv.setChildren([selfVar])
        selfEpv.setMember('d_epv')
        selfEpv.setPointer(1)
        constructorFunc = ASE.Compiler.Cxx.StructReference.StructReference()
        constructorFunc.setChildren([selfEpv])
        constructorFunc.setMember(type.getIdentifier())
        constructorFunc.setPointer(1)
        constructorInd = ASE.Compiler.Cxx.Indirection.Indirection()
        constructorInd.setChildren([self.Cxx.getGroup(constructorFunc)])
        constructorCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        constructorCall.setChildren([self.Cxx.getGroup(constructorInd)])
        constructorCall.setArguments([selfVar, self.Cxx.getVar('_exception')])
        compound.addChildren([self.Cxx.getExpStmt(setData, 'Setup implementation data'), self.Cxx.getExpStmt(constructorCall)])
        initFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getInit(type)).getChildren()[0])
        initFunc.setBody(compound)
        initFunc.setStatic(1)
        selfDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        selfDecl.setType(selfType.getChildren()[0])
        sizeofSelf = ASE.Compiler.Cxx.Sizeof.Sizeof()
        sizeofSelf.setTypeName(selfDecl)
        mallocSelf = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        mallocSelf.setChildren([self.Cxx.getVar('malloc')])
        mallocSelf.setArguments([sizeofSelf])
        selfInit = ASE.Compiler.Cxx.Initializer.Initializer()
        selfInit.setChildren([self.Cxx.castToType(mallocSelf, selfType)])
        initSelf = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        initSelf.setChildren(initFunc.getChildren())
        initSelf.setArguments([selfVar, selfVar, self.Cxx.getVar('_exception')])
        returnSelf = ASE.Compiler.Cxx.Return.Return()
        returnSelf.setChildren([selfVar])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setDeclarations([self.Cxx.getDeclaration(selfVar, selfType, selfInit)])
        compound.setChildren([self.Cxx.getExpStmt(initSelf), returnSelf])
        createFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getConstructor(type)).getChildren()[0])
        createFunc.setBody(compound)
        createFunc.setStatic(1)
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        for interface in list(type.getInterfaces()) + list(type.getParents()):
          infStruct = ASE.Compiler.Cxx.StructReference.StructReference()
          infStruct.setChildren([selfVar])
          infStruct.setMember(interface.getFullIdentifier().replace('.', '_').lower())
          infStruct.setPointer(1)
          infEpv = ASE.Compiler.Cxx.StructReference.StructReference()
          infEpv.setChildren([infStruct])
          infEpv.setMember('d_epv')
          infOldEpv = ASE.Compiler.Cxx.StructReference.StructReference()
          infOldEpv.setChildren([infStruct])
          infOldEpv.setMember('d_oldEpv')
          setEpv = ASE.Compiler.Cxx.Assignment.Assignment()
          setEpv.setChildren([infEpv, infOldEpv])
          compound.addChildren([self.Cxx.getExpStmt(setEpv)])
        selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
        selfEpv.setChildren([selfVar])
        selfEpv.setMember('d_epv')
        selfEpv.setPointer(1)
        destructorFunc = ASE.Compiler.Cxx.StructReference.StructReference()
        destructorFunc.setChildren([selfEpv])
        destructorFunc.setMember(type.getIdentifier() + '_')
        destructorFunc.setPointer(1)
        destructorInd = ASE.Compiler.Cxx.Indirection.Indirection()
        destructorInd.setChildren([self.Cxx.getGroup(destructorFunc)])
        destructorCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        destructorCall.setChildren([self.Cxx.getGroup(destructorInd)])
        destructorCall.setArguments([selfVar, self.Cxx.getVar('_exception')])
        compound.addChildren([self.Cxx.getExpStmt(destructorCall)])
        for parent in type.getParents():
          parentVar = self.Cxx.getVar('parent')
          parentType = self.IOR.getType(parent)
          parentObj = ASE.Compiler.Cxx.StructReference.StructReference()
          parentObj.setChildren([selfVar])
          parentObj.setMember(parent.getFullIdentifier().replace('.', '_').lower())
          parentObj.setPointer(1)
          parentInit = ASE.Compiler.Cxx.Initializer.Initializer()
          parentInit.setChildren([self.Cxx.getAddress(parentObj)])
          compound.addDeclarations([self.Cxx.getDeclaration(parentVar, parentType, parentInit)])
          extFuncVar = self.Cxx.getVar('extFunc')
          extFuncCastPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncCastPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncCastDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncCastDecl.setChildren([self.Cxx.getDeclGroup(extFuncCastPtr)])
          extFuncCastDecl.setType(self.IOR.getExternalType(parent))
          extFunc = ASE.Compiler.Cxx.Cast.Cast()
          extFunc.setTypeName(extFuncCastDecl)
          extFunc.setChildren([self.IOR.getClassMethodTable(parent)])
          extFuncInit = ASE.Compiler.Cxx.Initializer.Initializer()
          extFuncInit.setChildren([extFunc])
          extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
          extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
          extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
          extFuncDecl = ASE.Compiler.Cxx.Function.Function()
          extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
          extFuncDecl.setType(self.IOR.getExternalType(parent))
          extFuncDecl.setInitializer(extFuncInit)
          compound.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
          parentFiniVar = self.Cxx.getVar('parentFini')
          extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          extFuncCall.setChildren([extFuncVar])
          parentFiniFunc = ASE.Compiler.Cxx.StructReference.StructReference()
          parentFiniFunc.setChildren([self.Cxx.getGroup(extFuncCall)])
          parentFiniFunc.setMember('finiObject')
          parentFiniFunc.setPointer(1)
          parentFiniInit = ASE.Compiler.Cxx.Initializer.Initializer()
          parentFiniInit.setChildren([parentFiniFunc])
          parentFiniDecl = self.IOR.getFunctionDeclaration(parent, self.getFini(parent), 1)
          ASE.Compiler.Cxx.Function.Function(parentFiniDecl.getChildren()[0]).setInitializer(parentFiniInit)
          parentFiniDecl.setComments([])
          parentFiniDecl.getChildren()[0].getChildren()[0].getChildren()[0].setIdentifier(parentFiniVar.getIdentifier())
          compound.addDeclarations([parentFiniDecl])
          compound.addChildren([self.IOR.getErrorCheck(type, None, parentFiniVar, 'Unable to find finalizer for ' + parent.getFullIdentifier())])
          parentFiniInd = ASE.Compiler.Cxx.Indirection.Indirection()
          parentFiniInd.setChildren([parentFiniVar])
          parentFiniCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          parentFiniCall.setChildren([self.Cxx.getGroup(parentFiniInd)])
          parentFiniCall.setArguments([parentVar, self.Cxx.getVar('_exception')])
          compound.addChildren([self.Cxx.getExpStmt(parentFiniCall)])
        finiFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getFini(type)).getChildren()[0])
        finiFunc.setBody(compound)
        finiFunc.setStatic(1)
        objVar = ASE.Compiler.Cxx.StructReference.StructReference()
        objVar.setChildren([selfVar])
        objVar.setMember('d_object')
        objVar.setPointer(1)
        finiCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        finiCall.setChildren(finiFunc.getChildren())
        finiCall.setArguments([objVar, self.Cxx.getVar('_exception')])
        selfDecl = ASE.Compiler.Cxx.Declarator.Declarator()
        selfDecl.setType(selfType.getChildren()[0])
        sizeofSelf = ASE.Compiler.Cxx.Sizeof.Sizeof()
        sizeofSelf.setTypeName(selfDecl)
        clearSelf = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        clearSelf.setChildren([self.Cxx.getVar('memset')])
        clearSelf.setArguments([objVar, self.Cxx.getInteger(0), sizeofSelf])
        freeSelf = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        freeSelf.setChildren([self.Cxx.getVar('free')])
        freeSelf.setArguments([objVar])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren(map(self.Cxx.getExpStmt, [finiCall, clearSelf, freeSelf]))
        destroyFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getDestructor(type)).getChildren()[0])
        destroyFunc.setBody(compound)
        destroyFunc.setStatic(1)
        objVar = self.Cxx.getVar('obj')
        castVar = self.Cxx.getVar('cast')
        selfObj = ASE.Compiler.Cxx.StructReference.StructReference()
        selfObj.setChildren([self.Cxx.getGroup(self.Cxx.castToType(selfVar, self.IOR.getType(self.SIDL.getBaseInterface(type))))])
        selfObj.setMember('d_object')
        selfObj.setPointer(1)
        objInit = ASE.Compiler.Cxx.Initializer.Initializer()
        objInit.setChildren([selfObj])
        castReturn = ASE.Compiler.Cxx.Return.Return()
        castReturn.setChildren([castVar])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setDeclarations([self.Cxx.getDeclaration(objVar, self.IOR.getType(type), objInit, 'We are guaranteed here that the first member in any IOR is a ASE.BaseInterface object'), self.Cxx.getDeclaration(castVar, self.IOR.typeMap['opaque'])])
        compound.setChildren([self.getCastCheck(type, objVar), castReturn])
        castFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getCast(type)).getChildren()[0])
        castFunc.setBody(compound)
        castFunc.setStatic(1)
        strReturn = ASE.Compiler.Cxx.Return.Return()
        strReturn.setChildren([self.Cxx.getString(type.getFullIdentifier())])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren([strReturn])
        strFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, self.getStr(type)).getChildren()[0])
        strFunc.setBody(compound)
        strFunc.setStatic(1)
        defns = []
        for parent in type.getParents():
          if type.getFullIdentifier() in ['ASE.DLL', 'ASE.Loader']:
            if type.getFullIdentifier() in ['ASE.DLL']:
              loader = self.SIDL.getLoader(type)
              defns.extend(self.getSEPV(loader)[0])
              defns.extend(self.getExternal(loader)[0])
              defns.extend(self.getExternals(loader)[0])
            aseExc = self.SIDL.getASEException(type)
            defns.append(self.IOR.getGuardedInclude(self.SIDL.getBaseException(aseExc)))
            defns.extend(self.getForwardDeclarations(aseExc)[0])
            defns.extend(self.getEPV(aseExc)[0])
            defns.extend(self.getObject(aseExc)[0])
            defns.extend(self.getExternal(aseExc)[0])
            defns.extend(self.getExternals(aseExc)[0])
          else:
            defns.extend([self.IOR.getGuardedInclude(self.SIDL.getLoader(type)), self.IOR.getGuardedInclude(self.SIDL.getASEException(type))])
        defns.extend([castFunc, strFunc, finiFunc, destroyFunc, initFunc, createFunc])
    return ([self.Cxx.getDecl(objDecl, 'The IOR object structure for this type')], defns)
  
  def initArray(self, type, method, body, isBorrowed):
    import ASE.Compiler.Cxx.Addition
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Modulo
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.MultiplicationAssignment
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Subtraction
    inParams = method.getInParameters()
    dimenVar = self.Cxx.getVar(inParams[0].getIdentifier())
    lowerVar = self.Cxx.getVar(inParams[1].getIdentifier())
    upperVar = self.Cxx.getVar(inParams[2].getIdentifier())
    if len(inParams) > 3:
      strideVar = self.Cxx.getVar(inParams[3].getIdentifier())
    else:
      strideVar = None
    returnVar = self.Cxx.getVar(method.getReturnParameter().getIdentifier())
    arraySizeVar = self.Cxx.getVar('arraySize')
    sizeVar = self.Cxx.getVar('size')
    dVar = self.Cxx.getVar('d')
    arrayType = self.IOR.getType(method.getReturnParameter().getType())
    arrayStruct = ASE.Compiler.Cxx.Struct.Struct(arrayType.getChildren()[0])
    intDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    intDecl.setType(self.IOR.typeMap['int'])
    sizeofInt = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofInt.setTypeName(intDecl)
    structDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    structDecl.setType(arrayStruct)
    sizeofStruct = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofStruct.setTypeName(structDecl)
    elemDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    elemDecl.setType(self.IOR.getType(type))
    sizeofElem = ASE.Compiler.Cxx.Sizeof.Sizeof()
    sizeofElem.setTypeName(elemDecl)
    structOffset = ASE.Compiler.Cxx.Modulo.Modulo()
    structOffset.setChildren([self.Cxx.getGroup(sizeofStruct), sizeofInt])
    wordDiff = ASE.Compiler.Cxx.Subtraction.Subtraction()
    wordDiff.setChildren([sizeofInt, structOffset])
    wordOffset = ASE.Compiler.Cxx.Modulo.Modulo()
    wordOffset.setChildren([self.Cxx.getGroup(wordDiff), sizeofInt])
    arraySize = ASE.Compiler.Cxx.Addition.Addition()
    arraySize.setChildren([sizeofStruct, wordOffset])
    arraySizeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    arraySizeInit.setChildren([arraySize])
    sizeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    sizeInit.setChildren([self.Cxx.getInteger(1)])
    safeDimen = ASE.Compiler.Cxx.If.If()
    safeDimen.setBranch(dimenVar)
    safeDimen.setChildren([dimenVar, self.Cxx.getInteger(1)])
    safeDimen.setTernary(1)
    indexSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indexSize.setChildren([sizeofInt, self.Cxx.getGroup(safeDimen)])
    indicesSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    indicesSize.setChildren([self.Cxx.getInteger(3), indexSize])
    returnSize = ASE.Compiler.Cxx.Addition.Addition()
    returnSize.setChildren([arraySizeVar, indicesSize])
    returnAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    returnAlloc.setChildren([self.Cxx.getVar('malloc')])
    returnAlloc.setArguments([returnSize])
    returnInit = ASE.Compiler.Cxx.Initializer.Initializer()
    returnInit.setChildren([self.Cxx.castToType(returnAlloc, arrayType)])
    body.addDeclarations([self.Cxx.getDeclaration(arraySizeVar, self.IOR.typeMap['int'], arraySizeInit, isStatic = 1), self.Cxx.getDeclaration(returnVar, arrayType, returnInit), self.Cxx.getDeclaration(sizeVar, self.IOR.typeMap['int'], sizeInit), self.Cxx.getDeclaration(dVar, self.IOR.typeMap['int'])])
    arrayDimen = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayDimen.setChildren([returnVar])
    arrayDimen.setMember('d_dimen')
    arrayDimen.setPointer(1)
    arrayBorrow = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayBorrow.setChildren([returnVar])
    arrayBorrow.setMember('d_borrowed')
    arrayBorrow.setPointer(1)
    arrayLower = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayLower.setChildren([returnVar])
    arrayLower.setMember('d_lower')
    arrayLower.setPointer(1)
    arrayUpper = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayUpper.setChildren([returnVar])
    arrayUpper.setMember('d_upper')
    arrayUpper.setPointer(1)
    arrayStride = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayStride.setChildren([returnVar])
    arrayStride.setMember('d_stride')
    arrayStride.setPointer(1)
    dArrayLower = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dArrayLower.setChildren([arrayLower])
    dArrayLower.setIndex(dVar)
    dArrayUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dArrayUpper.setChildren([arrayUpper])
    dArrayUpper.setIndex(dVar)
    dArrayStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dArrayStride.setChildren([arrayStride])
    dArrayStride.setIndex(dVar)
    lowerFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    lowerFirst.setChildren([lowerVar])
    lowerFirst.setMember('d_firstElement')
    lowerFirst.setPointer(1)
    upperFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    upperFirst.setChildren([upperVar])
    upperFirst.setMember('d_firstElement')
    upperFirst.setPointer(1)
    dLower = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dLower.setChildren([lowerFirst])
    dLower.setIndex(dVar)
    dUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dUpper.setChildren([upperFirst])
    dUpper.setIndex(dVar)
    if not strideVar is None:
      strideFirst = ASE.Compiler.Cxx.StructReference.StructReference()
      strideFirst.setChildren([strideVar])
      strideFirst.setMember('d_firstElement')
      strideFirst.setPointer(1)
      dStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
      dStride.setChildren([strideFirst])
      dStride.setIndex(dVar)
    setDimen = ASE.Compiler.Cxx.Assignment.Assignment()
    setDimen.setChildren([arrayDimen, dimenVar])
    setBorrowed = ASE.Compiler.Cxx.Assignment.Assignment()
    setBorrowed.setChildren([arrayBorrow, self.Cxx.getInteger(isBorrowed)])
    incReturn = ASE.Compiler.Cxx.Addition.Addition()
    incReturn.setChildren([self.Cxx.castToType(returnVar, self.IOR.typeMap['char'], 1), arraySizeVar])
    setLower = ASE.Compiler.Cxx.Assignment.Assignment()
    setLower.setChildren([arrayLower, self.Cxx.castToType(self.Cxx.getGroup(incReturn), self.IOR.typeMap['int'], 1)])
    incLower = ASE.Compiler.Cxx.Addition.Addition()
    incLower.setChildren([arrayLower, self.Cxx.getGroup(safeDimen)])
    setUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    setUpper.setChildren([arrayUpper, incLower])
    incUpper = ASE.Compiler.Cxx.Addition.Addition()
    incUpper.setChildren([arrayUpper, self.Cxx.getGroup(safeDimen)])
    setStride = ASE.Compiler.Cxx.Assignment.Assignment()
    setStride.setChildren([arrayStride, incUpper])
    lowerFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    lowerFirst.setChildren([arrayLower])
    lowerFirst.setIndex(self.Cxx.getInteger(0))
    initLower = ASE.Compiler.Cxx.Assignment.Assignment()
    initLower.setChildren([lowerFirst, self.Cxx.getInteger(0)])
    upperFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    upperFirst.setChildren([arrayUpper])
    upperFirst.setIndex(self.Cxx.getInteger(0))
    initUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    initUpper.setChildren([upperFirst, self.Cxx.getInteger(0)])
    strideFirst = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    strideFirst.setChildren([arrayStride])
    strideFirst.setIndex(self.Cxx.getInteger(0))
    initStride = ASE.Compiler.Cxx.Assignment.Assignment()
    initStride.setChildren([strideFirst, self.Cxx.getInteger(1)])
    body.addChildren([self.Cxx.getExpStmt(setDimen, 'Initialize array structure')])
    body.addChildren(map(self.Cxx.getExpStmt, [setBorrowed, setLower, setUpper, setStride, initLower, initUpper, initStride]))
    setLower = ASE.Compiler.Cxx.Assignment.Assignment()
    setLower.setChildren([dArrayLower, dLower])
    setUpper = ASE.Compiler.Cxx.Assignment.Assignment()
    setUpper.setChildren([dArrayUpper, dUpper])
    if strideVar is None:
      setStride = ASE.Compiler.Cxx.Assignment.Assignment()
      setStride.setChildren([dArrayStride, sizeVar])
    else:
      setStride = ASE.Compiler.Cxx.Assignment.Assignment()
      setStride.setChildren([dArrayStride, dStride])
    rowDiff = ASE.Compiler.Cxx.Subtraction.Subtraction()
    rowDiff.setChildren([dUpper, dLower])
    rowSize = ASE.Compiler.Cxx.Addition.Addition()
    rowSize.setChildren([rowDiff, self.Cxx.getInteger(1)])
    setSize = ASE.Compiler.Cxx.MultiplicationAssignment.MultiplicationAssignment()
    setSize.setChildren([sizeVar, rowSize])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren(map(self.Cxx.getExpStmt, [setLower, setUpper, setStride, setSize]))
    dInit = ASE.Compiler.Cxx.Assignment.Assignment()
    dInit.setChildren([dVar, self.Cxx.getInteger(0)])
    dCmp = ASE.Compiler.Cxx.LessThan.LessThan()
    dCmp.setChildren([dVar, dimenVar])
    dInc = ASE.Compiler.Cxx.Increment.Increment()
    dInc.setChildren([dVar])
    dInc.setPostfix(1)
    sizeLoop = ASE.Compiler.Cxx.For.For()
    sizeLoop.setInitialization(dInit)
    sizeLoop.setBranch(dCmp)
    sizeLoop.setIncrement(dInc)
    sizeLoop.setChildren([compound])
    sizeLoop.setComments([self.Cxx.getComment('Determine array size and set bounds')])
    setSize = ASE.Compiler.Cxx.MultiplicationAssignment.MultiplicationAssignment()
    setSize.setChildren([sizeVar, sizeofElem])
    body.addChildren([sizeLoop, self.Cxx.getExpStmt(setSize)])
    return
  
  def m_createArray(self, type, method, body):
    import ASE.Compiler.Cxx.Addition
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.StructReference
    self.initArray(type, method, body, 0)
    returnVar = self.Cxx.getVar(method.getReturnParameter().getIdentifier())
    sizeVar = self.Cxx.getVar('size')
    arrayFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayFirst.setChildren([returnVar])
    arrayFirst.setMember('d_firstElement')
    arrayFirst.setPointer(1)
    dataAlloc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    dataAlloc.setChildren([self.Cxx.getVar('malloc')])
    dataAlloc.setArguments([sizeVar])
    setData = ASE.Compiler.Cxx.Assignment.Assignment()
    setData.setChildren([arrayFirst, self.Cxx.castToType(dataAlloc, self.IOR.getType(type), 1)])
    dataInit = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    dataInit.setChildren([self.Cxx.getVar('memset')])
    dataInit.setArguments([arrayFirst, self.Cxx.getInteger(0), sizeVar])
    returnArray = ASE.Compiler.Cxx.Return.Return()
    returnArray.setChildren([returnVar])
    body.addChildren([self.Cxx.getExpStmt(setData, 'Allocate and initialize data'), self.Cxx.getExpStmt(dataInit), returnArray])
    return
  
  def m_borrowArray(self, type, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Return
    self.initArray(type, method, body, 1)
    firstVar = self.Cxx.getVar(method.getInParameters()[4].getIdentifier())
    returnVar = self.Cxx.getVar(method.getReturnParameter().getIdentifier())
    arrayFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayFirst.setChildren([returnVar])
    arrayFirst.setMember('d_firstElement')
    arrayFirst.setPointer(1)
    setData = ASE.Compiler.Cxx.Assignment.Assignment()
    setData.setChildren([arrayFirst, self.Cxx.castToType(firstVar, self.IOR.getType(type), 1)])
    returnArray = ASE.Compiler.Cxx.Return.Return()
    returnArray.setChildren([returnVar])
    body.addChildren([self.Cxx.getExpStmt(setData), returnArray])
    return
  
  def m_destroyArray(self, type, method, body):
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Return
    arrayVar = self.Cxx.getVar('array')
    arrayMem = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayMem.setChildren([arrayVar])
    arrayMem.setMember('d_firstElement')
    arrayMem.setPointer(1)
    freeArray = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeArray.setChildren([self.Cxx.getVar('free')])
    freeArray.setArguments([arrayMem])
    freeStruct = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeStruct.setChildren([self.Cxx.getVar('free')])
    freeStruct.setArguments([arrayVar])
    body.addChildren([self.Cxx.getExpStmt(freeArray), self.Cxx.getExpStmt(freeStruct)])
    return
  
  def m_getDimen(self, type, method, body):
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Return
    arrayVar = self.Cxx.getVar(method.getInParameters()[0].getIdentifier())
    arrayDimen = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayDimen.setChildren([arrayVar])
    arrayDimen.setMember('d_dimen')
    arrayDimen.setPointer(1)
    returnDimen = ASE.Compiler.Cxx.Return.Return()
    returnDimen.setChildren([arrayDimen])
    body.addChildren([returnDimen])
    return
  
  def m_getLower(self, type, method, body):
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Return
    arrayVar = self.Cxx.getVar(method.getInParameters()[0].getIdentifier())
    indexVar = self.Cxx.getVar(method.getInParameters()[1].getIdentifier())
    arrayLower = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayLower.setChildren([arrayVar])
    arrayLower.setMember('d_lower')
    arrayLower.setPointer(1)
    lowerInd = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    lowerInd.setChildren([arrayLower])
    lowerInd.setIndex(indexVar)
    returnLower = ASE.Compiler.Cxx.Return.Return()
    returnLower.setChildren([lowerInd])
    body.addChildren([returnLower])
    return
  
  def m_getUpper(self, type, method, body):
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Return
    arrayVar = self.Cxx.getVar(method.getInParameters()[0].getIdentifier())
    indexVar = self.Cxx.getVar(method.getInParameters()[1].getIdentifier())
    arrayUpper = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayUpper.setChildren([arrayVar])
    arrayUpper.setMember('d_upper')
    arrayUpper.setPointer(1)
    upperInd = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    upperInd.setChildren([arrayUpper])
    upperInd.setIndex(indexVar)
    returnUpper = ASE.Compiler.Cxx.Return.Return()
    returnUpper.setChildren([upperInd])
    body.addChildren([returnUpper])
    return
  
  def findElement(self, type, method, body):
    import ASE.Compiler.Cxx.AdditionAssignment
    import ASE.Compiler.Cxx.ArrayReference
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.For
    import ASE.Compiler.Cxx.GreaterThan
    import ASE.Compiler.Cxx.Increment
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.LogicalOr
    import ASE.Compiler.Cxx.Multiplication
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Subtraction
    arrayVar = self.Cxx.getVar(method.getInParameters()[0].getIdentifier())
    indexVar = self.Cxx.getVar(method.getInParameters()[1].getIdentifier())
    elemVar = self.Cxx.getVar('_element')
    dVar = self.Cxx.getVar('d')
    elemType = ASE.Compiler.Cxx.Pointer.Pointer()
    elemType.setChildren([self.IOR.getType(type)])
    body.addDeclarations([self.Cxx.getDeclaration(elemVar, elemType), self.Cxx.getDeclaration(dVar, self.IOR.typeMap['int'])])
    arrayDimen = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayDimen.setChildren([arrayVar])
    arrayDimen.setMember('d_dimen')
    arrayDimen.setPointer(1)
    arrayLower = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayLower.setChildren([arrayVar])
    arrayLower.setMember('d_lower')
    arrayLower.setPointer(1)
    arrayUpper = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayUpper.setChildren([arrayVar])
    arrayUpper.setMember('d_upper')
    arrayUpper.setPointer(1)
    arrayStride = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayStride.setChildren([arrayVar])
    arrayStride.setMember('d_stride')
    arrayStride.setPointer(1)
    arrayFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    arrayFirst.setChildren([arrayVar])
    arrayFirst.setMember('d_firstElement')
    arrayFirst.setPointer(1)
    indexFirst = ASE.Compiler.Cxx.StructReference.StructReference()
    indexFirst.setChildren([indexVar])
    indexFirst.setMember('d_firstElement')
    indexFirst.setPointer(1)
    dIndex = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dIndex.setChildren([indexFirst])
    dIndex.setIndex(dVar)
    dLower = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dLower.setChildren([arrayLower])
    dLower.setIndex(dVar)
    dUpper = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dUpper.setChildren([arrayUpper])
    dUpper.setIndex(dVar)
    dStride = ASE.Compiler.Cxx.ArrayReference.ArrayReference()
    dStride.setChildren([arrayStride])
    dStride.setIndex(dVar)
    returnInit = ASE.Compiler.Cxx.Assignment.Assignment()
    returnInit.setChildren([elemVar, arrayFirst])
    body.addChildren([self.Cxx.getExpStmt(returnInit)])
    cmpLower = ASE.Compiler.Cxx.LessThan.LessThan()
    cmpLower.setChildren([dIndex, dLower])
    cmpUpper = ASE.Compiler.Cxx.GreaterThan.GreaterThan()
    cmpUpper.setChildren([dIndex, dUpper])
    cmpIndex = ASE.Compiler.Cxx.LogicalOr.LogicalOr()
    cmpIndex.setChildren([cmpLower, cmpUpper])
    rowIndex = ASE.Compiler.Cxx.Subtraction.Subtraction()
    rowIndex.setChildren([dIndex, dLower])
    rowSize = ASE.Compiler.Cxx.Multiplication.Multiplication()
    rowSize.setChildren([dStride, self.Cxx.getGroup(rowIndex)])
    incElem = ASE.Compiler.Cxx.AdditionAssignment.AdditionAssignment()
    incElem.setChildren([elemVar, rowSize])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface') and not type.getFullIdentifier() in ['ASE.BaseInterface', 'ASE.BaseException', 'ASE.BaseClass']:
      compound.addChildren([self.IOR.getErrorCheck(type, method, None, 'Invalid indices', cmpIndex)])
    compound.addChildren([self.Cxx.getExpStmt(incElem)])
    dInit = ASE.Compiler.Cxx.Assignment.Assignment()
    dInit.setChildren([dVar, self.Cxx.getInteger(0)])
    cmpDimen = ASE.Compiler.Cxx.LessThan.LessThan()
    cmpDimen.setChildren([dVar, arrayDimen])
    dInc = ASE.Compiler.Cxx.Increment.Increment()
    dInc.setChildren([dVar])
    dInc.setPostfix(1)
    dimenLoop = ASE.Compiler.Cxx.For.For()
    dimenLoop.setInitialization(dInit)
    dimenLoop.setBranch(cmpDimen)
    dimenLoop.setIncrement(dInc)
    dimenLoop.setChildren([compound])
    body.addChildren([dimenLoop])
    return
  
  def adjustRefCount(self, type, infVar, addRef, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    infObj = ASE.Compiler.Cxx.StructReference.StructReference()
    infObj.setChildren([self.Cxx.getGroup(infVar)])
    infObj.setMember('d_object')
    infObj.setPointer(1)
    infEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    infEpv.setChildren([self.Cxx.getGroup(self.Cxx.castToType(infObj, self.IOR.getType(self.SIDL.getBaseInterface(type))))])
    infEpv.setMember('d_epv')
    infEpv.setPointer(1)
    increfFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    increfFuncPtr.setChildren([infEpv])
    if addRef:
      increfFuncPtr.setMember('addReference')
    else:
      increfFuncPtr.setMember('deleteReference')
    increfFuncPtr.setPointer(1)
    increfFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    increfFunc.setChildren([self.Cxx.getGroup(increfFuncPtr)])
    increfInf = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfInf.setChildren([self.Cxx.getGroup(increfFunc)])
    increfInf.setArguments([infObj, self.Cxx.castToType(self.Cxx.getVar('_exception'), self.IOR.typeMap['opaque'], 1)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(increfInf)])
    checkInf = ASE.Compiler.Cxx.If.If()
    checkInf.setBranch(infVar)
    checkInf.setChildren([compound])
    body.addChildren([checkInf])
    return
  
  def m_getElement(self, type, method, body):
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Return
    elemVar = self.Cxx.getVar('_element')
    returnVar = self.Cxx.getVar('_result')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.IOR.getType(method.getReturnParameter().getType()))])
    self.findElement(type, method, body)
    elemInd = ASE.Compiler.Cxx.Indirection.Indirection()
    elemInd.setChildren([elemVar])
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.adjustRefCount(type, elemInd, 0, body)
    returnElem = ASE.Compiler.Cxx.Return.Return()
    returnElem.setChildren([elemInd])
    body.addChildren([returnElem])
    return
  
  def m_getElement4(self, type, method, body):
    return
  
  def m_setElement(self, type, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Indirection
    valueVar = self.Cxx.getVar(method.getInParameters()[2].getIdentifier())
    elemVar = self.Cxx.getVar('_element')
    elemInd = ASE.Compiler.Cxx.Indirection.Indirection()
    elemInd.setChildren([elemVar])
    self.findElement(type, method, body)
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      self.adjustRefCount(type, valueVar, 1, body)
      self.adjustRefCount(type, elemInd, 0, body)
    setElem = ASE.Compiler.Cxx.Assignment.Assignment()
    setElem.setChildren([elemInd, valueVar])
    body.addChildren([self.Cxx.getExpStmt(setElem)])
    return
  
  def m_setElement4(self, type, method, body):
    return
  
  def getArray(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    if type.getBaseType() and type.getIdentifier() == 'void':
      return ([], [])
    arrayStruct = ASE.Compiler.Cxx.Struct.Struct()
    arrayStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__array')
    dataType = ASE.Compiler.Cxx.Pointer.Pointer()
    dataType.setChildren([self.IOR.getType(type)])
    dataDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    dataDecl.setIdentifier('d_firstElement')
    dataDecl.setType(dataType)
    intArrayType = ASE.Compiler.Cxx.Pointer.Pointer()
    intArrayType.setChildren([self.IOR.typeMap['int']])
    lowerDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    lowerDecl.setIdentifier('d_lower')
    lowerDecl.setType(intArrayType)
    upperDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    upperDecl.setIdentifier('d_upper')
    upperDecl.setType(intArrayType)
    strideDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    strideDecl.setIdentifier('d_stride')
    strideDecl.setType(intArrayType)
    borrowedDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    borrowedDecl.setIdentifier('d_borrowed')
    borrowedDecl.setType(self.IOR.typeMap['bool'])
    dimenDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    dimenDecl.setIdentifier('d_dimen')
    dimenDecl.setType(self.IOR.typeMap['int'])
    arrayStruct.addChildren(map(self.Cxx.getDecl, [dataDecl, lowerDecl, upperDecl, strideDecl, borrowedDecl, dimenDecl]))
    arrayDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    arrayDecl.setType(arrayStruct)
    decls = [self.Cxx.getDecl(arrayDecl, 'The IOR array structure for this type')]
    if type.getFullIdentifier() == 'ASE.ProjectState':
      decls.append(self.IOR.getGuardedInclude(self.SIDL.getASEException(type)))
    defns = []
    if not (type.getBaseType() and type.getIdentifier() == 'void'):
      if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        for parent in type.getParents():
          if not type.getFullIdentifier() in ['ASE.DLL', 'ASE.Loader']:
            defns.extend([self.IOR.getGuardedInclude(self.SIDL.getASEException(type))])
      for method in self.getArrayMethods(type):
        import ASE.Compiler.Cxx.CompoundStatement
        import ASE.Compiler.Cxx.Function
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        funcDefn = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(type, method).getChildren()[0])
        funcDefn.setBody(compound)
        funcDefn.setStatic(1)
        getattr(self, 'm_' + method.getIdentifier())(type, method, compound)
        defns.append(funcDefn)
    return (decls, defns)
  
  def getArrayMethods(self, type):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Array
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    arrayMethods = []
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      exc = [self.SIDL.getBaseException(self.SIDL.getASEException(type))]
    else:
      exc = []
    intArrayType = ASE.Compiler.SIDL.Array.Array()
    intArrayType.setType(self.SIDL.typeMap['int'])
    intReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    intReturn.setIdentifier('_return_value')
    intReturn.setType(self.SIDL.typeMap['int'])
    intReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    arrayType = ASE.Compiler.SIDL.Array.Array()
    arrayType.setType(type)
    arrayParam = ASE.Compiler.SIDL.Parameter.Parameter()
    arrayParam.setIdentifier('array')
    arrayParam.setType(arrayType)
    arrayParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    arrayReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    arrayReturn.setIdentifier('_return_value')
    arrayReturn.setType(arrayType)
    arrayReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    elemParam = ASE.Compiler.SIDL.Parameter.Parameter()
    elemParam.setIdentifier('value')
    elemParam.setType(type)
    elemParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    elemReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    elemReturn.setIdentifier('_return_value')
    elemReturn.setType(type)
    elemReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    dimenParam = ASE.Compiler.SIDL.Parameter.Parameter()
    dimenParam.setIdentifier('dimen')
    dimenParam.setType(self.SIDL.typeMap['int'])
    dimenParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    lowerParam = ASE.Compiler.SIDL.Parameter.Parameter()
    lowerParam.setIdentifier('lower')
    lowerParam.setType(intArrayType)
    lowerParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    upperParam = ASE.Compiler.SIDL.Parameter.Parameter()
    upperParam.setIdentifier('upper')
    upperParam.setType(intArrayType)
    upperParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    strideParam = ASE.Compiler.SIDL.Parameter.Parameter()
    strideParam.setIdentifier('stride')
    strideParam.setType(intArrayType)
    strideParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    firstParam = ASE.Compiler.SIDL.Parameter.Parameter()
    firstParam.setIdentifier('first')
    firstParam.setType(self.SIDL.typeMap['opaque'])
    firstParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indicesParam = ASE.Compiler.SIDL.Parameter.Parameter()
    indicesParam.setIdentifier('indices')
    indicesParam.setType(intArrayType)
    indicesParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam1 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam1.setIdentifier('i1')
    indexParam1.setType(self.SIDL.typeMap['int'])
    indexParam1.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam2 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam2.setIdentifier('i2')
    indexParam2.setType(self.SIDL.typeMap['int'])
    indexParam2.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam3 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam3.setIdentifier('i3')
    indexParam3.setType(self.SIDL.typeMap['int'])
    indexParam3.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    indexParam4 = ASE.Compiler.SIDL.Parameter.Parameter()
    indexParam4.setIdentifier('i4')
    indexParam4.setType(self.SIDL.typeMap['int'])
    indexParam4.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Create a dense array of the given dimension with specified index bounds\nThis array owns and manages its data\nAll object pointers are initialized to NULL')
    createArray = ASE.Compiler.SIDL.Method.Method()
    createArray.setIdentifier('createArray')
    createArray.setReturnParameter(arrayReturn)
    createArray.setChildren([dimenParam, lowerParam, upperParam])
    createArray.setExceptions(exc)
    createArray.setStatic(1)
    createArray.setComments([comment])
    arrayMethods.append(createArray)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Create an array using user-provided memory')
    borrowArray = ASE.Compiler.SIDL.Method.Method()
    borrowArray.setIdentifier('borrowArray')
    borrowArray.setReturnParameter(arrayReturn)
    borrowArray.setChildren([dimenParam, lowerParam, upperParam, strideParam, firstParam])
    borrowArray.setExceptions(exc)
    borrowArray.setStatic(1)
    borrowArray.setComments([comment])
    arrayMethods.append(borrowArray)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Destroy the array (NULL arrays are ignored)')
    destroyArray = ASE.Compiler.SIDL.Method.Method()
    destroyArray.setIdentifier('destroyArray')
    destroyArray.setChildren([arrayParam])
    destroyArray.setExceptions(exc)
    destroyArray.setStatic(1)
    destroyArray.setComments([comment])
    arrayMethods.append(destroyArray)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Return the number of dimensions in the array\nIf the array pointer is NULL, zero is returned')
    getDimen = ASE.Compiler.SIDL.Method.Method()
    getDimen.setIdentifier('getDimen')
    getDimen.setReturnParameter(intReturn)
    getDimen.setChildren([arrayParam])
    getDimen.setExceptions(exc)
    getDimen.setStatic(1)
    getDimen.setComments([comment])
    arrayMethods.append(getDimen)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Return the lower bound on dimension ind\nIf the array pointer is NULL or dimen is not a valid dimension, zero is returned')
    getLower = ASE.Compiler.SIDL.Method.Method()
    getLower.setIdentifier('getLower')
    getLower.setReturnParameter(intReturn)
    getLower.setChildren([arrayParam, dimenParam])
    getLower.setExceptions(exc)
    getLower.setStatic(1)
    getLower.setComments([comment])
    arrayMethods.append(getLower)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Return the upper bound on dimension ind\nIf the array pointer is NULL or dimen is not a valid dimension, -1 is returned')
    getUpper = ASE.Compiler.SIDL.Method.Method()
    getUpper.setIdentifier('getUpper')
    getUpper.setReturnParameter(intReturn)
    getUpper.setChildren([arrayParam, dimenParam])
    getUpper.setExceptions(exc)
    getUpper.setStatic(1)
    getUpper.setComments([comment])
    arrayMethods.append(getUpper)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Get an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the return value is non-NULL, the client owns one reference to the object/interface\nThe client must decrement the reference count when done with the reference')
    getElement = ASE.Compiler.SIDL.Method.Method()
    getElement.setIdentifier('getElement')
    getElement.setReturnParameter(elemReturn)
    getElement.setChildren([arrayParam, indicesParam])
    getElement.setExceptions(exc)
    getElement.setStatic(1)
    getElement.setComments([comment])
    arrayMethods.append(getElement)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Get an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the return value is non-NULL, the client owns one reference to the object/interface\nThe client must decrement the reference count when done with the reference')
    getElement4 = ASE.Compiler.SIDL.Method.Method()
    getElement4.setIdentifier('getElement4')
    getElement4.setReturnParameter(elemReturn)
    getElement4.setChildren([arrayParam, indexParam1, indexParam2, indexParam3, indexParam4])
    getElement4.setExceptions(exc)
    getElement4.setStatic(1)
    getElement4.setComments([comment])
    arrayMethods.append(getElement4)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Set an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the incoming value is non-NULL, this function will increment the reference code of the object/interface\nIf it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented')
    setElement = ASE.Compiler.SIDL.Method.Method()
    setElement.setIdentifier('setElement')
    setElement.setChildren([arrayParam, indicesParam, elemParam])
    setElement.setExceptions(exc)
    setElement.setStatic(1)
    setElement.setComments([comment])
    arrayMethods.append(setElement)
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Set an element of a multi-dimensional array\nThis will use the indices provided up to the actual dimension of the array, ignoring extra indices\nIf the incoming value is non-NULL, this function will increment the reference code of the object/interface\nIf it is overwriting a non-NULL pointer, the reference count of the object/interface being overwritten will be decremented')
    setElement4 = ASE.Compiler.SIDL.Method.Method()
    setElement4.setIdentifier('setElement4')
    setElement4.setChildren([arrayParam, indexParam1, indexParam2, indexParam3, indexParam4, elemParam])
    setElement4.setExceptions(exc)
    setElement4.setStatic(1)
    setElement4.setComments([comment])
    arrayMethods.append(setElement4)
    return arrayMethods
  
  def m_isSame(self, interface, method, body):
    import ASE.Compiler.Cxx.Equality
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    selfObj = ASE.Compiler.Cxx.StructReference.StructReference()
    selfObj.setChildren([self.Cxx.getVar('self')])
    selfObj.setMember('d_object')
    selfObj.setPointer(1)
    argObj = ASE.Compiler.Cxx.StructReference.StructReference()
    argObj.setChildren([self.Cxx.getVar(method.getChildren()[0].getIdentifier())])
    argObj.setMember('d_object')
    argObj.setPointer(1)
    objEqual = ASE.Compiler.Cxx.Equality.Equality()
    objEqual.setChildren([selfObj, argObj])
    returnComp = ASE.Compiler.Cxx.Return.Return()
    returnComp.setChildren([objEqual])
    body.addChildren([returnComp])
    return
  
  def m_isInstanceOf(self, interface, method, body):
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    selfEpv.setChildren([self.Cxx.getVar('self')])
    selfEpv.setMember('d_epv')
    selfEpv.setPointer(1)
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([selfEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    castFuncInd.setChildren([self.Cxx.getGroup(castFunc)])
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFuncInd)])
    castCall.setArguments([self.Cxx.getVar('self'), self.Cxx.getVar(method.getChildren()[0].getIdentifier()), self.Cxx.getVar('_exception')])
    checkCast = ASE.Compiler.Cxx.If.If()
    checkCast.setBranch(castCall)
    checkCast.setChildren([self.Cxx.getInteger(1), self.Cxx.getInteger(0)])
    checkCast.setTernary(1)
    returnCast = ASE.Compiler.Cxx.Return.Return()
    returnCast.setChildren([checkCast])
    body.addChildren([returnCast])
    return
  
  def m_getClassName(self, interface, method, body):
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Return
    dupCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    dupCall.setChildren([self.Cxx.getVar('strdup')])
    dupCall.setArguments([self.Cxx.getString(interface.getFullIdentifier())])
    returnName = ASE.Compiler.Cxx.Return.Return()
    returnName.setChildren([dupCall])
    body.addChildren([returnName])
    return
  
  def m_getSuper(self, interface, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    superSelfVar = self.Cxx.getVar('superSelf')
    createCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createCall.setChildren([self.Cxx.getVar('createObject')])
    createCall.setArguments([self.Cxx.getVar('_exception')])
    superSelfInit = ASE.Compiler.Cxx.Initializer.Initializer()
    superSelfInit.setChildren([createCall])
    body.addDeclarations([self.Cxx.getDeclaration(superSelfVar, self.IOR.getType(interface), superSelfInit)])
    selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    selfEpv.setChildren([superSelfVar])
    selfEpv.setMember('d_epv')
    selfEpv.setPointer(1)
    selfOldEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    selfOldEpv.setChildren([superSelfVar])
    selfOldEpv.setMember('d_oldEpv')
    selfOldEpv.setPointer(1)
    setEpv = ASE.Compiler.Cxx.Assignment.Assignment()
    setEpv.setChildren([selfEpv, selfOldEpv])
    body.addChildren([self.Cxx.getExpStmt(setEpv)])
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([selfEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([castFunc])
    castCall.setArguments([superSelfVar, self.Cxx.getString('ASE.BaseInterface'), self.Cxx.getVar('_exception')])
    returnCast = ASE.Compiler.Cxx.Return.Return()
    returnCast.setChildren([self.Cxx.getExpStmt(castCall)])
    body.addChildren([returnCast])
    return
  
  def m_createClass(self, interface, method, body):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    extFuncVar = self.Cxx.getVar('_extFunc')
    excVar = self.Cxx.getVar('_exception')
    nameVar = self.Cxx.getVar(method.getChildren()[0].getIdentifier())
    genericExtStruct = ASE.Compiler.Cxx.Struct.Struct()
    genericExtStruct.setIdentifier('generic__external')
    genericExtStruct.setChildren([self.IOR.getFunctionDeclaration(interface, self.getConstructor(interface), 1)])
    genericExtDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    genericExtDecl.setType(genericExtStruct)
    body.addDeclarations([self.Cxx.getDecl(genericExtDecl)])
    genericExtStruct = ASE.Compiler.Cxx.Struct.Struct()
    genericExtStruct.setIdentifier('generic__external')
    genericExt = ASE.Compiler.Cxx.Pointer.Pointer()
    genericExt.setChildren([genericExtStruct])
    sidlExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
    staticEPV = ASE.Compiler.Cxx.StructReference.StructReference()
    staticEPV.setChildren([sidlExtCall])
    staticEPV.setMember('getStaticEPV')
    staticEPV.setPointer(1)
    staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    staticEPVCall.setChildren([staticEPV])
    staticEPVCall.setArguments([excVar])
    classMethods = ASE.Compiler.Cxx.StructReference.StructReference()
    classMethods.setChildren([staticEPVCall])
    classMethods.setMember('getClassMethodTable')
    classMethods.setPointer(1)
    classMethodsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    classMethodsCall.setChildren([classMethods])
    classMethodsCall.setArguments([nameVar, excVar])
    extFuncType = ASE.Compiler.Cxx.Function.Function()
    extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
    extFuncType.setType(genericExt)
    extFunc = ASE.Compiler.Cxx.Cast.Cast()
    extFunc.setTypeName(extFuncType)
    extFunc.setChildren([classMethodsCall])
    extFuncInit = ASE.Compiler.Cxx.Initializer.Initializer()
    extFuncInit.setChildren([extFunc])
    extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
    extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
    extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
    extFuncDecl = ASE.Compiler.Cxx.Function.Function()
    extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
    extFuncDecl.setType(genericExt)
    extFuncDecl.setInitializer(extFuncInit)
    body.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
    extFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    extFuncInd.setChildren([extFuncVar])
    extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFuncCall.setChildren([self.Cxx.getGroup(extFuncInd)])
    createFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    createFuncPtr.setChildren([extFuncCall])
    createFuncPtr.setMember('createObject')
    createFuncPtr.setPointer(1)
    createFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    createFunc.setChildren([self.Cxx.getGroup(createFuncPtr)])
    createFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createFuncCall.setChildren([self.Cxx.getGroup(createFunc)])
    createFuncCall.setArguments([excVar])
    returnClass = ASE.Compiler.Cxx.Return.Return()
    returnClass.setChildren([self.Cxx.castToType(createFuncCall, self.IOR.getType(self.SIDL.getAncestor(interface, 'ASE.BaseClass')))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([returnClass])
    checkIor = ASE.Compiler.Cxx.If.If()
    checkIor.setBranch(extFuncVar)
    checkIor.setChildren([compound])
    returnNull = ASE.Compiler.Cxx.Return.Return()
    returnNull.setChildren([self.Cxx.getNull()])
    body.addChildren([checkIor, returnNull])
    return
  
  def isIORMethod(self, method):
    from ASE.Compiler.CodePurpose import IMPLEMENTATION, IOR, SKELETON
    purpose = method.getPurpose()
    return not (SKELETON in purpose or IMPLEMENTATION in purpose) and IOR in purpose
  
  def getIORMethods(self, interface):
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      import ASE.Compiler.SIDL.Class
      klass = ASE.Compiler.SIDL.Class.Class(interface)
      implMethods = klass.getImplementedMethods()
      return [method for method in klass.getMethods() if self.isIORMethod(method) and (method in implMethods or not method.getVirtual())]
    return []
  
  def getIORImplementations(self, interface):
    defns = []
    for method in self.getTemplateMethods(self.getIORMethods(interface)):
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      getattr(self, 'm_' + method.getIdentifier())(interface, method, compound)
      funcDefn = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(interface, method).getChildren()[0])
      funcDefn.setBody(compound)
      funcDefn.setStatic(1)
      defns.append(funcDefn)
    return ([], defns)
  
  def getExternal(self, type):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Struct
    if type.getBaseType() and type.getIdentifier() == 'void':
      return ([], [])
    comment = 'The entry points into the DLL containing this type'
    externalStruct = ASE.Compiler.Cxx.Struct.Struct()
    externalStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
    if type.isInstanceOf('ASE.Compiler.SIDL.Class'):
      externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getConstructor(type), 1)])
      externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getInit(type), 1)])
      externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getFini(type), 1)])
      if self.hasStaticMethod(type):
        externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, self.getSEPVFunc(type), 1)])
    externalStruct.addChildren([self.IOR.getFunctionDeclaration(type, method, 1) for method in self.getArrayMethods(type)])
    externalDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    externalDecl.setType(externalStruct)
    externalStruct = ASE.Compiler.Cxx.Struct.Struct()
    externalStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
    externalInit = ASE.Compiler.Cxx.Initializer.Initializer()
    if type.isInstanceOf('ASE.Compiler.SIDL.Class'):
      externalInit.addChildren([self.Cxx.getVar(self.getConstructor(type).getIdentifier())])
      externalInit.addChildren([self.Cxx.getVar(self.getInit(type).getIdentifier())])
      externalInit.addChildren([self.Cxx.getVar(self.getFini(type).getIdentifier())])
      if self.hasStaticMethod(type):
        externalInit.addChildren([self.Cxx.getVar(self.getSEPVFunc(type).getIdentifier())])
    externalInit.addChildren([self.Cxx.getVar(method.getIdentifier()) for method in self.getArrayMethods(type)])
    externalDefn = ASE.Compiler.Cxx.Declarator.Declarator()
    externalDefn.setIdentifier('externalEntryPoints')
    externalDefn.setType(externalStruct)
    externalDefn.setStatic(1)
    externalDefn.setInitializer(externalInit)
    return ([self.Cxx.getDecl(externalDecl, comment)], [self.Cxx.getDecl(externalDefn, comment)])
  
  def getExternals(self, type):
    import ASE.Compiler.Cxx.Address
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Struct
    if type.getBaseType() and type.getIdentifier() == 'void':
      return ([], [])
    externalStruct = ASE.Compiler.Cxx.Struct.Struct()
    externalStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
    externalStruct.setConst(1)
    externalType = ASE.Compiler.Cxx.Pointer.Pointer()
    externalType.setChildren([externalStruct])
    externalName = ASE.Compiler.Cxx.Declarator.Declarator()
    externalName.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__externals')
    externalDecl = ASE.Compiler.Cxx.Function.Function()
    externalDecl.setChildren([externalName])
    externalDecl.setType(externalType)
    extAddr = ASE.Compiler.Cxx.Address.Address()
    extAddr.setChildren([self.Cxx.getVar('externalEntryPoints')])
    returnExt = ASE.Compiler.Cxx.Return.Return()
    returnExt.setChildren([extAddr])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([returnExt])
    externalDefn = ASE.Compiler.Cxx.Function.Function()
    externalDefn.setChildren([externalName])
    externalDefn.setType(externalType)
    externalDefn.setBody(compound)
    return ([self.Cxx.getDecl(externalDecl, 'This function, called upon loading the DLL, returns a pointer to the static structure of DLL entry points')], [externalDefn])
  
  def getHeaderFile(self, type, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Define
    import ASE.Compiler.Cxx.Header
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + type.getFullIdentifier().replace('.', '_'))
    guard.setValue(1)
    define = ASE.Compiler.Cxx.Define.Define()
    define.setIdentifier(guard.getBranchExpression())
    externBegin = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externBegin.setIdentifier('ifdef')
    externBegin.setBranchExpression('__cplusplus')
    externBegin.setValue(0)
    externBegin.setInactiveCode('extern "C" {')
    externEnd = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externEnd.setIdentifier('ifdef')
    externEnd.setBranchExpression('__cplusplus')
    externEnd.setValue(0)
    externEnd.setInactiveCode('}')
    typeInclude = ASE.Compiler.Cxx.Include.Include()
    typeInclude.setIdentifier('<sys/types.h>')
    guard.addChildren([define, externBegin, typeInclude])
    guard.addChildren(self.getForwardDeclarations(type)[0])
    for dependency in type.getDependencies():
      guard.addChildren(self.getForwardDeclarations(dependency, noExceptions = 1)[0])
      if ASE.Compiler.SIDL.Type.Type(dependency).getBaseType() or dependency.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
        guard.addChildren([self.IOR.getGuardedInclude(dependency)])
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      import ASE.Compiler.SIDL.Interface
      interface = ASE.Compiler.SIDL.Interface.Interface(type)
      if len(interface.getParents()):
        guard.addChildren([self.IOR.getGuardedInclude(parent) for parent in interface.getParents()])
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        import ASE.Compiler.SIDL.Class
        klass = ASE.Compiler.SIDL.Class.Class(interface)
        if len(klass.getInterfaces()):
          guard.addChildren([self.IOR.getGuardedInclude(inf) for inf in klass.getInterfaces()])
    guard.addChildren(decls)
    guard.addChildren([externEnd])
    header = ASE.Compiler.Cxx.Header.Header()
    header.setFilename(self.IOR.getFileBasename(type) + '.h')
    header.setChildren([guard])
    header.setPurpose(ASE.Compiler.CodePurpose.IOR)
    return header
  
  def getSourceFile(self, interface, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Source
    stdInclude = ASE.Compiler.Cxx.Include.Include()
    stdInclude.setIdentifier('<stdlib.h>')
    stdInclude.setComments([self.Cxx.getComment('Needed for NULL')])
    strInclude = ASE.Compiler.Cxx.Include.Include()
    strInclude.setIdentifier('<string.h>')
    strInclude.setComments([self.Cxx.getComment('Needed for implementation of getClassName()')])
    source = ASE.Compiler.Cxx.Source.Source()
    source.setFilename(self.IOR.getFileBasename(interface) + '.c')
    source.addChildren([stdInclude, strInclude, self.IOR.getGuardedInclude(interface)])
    source.addChildren(decls)
    source.setPurpose(ASE.Compiler.CodePurpose.IOR)
    return source
  
  def createBaseTypes(self):
    for type in self.SIDL.typeMap.values():
      self.visitInterface(type)
    return
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not method.getReturnParameter() is None:
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not parameter is None:
      type = parameter.getType()
      if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not method in self.templateMethods:
        self.template(method, method.getTemplateIdentifiers()[0])
    return
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 

class IOR(object):
  
  def __init__(self):
    self.Cxx = Cxx()
    self.SIDL = SIDL()
    self.typeMap = self.createTypeMap()
    self.epv = {}

    self.external = {}

    self.sepv = {}

    return
  
  def raiseException(self, message, trace = ''):
    import sys
    e = ASE.Compiler.CompilerException.CompilerException()
    e.setMessage(message)
    if isinstance(trace, list):
      for line in trace:
        e.addStackLine(line.strip())
    elif trace:
      e.addStackLine(trace)
    e.addToTrace(__file__, sys._getframe(1).f_lineno, sys._getframe().f_code.co_name)
    raise ASE.Compiler.CompilerException.Exception, e
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int')
    iorType.setBaseType(1)
    typeMap['bool'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('char')
    iorType.setBaseType(1)
    typeMap['char'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_dcomplex')
    typeMap['dcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('double')
    iorType.setBaseType(1)
    typeMap['double'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_fcomplex')
    typeMap['fcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('float')
    iorType.setBaseType(1)
    typeMap['float'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int32_t')
    iorType.setBaseType(1)
    typeMap['int'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int64_t')
    iorType.setBaseType(1)
    typeMap['long'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('void')
    iorType.setBaseType(1)
    typeMap['void'] = iorType
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([typeMap['void']])
    typeMap['opaque'] = iorType
    constChar = ASE.Compiler.Cxx.Type.Type()
    constChar.setIdentifier('char')
    constChar.setBaseType(1)
    constChar.setConst(1)
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([constChar])
    typeMap['string'] = iorType
    return typeMap
  
  def getType(self, type):
    '''
    Return the C IOR type corresponding to this sidl type
    '''
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    iorType = None
    if type.getBaseType():
      try:
        iorType = self.typeMap[type.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      iorType = ASE.Compiler.Cxx.Enumeration.Enumeration()
      iorType.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__enum')
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(entryType.getFullIdentifier().replace('.', '_') + '__array')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__object')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    else:
      import ASE.Compiler.Cxx.Type
      iorType = ASE.Compiler.Cxx.Type.Type()
      iorType.setIdentifier(type.getIdentifier())
    return iorType
  
  def getEpvType(self, type):
    if not type in self.epv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      epvStruct = ASE.Compiler.Cxx.Struct.Struct()
      epvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__epv')
      epvType = ASE.Compiler.Cxx.Pointer.Pointer()
      epvType.setChildren([epvStruct])
      self.epv[type] = epvType
    return self.epv[type]
  
  def getExternalType(self, type):
    if not type in self.external:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      extStruct = ASE.Compiler.Cxx.Struct.Struct()
      extStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
      extStruct.setConst(1)
      extType = ASE.Compiler.Cxx.Pointer.Pointer()
      extType.setChildren([extStruct])
      self.external[type] = extType
    return self.external[type]
  
  def getStaticExternalType(self, type):
    if not type in self.sepv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
      sepvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__sepv')
      sepvStruct.setConst(1)
      sepvType = ASE.Compiler.Cxx.Pointer.Pointer()
      sepvType.setChildren([sepvStruct])
      self.sepv[type] = sepvType
    return self.sepv[type]
  
  def getParameter(self, parameter, mode = None):
    '''
    Return the C IOR parameter corresponding to this sidl parameter
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Comment
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    if mode is None:
      type = parameter.getType()
      mode = parameter.getAccessMode()
    else:
      type = parameter
    comment = ASE.Compiler.Cxx.Comment.Comment()
    iorType = self.getType(type)
    if mode == ASE.Compiler.ParameterAccessMode.IN:
      comment.setComment('in')
    elif mode == ASE.Compiler.ParameterAccessMode.OUT:
      comment.setComment('out')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    elif mode == ASE.Compiler.ParameterAccessMode.INOUT:
      comment.setComment('inout')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(parameter.getIdentifier())
    iorParam.setType(iorType)
    iorParam.setComments([comment])
    return iorParam
  
  def getFunctionDeclaration(self, type, method, isPointer = 0, qualifyName = 0):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    func = ASE.Compiler.Cxx.Function.Function()
    if qualifyName:
      name = type.getFullIdentifier().replace('.', '_') + '_' + method.getIdentifier()
    else:
      name = method.getIdentifier()
    if isPointer:
      funcPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      funcPtr.setIdentifier(name)
      funcPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      func.setChildren([self.Cxx.getDeclGroup(funcPtr)])
    else:
      decl = ASE.Compiler.Cxx.Declarator.Declarator()
      decl.setIdentifier(name)
      func.setChildren([decl])
    if method.getReturnParameter() is None:
      func.setType(self.typeMap['void'])
    else:
      func.setType(self.getType(method.getReturnParameter().getType()))
    if not method.getStatic():
      selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
      selfParam.setIdentifier('self')
      selfParam.setType(self.getType(type))
      func.addParameters([selfParam])
    for child in method.getChildren():
      func.addParameters([self.getParameter(ASE.Compiler.SIDL.Parameter.Parameter(child))])
    excParam = ASE.Compiler.SIDL.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setAccessMode(ASE.Compiler.ParameterAccessMode.OUT)
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface') and not type.getFullIdentifier() == 'ASE.BaseInterface':
      excParam.setType(self.SIDL.getBaseException(method.getExceptions()[0]))
    else:
      excParam.setType(self.SIDL.typeMap['opaque'])
    func.addParameters([self.getParameter(excParam)])
    comment = method.getFullIdentifier() + ': '
    comment += '\n'.join([c.getComment() for c in method.getComments()])
    return self.Cxx.getDecl(func, comment)
  
  def getFileBasename(self, interface):
    import os
    idChain = interface.getFullIdentifier().split('.')
    if not len(idChain[:-1]):
      return idChain[-1] + '_IOR'
    return os.path.join(apply(os.path.join, idChain[:-1]), idChain[-1] + '_IOR')
  
  def getGuardedInclude(self, interface):
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_'))
    guard.setValue(1)
    guard.addChildren([self.Cxx.getInclude(self.getFileBasename(interface) + '.h')])
    return guard
  
  def getErrorCheck(self, interface, method, branchVar, errorMessage, branchExp = None):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.StructReference
    excVar = self.Cxx.getVar('_exception')
    localExcVar = self.Cxx.getVar('_local_exception')
    excType = self.SIDL.getASEException(interface)
    extFunc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFunc.setChildren([self.Cxx.getVar(excType.getFullIdentifier().replace('.', '_') + '__externals')])
    createFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    createFunc.setChildren([extFunc])
    createFunc.setMember('createObject')
    createFunc.setPointer(1)
    createExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createExc.setChildren([createFunc])
    createExc.setArguments([self.Cxx.getNull()])
    localExcInit = ASE.Compiler.Cxx.Initializer.Initializer()
    localExcInit.setChildren([createExc])
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([localExcVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    messageFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    messageFunc.setChildren([excEpv])
    messageFunc.setMember('setMessage')
    messageFunc.setPointer(1)
    setMessage = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setMessage.setChildren([messageFunc])
    setMessage.setArguments([localExcVar, self.Cxx.getString(errorMessage), excVar])
    excInd = ASE.Compiler.Cxx.Indirection.Indirection()
    excInd.setChildren([excVar])
    negExcInd = ASE.Compiler.Cxx.Negation.Negation()
    negExcInd.setChildren([self.Cxx.getGroup(excInd)])
    setExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setExc.setChildren([excInd, self.Cxx.castToType(localExcVar, self.getType(self.SIDL.getBaseException(excType)))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setExc)])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(negExcInd)
    checkExc.setChildren([compound])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    if not method is None and not method.getReturnParameter() is None:
      returnStmt.setChildren([self.Cxx.getVar('_result')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(localExcVar, self.getType(excType), localExcInit)])
    compound.setChildren([self.Cxx.getExpStmt(setMessage), checkExc, returnStmt])
    checkVar = ASE.Compiler.Cxx.If.If()
    if branchExp is None:
      negVar = ASE.Compiler.Cxx.Negation.Negation()
      negVar.setChildren([branchVar])
      checkVar.setBranch(negVar)
    else:
      checkVar.setBranch(branchExp)
    checkVar.setChildren([compound])
    return checkVar
  
  def getClassMethodTable(self, klass):
    excVar = self.Cxx.getVar('_exception')
    loaderExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    loaderExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
    staticEPVFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    staticEPVFunc.setChildren([loaderExtCall])
    staticEPVFunc.setMember('getStaticEPV')
    staticEPVFunc.setPointer(1)
    staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    staticEPVCall.setChildren([staticEPVFunc])
    staticEPVCall.setArguments([excVar])
    tableFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    tableFunc.setChildren([staticEPVCall])
    tableFunc.setMember('getClassMethodTable')
    tableFunc.setPointer(1)
    tableFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    tableFuncCall.setChildren([tableFunc])
    tableFuncCall.setArguments([self.Cxx.getString(klass.getFullIdentifier()), excVar])
    return tableFuncCall

class SIDL(object):
  
  def __init__(self):
    self.typeMap = self.createTypeMap()
    return
  
  def createTypeMap(self):
    import ASE.Compiler.SIDL.Type
    typeMap = {}

    for typeName in ['int', 'bool', 'char', 'dcomplex', 'double', 'fcomplex', 'float', 'long', 'opaque', 'string', 'void']:
      sidlType = ASE.Compiler.SIDL.Type.Type()
      sidlType.setIdentifier(typeName)
      sidlType.setBaseType(1)
      if not typeName == 'int':
        sidlType.setDependencies([typeMap['int']])
      typeMap[typeName] = sidlType
    return typeMap
  
  def getLoader(self, interface):
    for child in self.getBaseInterface(interface).getParent().getChildren():
      if child.getIdentifier() == 'Loader':
        import ASE.Compiler.SIDL.Class
        return ASE.Compiler.SIDL.Class.Class(child)
    raise RuntimeError('Could not find Loader using ' + interface.getFullIdentifier())
  
  def getBasePackage(self, interface):
    import ASE.Compiler.SIDL.Package
    return ASE.Compiler.SIDL.Package.Package(self.getAncestor(interface, 'ASE.BaseInterface').getParent())
  
  def getBaseInterface(self, interface):
    return self.getAncestor(interface, 'ASE.BaseInterface')
  
  def getBaseException(self, exception):
    return self.getAncestor(exception, 'ASE.BaseException')
  
  def getASEException(self, interface):
    asePackage = self.getBasePackage(interface)
    for child in asePackage.getChildren():
      if child.getIdentifier() == 'ASEException':
        import ASE.Compiler.SIDL.Class
        return ASE.Compiler.SIDL.Class.Class(child)
    return
  
  def getAncestor(self, interface, typeName):
    import ASE.Compiler.SIDL.Interface
    interface = ASE.Compiler.SIDL.Interface.Interface(interface)
    if interface.getFullIdentifier() == typeName:
      return interface
    parents = []
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      import ASE.Compiler.SIDL.Class
      parents.extend(ASE.Compiler.SIDL.Class.Class(interface).getInterfaces())
    parents.extend(interface.getParents())
    for parent in parents:
      base = self.getAncestor(parent, typeName)
      if base:
        return base
    return

class Cxx(object):
  
  def __init__(self):
    self.typeMap = self.createTypeMap()
    self.nullVar = self.getVar('NULL')
    self.integers = {}

    return
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('bool')
    cxxType.setBaseType(1)
    typeMap['bool'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('char')
    cxxType.setBaseType(1)
    typeMap['char'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<double>')
    typeMap['dcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('double')
    cxxType.setBaseType(1)
    typeMap['double'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<float>')
    typeMap['fcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('float')
    cxxType.setBaseType(1)
    typeMap['float'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('int')
    cxxType.setBaseType(1)
    typeMap['int'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('long long')
    cxxType.setBaseType(1)
    typeMap['long'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('void')
    cxxType.setBaseType(1)
    typeMap['void'] = cxxType
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['void']])
    typeMap['opaque'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('std::str')
    cxxType.setBaseType(1)
    typeMap['string'] = cxxType
    return typeMap
  
  def getNull(self):
    return self.nullVar
  
  def getInteger(self, num):
    if not num in self.integers:
      import ASE.Compiler.Cxx.Constant
      integer = ASE.Compiler.Cxx.Constant.Constant()
      integer.setValue_int(num)
      self.integers[num] = integer
    return self.integers[num]
  
  def getInclude(self, filename):
    import ASE.Compiler.Cxx.Include
    include = ASE.Compiler.Cxx.Include.Include()
    include.setIdentifier('"' + filename + '"')
    return include
  
  def getString(self, s):
    import ASE.Compiler.Cxx.Constant
    string = ASE.Compiler.Cxx.Constant.Constant()
    string.setIdentifier(s)
    return string
  
  def getComment(self, comment):
    import ASE.Compiler.Cxx.Comment
    c = ASE.Compiler.Cxx.Comment.Comment()
    c.setComment(comment)
    return c
  
  def getVar(self, name):
    import ASE.Compiler.Cxx.Variable
    var = ASE.Compiler.Cxx.Variable.Variable()
    var.setIdentifier(name)
    return var
  
  def getAddress(self, exp):
    import ASE.Compiler.Cxx.Address
    addr = ASE.Compiler.Cxx.Address.Address()
    addr.setChildren([exp])
    return addr
  
  def getGroup(self, exp):
    import ASE.Compiler.Cxx.Group
    group = ASE.Compiler.Cxx.Group.Group()
    group.setChildren([exp])
    return group
  
  def getDeclGroup(self, exp):
    import ASE.Compiler.Cxx.DeclaratorGroup
    group = ASE.Compiler.Cxx.DeclaratorGroup.DeclaratorGroup()
    group.setChildren([exp])
    return group
  
  def getDecl(self, declarator, comment = None):
    '''
    Make a declaration from a declarator
    '''
    import ASE.Compiler.Cxx.Declaration
    decl = ASE.Compiler.Cxx.Declaration.Declaration()
    decl.setChildren([declarator])
    if not comment is None:
      import ASE.Compiler.Cxx.Comment
      c = ASE.Compiler.Cxx.Comment.Comment()
      c.setComment(comment)
      decl.setComments([c])
    return decl
  
  def getInitializer(self, type):
    import ASE.Compiler.Cxx.Initializer
    init = ASE.Compiler.Cxx.Initializer.Initializer()
    if type.getBaseType():
      typeName = type.getIdentifier()
      if typeName in ['bool', 'char', 'int', 'long', 'int32_t', 'int64_t']:
        init.setChildren([self.getInteger(0)])
      elif typeName in ['double', 'float']:
        import ASE.Compiler.Cxx.Constant
        zero = ASE.Compiler.Cxx.Constant.Constant()
        zero.setValue_double(0.0)
        init.setChildren([zero])
      elif typeName in ['dcomplex', 'fcomplex']:
        import ASE.Compiler.Cxx.Constant
        zero1 = ASE.Compiler.Cxx.Constant.Constant()
        zero1.setValue_double(0.0)
        zero2 = ASE.Compiler.Cxx.Constant.Constant()
        zero2.setValue_double(0.0)
        init.setChildren([zero1, zero2])
      elif typeName == 'string':
        init.setChildren([self.getNull()])
    elif type.isInstanceOf('ASE.Compiler.Cxx.Pointer'):
      init.setChildren([self.getNull()])
    else:
      init.setChildren([self.getInteger(0)])
    return init
  
  def getDeclaration(self, var, type, initializer = None, comment = None, isStatic = 0):
    import ASE.Compiler.Cxx.Declarator
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    decl.setIdentifier(var.getIdentifier())
    decl.setType(type)
    decl.setStatic(isStatic)
    if initializer is None:
      if not type.isInstanceOf('ASE.Compiler.Cxx.Struct'):
        decl.setInitializer(self.getInitializer(type))
    else:
      decl.setInitializer(initializer)
    return self.getDecl(decl, comment)
  
  def getExpStmt(self, exp, comment = None):
    '''
    Make a statement from an expression
    '''
    import ASE.Compiler.Cxx.ExpressionStatement
    stmt = ASE.Compiler.Cxx.ExpressionStatement.ExpressionStatement()
    stmt.setChildren([exp])
    if not comment is None:
      stmt.setComments([self.getComment(comment)])
    return stmt
  
  def castToType(self, exp, type, numPointers = 0):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    for n in range(numPointers):
      pointer = ASE.Compiler.Cxx.Pointer.Pointer()
      pointer.setChildren([type])
      type = pointer
    decl.setType(type)
    cast = ASE.Compiler.Cxx.Cast.Cast()
    cast.setChildren([exp])
    cast.setTypeName(decl)
    return cast
  
  def castToNamedType(self, exp, typeName, numPointers = 0, isConst = 0, isStruct = 0):
    if isStruct:
      import ASE.Compiler.Cxx.Struct
      type = ASE.Compiler.Cxx.Struct.Struct()
      type.setIdentifier(typeName)
    else:
      if typeName in self.typeMap and not isConst:
        type = self.typeMap[typeName]
      else:
        import ASE.Compiler.Cxx.Type
        type = ASE.Compiler.Cxx.Type.Type()
        type.setIdentifier(typeName)
        type.setBaseType(typeName in self.typeMap.keys())
    if isConst:
      type.setConst(1)
    return self.castToType(exp, type, numPointers)
  
  def castToVoidPointer(self, exp):
    return self.castToNamedType(exp, 'void', 1)
  
  def castToString(self, constString):
    return self.castToNamedType(self.getString(constString), 'char', 1)
