# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.StubGenerator
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class StubGenerator:
  '''
  This visitor transforms a SIDL AST into a set of C Stub ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.sourceLanguages = ['Cxx']
    self.templateMethods = {}

    self.SIDL = SIDL()
    self.Cxx = Cxx()
    self.IOR = IOR()
  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = list(sourceLanguages)
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    if not language in self.source:
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    if not language in self.source:
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not language in self.sourceLanguages:
      raise RuntimeError('Invalid source language: ' + str(language))
    if not language in self.source:
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitInterface(self, node):
    '''
    Each interface becomes a source and header file including the methods and cast method
    '''
    decls = []
    defns = []
    
    def extend(d):
      decls.extend(d[0])
      defns.extend(d[1])
      return None
    [self.createTemplateMethod(method) for method in node.getMethods()]
    extend(self.getConstructorMethod(node))
    for method in self.getTemplateMethods(node.getMethods()):
      extend(self.getStubMethod(node, method))
    self.addSource('Cxx', [self.getHeaderFile(node, decls)])
    self.addSource('Cxx', [self.getSourceFile(node, defns)])
  
  def visitClass(self, node):
    '''
    Each class becomes a source and header file including the methods and constructor method
    '''
    self.visitInterface(node)
  # Ending class methods section
  pass
  
  def getFileBasename(self, interface):
    import os
    idChain = interface.getFullIdentifier().split('.')
    if not len(idChain[:-1]):
      return idChain[-1] + '_Stub'
    return os.path.join(apply(os.path.join, idChain[:-1]), idChain[-1] + '_Stub')
  
  def getGuardedInclude(self, interface):
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_') + '_stub')
    guard.setValue(1)
    guard.addChildren([self.Cxx.getInclude(self.getFileBasename(interface) + '.h')])
    return guard
  
  def getConstructor(self, interface):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Comment
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    comment = ASE.Compiler.SIDL.Comment.Comment()
    comment.setComment('Constructor and cast method')
    iorParam = ASE.Compiler.SIDL.Parameter.Parameter()
    iorParam.setIdentifier('obj')
    iorParam.setType(self.SIDL.getBaseInterface(interface))
    iorParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    iorReturn = ASE.Compiler.SIDL.Parameter.Parameter()
    iorReturn.setIdentifier('ior')
    iorReturn.setType(self.SIDL.typeMap['opaque'])
    iorReturn.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    constructor = ASE.Compiler.SIDL.Method.Method()
    constructor.setIdentifier(interface.getIdentifier())
    constructor.setReturnParameter(iorReturn)
    constructor.setChildren([iorParam])
    constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    constructor.setParent(interface)
    constructor.setComments([comment])
    constructor.setStatic(1)
    return constructor
  
  def getConstructorMethod(self, interface):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    method = self.getConstructor(interface)
    stubDecl = self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1)
    stubFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1).getChildren()[0])
    objVar = self.Cxx.getVar(stubFunc.getParameters()[0].getIdentifier())
    extFuncVar = self.Cxx.getVar('_extFunc')
    excVar = self.Cxx.getVar(stubFunc.getParameters()[-1].getIdentifier())
    selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    selfEpv.setChildren([objVar])
    selfEpv.setMember('d_epv')
    selfEpv.setPointer(1)
    castPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    castPtr.setChildren([selfEpv])
    castPtr.setMember('castObject')
    castPtr.setPointer(1)
    castFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    castFunc.setChildren([self.Cxx.getGroup(castPtr)])
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFunc)])
    castCall.setArguments([objVar, self.Cxx.getString(interface.getFullIdentifier()), excVar])
    castReturn = ASE.Compiler.Cxx.Return.Return()
    castReturn.setChildren([castCall])
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      trueBranch.setChildren([castReturn])
      falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
      extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      extFuncDecl = ASE.Compiler.Cxx.Function.Function()
      extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
      extFuncDecl.setType(self.IOR.getExternalType(interface))
      falseBranch.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
      extFuncType = ASE.Compiler.Cxx.Function.Function()
      extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
      extFuncType.setType(self.IOR.getExternalType(interface))
      sidlExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
      staticEPV = ASE.Compiler.Cxx.StructReference.StructReference()
      staticEPV.setChildren([sidlExtCall])
      staticEPV.setMember('getStaticEPV')
      staticEPV.setPointer(1)
      staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      staticEPVCall.setChildren([staticEPV])
      staticEPVCall.setArguments([self.Cxx.getAddress(excVar)])
      classMethods = ASE.Compiler.Cxx.StructReference.StructReference()
      classMethods.setChildren([staticEPVCall])
      classMethods.setMember('getClassMethodTable')
      classMethods.setPointer(1)
      classMethodsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      classMethodsCall.setChildren([classMethods])
      classMethodsCall.setArguments([self.Cxx.getString(interface.getFullIdentifier()), self.Cxx.getAddress(excVar)])
      extFunc = ASE.Compiler.Cxx.Cast.Cast()
      extFunc.setTypeName(extFuncType)
      extFunc.setChildren([classMethodsCall])
      setExternals = ASE.Compiler.Cxx.Assignment.Assignment()
      setExternals.setChildren([extFuncVar, extFunc])
      extFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
      extFuncInd.setChildren([extFuncVar])
      extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      extFuncCall.setChildren([self.Cxx.getGroup(extFuncInd)])
      createPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      createPtr.setChildren([extFuncCall])
      createPtr.setMember('createObject')
      createPtr.setPointer(1)
      createFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      createFunc.setChildren([self.Cxx.getGroup(createPtr)])
      createCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createCall.setChildren([self.Cxx.getGroup(createFunc)])
      createCall.setArguments([excVar])
      createReturn = ASE.Compiler.Cxx.Return.Return()
      createReturn.setChildren([createCall])
      falseBranch.setChildren([self.Cxx.getExpStmt(setExternals), createReturn])
      checkObj = ASE.Compiler.Cxx.If.If()
      checkObj.setBranch(objVar)
      checkObj.setChildren([trueBranch, falseBranch])
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([checkObj])
    else:
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setDeclarations([self.Cxx.getDeclaration(self.Cxx.getVar('_result'), self.IOR.getType(interface))])
      compound.setChildren([self.IOR.getErrorCheck(interface, method, objVar, 'Interfaces require an argument for the constructor'), castReturn])
    stubFunc.setBody(compound)
    return ([stubDecl], [stubFunc])
  
  def getStubMethod(self, interface, method):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    stubDecl = self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1)
    stubFunc = ASE.Compiler.Cxx.Function.Function(self.IOR.getFunctionDeclaration(interface, method, qualifyName = 1).getChildren()[0])
    if not method.getStatic():
      epv = ASE.Compiler.Cxx.StructReference.StructReference()
      epv.setChildren([self.Cxx.getVar('self')])
      epv.setMember('d_epv')
      epv.setPointer(1)
    else:
      extFunc = ASE.Compiler.Cxx.Declarator.Declarator()
      extFunc.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '__externals')
      extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      extFuncCall.setChildren([extFunc])
      sepvFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      sepvFuncPtr.setChildren([extFuncCall])
      sepvFuncPtr.setMember('getStaticEPV')
      sepvFuncPtr.setPointer(1)
      epv = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      epv.setChildren([sepvFuncPtr])
      epv.setArguments([self.Cxx.getVar('_exception')])
    epvFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    epvFuncPtr.setChildren([epv])
    epvFuncPtr.setMember(method.getIdentifier())
    epvFuncPtr.setPointer(1)
    epvFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    epvFunc.setChildren([self.Cxx.getGroup(epvFuncPtr)])
    epvFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    epvFuncCall.setChildren([self.Cxx.getGroup(epvFunc)])
    args = []
    for (p, param) in enumerate(stubFunc.getParameters()):
      if p == 0 and not method.getStatic():
        selfObj = ASE.Compiler.Cxx.StructReference.StructReference()
        selfObj.setChildren([self.Cxx.getVar('self')])
        selfObj.setMember('d_object')
        selfObj.setPointer(1)
        args.append(selfObj)
      else:
        args.append(self.Cxx.getVar(param.getIdentifier()))
    epvFuncCall.setArguments(args)
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    if method.getReturnParameter() is None:
      compound.setChildren([self.Cxx.getExpStmt(epvFuncCall)])
    else:
      epvReturn = ASE.Compiler.Cxx.Return.Return()
      epvReturn.setChildren([epvFuncCall])
      compound.setChildren([epvReturn])
    stubFunc.setBody(compound)
    return ([stubDecl], [stubFunc])
  
  def getHeaderFile(self, interface, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Define
    import ASE.Compiler.Cxx.Header
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_') + '_stub')
    guard.setValue(1)
    define = ASE.Compiler.Cxx.Define.Define()
    define.setIdentifier(guard.getBranchExpression())
    externBegin = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externBegin.setIdentifier('ifdef')
    externBegin.setBranchExpression('__cplusplus')
    externBegin.setValue(0)
    externBegin.setInactiveCode('extern "C" {')
    externEnd = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    externEnd.setIdentifier('ifdef')
    externEnd.setBranchExpression('__cplusplus')
    externEnd.setValue(0)
    externEnd.setInactiveCode('}')
    guard.addChildren([define, externBegin])
    guard.addChildren([self.IOR.getGuardedInclude(interface)])
    guard.addChildren(decls)
    guard.addChildren([externEnd])
    header = ASE.Compiler.Cxx.Header.Header()
    header.setFilename(self.getFileBasename(interface) + '.h')
    header.setChildren([guard])
    header.setPurpose(ASE.Compiler.CodePurpose.STUB)
    return header
  
  def getSourceFile(self, interface, defns):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Source
    stdInclude = ASE.Compiler.Cxx.Include.Include()
    stdInclude.setIdentifier('<stdlib.h>')
    source = ASE.Compiler.Cxx.Source.Source()
    source.setFilename(self.getFileBasename(interface) + '.c')
    source.addChildren([stdInclude, self.getGuardedInclude(interface), self.IOR.getGuardedInclude(self.SIDL.getASEException(interface)), self.IOR.getGuardedInclude(self.SIDL.getLoader(interface))])
    source.addChildren(defns)
    source.setPurpose(ASE.Compiler.CodePurpose.STUB)
    return source
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not method.getReturnParameter() is None:
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not parameter is None:
      type = parameter.getType()
      if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not method in self.templateMethods:
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 

class SIDL(object):
  
  def __init__(self):
    self.typeMap = self.createTypeMap()
    return None
  
  def createTypeMap(self):
    import ASE.Compiler.SIDL.Type
    typeMap = {}

    for typeName in ['int', 'bool', 'char', 'dcomplex', 'double', 'fcomplex', 'float', 'long', 'opaque', 'string', 'void']:
      sidlType = ASE.Compiler.SIDL.Type.Type()
      sidlType.setIdentifier(typeName)
      sidlType.setBaseType(1)
      if not typeName == 'int':
        sidlType.setDependencies([typeMap['int']])
      typeMap[typeName] = sidlType
    return typeMap
  
  def getLoader(self, interface):
    for child in self.getBaseInterface(interface).getParent().getChildren():
      if child.getIdentifier() == 'Loader':
        import ASE.Compiler.SIDL.Class
        return ASE.Compiler.SIDL.Class.Class(child)
    raise RuntimeError('Could not find Loader using ' + interface.getFullIdentifier())
  
  def getBasePackage(self, interface):
    import ASE.Compiler.SIDL.Package
    return ASE.Compiler.SIDL.Package.Package(self.getAncestor(interface, 'ASE.BaseInterface').getParent())
  
  def getBaseInterface(self, interface):
    return self.getAncestor(interface, 'ASE.BaseInterface')
  
  def getBaseException(self, exception):
    return self.getAncestor(exception, 'ASE.BaseException')
  
  def getASEException(self, interface):
    asePackage = self.getBasePackage(interface)
    for child in asePackage.getChildren():
      if child.getIdentifier() == 'ASEException':
        import ASE.Compiler.SIDL.Class
        return ASE.Compiler.SIDL.Class.Class(child)
    return None
  
  def getAncestor(self, interface, typeName):
    import ASE.Compiler.SIDL.Interface
    interface = ASE.Compiler.SIDL.Interface.Interface(interface)
    if interface.getFullIdentifier() == typeName:
      return interface
    parents = []
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      import ASE.Compiler.SIDL.Class
      parents.extend(ASE.Compiler.SIDL.Class.Class(interface).getInterfaces())
    parents.extend(interface.getParents())
    for parent in parents:
      base = self.getAncestor(parent, typeName)
      if base:
        return base
    return None

class Cxx(object):
  
  def __init__(self):
    self.typeMap = self.createTypeMap()
    self.nullVar = self.getVar('NULL')
    self.integers = {}

    return None
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('bool')
    cxxType.setBaseType(1)
    typeMap['bool'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('char')
    cxxType.setBaseType(1)
    typeMap['char'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<double>')
    typeMap['dcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('double')
    cxxType.setBaseType(1)
    typeMap['double'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<float>')
    typeMap['fcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('float')
    cxxType.setBaseType(1)
    typeMap['float'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('int')
    cxxType.setBaseType(1)
    typeMap['int'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('long long')
    cxxType.setBaseType(1)
    typeMap['long'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('void')
    cxxType.setBaseType(1)
    typeMap['void'] = cxxType
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['void']])
    typeMap['opaque'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('std::str')
    cxxType.setBaseType(1)
    typeMap['string'] = cxxType
    return typeMap
  
  def getNull(self):
    return self.nullVar
  
  def getInteger(self, num):
    if not num in self.integers:
      import ASE.Compiler.Cxx.Constant
      integer = ASE.Compiler.Cxx.Constant.Constant()
      integer.setValue_int(num)
      self.integers[num] = integer
    return self.integers[num]
  
  def getInclude(self, filename):
    import ASE.Compiler.Cxx.Include
    include = ASE.Compiler.Cxx.Include.Include()
    include.setIdentifier('<' + filename + '>')
    return include
  
  def getString(self, s):
    import ASE.Compiler.Cxx.Constant
    string = ASE.Compiler.Cxx.Constant.Constant()
    string.setIdentifier(s)
    return string
  
  def getComment(self, comment):
    import ASE.Compiler.Cxx.Comment
    c = ASE.Compiler.Cxx.Comment.Comment()
    c.setComment(comment)
    return c
  
  def getVar(self, name):
    import ASE.Compiler.Cxx.Variable
    var = ASE.Compiler.Cxx.Variable.Variable()
    var.setIdentifier(name)
    return var
  
  def getAddress(self, exp):
    import ASE.Compiler.Cxx.Address
    addr = ASE.Compiler.Cxx.Address.Address()
    addr.setChildren([exp])
    return addr
  
  def getGroup(self, exp):
    import ASE.Compiler.Cxx.Group
    group = ASE.Compiler.Cxx.Group.Group()
    group.setChildren([exp])
    return group
  
  def getDeclGroup(self, exp):
    import ASE.Compiler.Cxx.DeclaratorGroup
    group = ASE.Compiler.Cxx.DeclaratorGroup.DeclaratorGroup()
    group.setChildren([exp])
    return group
  
  def getDecl(self, declarator, comment = None):
    '''
    Make a declaration from a declarator
    '''
    import ASE.Compiler.Cxx.Declaration
    decl = ASE.Compiler.Cxx.Declaration.Declaration()
    decl.setChildren([declarator])
    if not comment is None:
      import ASE.Compiler.Cxx.Comment
      c = ASE.Compiler.Cxx.Comment.Comment()
      c.setComment(comment)
      decl.setComments([c])
    return decl
  
  def getInitializer(self, type):
    import ASE.Compiler.Cxx.Initializer
    init = ASE.Compiler.Cxx.Initializer.Initializer()
    if type.getBaseType():
      typeName = type.getIdentifier()
      if typeName in ['bool', 'char', 'int', 'long', 'int32_t', 'int64_t']:
        init.setChildren([self.getInteger(0)])
      elif typeName in ['double', 'float']:
        import ASE.Compiler.Cxx.Constant
        zero = ASE.Compiler.Cxx.Constant.Constant()
        zero.setValue_double(0.0)
        init.setChildren([zero])
      elif typeName in ['dcomplex', 'fcomplex']:
        import ASE.Compiler.Cxx.Constant
        zero1 = ASE.Compiler.Cxx.Constant.Constant()
        zero1.setValue_double(0.0)
        zero2 = ASE.Compiler.Cxx.Constant.Constant()
        zero2.setValue_double(0.0)
        init.setChildren([zero1, zero2])
      elif typeName == 'string':
        init.setChildren([self.getNull()])
    elif type.isInstanceOf('ASE.Compiler.Cxx.Pointer'):
      init.setChildren([self.getNull()])
    else:
      init.setChildren([self.getInteger(0)])
    return init
  
  def getDeclaration(self, var, type, initializer = None, comment = None, isStatic = 0):
    import ASE.Compiler.Cxx.Declarator
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    decl.setIdentifier(var.getIdentifier())
    decl.setType(type)
    decl.setStatic(isStatic)
    if initializer is None:
      if not type.isInstanceOf('ASE.Compiler.Cxx.Struct'):
        decl.setInitializer(self.getInitializer(type))
    else:
      decl.setInitializer(initializer)
    return self.getDecl(decl, comment)
  
  def getExpStmt(self, exp, comment = None):
    '''
    Make a statement from an expression
    '''
    import ASE.Compiler.Cxx.ExpressionStatement
    stmt = ASE.Compiler.Cxx.ExpressionStatement.ExpressionStatement()
    stmt.setChildren([exp])
    if not comment is None:
      stmt.setComments([self.getComment(comment)])
    return stmt
  
  def castToType(self, exp, type, numPointers = 0):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    for n in range(numPointers):
      pointer = ASE.Compiler.Cxx.Pointer.Pointer()
      pointer.setChildren([type])
      type = pointer
    decl.setType(type)
    cast = ASE.Compiler.Cxx.Cast.Cast()
    cast.setChildren([exp])
    cast.setTypeName(decl)
    return cast
  
  def castToNamedType(self, exp, typeName, numPointers = 0, isConst = 0, isStruct = 0):
    if isStruct:
      import ASE.Compiler.Cxx.Struct
      type = ASE.Compiler.Cxx.Struct.Struct()
      type.setIdentifier(typeName)
    else:
      if typeName in self.typeMap and not isConst:
        type = self.typeMap[typeName]
      else:
        import ASE.Compiler.Cxx.Type
        type = ASE.Compiler.Cxx.Type.Type()
        type.setIdentifier(typeName)
        type.setBaseType(typeName in self.typeMap.keys())
    if isConst:
      type.setConst(1)
    return self.castToType(exp, type, numPointers)
  
  def castToVoidPointer(self, exp):
    return self.castToNamedType(exp, 'void', 1)
  
  def castToString(self, constString):
    return self.castToNamedType(self.getString(constString), 'char', 1)

class IOR(object):
  
  def __init__(self):
    self.Cxx = Cxx()
    self.SIDL = SIDL()
    self.typeMap = self.createTypeMap()
    self.epv = {}

    self.external = {}

    self.sepv = {}

    return None
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int')
    iorType.setBaseType(1)
    typeMap['bool'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('char')
    iorType.setBaseType(1)
    typeMap['char'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_dcomplex')
    typeMap['dcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('double')
    iorType.setBaseType(1)
    typeMap['double'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_fcomplex')
    typeMap['fcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('float')
    iorType.setBaseType(1)
    typeMap['float'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int32_t')
    iorType.setBaseType(1)
    typeMap['int'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int64_t')
    iorType.setBaseType(1)
    typeMap['long'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('void')
    iorType.setBaseType(1)
    typeMap['void'] = iorType
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([typeMap['void']])
    typeMap['opaque'] = iorType
    constChar = ASE.Compiler.Cxx.Type.Type()
    constChar.setIdentifier('char')
    constChar.setBaseType(1)
    constChar.setConst(1)
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([constChar])
    typeMap['string'] = iorType
    return typeMap
  
  def getType(self, type):
    '''
    Return the C IOR type corresponding to this sidl type
    '''
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    iorType = None
    if type.getBaseType():
      try:
        iorType = self.typeMap[type.getIdentifier()]
      except KeyError:
        raise RuntimeError('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      iorType = ASE.Compiler.Cxx.Enumeration.Enumeration()
      iorType.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__enum')
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(entryType.getFullIdentifier().replace('.', '_') + '__array')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__object')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    else:
      import ASE.Compiler.Cxx.Type
      iorType = ASE.Compiler.Cxx.Type.Type()
      iorType.setIdentifier(type.getIdentifier())
    return iorType
  
  def getEpvType(self, type):
    if not type in self.epv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      epvStruct = ASE.Compiler.Cxx.Struct.Struct()
      epvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__epv')
      epvType = ASE.Compiler.Cxx.Pointer.Pointer()
      epvType.setChildren([epvStruct])
      self.epv[type] = epvType
    return self.epv[type]
  
  def getExternalType(self, type):
    if not type in self.external:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      extStruct = ASE.Compiler.Cxx.Struct.Struct()
      extStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
      extStruct.setConst(1)
      extType = ASE.Compiler.Cxx.Pointer.Pointer()
      extType.setChildren([extStruct])
      self.external[type] = extType
    return self.external[type]
  
  def getStaticExternalType(self, type):
    if not type in self.sepv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
      sepvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__sepv')
      sepvStruct.setConst(1)
      sepvType = ASE.Compiler.Cxx.Pointer.Pointer()
      sepvType.setChildren([sepvStruct])
      self.sepv[type] = sepvType
    return self.sepv[type]
  
  def getParameter(self, parameter, mode = None):
    '''
    Return the C IOR parameter corresponding to this sidl parameter
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Comment
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    if mode is None:
      type = parameter.getType()
      mode = parameter.getAccessMode()
    else:
      type = parameter
    comment = ASE.Compiler.Cxx.Comment.Comment()
    iorType = self.getType(type)
    if mode == ASE.Compiler.ParameterAccessMode.IN:
      comment.setComment('in')
    elif mode == ASE.Compiler.ParameterAccessMode.OUT:
      comment.setComment('out')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    elif mode == ASE.Compiler.ParameterAccessMode.INOUT:
      comment.setComment('inout')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(parameter.getIdentifier())
    iorParam.setType(iorType)
    iorParam.setComments([comment])
    return iorParam
  
  def getFunctionDeclaration(self, type, method, isPointer = 0, qualifyName = 0):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    func = ASE.Compiler.Cxx.Function.Function()
    if qualifyName:
      name = type.getFullIdentifier().replace('.', '_') + '_' + method.getIdentifier()
    else:
      name = method.getIdentifier()
    if isPointer:
      funcPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      funcPtr.setIdentifier(name)
      funcPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      func.setChildren([self.Cxx.getDeclGroup(funcPtr)])
    else:
      func.setChildren([self.Cxx.getVar(name)])
    if method.getReturnParameter() is None:
      func.setType(self.typeMap['void'])
    else:
      func.setType(self.getType(method.getReturnParameter().getType()))
    if not method.getStatic():
      selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
      selfParam.setIdentifier('self')
      selfParam.setType(self.getType(type))
      func.addParameters([selfParam])
    for child in method.getChildren():
      func.addParameters([self.getParameter(ASE.Compiler.SIDL.Parameter.Parameter(child))])
    excParam = ASE.Compiler.SIDL.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setAccessMode(ASE.Compiler.ParameterAccessMode.OUT)
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface') and not type.getFullIdentifier() == 'ASE.BaseInterface':
      excParam.setType(self.SIDL.getBaseException(method.getExceptions()[0]))
    else:
      excParam.setType(self.SIDL.typeMap['opaque'])
    func.addParameters([self.getParameter(excParam)])
    comment = method.getFullIdentifier() + ': '
    comment += '\n'.join([c.getComment() for c in method.getComments()])
    return self.Cxx.getDecl(func, comment)
  
  def getFileBasename(self, interface):
    import os
    idChain = interface.getFullIdentifier().split('.')
    if not len(idChain[:-1]):
      return idChain[-1] + '_IOR'
    return os.path.join(apply(os.path.join, idChain[:-1]), idChain[-1] + '_IOR')
  
  def getGuardedInclude(self, interface):
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_'))
    guard.setValue(1)
    guard.addChildren([self.Cxx.getInclude(self.getFileBasename(interface) + '.h')])
    return guard
  
  def getErrorCheck(self, interface, method, branchVar, errorMessage, branchExp = None):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.StructReference
    excVar = self.Cxx.getVar('_exception')
    localExcVar = self.Cxx.getVar('_local_exception')
    excType = self.SIDL.getASEException(interface)
    extFunc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFunc.setChildren([self.Cxx.getVar(excType.getFullIdentifier().replace('.', '_') + '__externals')])
    createFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    createFunc.setChildren([extFunc])
    createFunc.setMember('createObject')
    createFunc.setPointer(1)
    createExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createExc.setChildren([createFunc])
    createExc.setArguments([self.Cxx.getNull()])
    localExcInit = ASE.Compiler.Cxx.Initializer.Initializer()
    localExcInit.setChildren([createExc])
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([localExcVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    messageFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    messageFunc.setChildren([excEpv])
    messageFunc.setMember('setMessage')
    messageFunc.setPointer(1)
    setMessage = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setMessage.setChildren([messageFunc])
    setMessage.setArguments([localExcVar, self.Cxx.getString(errorMessage), excVar])
    excInd = ASE.Compiler.Cxx.Indirection.Indirection()
    excInd.setChildren([excVar])
    negExcInd = ASE.Compiler.Cxx.Negation.Negation()
    negExcInd.setChildren([self.Cxx.getGroup(excInd)])
    setExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setExc.setChildren([excInd, self.Cxx.castToType(localExcVar, self.getType(self.SIDL.getBaseException(excType)))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setExc)])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(negExcInd)
    checkExc.setChildren([compound])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    if not method is None and not method.getReturnParameter() is None:
      returnStmt.setChildren([self.Cxx.getVar('_result')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(localExcVar, self.getType(excType), localExcInit)])
    compound.setChildren([self.Cxx.getExpStmt(setMessage), checkExc, returnStmt])
    checkVar = ASE.Compiler.Cxx.If.If()
    if branchExp is None:
      negVar = ASE.Compiler.Cxx.Negation.Negation()
      negVar.setChildren([branchVar])
      checkVar.setBranch(negVar)
    else:
      checkVar.setBranch(branchExp)
    checkVar.setChildren([compound])
    return checkVar
  
  def getClassMethodTable(self, klass):
    excVar = self.Cxx.getVar('_exception')
    loaderExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    loaderExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
    staticEPVFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    staticEPVFunc.setChildren([loaderExtCall])
    staticEPVFunc.setMember('getStaticEPV')
    staticEPVFunc.setPointer(1)
    staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    staticEPVCall.setChildren([staticEPVFunc])
    staticEPVCall.setArguments([excVar])
    tableFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    tableFunc.setChildren([staticEPVCall])
    tableFunc.setMember('getClassMethodTable')
    tableFunc.setPointer(1)
    tableFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    tableFuncCall.setChildren([tableFunc])
    tableFuncCall.setArguments([self.Cxx.getString(klass.getFullIdentifier()), excVar])
    return tableFuncCall
