class Cxx(object):
  def __init__(self):
    self.typeMap = self.createTypeMap()
    self.nullVar = self.getVar('NULL')
    self.integers = {}

    return

  def clear(self):
    self.integers = {}
    return
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('bool')
    cxxType.setBaseType(1)
    typeMap['bool'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('char')
    cxxType.setBaseType(1)
    typeMap['char'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<double>')
    typeMap['dcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('double')
    cxxType.setBaseType(1)
    typeMap['double'] = cxxType
    cxxType = ASE.Compiler.Cxx.Struct.Struct()
    cxxType.setIdentifier('complex<float>')
    typeMap['fcomplex'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('float')
    cxxType.setBaseType(1)
    typeMap['float'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('int')
    cxxType.setBaseType(1)
    typeMap['int'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('long long')
    cxxType.setBaseType(1)
    typeMap['long'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('void')
    cxxType.setBaseType(1)
    typeMap['void'] = cxxType
    cxxType = ASE.Compiler.Cxx.Pointer.Pointer()
    cxxType.setChildren([typeMap['void']])
    typeMap['opaque'] = cxxType
    cxxType = ASE.Compiler.Cxx.Type.Type()
    cxxType.setIdentifier('std::str')
    cxxType.setBaseType(1)
    typeMap['string'] = cxxType
    return typeMap
  
  def getNull(self):
    return self.nullVar
  
  def getInteger(self, num):
    if not num in self.integers:
      import ASE.Compiler.Cxx.Constant
      integer = ASE.Compiler.Cxx.Constant.Constant()
      integer.setValue_int(num)
      self.integers[num] = integer
    return self.integers[num]
  
  def getDouble(self, num):
    import ASE.Compiler.Cxx.Constant

    double = ASE.Compiler.Cxx.Constant.Constant()
    double.setValue_double(num)
    return double

  def getInclude(self, filename):
    import ASE.Compiler.Cxx.Include
    include = ASE.Compiler.Cxx.Include.Include()
    include.setIdentifier('"' + filename + '"')
    return include
  
  def getString(self, s):
    import ASE.Compiler.Cxx.Constant
    string = ASE.Compiler.Cxx.Constant.Constant()
    string.setIdentifier(s)
    string.setValue_string(s)
    return string
  
  def getComment(self, comment):
    import ASE.Compiler.Cxx.Comment
    c = ASE.Compiler.Cxx.Comment.Comment()
    c.setComment(comment)
    return c
  
  def getVar(self, name):
    import ASE.Compiler.Cxx.Variable
    var = ASE.Compiler.Cxx.Variable.Variable()
    var.setIdentifier(name)
    return var
  
  def getAddress(self, exp):
    import ASE.Compiler.Cxx.Address
    addr = ASE.Compiler.Cxx.Address.Address()
    addr.setChildren([exp])
    return addr
  
  def getGroup(self, exp):
    import ASE.Compiler.Cxx.Group
    group = ASE.Compiler.Cxx.Group.Group()
    group.setChildren([exp])
    return group
  
  def getDeclGroup(self, exp):
    import ASE.Compiler.Cxx.DeclaratorGroup
    group = ASE.Compiler.Cxx.DeclaratorGroup.DeclaratorGroup()
    group.setChildren([exp])
    return group
  
  def getDecl(self, declarator, comment = None):
    '''
    Make a declaration from a declarator
    '''
    import ASE.Compiler.Cxx.Declaration
    decl = ASE.Compiler.Cxx.Declaration.Declaration()
    decl.setChildren([declarator])
    if not comment is None:
      import ASE.Compiler.Cxx.Comment
      c = ASE.Compiler.Cxx.Comment.Comment()
      c.setComment(comment)
      decl.setComments([c])
    return decl
  
  def getInitializer(self, type):
    import ASE.Compiler.Cxx.Constant
    import ASE.Compiler.Cxx.Initializer
    init = ASE.Compiler.Cxx.Initializer.Initializer()
    if type.getBaseType():
      typeName = type.getIdentifier()
      if typeName in ['bool', 'char', 'int', 'long', 'long long', 'int32_t', 'int64_t']:
        init.setChildren([self.getInteger(0)])
      elif typeName in ['double', 'float']:
        zero = ASE.Compiler.Cxx.Constant.Constant()
        zero.setValue_double(0.0)
        init.setChildren([zero])
      elif typeName in ['dcomplex', 'fcomplex']:
        zero1 = ASE.Compiler.Cxx.Constant.Constant()
        zero1.setValue_double(0.0)
        zero2 = ASE.Compiler.Cxx.Constant.Constant()
        zero2.setValue_double(0.0)
        init.setChildren([zero1, zero2])
      elif typeName in ['string']:
        init.setChildren([self.getNull()])
    elif type.isInstanceOf('ASE.Compiler.Cxx.Pointer'):
      init.setChildren([self.getNull()])
    elif type.isInstanceOf('ASE.Compiler.Cxx.Struct'):
      if type.getIdentifier() in ['SIDL_dcomplex', 'SIDL_fcomplex']:
        zero1 = ASE.Compiler.Cxx.Constant.Constant()
        zero1.setValue_double(0.0)
        zero2 = ASE.Compiler.Cxx.Constant.Constant()
        zero2.setValue_double(0.0)
        init.setChildren([zero1, zero2])
    elif type.isInstanceOf('ASE.Compiler.Cxx.Type'):
      if type.getIdentifier() in ['Py_complex']:
        zero1 = ASE.Compiler.Cxx.Constant.Constant()
        zero1.setValue_double(0.0)
        zero2 = ASE.Compiler.Cxx.Constant.Constant()
        zero2.setValue_double(0.0)
        init.setChildren([zero1, zero2])
      else:
        init.setChildren([self.getInteger(0)])
    else:
      init.setChildren([self.getInteger(0)])
    return init
  
  def getDeclaration(self, var, type, initializer = None, comment = None, isStatic = 0):
    import ASE.Compiler.Cxx.Declarator
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    decl.setIdentifier(var.getIdentifier())
    decl.setType(type)
    decl.setStatic(isStatic)
    if initializer is None:
      decl.setInitializer(self.getInitializer(type))
    else:
      decl.setInitializer(initializer)
    return self.getDecl(decl, comment)
  
  def getExpStmt(self, exp, comment = None):
    '''
    Make a statement from an expression
    '''
    import ASE.Compiler.Cxx.ExpressionStatement
    stmt = ASE.Compiler.Cxx.ExpressionStatement.ExpressionStatement()
    stmt.setChildren([exp])
    if not comment is None:
      stmt.setComments([self.getComment(comment)])
    return stmt
  
  def castToType(self, exp, type, numPointers = 0):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Pointer
    decl = ASE.Compiler.Cxx.Declarator.Declarator()
    for n in range(numPointers):
      pointer = ASE.Compiler.Cxx.Pointer.Pointer()
      pointer.setChildren([type])
      type = pointer
    decl.setType(type)
    cast = ASE.Compiler.Cxx.Cast.Cast()
    cast.setChildren([exp])
    cast.setTypeName(decl)
    return cast
  
  def castToNamedType(self, exp, typeName, numPointers = 0, isConst = 0, isStruct = 0):
    if isStruct:
      import ASE.Compiler.Cxx.Struct
      type = ASE.Compiler.Cxx.Struct.Struct()
      type.setIdentifier(typeName)
    else:
      if typeName in self.typeMap and not isConst:
        type = self.typeMap[typeName]
      else:
        import ASE.Compiler.Cxx.Type
        type = ASE.Compiler.Cxx.Type.Type()
        type.setIdentifier(typeName)
        type.setBaseType(typeName in self.typeMap.keys())
    if isConst:
      type.setConst(1)
    return self.castToType(exp, type, numPointers)
  
  def castToVoidPointer(self, exp):
    return self.castToNamedType(exp, 'void', 1)
  
  def castToString(self, constString):
    return self.castToNamedType(self.getString(constString), 'char', 1)
