class IOR(object):
  def __init__(self):
    import ASE.Compiler.Python.Cxx
    import ASE.Compiler.Python.SIDL
    self.SIDL = ASE.Compiler.Python.SIDL.SIDL()
    self.Cxx = ASE.Compiler.Python.Cxx.Cxx()
    self.typeMap = self.createTypeMap()
    self.external = {}

    self.epv = {}

    self.sepv = {}

    return
  
  def raiseException(self, message, trace = ''):
    import sys
    e = ASE.Compiler.CompilerException.CompilerException()
    e.setMessage(message)
    if isinstance(trace, list):
      for line in trace:
        e.addStackLine(line.strip())
    elif trace:
      e.addStackLine(trace)
    e.addStackEntry(__file__, sys._getframe(1).f_lineno, sys._getframe(1).f_code.co_name)
    raise ASE.Compiler.CompilerException.Exception, e
  
  def createTypeMap(self):
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.Type
    typeMap = {}

    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int')
    iorType.setBaseType(1)
    typeMap['bool'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('char')
    iorType.setBaseType(1)
    typeMap['char'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_dcomplex')
    typeMap['dcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('double')
    iorType.setBaseType(1)
    typeMap['double'] = iorType
    iorType = ASE.Compiler.Cxx.Struct.Struct()
    iorType.setIdentifier('SIDL_fcomplex')
    typeMap['fcomplex'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('float')
    iorType.setBaseType(1)
    typeMap['float'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int32_t')
    iorType.setBaseType(1)
    typeMap['int'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('int64_t')
    iorType.setBaseType(1)
    typeMap['long'] = iorType
    iorType = ASE.Compiler.Cxx.Type.Type()
    iorType.setIdentifier('void')
    iorType.setBaseType(1)
    typeMap['void'] = iorType
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([typeMap['void']])
    typeMap['opaque'] = iorType
    constChar = ASE.Compiler.Cxx.Type.Type()
    constChar.setIdentifier('char')
    constChar.setBaseType(1)
    constChar.setConst(1)
    iorType = ASE.Compiler.Cxx.Pointer.Pointer()
    iorType.setChildren([constChar])
    typeMap['string'] = iorType
    return typeMap
  
  def getType(self, type):
    '''
    Return the C IOR type corresponding to this sidl type
    '''
    import ASE.Compiler.Cxx.Enumeration
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    iorType = None
    if type.getBaseType():
      try:
        iorType = self.typeMap[type.getIdentifier()]
      except KeyError:
        self.raiseException('No base type: ' + str(type.getIdentifier()))
    elif type.isInstanceOf('ASE.Compiler.SIDL.Enumeration'):
      import ASE.Compiler.Cxx.Enumeration
      iorType = ASE.Compiler.Cxx.Enumeration.Enumeration()
      iorType.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__enum')
    elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(entryType.getFullIdentifier().replace('.', '_') + '__array')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    elif type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      iorStruct = ASE.Compiler.Cxx.Struct.Struct()
      iorStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__object')
      iorType = ASE.Compiler.Cxx.Pointer.Pointer()
      iorType.setChildren([iorStruct])
    else:
      import ASE.Compiler.Cxx.Type
      iorType = ASE.Compiler.Cxx.Type.Type()
      iorType.setIdentifier(type.getIdentifier())
    return iorType

  def getEpvType(self, type):
    if not type in self.epv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct

      epvStruct = ASE.Compiler.Cxx.Struct.Struct()
      epvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_')+'__epv')
      epvType   = ASE.Compiler.Cxx.Pointer.Pointer()
      epvType.setChildren([epvStruct])
      self.epv[type] = epvType
    return self.epv[type]
  
  def getExternalType(self, type):
    if not type in self.external:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      extStruct = ASE.Compiler.Cxx.Struct.Struct()
      extStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__external')
      extStruct.setConst(1)
      extType = ASE.Compiler.Cxx.Pointer.Pointer()
      extType.setChildren([extStruct])
      self.external[type] = extType
    return self.external[type]
  
  def getStaticExternalType(self, type):
    if not type in self.sepv:
      import ASE.Compiler.Cxx.Pointer
      import ASE.Compiler.Cxx.Struct
      sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
      sepvStruct.setIdentifier(type.getFullIdentifier().replace('.', '_') + '__sepv')
      sepvStruct.setConst(1)
      sepvType = ASE.Compiler.Cxx.Pointer.Pointer()
      sepvType.setChildren([sepvStruct])
      self.sepv[type] = sepvType
    return self.sepv[type]
  
  def getParameter(self, parameter, mode = None):
    '''
    Return the C IOR parameter corresponding to this sidl parameter
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Comment
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    if mode is None:
      type = parameter.getType()
      mode = parameter.getAccessMode()
    else:
      type = parameter
    comment = ASE.Compiler.Cxx.Comment.Comment()
    iorType = self.getType(type)
    if mode == ASE.Compiler.ParameterAccessMode.IN:
      comment.setComment('in')
    elif mode == ASE.Compiler.ParameterAccessMode.OUT:
      comment.setComment('out')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    elif mode == ASE.Compiler.ParameterAccessMode.INOUT:
      comment.setComment('inout')
      newType = ASE.Compiler.Cxx.Pointer.Pointer()
      newType.setChildren([iorType])
      iorType = newType
    iorParam = ASE.Compiler.Cxx.Parameter.Parameter()
    iorParam.setIdentifier(parameter.getIdentifier())
    iorParam.setType(iorType)
    iorParam.setComments([comment])
    return iorParam
  
  def getFunctionDeclaration(self, type, method, isPointer = 0, qualifyName = 0):
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    func = ASE.Compiler.Cxx.Function.Function()
    if qualifyName:
      name = type.getFullIdentifier().replace('.', '_') + '_' + method.getIdentifier()
    else:
      name = method.getIdentifier()
    if isPointer:
      funcPtr = ASE.Compiler.Cxx.Declarator.Declarator()
      funcPtr.setIdentifier(name)
      funcPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
      func.setChildren([self.Cxx.getDeclGroup(funcPtr)])
    else:
      decl = ASE.Compiler.Cxx.Declarator.Declarator()
      decl.setIdentifier(name)
      func.setChildren([decl])
    if method.getReturnParameter() is None:
      func.setType(self.typeMap['void'])
    else:
      func.setType(self.getType(method.getReturnParameter().getType()))
    if not method.getStatic():
      selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
      selfParam.setIdentifier('self')
      selfParam.setType(self.getType(type))
      func.addParameters([selfParam])
    for child in method.getChildren():
      func.addParameters([self.getParameter(ASE.Compiler.SIDL.Parameter.Parameter(child))])
    excParam = ASE.Compiler.SIDL.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setAccessMode(ASE.Compiler.ParameterAccessMode.OUT)
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface') and not type.getFullIdentifier() == 'ASE.BaseInterface':
      excParam.setType(self.SIDL.getBaseException(method.getExceptions()[0]))
    else:
      excParam.setType(self.SIDL.typeMap['opaque'])
    func.addParameters([self.getParameter(excParam)])
    comment = method.getFullIdentifier() + ': '
    comment += '\n'.join([c.getComment() for c in method.getComments()])
    return self.Cxx.getDecl(func, comment)
  
  def getFileBasename(self, interface):
    import os
    idChain = interface.getFullIdentifier().split('.')
    if not len(idChain[:-1]):
      return idChain[-1] + '_IOR'
    return os.path.join(apply(os.path.join, idChain[:-1]), idChain[-1] + '_IOR')
  
  def getGuardedInclude(self, interface):
    import ASE.Compiler.Cxx.PreprocessorConditional
    guard = ASE.Compiler.Cxx.PreprocessorConditional.PreprocessorConditional()
    guard.setIdentifier('ifndef')
    guard.setBranchExpression('included_' + interface.getFullIdentifier().replace('.', '_'))
    guard.setValue(1)
    guard.addChildren([self.Cxx.getInclude(self.getFileBasename(interface) + '.h')])
    return guard

  def getErrorCheck(self, interface, method, branchVar, errorMessage, branchExp = None):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.StructReference

    excType      = self.SIDL.getASEException(interface)
    excVar       = self.Cxx.getVar('_exception')
    localExcVar  = self.Cxx.getVar('_local_exception')
    extFunc      = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFunc.setChildren([self.Cxx.getVar(excType.getFullIdentifier().replace('.', '_')+'__externals')])
    createFunc   = ASE.Compiler.Cxx.StructReference.StructReference()
    createFunc.setChildren([extFunc])
    createFunc.setMember('createObject')
    createFunc.setPointer(1)
    createExc    = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createExc.setChildren([createFunc])
    createExc.setArguments([self.Cxx.getNull()])
    localExcInit = ASE.Compiler.Cxx.Initializer.Initializer()
    localExcInit.setChildren([createExc])
    excEpv       = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([localExcVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    messageFunc  = ASE.Compiler.Cxx.StructReference.StructReference()
    messageFunc.setChildren([excEpv])
    messageFunc.setMember('setMessage')
    messageFunc.setPointer(1)
    setMessage   = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setMessage.setChildren([messageFunc])
    setMessage.setArguments([localExcVar, self.Cxx.getString(errorMessage), excVar])
    excInd       = ASE.Compiler.Cxx.Indirection.Indirection()
    excInd.setChildren([excVar])
    setExc       = ASE.Compiler.Cxx.Assignment.Assignment()
    setExc.setChildren([excInd, self.Cxx.castToType(localExcVar, self.getType(self.SIDL.getBaseException(excType)))])
    compound     = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setExc)])
    checkExc     = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    returnStmt   = ASE.Compiler.Cxx.Return.Return()
    if not method is None and not method.getReturnParameter() is None:
      returnStmt.setChildren([self.Cxx.getVar('_return_value')])
    compound     = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(localExcVar, self.getType(excType), localExcInit)])
    compound.setChildren([self.Cxx.getExpStmt(setMessage), checkExc, returnStmt])
    checkVar     = ASE.Compiler.Cxx.If.If()
    if branchExp is None:
      negVar     = ASE.Compiler.Cxx.Negation.Negation()
      negVar.setChildren([branchVar])
      checkVar.setBranch(negVar)
    else:
      checkVar.setBranch(branchExp)
    checkVar.setChildren([compound])
    return checkVar

  def getClassMethodTable(self, klass):
    excVar        = self.Cxx.getVar('_exception')
    loaderExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    loaderExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
    staticEPVFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    staticEPVFunc.setChildren([loaderExtCall])
    staticEPVFunc.setMember('getStaticEPV')
    staticEPVFunc.setPointer(1)
    staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    staticEPVCall.setChildren([staticEPVFunc])
    staticEPVCall.setArguments([excVar])
    tableFunc     = ASE.Compiler.Cxx.StructReference.StructReference()
    tableFunc.setChildren([staticEPVCall])
    tableFunc.setMember('getClassMethodTable')
    tableFunc.setPointer(1)
    tableFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    tableFuncCall.setChildren([tableFunc])
    tableFuncCall.setArguments([self.Cxx.getString(klass.getFullIdentifier()), excVar])
    return tableFuncCall
