# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.Printer
import ASE.Compiler.Python.Add
import ASE.Compiler.Python.And
import ASE.Compiler.Python.AssAttr
import ASE.Compiler.Python.AssList
import ASE.Compiler.Python.AssName
import ASE.Compiler.Python.AssTuple
import ASE.Compiler.Python.Assign
import ASE.Compiler.Python.AugAssign
import ASE.Compiler.Python.Bitand
import ASE.Compiler.Python.Bitor
import ASE.Compiler.Python.Bitxor
import ASE.Compiler.Python.Break
import ASE.Compiler.Python.CallFunc
import ASE.Compiler.Python.Class
import ASE.Compiler.Python.Compare
import ASE.Compiler.Python.Const
import ASE.Compiler.Python.Continue
import ASE.Compiler.Python.DepthFirstVisitor
import ASE.Compiler.Python.Dict
import ASE.Compiler.Python.Discard
import ASE.Compiler.Python.Div
import ASE.Compiler.Python.Ellipsis
import ASE.Compiler.Python.For
import ASE.Compiler.Python.From
import ASE.Compiler.Python.Function
import ASE.Compiler.Python.GetAttr
import ASE.Compiler.Python.Global
import ASE.Compiler.Python.If
import ASE.Compiler.Python.Import
import ASE.Compiler.Python.Invert
import ASE.Compiler.Python.Keyword
import ASE.Compiler.Python.Lambda
import ASE.Compiler.Python.LeftShift
import ASE.Compiler.Python.List
import ASE.Compiler.Python.ListComp
import ASE.Compiler.Python.ListCompFor
import ASE.Compiler.Python.ListCompIf
import ASE.Compiler.Python.Mod
import ASE.Compiler.Python.Module
import ASE.Compiler.Python.Mul
import ASE.Compiler.Python.Name
import ASE.Compiler.Python.Not
import ASE.Compiler.Python.Or
import ASE.Compiler.Python.Pass
import ASE.Compiler.Python.Power
import ASE.Compiler.Python.Print
import ASE.Compiler.Python.Printnl
import ASE.Compiler.Python.PyNone
import ASE.Compiler.Python.Raise
import ASE.Compiler.Python.Return
import ASE.Compiler.Python.RightShift
import ASE.Compiler.Python.Slice
import ASE.Compiler.Python.Stmt
import ASE.Compiler.Python.Sub
import ASE.Compiler.Python.Subscript
import ASE.Compiler.Python.TryExcept
import ASE.Compiler.Python.TryFinally
import ASE.Compiler.Python.Tuple
import ASE.Compiler.Python.UnaryAdd
import ASE.Compiler.Python.UnarySub
import ASE.Compiler.Python.Vertex
import ASE.Compiler.Python.While
import ASE.Compiler.Python.Yield
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class Printer:
  '''
  This class prints a Python AST to standard output as Python source
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    import sys
    self.tab = '  '
    self.indentSize = 0
    self.output = sys.stdout
  
  def getTab(self):
    '''
    Retrieve the tab string
    '''
    return self.tab
  
  def setTab(self, tab):
    '''
    Set the tab string
    '''
    self.tab = tab
  
  def getIndentSize(self):
    '''
    Retrieve the number of tabs of the current indent
    '''
    return self.indentSize
  
  def setIndentSize(self, indentSize):
    '''
    Set the number of tabs of the current indent
    Throws an exception on a negative indent size
    '''
    if self.indentSize < 0:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Indent size must be positive: ' + str(self.indentSize))
      raise e
    self.indentSize = indentSize
  
  def increaseIndent(self):
    '''
    Increase the index size by one
    '''
    self.indentSize += 1
  
  def decreaseIndent(self):
    '''
    Decrease the index size by one
    Throws an exception on a negative indent size
    '''
    self.indentSize -= 1
    if self.indentSize < 0:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Indent size decreased below zero: ' + str(self.indentSize))
      raise e
  
  def indent(self):
    '''
    Output the current indent (the indent size in tabs)
    '''
    for i in range(self.getIndentSize()):
      self.__IORself.write(self.getTab())
    return None
  
  def write(self, s):
    '''
    Output a string
    '''
    self.output.write(s)
  
  def println(self, s):
    '''
    Output a string, properly indented and followed by a newline
    '''
    self.__IORself.indent()
    self.__IORself.write(s)
    self.__IORself.write('\n')
  
  def visitAdd(self, node):
    self.visitNaryExpression(node)
  
  def visitAnd(self, node):
    self.visitNaryExpression(node)
  
  def visitAssAttr(self, node):
    flags = node.getFlags()
    if flags == 'OP_DELETE':
      self.__IORself.indent()
    self.printFlags(flags)
    self.__IORself.visitVertex(node)
    self.__IORself.write('.' + node.getIdentifier())
    if flags == 'OP_DELETE':
      self.__IORself.write('\n')
  
  def visitAssList(self, node):
    self.visitList(node)
  
  def visitAssName(self, node):
    flags = node.getFlags()
    if flags == 'OP_DELETE':
      self.__IORself.indent()
    self.printFlags(flags)
    self.__IORself.write(node.getIdentifier())
    if flags == 'OP_DELETE':
      self.__IORself.write('\n')
  
  def visitAssTuple(self, node):
    self.visitTuple(node)
  
  def visitAssign(self, node):
    self.indent()
    for target in node.getChildren():
      target.accept(self.__IORself)
      self.__IORself.write(' = ')
    node.getValue().accept(self.__IORself)
    self.__IORself.write('\n')
  
  def visitAugAssign(self, node):
    self.indent()
    for target in node.getChildren():
      target.accept(self.__IORself)
      self.__IORself.write(' ' + node.getOp() + ' ')
    node.getValue().accept(self.__IORself)
    self.__IORself.write('\n')
  
  def visitBitand(self, node):
    self.visitBinaryExpression(node)
  
  def visitBitor(self, node):
    self.visitBinaryExpression(node)
  
  def visitBitxor(self, node):
    self.visitBinaryExpression(node)
  
  def visitBreak(self, node):
    self.__IORself.println('break')
  
  def visitCallFunc(self, node):
    self.__IORself.visitVertex(node)
    self.__IORself.write('(')
    for (a, arg) in enumerate(node.getArguments()):
      if a > 0:
        self.__IORself.write(', ')
      arg.accept(self.__IORself)
    if not node.getOptionalArguments() is None:
      self.__IORself.write(', *')
      node.getOptionalArguments().accept(self.__IORself)
    if not node.getKeywordArguments() is None:
      self.__IORself.write(', *')
      node.getKeywordArguments().accept(self.__IORself)
    self.__IORself.write(')')
  
  def visitClass(self, node):
    parents = ''
    if len(node.getParents()):
      parents = '(' + ','.join([parent.getIdentifier() for parent in node.getParents()]) + ')'
    self.__IORself.println('')
    self.__IORself.println('class ' + node.getIdentifier() + parents + ':')
    self.increaseIndent()
    self.printDocComment(node)
    self.__IORself.visitVertex(node)
    self.decreaseIndent()
  
  def visitCompare(self, node):
    self.__IORself.visitVertex(node)
    for op in node.getOps():
      self.__IORself.write(' ' + op + ' ')
      node.getTestExpr(op).accept(self.__IORself)
  
  def visitConst(self, node):
    self.__IORself.write(node.getValue_string())
  
  def visitPyNone(self, node):
    self.__IORself.write('None')
  
  def visitContinue(self, node):
    self.__IORself.println('continue')
  
  def visitDict(self, node):
    self.__IORself.write('{')
    for (c, child) in enumerate(node.getChildren()):
      if c > 0:
        if c % 2:
          self.__IORself.write(': ')
        else:
          self.__IORself.write(',\n')
          self.__IORself.indent()
          self.__IORself.write('  ')
      child.accept(self.__IORself)
    self.__IORself.write('}\n')
  
  def visitDiscard(self, node):
    self.__IORself.indent()
    self.__IORself.visitVertex(node)
    self.__IORself.write('\n')
  
  def visitDiv(self, node):
    self.visitBinaryExpression(node)
  
  def visitEllipsis(self, node):
    self.__IORself.write('...')
  
  def visitFor(self, node):
    self.__IORself.indent()
    self.__IORself.write('for ')
    node.getVar().accept(self.__IORself)
    self.__IORself.write(' in ')
    node.getList().accept(self.__IORself)
    self.__IORself.write(':\n')
    self.__IORself.increaseIndent()
    self.__IORself.visitVertex(node)
    self.__IORself.decreaseIndent()
  
  def visitListCompFor(self, node):
    self.__IORself.write('for ')
    node.getVar().accept(self.__IORself)
    self.__IORself.write(' in ')
    node.getList().accept(self.__IORself)
    self.__IORself.visitVertex(node)
  
  def visitFunction(self, node):
    self.printFlags(node.getFlags())
    arguments = node.getArgumentNames()
    defaults = node.getDefaults()
    numArgs = len(arguments) - len(defaults)
    self.__IORself.println('')
    self.__IORself.indent()
    self.__IORself.write('def ' + node.getIdentifier() + '(')
    for (a, arg) in enumerate(arguments[:numArgs]):
      if a > 0:
        self.__IORself.write(', ')
      self.__IORself.write(arg)
    for (a, arg) in enumerate(zip(arguments[numArgs:], defaults)):
      if a > 0 or numArgs:
        self.__IORself.write(', ')
      self.__IORself.write(arg[0])
      self.__IORself.write(' = ')
      arg[1].accept(self.__IORself)
    self.__IORself.write('):\n')
    self.increaseIndent()
    self.printDocComment(node)
    self.__IORself.visitVertex(node)
    self.decreaseIndent()
  
  def visitLambda(self, node):
    arguments = node.getArgumentNames()
    defaults = node.getDefaults()
    numArgs = len(arguments) - len(defaults)
    self.__IORself.write('lambda ')
    for (a, arg) in enumerate(arguments[:numArgs]):
      if a > 0:
        self.__IORself.write(', ')
      self.__IORself.write(arg)
    for (a, arg) in enumerate(zip(arguments[numArgs:], defaults)):
      if a > 0 or numArgs:
        self.__IORself.write(', ')
      self.__IORself.write(arg[0])
      self.__IORself.write(' = ')
      arg[1].accept(self.__IORself)
    self.__IORself.write(': ')
    self.__IORself.visitVertex(node)
  
  def visitGetAttr(self, node):
    self.__IORself.visitVertex(node)
    self.__IORself.write('.' + node.getName())
  
  def visitGlobal(self, node):
    self.__IORself.println('global ' + ','.join(node.getNames()))
  
  def visitIf(self, node):
    children = node.getChildren()
    for i in range(len(children) / 2):
      self.__IORself.indent()
      if i == 0:
        self.__IORself.write('if ')
      else:
        self.__IORself.write('elif ')
      children[i * 2].accept(self.__IORself)
      self.__IORself.write(':\n')
      self.increaseIndent()
      children[i * 2 + 1].accept(self.__IORself)
      self.decreaseIndent()
    if len(children) % 2 == 1:
      self.__IORself.println('else:')
      self.increaseIndent()
      children[-1].accept(self.__IORself)
      self.decreaseIndent()
  
  def visitListCompIf(self, node):
    self.__IORself.write(' if ')
    self.__IORself.visitVertex(node)
  
  def visitImport(self, node):
    self.__IORself.indent()
    self.__IORself.write('import ')
    for (n, name) in enumerate(node.getModuleNames()):
      if n > 0:
        self.__IORself.write(', ')
      if node.getAlias(name):
        self.__IORself.write(name + ' as ' + node.getAlias(name))
      else:
        self.__IORself.write(name)
    self.__IORself.write('\n')
  
  def visitFrom(self, node):
    self.__IORself.indent()
    self.__IORself.write('from ' + node.getModuleNames()[0] + ' import ')
    for (n, name) in enumerate(node.getImportedNames()):
      if n > 0:
        self.__IORself.write(', ')
      if node.getAlias(name):
        self.__IORself.write(name + ' as ' + node.getAlias(name))
      else:
        self.__IORself.write(name)
    self.__IORself.write('\n')
  
  def visitInvert(self, node):
    self.visitUnaryExpression(node)
  
  def visitKeyword(self, node):
    self.__IORself.write(node.getIdentifier())
    self.__IORself.write(' = ')
    self.__IORself.visitVertex(node)
  
  def visitLeftShift(self, node):
    self.visitBinaryExpression(node)
  
  def visitList(self, node):
    self.__IORself.write('[')
    for (c, child) in enumerate(node.getChildren()):
      if c > 0:
        self.__IORself.write(', ')
      child.accept(self.__IORself)
    self.__IORself.write(']')
  
  def visitListComp(self, node):
    self.__IORself.write('[')
    self.__IORself.visitVertex(node)
    for qualifier in node.getQualifiers():
      self.__IORself.write(' ')
      qualifier.accept(self.__IORself)
    self.__IORself.write(']')
  
  def visitMod(self, node):
    self.visitBinaryExpression(node)
  
  def visitModule(self, node):
    self.printDocComment(node)
    self.__IORself.visitVertex(node)
  
  def visitMul(self, node):
    self.visitBinaryExpression(node)
  
  def visitName(self, node):
    self.__IORself.write(node.getIdentifier())
  
  def visitNot(self, node):
    self.visitUnaryExpression(node)
  
  def visitOr(self, node):
    self.visitNaryExpression(node)
  
  def visitPass(self, node):
    self.writeComments(node, newline = 1)
    self.__IORself.println('pass')
  
  def visitPower(self, node):
    self.visitBinaryExpression(node)
  
  def visitPrint(self, node):
    self.__IORself.indent()
    self.__IORself.write('print ')
    for child in node.getChildren():
      child.accept(self.__IORself)
      self.__IORself.write(', ')
    self.__IORself.write('\n')
  
  def visitPrintnl(self, node):
    self.__IORself.indent()
    self.__IORself.write('print ')
    for (c, child) in enumerate(node.getChildren()):
      if c > 0:
        self.__IORself.write(', ')
      child.accept(self.__IORself)
    self.__IORself.write('\n')
  
  def visitRaise(self, node):
    self.__IORself.indent()
    self.__IORself.write('raise ')
    for (c, child) in enumerate(node.getChildren()):
      if c > 0:
        self.__IORself.write(', ')
      child.accept(self.__IORself)
    self.__IORself.write('\n')
  
  def visitReturn(self, node):
    self.__IORself.indent()
    self.__IORself.write('return ')
    if len(node.getChildren()) == 1 and node.getChildren()[0].isInstanceOf('ASE.Compiler.Python.PyNone'):
      pass
    else:
      self.__IORself.visitVertex(node)
    self.__IORself.write('\n')
  
  def visitRightShift(self, node):
    self.visitBinaryExpression(node)
  
  def visitSlice(self, node):
    flags = node.getFlags()
    if flags == 'OP_DELETE':
      self.__IORself.indent()
    self.printFlags(flags)
    self.__IORself.visitVertex(node)
    self.__IORself.write('[')
    if not node.getLower() is None:
      node.getLower().accept(self.__IORself)
    self.__IORself.write(':')
    if not node.getUpper() is None:
      node.getUpper().accept(self.__IORself)
    self.__IORself.write(']')
    if flags == 'OP_DELETE':
      self.__IORself.write('\n')
  
  def visitStmt(self, node):
    self.__IORself.visitVertex(node)
  
  def visitSub(self, node):
    self.visitBinaryExpression(node)
  
  def visitSubscript(self, node):
    flags = node.getFlags()
    if flags == 'OP_DELETE':
      self.__IORself.indent()
    self.printFlags(flags)
    self.__IORself.visitVertex(node)
    self.__IORself.write('[')
    for (s, subscript) in enumerate(node.getSubscripts()):
      if s > 0:
        self.__IORself.write(', ')
      subscript.accept(self.__IORself)
    self.__IORself.write(']')
    if flags == 'OP_DELETE':
      self.__IORself.write('\n')
  
  def visitTryExcept(self, node):
    self.__IORself.println('try:')
    self.increaseIndent()
    self.__IORself.visitVertex(node)
    self.decreaseIndent()
    for handler in node.getHandlers():
      self.__IORself.indent()
      self.__IORself.write('except')
      if not handler.getType() is None:
        self.__IORself.write(' ')
        handler.getType().accept(self.__IORself)
      if not handler.getVariable() is None:
        self.__IORself.write(', ')
        handler.getVariable().accept(self.__IORself)
      self.__IORself.write(':\n')
      self.increaseIndent()
      self.__IORself.visitVertex(handler)
      self.decreaseIndent()
  
  def visitTryFinally(self, node):
    self.__IORself.println('try:')
    self.increaseIndent()
    self.__IORself.visitVertex(node)
    self.decreaseIndent()
    self.__IORself.println('finally:')
    self.increaseIndent()
    node.getFinal().accept(self.__IORself)
    self.decreaseIndent()
  
  def visitTuple(self, node):
    self.__IORself.write('(')
    children = node.getChildren()
    for c in range(len(children)):
      if c > 0:
        self.__IORself.write(', ')
      children[c].accept(self.__IORself)
      if len(children) == 1:
        self.__IORself.write(', ')
    self.__IORself.write(')')
  
  def visitUnaryAdd(self, node):
    self.visitUnaryExpression(node)
  
  def visitUnarySub(self, node):
    self.visitUnaryExpression(node)
  
  def visitWhile(self, node):
    self.__IORself.indent()
    self.__IORself.write('while ')
    node.getTest().accept(self.__IORself)
    self.__IORself.write(':\n')
    self.increaseIndent()
    self.__IORself.visitVertex(node)
    self.decreaseIndent()
  
  def visitYield(self, node):
    self.__IORself.indent()
    self.__IORself.write('yield ')
    self.__IORself.visitVertex(node)
    self.__IORself.write('\n')
  
  def printDocComment(self, node):
    comments = node.getComments()
    if len(comments):
      self.__IORself.println('\'\'\'')
      for c in comments:
        self.__IORself.println(self.escapeString(c.getComment().strip()))
      self.__IORself.println('\'\'\'')
  # Ending class methods section
  pass
  
  def escapeString(self, line):
    '''
    We only escape single quotes, since we are always single-quoting and double quotes are harmless
    '''
    return line.replace('\'', '\\\'')
  
  def writeComments(self, vertex, newline = 0):
    for comment in vertex.getComments():
      if newline:
        [self.__IORself.println('# ' + self.escapeString(line)) for line in comment.getComment().split('\n')]
      else:
        self.__IORself.write('# ' + self.escapeString(comment.getComment()))
    return None
  
  def visitUnaryExpression(self, vertex):
    children = vertex.getChildren()
    if not len(children) == 1:
      e = ASE.Compiler.CompilerException.CompilerException()
      e.setMessage('Unary expressions must have exactly one child')
      raise e
    self.__IORself.write(vertex.getIdentifier())
    if vertex.getIdentifier().isalpha():
      self.__IORself.write(' ')
    children[0].accept(self.__IORself)
    return None
  
  def visitBinaryExpression(self, vertex):
    children = vertex.getChildren()
    if not len(children) == 2:
      e = ASE.Compiler.CompilerException.CompilerException()
      e.setMessage('Binary expressions must have exactly two children')
      raise e
    children[0].accept(self.__IORself)
    self.__IORself.write(' ' + vertex.getIdentifier() + ' ')
    children[1].accept(self.__IORself)
    return None
  
  def visitNaryExpression(self, vertex):
    children = vertex.getChildren()
    if len(children) < 2:
      e = ASE.Compiler.CompilerException.CompilerException()
      e.setMessage('Nary expressions must have at least two children')
      raise e
    children[0].accept(self.__IORself)
    for child in children[1:]:
      self.__IORself.write(' ' + vertex.getIdentifier() + ' ')
      child.accept(self.__IORself)
    return None
  
  def printFlags(self, flags):
    if flags == 'OP_DELETE':
      self.__IORself.write('del ')
