import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Type

class SIDL(object):
  def __init__(self):
    self.typeMap = self.createTypeMap()
    return None
  
  def createTypeMap(self):
    '''
    Notice that array functions take integer array arguments, and thus all types depend on int
    '''
    typeMap = {}

    for typeName in ['int', 'bool', 'char', 'dcomplex', 'double', 'fcomplex', 'float', 'long', 'opaque', 'string', 'void']:
      sidlType = ASE.Compiler.SIDL.Type.Type()
      sidlType.setIdentifier(typeName)
      sidlType.setBaseType(1)
      if not typeName == 'int':
        sidlType.setDependencies([typeMap['int']])
      typeMap[typeName] = sidlType
    return typeMap
  
  def getLoader(self, interface):
    for child in self.getBaseInterface(interface).getParent().getChildren():
      if child.getIdentifier() == 'Loader':
        return ASE.Compiler.SIDL.Class.Class(child)
    raise RuntimeError('Could not find Loader using ' + interface.getFullIdentifier())
  
  def getBasePackage(self, interface):
    return ASE.Compiler.SIDL.Package.Package(self.getAncestor(interface, 'ASE.BaseInterface').getParent())
  
  def getBaseParent(self, interface):
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      return self.getAncestor(interface, 'ASE.BaseClass')
    return self.getBaseInterface(interface)
  
  def getBaseInterface(self, interface):
    return self.getAncestor(interface, 'ASE.BaseInterface')
  
  def getBaseClass(self, interface):
    package = self.getBasePackage(interface)
    for child in package.getChildren():
      if child.getIdentifier() == 'BaseClass':
        return ASE.Compiler.SIDL.Class.Class(child)
    return None
  
  def getBaseException(self, exception):
    return self.getAncestor(exception, 'ASE.BaseException')
  
  def getASEException(self, interface):
    asePackage = self.getBasePackage(interface)
    for child in asePackage.getChildren():
      if child.getIdentifier() == 'ASEException':
        return ASE.Compiler.SIDL.Class.Class(child)
    return None
  
  def getAncestor(self, interface, typeName):
    interface = ASE.Compiler.SIDL.Interface.Interface(interface)
    if interface.getFullIdentifier() == typeName:
      return interface
    parents = []
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      parents.extend(ASE.Compiler.SIDL.Class.Class(interface).getInterfaces())
    parents.extend(interface.getParents())
    for parent in parents:
      base = self.getAncestor(parent, typeName)
      if base:
        return base
    return None
