# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.SkeletonGenerator
import ASE.Compiler.Vertex
# Ending imports section
pass
import ASE.Compiler.Python.Python
import ASE.Compiler.Python.SIDL
# Starting static methods section
pass
# Ending static methods section
pass

class SkeletonGenerator:
  '''
  This visitor transforms a SIDL AST into a set of Python Stub ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.packages = [[]]
    self.sourceLanguages = ['Cxx']
    self.Skeleton = Skeleton()
    self.Python = ASE.Compiler.Python.Python.Python()
    self.IOR = self.Python.IOR
    self.SIDL = self.Python.SIDL
    self.Cxx = self.Python.Cxx
    self.pyObjType = self.Python.getObjectType()
    self.constructors = {}

    self.destructors = {}

    self.templateMethods = {}

  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not language in self.source:
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not language in self.source:
      self.source[language] = []
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not language in self.source:
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not language in self.source:
      self.source[language] = []
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitClass(self, node):
    import ASE.Compiler.SIDL.TypeUniverse
    import ASE.Compiler.Python.Const
    name = ASE.Compiler.Python.Const.Const()
    name.setValue_string('\'' + node.getIdentifier() + '\'')
    self.packageAppend(name)
    defns = []
    defns.extend([self.Python.getRefTypeEnumeration(), self.Python.getSIDLObject(node)])
    defns.extend([self.Python.getNew(node)])
    wrapIORFuncs = {}

    extractIORFuncs = {}

    wrapArrayFuncs = {}

    extractArrayFuncs = {}

    for method in list(node.getMethods()) + [self.getConstructorMethod(node), self.getDestructorMethod(node)]:
      for type in [parameter.getType() for parameter in method.getInParameters() if parameter.getType().getBaseType()]:
        if type.getIdentifier() == 'opaque':
          wrapIORFuncs[type] = self.Python.getWrapOpaque(type)
      for type in [parameter.getType() for parameter in method.getInParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Interface')]:
        wrapIORFuncs[type] = self.Python.getWrapIOR(type)
      for type in [parameter.getType() for parameter in method.getOutParameters() if parameter.getType().getBaseType()]:
        if type.getIdentifier() == 'opaque':
          extractIORFuncs[type] = self.Python.getExtractOpaque(type)
      for type in [parameter.getType() for parameter in method.getOutParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Interface')]:
        extractIORFuncs[type] = self.Python.getExtractIOR(type, node)
      for type in [parameter.getType() for parameter in method.getInParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array')]:
        entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
        if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
          for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
            wrapArrayFuncs[t] = self.Python.getWrapArray(t)
        else:
          wrapArrayFuncs[entryType] = self.Python.getWrapArray(entryType)
        if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          wrapIORFuncs[entryType] = self.Python.getWrapIOR(entryType)
      for type in [parameter.getType() for parameter in method.getOutParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array')]:
        entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
        if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
          for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
            extractArrayFuncs[t] = self.Python.getExtractArray(t, node)
        else:
          extractArrayFuncs[entryType] = self.Python.getExtractArray(entryType, node)
        if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          extractIORFuncs[entryType] = self.Python.getExtractIOR(entryType, node)
    defns.extend(wrapIORFuncs.values())
    defns.extend(extractIORFuncs.values())
    for (entryType, func) in wrapArrayFuncs.items():
      defns.extend([self.IOR.getGuardedInclude(entryType), func])
    for (entryType, func) in extractArrayFuncs.items():
      defns.extend([self.IOR.getGuardedInclude(entryType), func])
    [self.createTemplateMethod(method) for method in node.getImplementedMethods()]
    for method in self.getTemplateMethods(node.getImplementedMethods()):
      defns.extend(self.getMethodSkeleton(node, method))
    defns.extend(self.getConstructorSkeleton(node))
    defns.extend(self.getDestructorSkeleton(node))
    defns.extend(self.getSetEpvSkeleton(node))
    defns.extend(self.getSetSepvSkeleton(node))
    self.addSource('Cxx', [self.getSourceFile(node, defns)])
  # Ending class methods section
  pass
  
  def hasStaticMethod(self, type):
    '''
    We could use  if method.getStatic() and not method.getInternal():
    '''
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      for method in type.getMethods():
        if method.getStatic():
          return 1
    return 0
  
  def pushPackage(self):
    return self.packages.append([])
  
  def popPackage(self):
    return self.packages.pop()
  
  def packageAppend(self, name):
    return self.packages[-1].append(name)
  
  def isOut(self, parameter):
    import ASE.Compiler.ParameterAccessMode
    mode = parameter.getAccessMode()
    return mode == ASE.Compiler.ParameterAccessMode.OUT or mode == ASE.Compiler.ParameterAccessMode.INOUT or mode == ASE.Compiler.ParameterAccessMode.RETURN
  
  def getSourceFile(self, klass, decls):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Source
    import operator
    import sets
    source = ASE.Compiler.Cxx.Source.Source()
    source.setFilename(self.Skeleton.getFileBasename(klass) + '.c')
    pyInclude = ASE.Compiler.Cxx.Include.Include()
    pyInclude.setIdentifier('<Python.h>')
    numericInclude = ASE.Compiler.Cxx.Include.Include()
    numericInclude.setIdentifier('<Numeric/arrayobject.h>')
    source.addChildren([pyInclude, numericInclude, self.IOR.getGuardedInclude(klass)])
    source.addChildren([self.IOR.getGuardedInclude(self.SIDL.getLoader(klass)), self.IOR.getGuardedInclude(self.SIDL.getASEException(klass))])
    source.addChildren([self.IOR.getGuardedInclude(exc) for exc in sets.Set(reduce(operator.concat, [list(method.getExceptions()) for method in klass.getImplementedMethods()], []))])
    source.addChildren(decls)
    source.setPurpose(ASE.Compiler.CodePurpose.SKELETON)
    return source
  
  def getMethodReturn(self, klass, method, body):
    '''
    This method:
       - Declares the Python and IOR return values
       - Declares local arguments, builds format, and adds any conversion code
       - Parses and converts the local Python return value
       - Cleans up arguments
       It also contains special purpose destruction code
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.LessThanOrEqual
    import ASE.Compiler.Cxx.LogicalAnd
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    returnVar = self.Cxx.getVar('_return_value')
    pyReturnVar = self.Cxx.getVar('_pyReturn')
    body.addDeclarations([self.Cxx.getDeclaration(pyReturnVar, self.pyObjType)])
    format = ''
    localArgs = []
    for parameter in method.getOutParameters():
      format += self.Python.getParseFormat(parameter)
      localArgs.extend(self.Python.getParseArguments(parameter, isArgument = not parameter.getAccessMode() == ASE.Compiler.ParameterAccessMode.RETURN))
    if len(method.getOutParameters()):
      if len(method.getOutParameters()) == 1:
        tupleVar = self.Cxx.getVar('returnTuple')
        createTuple = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        createTuple.setChildren([self.Cxx.getVar('PyTuple_New')])
        createTuple.setArguments([self.Cxx.getInteger(1)])
        setTuple = ASE.Compiler.Cxx.Assignment.Assignment()
        setTuple.setChildren([tupleVar, createTuple])
        addReturn = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        addReturn.setChildren([self.Cxx.getVar('PyTuple_SetItem')])
        addReturn.setArguments([tupleVar, self.Cxx.getInteger(0), pyReturnVar])
        body.addDeclarations([self.Cxx.getDeclaration(tupleVar, self.pyObjType)])
        body.addChildren([self.Cxx.getExpStmt(setTuple), self.Cxx.getExpStmt(addReturn)])
      else:
        tupleVar = pyReturnVar
      checkVar = self.Cxx.getVar('_okay')
      parseCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      parseCall.setChildren([self.Cxx.getVar('PyArg_ParseTuple')])
      parseCall.setArguments([tupleVar, self.Cxx.castToString(format)] + localArgs)
      parseExp = ASE.Compiler.Cxx.Assignment.Assignment()
      parseExp.setChildren([checkVar, parseCall])
      errorMsg = 'Could not parse return value for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
      body.addChildren([self.Cxx.getExpStmt(parseExp, 'Parse arguments'), self.Python.getClearError(), self.IOR.getErrorCheck(klass, method, checkVar, errorMsg)])
      if len(method.getOutParameters()) == 1:
        decrefTuple = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        decrefTuple.setChildren([self.Cxx.getVar('Py_DECREF')])
        decrefTuple.setArguments([tupleVar])
        body.addChildren([self.Cxx.getExpStmt(decrefTuple, 'Cleanup local return value')])
      else:
        notNone = ASE.Compiler.Cxx.Inequality.Inequality()
        notNone.setChildren([pyReturnVar, self.Cxx.getVar('Py_None')])
        bothChecks = ASE.Compiler.Cxx.LogicalAnd.LogicalAnd()
        bothChecks.setChildren([pyReturnVar, self.Cxx.getGroup(notNone)])
        decrefReturn = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        decrefReturn.setChildren([self.Cxx.getVar('Py_DECREF')])
        decrefReturn.setArguments([pyReturnVar])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren([self.Cxx.getExpStmt(decrefReturn)])
        checkReturn = ASE.Compiler.Cxx.If.If()
        checkReturn.setBranch(bothChecks)
        checkReturn.setChildren([compound])
        checkReturn.setComments([self.Cxx.getComment('Cleanup local return value')])
        body.addChildren([checkReturn])
    else:
      isNone = ASE.Compiler.Cxx.Equality.Equality()
      isNone.setChildren([pyReturnVar, self.Cxx.getVar('Py_None')])
      notNone = ASE.Compiler.Cxx.Negation.Negation()
      notNone.setChildren([self.Cxx.getGroup(isNone)])
      errorMsg = 'Invalid return from void method ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, errorMsg, notNone)])
    for parameter in method.getOutParameters():
      self.Python.getConvertParseArgument(parameter, body, isArgument = not parameter.getAccessMode() == ASE.Compiler.ParameterAccessMode.RETURN)
    decrefArgs = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefArgs.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefArgs.setArguments([self.Cxx.getVar('_args')])
    body.addChildren([self.Cxx.getExpStmt(decrefArgs, 'Cleanup arguments')])
    for parameter in method.getInParameters():
      self.Python.getCleanupBuildParameter(parameter, body, isArgument = 1)
    if klass.getFullIdentifier() == 'ASE.BaseClass' and method.getFullIdentifier() == 'ASE.BaseInterface.deleteReference':
      selfEpv = ASE.Compiler.Cxx.StructReference.StructReference()
      selfEpv.setChildren([self.Cxx.getVar('_self')])
      selfEpv.setMember('d_epv')
      selfEpv.setPointer(1)
      refCountFunc = ASE.Compiler.Cxx.StructReference.StructReference()
      refCountFunc.setChildren([selfEpv])
      refCountFunc.setMember('getReferenceCount')
      refCountFunc.setPointer(1)
      refCountCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      refCountCall.setChildren([refCountFunc])
      refCountCall.setArguments([self.Cxx.getVar('_self'), self.Cxx.getVar('_exception')])
      isCountZero = ASE.Compiler.Cxx.LessThanOrEqual.LessThanOrEqual()
      isCountZero.setChildren([refCountCall, self.Cxx.getInteger(0)])
      destroyFunc = ASE.Compiler.Cxx.StructReference.StructReference()
      destroyFunc.setChildren([selfEpv])
      destroyFunc.setMember('destroyObject')
      destroyFunc.setPointer(1)
      destroyCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      destroyCall.setChildren([destroyFunc])
      destroyCall.setArguments([self.Cxx.getVar('_self'), self.Cxx.getVar('_exception')])
      compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound.setChildren([self.Cxx.getExpStmt(destroyCall)])
      checkRefCount = ASE.Compiler.Cxx.If.If()
      checkRefCount.setBranch(isCountZero)
      checkRefCount.setChildren([compound])
      checkRefCount.setComments([self.Cxx.getComment('Cleanup implementation')])
      body.addChildren([checkRefCount, self.Python.getRestoreError()])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    if not method.getReturnParameter() is None:
      returnStmt.setChildren([returnVar])
    body.addChildren([returnStmt])
    return None
  
  def copyPythonTraceback(self, excVar, tbVar):
    import ASE.Compiler.Cxx.And
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.While
    codeVar = self.Cxx.getVar('code')
    getAttrVar = self.Cxx.getVar('PyObject_GetAttrString')
    getStringVar = self.Cxx.getVar('PyString_AsString')
    tbNotNone = ASE.Compiler.Cxx.Inequality.Inequality()
    tbNotNone.setChildren([tbVar, self.Cxx.getVar('Py_None')])
    tbValid = ASE.Compiler.Cxx.LogicalAnd.LogicalAnd()
    tbValid.setChildren([tbVar, self.Cxx.getGroup(tbNotNone)])
    getFrame = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getFrame.setChildren([getAttrVar])
    getFrame.setArguments([tbVar, self.Cxx.getString('tb_frame')])
    getCode = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getCode.setChildren([getAttrVar])
    getCode.setArguments([getFrame, self.Cxx.getString('f_code')])
    codeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    codeInit.setChildren([getCode])
    excVarInd = ASE.Compiler.Cxx.Indirection.Indirection()
    excVarInd.setChildren([excVar])
    excVarObj = ASE.Compiler.Cxx.StructReference.StructReference()
    excVarObj.setChildren([self.Cxx.getGroup(excVarInd)])
    excVarObj.setMember('d_object')
    excVarObj.setPointer(1)
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([self.Cxx.getGroup(excVarInd)])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    addTraceFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    addTraceFunc.setChildren([excEpv])
    addTraceFunc.setMember('addStackEntry')
    addTraceFunc.setPointer(1)
    filenameCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    filenameCall.setChildren([getAttrVar])
    filenameCall.setArguments([codeVar, self.Cxx.getString('co_filename')])
    getFilename = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getFilename.setChildren([getStringVar])
    getFilename.setArguments([filenameCall])
    linenoCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    linenoCall.setChildren([getAttrVar])
    linenoCall.setArguments([tbVar, self.Cxx.getString('tb_lineno')])
    getLineno = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getLineno.setChildren([self.Cxx.getVar('PyInt_AsLong')])
    getLineno.setArguments([linenoCall])
    nameCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    nameCall.setChildren([getAttrVar])
    nameCall.setArguments([codeVar, self.Cxx.getString('co_name')])
    getName = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getName.setChildren([getStringVar])
    getName.setArguments([nameCall])
    addTrace = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    addTrace.setChildren([addTraceFunc])
    addTrace.setArguments([excVarObj, getFilename, getLineno, getName, self.Cxx.getNull()])
    getNextTb = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getNextTb.setChildren([getAttrVar])
    getNextTb.setArguments([tbVar, self.Cxx.getString('tb_next')])
    incTrace = ASE.Compiler.Cxx.Assignment.Assignment()
    incTrace.setChildren([tbVar, getNextTb])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(codeVar, self.pyObjType, codeInit)])
    compound.setChildren([self.Cxx.getExpStmt(addTrace), self.Cxx.getExpStmt(incTrace)])
    tbLoop = ASE.Compiler.Cxx.While.While()
    tbLoop.setBranch(tbValid)
    tbLoop.setChildren([compound])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([tbLoop])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    return checkExc
  
  def setException(self, excVar, localExcVar, localExcType):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    excInd = ASE.Compiler.Cxx.Indirection.Indirection()
    excInd.setChildren([excVar])
    setExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setExc.setChildren([excInd, self.Cxx.castToType(localExcVar, self.Python.getType(self.SIDL.getBaseException(localExcType)))])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(setExc)])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    return checkExc
  
  def throwDefaultException(self, klass, excVar, valVar):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    localExcVar = self.Cxx.getVar('_local_exception')
    excType = self.SIDL.getASEException(klass)
    extFunc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFunc.setChildren([self.Cxx.getVar(excType.getFullIdentifier().replace('.', '_') + '__externals')])
    createFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    createFunc.setChildren([extFunc])
    createFunc.setMember('createObject')
    createFunc.setPointer(1)
    createExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createExc.setChildren([createFunc])
    createExc.setArguments([self.Cxx.getNull()])
    setLocalExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setLocalExc.setChildren([localExcVar, createExc])
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([localExcVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    messageFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    messageFunc.setChildren([excEpv])
    messageFunc.setMember('setMessage')
    messageFunc.setPointer(1)
    excPyStr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    excPyStr.setChildren([self.Cxx.getVar('PyObject_Str')])
    excPyStr.setArguments([valVar])
    excStr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    excStr.setChildren([self.Cxx.getVar('PyString_AsString')])
    excStr.setArguments([excPyStr])
    setMessage = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setMessage.setChildren([messageFunc])
    setMessage.setArguments([localExcVar, excStr, self.Cxx.getNull()])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setDeclarations([self.Cxx.getDeclaration(localExcVar, self.Python.getType(excType))])
    compound.setChildren([self.Cxx.getExpStmt(setLocalExc), self.Cxx.getExpStmt(setMessage), self.setException(excVar, localExcVar, excType)])
    return compound
  
  def throwException(self, klass, exception, excVar, typeVar, valVar, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    moduleVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_module')
    pythonTypeVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')
    localExcVar = self.Cxx.getVar('_local_exception')
    sidlExcVar = self.Cxx.getVar('_sidlExc')
    isClassVar = self.Cxx.getVar('isClass')
    body.addDeclarations([self.Cxx.getDeclaration(moduleVar, self.pyObjType), self.Cxx.getDeclaration(pythonTypeVar, self.pyObjType)])
    moduleCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleCall.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
    moduleCall.setArguments([self.Cxx.castToString(exception.getFullIdentifier())])
    setModule = ASE.Compiler.Cxx.Assignment.Assignment()
    setModule.setChildren([moduleVar, moduleCall])
    moduleDict = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleDict.setChildren([self.Cxx.getVar('PyModule_GetDict')])
    moduleDict.setArguments([moduleVar])
    typeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    typeCall.setChildren([self.Cxx.getVar('PyMapping_GetItemString')])
    typeCall.setArguments([moduleDict, self.Cxx.castToString('Exception')])
    setType = ASE.Compiler.Cxx.Assignment.Assignment()
    setType.setChildren([pythonTypeVar, typeCall])
    decrefModule = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefModule.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefModule.setArguments([moduleVar])
    body.addChildren(map(self.Cxx.getExpStmt, [setModule, setType, decrefModule]))
    checkInstance = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    checkInstance.setChildren([self.Cxx.getVar('PyObject_IsInstance')])
    checkInstance.setArguments([valVar, pythonTypeVar])
    isClassInit = ASE.Compiler.Cxx.Initializer.Initializer()
    isClassInit.setChildren([checkInstance])
    checkType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    checkType.setChildren([self.Cxx.getVar('PyErr_GivenExceptionMatches')])
    checkType.setArguments([typeVar, pythonTypeVar])
    getExcArgs = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getExcArgs.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    getExcArgs.setArguments([valVar, self.Cxx.castToString('args')])
    getSidlExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getSidlExc.setChildren([self.Cxx.getVar('PySequence_GetItem')])
    getSidlExc.setArguments([getExcArgs, self.Cxx.getInteger(0)])
    setSidlExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setSidlExc.setChildren([sidlExcVar, self.Cxx.castToType(getSidlExc, self.Python.getSIDLObjectType(klass))])
    trueBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    trueBranch.setChildren([self.Cxx.getExpStmt(setSidlExc)])
    setSidlExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setSidlExc.setChildren([sidlExcVar, self.Cxx.castToType(valVar, self.Python.getSIDLObjectType(klass))])
    falseBranch = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    falseBranch.setChildren([self.Cxx.getExpStmt(setSidlExc)])
    checkValue = ASE.Compiler.Cxx.If.If()
    checkValue.setBranch(isClassVar)
    checkValue.setChildren([trueBranch, falseBranch])
    increfValue = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfValue.setChildren([self.Cxx.getVar('PyObject_CallMethod')])
    increfValue.setArguments([self.Cxx.castToType(sidlExcVar, self.pyObjType), self.Cxx.castToString('addReference'), self.Cxx.castToString('')])
    valIor = ASE.Compiler.Cxx.StructReference.StructReference()
    valIor.setChildren([sidlExcVar])
    valIor.setMember('ior')
    valIor.setPointer(1)
    setExc = ASE.Compiler.Cxx.Assignment.Assignment()
    setExc.setChildren([localExcVar, self.Cxx.castToType(valIor, self.Python.getType(exception))])
    localEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    localEpv.setChildren([localExcVar])
    localEpv.setMember('d_epv')
    localEpv.setPointer(1)
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([localEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    castFuncInd.setChildren([self.Cxx.getGroup(castFunc)])
    localObj = ASE.Compiler.Cxx.StructReference.StructReference()
    localObj.setChildren([localExcVar])
    localObj.setMember('d_object')
    localObj.setPointer(1)
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFuncInd)])
    castCall.setArguments([localObj, self.Cxx.getString(exception.getFullIdentifier()), self.Cxx.getNull()])
    castExc = ASE.Compiler.Cxx.Assignment.Assignment()
    castExc.setChildren([localExcVar, castCall])
    decrefSidlExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefSidlExc.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefSidlExc.setArguments([sidlExcVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(decrefSidlExc)])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(isClassVar)
    checkExc.setChildren([compound])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.addDeclarations([self.Cxx.getDeclaration(localExcVar, self.Python.getType(exception)), self.Cxx.getDeclaration(sidlExcVar, self.Python.getSIDLObjectType(klass)), self.Cxx.getDeclaration(isClassVar, self.Python.typeMap['bool'], isClassInit)])
    compound.addChildren([checkValue])
    compound.addChildren(map(self.Cxx.getExpStmt, [increfValue, setExc, castExc]))
    compound.addChildren([checkExc, self.setException(excVar, localExcVar, exception)])
    checkExcType = ASE.Compiler.Cxx.If.If()
    checkExcType.setBranch(checkType)
    checkExcType.setChildren([compound])
    return checkExcType
  
  def getMethodHandleExceptions(self, klass, method, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.Return
    excVar = self.Cxx.getVar('_exception')
    typeVar = self.Cxx.getVar('_excType')
    valVar = self.Cxx.getVar('_excValue')
    tbVar = self.Cxx.getVar('_excTraceback')
    excBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    excBody.addDeclarations([self.Cxx.getDeclaration(typeVar, self.pyObjType), self.Cxx.getDeclaration(valVar, self.pyObjType), self.Cxx.getDeclaration(tbVar, self.pyObjType)])
    fetchExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    fetchExc.setChildren([self.Cxx.getVar('PyErr_Fetch')])
    fetchExc.setArguments([self.Cxx.getAddress(typeVar), self.Cxx.getAddress(valVar), self.Cxx.getAddress(tbVar)])
    excBody.addChildren([self.Cxx.getExpStmt(fetchExc)])
    checkExcType = None
    for exception in method.getExceptions():
      check = self.throwException(klass, exception, excVar, typeVar, valVar, excBody)
      if checkExcType is None:
        checkExcType = check
        fullExcCheck = checkExcType
      else:
        checkExcType.addChildren([check])
        checkExcType = check
    checkExcType.addChildren([self.throwDefaultException(klass, excVar, valVar)])
    excBody.addChildren([fullExcCheck, self.copyPythonTraceback(excVar, tbVar)])
    for exception in method.getExceptions():
      decrefType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefType.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefType.setArguments([self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')])
      excBody.addChildren([self.Cxx.getExpStmt(decrefType)])
    decrefType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefType.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefType.setArguments([typeVar])
    decrefVal = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefVal.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefVal.setArguments([valVar])
    decrefTb = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefTb.setChildren([self.Cxx.getVar('Py_XDECREF')])
    decrefTb.setArguments([tbVar])
    tbNotNone = ASE.Compiler.Cxx.Inequality.Inequality()
    tbNotNone.setChildren([tbVar, self.Cxx.getVar('Py_None')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(decrefTb)])
    checkTb = ASE.Compiler.Cxx.If.If()
    checkTb.setBranch(tbNotNone)
    checkTb.setChildren([compound])
    returnStmt = ASE.Compiler.Cxx.Return.Return()
    if not method is None and not method.getReturnParameter() is None:
      returnStmt.setChildren([self.Cxx.getVar('_return_value')])
    excBody.addChildren([self.Cxx.getExpStmt(decrefType), self.Cxx.getExpStmt(decrefVal), checkTb, returnStmt])
    isErr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    isErr.setChildren([self.Cxx.getVar('PyErr_Occurred')])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(isErr)
    checkExc.setChildren([excBody])
    body.addChildren([checkExc])
    return None
  
  def getMethodDispatch(self, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    methodVar = self.Cxx.getVar('_method')
    argsVar = self.Cxx.getVar('_args')
    returnVar = self.Cxx.getVar('_pyReturn')
    callMethod = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    callMethod.setChildren([self.Cxx.getVar('PyObject_CallObject')])
    callMethod.setArguments([methodVar, argsVar])
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([returnVar, callMethod])
    decrefMethod = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefMethod.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefMethod.setArguments([methodVar])
    body.addChildren([self.Cxx.getExpStmt(setReturn, 'Dispatch to implementation'), self.Cxx.getExpStmt(decrefMethod)])
    return None
  
  def getMethodPackArguments(self, klass, method, body, isConstructor = 0):
    '''
    This method
       - Declares local arguments, builds format, and adds any conversion code
       - Converts arguments from IOR to Python
       - Builds Python arguments
    '''
    import ASE.Compiler.Cxx.Assignment
    argsVar = self.Cxx.getVar('_args')
    checkVar = self.Cxx.getVar('_okay')
    format = ''
    localArgs = []
    for parameter in method.getInParameters():
      format += self.Python.getParseFormat(parameter)
      localArgs.extend(self.Python.getBuildArguments(parameter, isArgument = 1))
    format = '(' + format + ')'
    for parameter in method.getInParameters():
      type = parameter.getType()
      var = self.Cxx.getVar(parameter.getIdentifier())
      self.Python.getConvertBuildParameter(parameter, body, isArgument = 1)
      if not isConstructor and type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        import ASE.Compiler.Cxx.CompoundStatement
        import ASE.Compiler.Cxx.FunctionCall
        import ASE.Compiler.Cxx.If
        import ASE.Compiler.Cxx.Indirection
        import ASE.Compiler.Cxx.StructReference
        objVar = self.Cxx.castToType(var, self.IOR.getType(self.SIDL.getBaseInterface(type)))
        iorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
        iorEpv.setChildren([self.Cxx.getGroup(objVar)])
        iorEpv.setMember('d_epv')
        iorEpv.setPointer(1)
        addRefFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
        addRefFuncPtr.setChildren([iorEpv])
        addRefFuncPtr.setMember('addReference')
        addRefFuncPtr.setPointer(1)
        addRefFunc = ASE.Compiler.Cxx.Indirection.Indirection()
        addRefFunc.setChildren([self.Cxx.getGroup(addRefFuncPtr)])
        iorObj = ASE.Compiler.Cxx.StructReference.StructReference()
        iorObj.setChildren([self.Cxx.getGroup(objVar)])
        iorObj.setMember('d_object')
        iorObj.setPointer(1)
        addRefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        addRefCall.setChildren([self.Cxx.getGroup(addRefFunc)])
        addRefCall.setArguments([iorObj, self.Cxx.getNull()])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren([self.Cxx.getExpStmt(addRefCall)])
        checkParam = ASE.Compiler.Cxx.If.If()
        checkParam.setBranch(var)
        checkParam.setChildren([compound])
        body.addChildren([checkParam])
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildCall.setArguments([self.Cxx.castToString(format)] + localArgs)
    setArgs = ASE.Compiler.Cxx.Assignment.Assignment()
    setArgs.setChildren([argsVar, buildCall])
    errorMsg = 'Unable to parse arguments for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addDeclarations([self.Cxx.getDeclaration(argsVar, self.pyObjType), self.Cxx.getDeclaration(checkVar, self.Python.typeMap['bool'])])
    body.addChildren([self.Cxx.getExpStmt(setArgs, 'Pack arguments for dispatch to implementation'), self.IOR.getErrorCheck(klass, method, argsVar, errorMsg)])
    return None
  
  def getMethodDummyArguments(self, klass, method, body):
    import ASE.Compiler.Cxx.Assignment
    argsVar = self.Cxx.getVar('_args')
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildCall.setArguments([self.Cxx.castToString('()')])
    setArgs = ASE.Compiler.Cxx.Assignment.Assignment()
    setArgs.setChildren([argsVar, buildCall])
    errorMsg = 'Failed to create dummy arguments for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addDeclarations([self.Cxx.getDeclaration(argsVar, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(setArgs, 'Dummy arguments dispatch to implementation'), self.IOR.getErrorCheck(klass, method, argsVar, errorMsg)])
    return None
  
  def getMethodImplMethod(self, klass, method, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    contextVar = self.Cxx.getVar('_context')
    methodVar = self.Cxx.getVar('_method')
    getMethod = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getMethod.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
    getMethod.setArguments([contextVar, self.Cxx.castToString(method.getIdentifier())])
    setMethod = ASE.Compiler.Cxx.Assignment.Assignment()
    setMethod.setChildren([methodVar, getMethod])
    errorMsg = 'Null implementation method for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addDeclarations([self.Cxx.getDeclaration(methodVar, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(setMethod, 'Get method object'), self.Python.getClearError(), self.IOR.getErrorCheck(klass, method, methodVar, errorMsg)])
    return None
  
  def getMethodImplObject(self, klass, method, body, forceStatic = 0):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    contextVar = self.Cxx.getVar('_context')
    if not method.getStatic() and not forceStatic:
      selfVar = self.Cxx.getVar('_self')
      dataVar = ASE.Compiler.Cxx.StructReference.StructReference()
      dataVar.setChildren([selfVar])
      dataVar.setMember('d_data')
      dataVar.setPointer(1)
      checkSelf = ASE.Compiler.Cxx.If.If()
      checkSelf.setBranch(selfVar)
      checkSelf.setChildren([dataVar, self.Cxx.getNull()])
      checkSelf.setTernary(1)
      setContext = ASE.Compiler.Cxx.Assignment.Assignment()
      setContext.setChildren([contextVar, checkSelf])
    else:
      importMod = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      importMod.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
      importMod.setArguments([self.Cxx.castToString(klass.getFullIdentifier() + '_impl')])
      setContext = ASE.Compiler.Cxx.Assignment.Assignment()
      setContext.setChildren([contextVar, importMod])
    body.addDeclarations([self.Cxx.getDeclaration(contextVar, self.pyObjType)])
    errorMsg = 'Null implementation object for ' + klass.getFullIdentifier() + '.' + method.getIdentifier()
    body.addChildren([self.Cxx.getExpStmt(setContext, 'Get implementation object'), self.Python.getClearError(), self.IOR.getErrorCheck(klass, method, contextVar, errorMsg)])
    return None
  
  def getMethodSignature(self, klass, method):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    methodName = ASE.Compiler.Cxx.Declarator.Declarator()
    methodName.setIdentifier(method.getFullIdentifier().replace('.', '_'))
    methodSkeleton = ASE.Compiler.Cxx.Function.Function()
    methodSkeleton.setChildren([methodName])
    if method.getReturnParameter() is None:
      methodSkeleton.setType(self.IOR.typeMap['void'])
    else:
      methodSkeleton.setType(self.IOR.getType(method.getReturnParameter().getType()))
    methodSkeleton.setStatic(1)
    if not method.getStatic():
      selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
      selfParam.setIdentifier('_self')
      selfParam.setType(self.IOR.getType(klass))
      methodSkeleton.addParameters([selfParam])
    for child in method.getChildren():
      parameter = ASE.Compiler.SIDL.Parameter.Parameter(child)
      param = ASE.Compiler.Cxx.Parameter.Parameter()
      param.setIdentifier(parameter.getIdentifier())
      if self.isOut(parameter):
        paramType = ASE.Compiler.Cxx.Pointer.Pointer()
        paramType.setChildren([self.IOR.getType(parameter.getType())])
        param.setType(paramType)
      else:
        param.setType(self.IOR.getType(parameter.getType()))
      methodSkeleton.addParameters([param])
    excType = ASE.Compiler.Cxx.Pointer.Pointer()
    excType.setChildren([self.IOR.getType(self.SIDL.getBaseException(method.getExceptions()[0]))])
    excParam = ASE.Compiler.Cxx.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setType(excType)
    methodSkeleton.addParameters([excParam])
    return methodSkeleton
  
  def getMethodSkeleton(self, klass, method):
    import ASE.Compiler.Cxx.CompoundStatement
    methodSkeleton = self.getMethodSignature(klass, method)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    if klass.getFullIdentifier() == 'ASE.BaseClass' and method.getFullIdentifier() == 'ASE.BaseInterface.deleteReference':
      typeVar = self.Cxx.getVar('_saveExcType')
      valVar = self.Cxx.getVar('_saveExcValue')
      tbVar = self.Cxx.getVar('_saveExcTraceback')
      body.addDeclarations([self.Cxx.getDeclaration(typeVar, self.pyObjType), self.Cxx.getDeclaration(valVar, self.pyObjType), self.Cxx.getDeclaration(tbVar, self.pyObjType)])
      body.addChildren([self.Python.getSuspendError()])
    self.getMethodImplObject(klass, method, body)
    self.getMethodImplMethod(klass, method, body)
    self.getMethodPackArguments(klass, method, body)
    self.getMethodDispatch(method, body)
    self.getMethodHandleExceptions(klass, method, body)
    self.getMethodReturn(klass, method, body)
    return [methodSkeleton]
  
  def getConstructorMethod(self, klass):
    if not klass in self.constructors:
      import ASE.Compiler.ParameterAccessMode
      import ASE.Compiler.SIDL.Method
      import ASE.Compiler.SIDL.Parameter
      objVar = self.Cxx.getVar('obj')
      objParam = ASE.Compiler.SIDL.Parameter.Parameter()
      objParam.setIdentifier(objVar.getIdentifier())
      objParam.setType(klass)
      objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
      constructor = ASE.Compiler.SIDL.Method.Method()
      constructor.setIdentifier(klass.getIdentifier())
      constructor.setChildren([objParam])
      constructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
      constructor.setParent(klass)
      constructor.setStatic(1)
      self.constructors[klass] = constructor
    return self.constructors[klass]
  
  def getDestructorMethod(self, klass):
    if not klass in self.destructors:
      import ASE.Compiler.SIDL.Method
      destructor = ASE.Compiler.SIDL.Method.Method()
      destructor.setIdentifier(klass.getIdentifier() + '_')
      destructor.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(klass))])
      destructor.setParent(klass)
      self.destructors[klass] = destructor
    return self.destructors[klass]
  
  def getConstructorSkeleton(self, klass):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    method = self.getConstructorMethod(klass)
    methodSkeleton = self.getMethodSignature(klass, method)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    self.getMethodImplObject(klass, method, body)
    self.getMethodImplMethod(klass, method, body)
    self.getMethodPackArguments(klass, method, body, isConstructor = 1)
    methodVar = self.Cxx.getVar('_method')
    argsVar = self.Cxx.getVar('_args')
    returnVar = self.Cxx.getVar('_return_value')
    pyReturnVar = self.Cxx.getVar('_pyReturn')
    objVar = self.Cxx.getVar(method.getChildren()[0].getIdentifier())
    if klass.getFullIdentifier() == 'ASE.BaseClass':
      longType = ASE.Compiler.Cxx.Type.Type()
      longType.setIdentifier('long')
      longType.setBaseType(1)
      objectsVar = self.Cxx.getVar('objects')
      addrVar = self.Cxx.getVar('addr')
      oneVar = self.Cxx.getVar('one')
      createOne = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createOne.setChildren([self.Cxx.getVar('PyInt_FromLong')])
      createOne.setArguments([self.Cxx.getInteger(1)])
      oneInit = ASE.Compiler.Cxx.Initializer.Initializer()
      oneInit.setChildren([createOne])
      createAddr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createAddr.setChildren([self.Cxx.getVar('PyInt_FromLong')])
      createAddr.setArguments([self.Cxx.castToType(objVar, longType)])
      addrInit = ASE.Compiler.Cxx.Initializer.Initializer()
      addrInit.setChildren([createAddr])
      getObjects = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      getObjects.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
      getObjects.setArguments([self.Cxx.getVar('_context'), self.Cxx.getString('objects')])
      setObjects = ASE.Compiler.Cxx.Assignment.Assignment()
      setObjects.setChildren([objectsVar, getObjects])
      body.addChildren([self.Cxx.getExpStmt(setObjects)])
      branchExp = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      branchExp.setChildren([self.Cxx.getVar('PyMapping_HasKey')])
      branchExp.setArguments([objectsVar, addrVar])
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, 'Redundant object creation', branchExp)])
      setItem = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      setItem.setChildren([self.Cxx.getVar('PyDict_SetItem')])
      setItem.setArguments([objectsVar, addrVar, oneVar])
      decrefAddr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefAddr.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefAddr.setArguments([addrVar])
      decrefOne = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefOne.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefOne.setArguments([oneVar])
      decrefObjects = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefObjects.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefObjects.setArguments([objectsVar])
      body.addDeclarations([self.Cxx.getDeclaration(objectsVar, self.pyObjType), self.Cxx.getDeclaration(addrVar, self.pyObjType, addrInit), self.Cxx.getDeclaration(oneVar, self.pyObjType, oneInit)])
      body.addChildren([self.Cxx.getExpStmt(s) for s in [setItem, decrefAddr, decrefOne, decrefObjects]])
    createObj = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createObj.setChildren([self.Cxx.getVar('PyInstance_New')])
    createObj.setArguments([methodVar, argsVar, self.Cxx.getNull()])
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([returnVar, createObj])
    setPyReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setPyReturn.setChildren([pyReturnVar, self.Cxx.getVar('Py_None')])
    decrefMethod = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefMethod.setChildren([self.Cxx.getVar('Py_DECREF')])
    decrefMethod.setArguments([methodVar])
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(setReturn, 'Create a new instance'), self.Cxx.getExpStmt(setPyReturn), self.Cxx.getExpStmt(decrefMethod)])
    errorMsg = 'Could not create instance of ' + klass.getFullIdentifier() + '_impl.' + klass.getIdentifier()
    body.addChildren([self.Python.getClearError(), self.IOR.getErrorCheck(klass, method, returnVar, errorMsg)])
    dataVar = ASE.Compiler.Cxx.StructReference.StructReference()
    dataVar.setChildren([objVar])
    dataVar.setMember('d_data')
    dataVar.setPointer(1)
    setObj = ASE.Compiler.Cxx.Assignment.Assignment()
    setObj.setChildren([dataVar, returnVar])
    body.addChildren([self.Cxx.getExpStmt(setObj)])
    self.getMethodHandleExceptions(klass, method, body)
    self.getMethodReturn(klass, method, body)
    return [methodSkeleton]
  
  def getDestructorSkeleton(self, klass):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.Type
    method = self.getDestructorMethod(klass)
    methodSkeleton = self.getMethodSignature(klass, method)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    selfVar = self.Cxx.getVar('_self')
    if klass.getFullIdentifier() == 'ASE.BaseClass':
      self.getMethodImplObject(klass, method, body, forceStatic = 1)
      longType = ASE.Compiler.Cxx.Type.Type()
      longType.setIdentifier('long')
      longType.setBaseType(1)
      objectsVar = self.Cxx.getVar('objects')
      addrVar = self.Cxx.getVar('addr')
      twoVar = self.Cxx.getVar('two')
      createTwo = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createTwo.setChildren([self.Cxx.getVar('PyInt_FromLong')])
      createTwo.setArguments([self.Cxx.getInteger(2)])
      twoInit = ASE.Compiler.Cxx.Initializer.Initializer()
      twoInit.setChildren([createTwo])
      createAddr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createAddr.setChildren([self.Cxx.getVar('PyInt_FromLong')])
      createAddr.setArguments([self.Cxx.castToType(selfVar, longType)])
      addrInit = ASE.Compiler.Cxx.Initializer.Initializer()
      addrInit.setChildren([createAddr])
      getObjects = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      getObjects.setChildren([self.Cxx.getVar('PyObject_GetAttrString')])
      getObjects.setArguments([self.Cxx.getVar('_context'), self.Cxx.getString('objects')])
      setObjects = ASE.Compiler.Cxx.Assignment.Assignment()
      setObjects.setChildren([objectsVar, getObjects])
      body.addChildren([self.Cxx.getExpStmt(setObjects)])
      hasKey = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      hasKey.setChildren([self.Cxx.getVar('PyMapping_HasKey')])
      hasKey.setArguments([objectsVar, addrVar])
      branchExp = ASE.Compiler.Cxx.Negation.Negation()
      branchExp.setChildren([hasKey])
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, 'Destruction of an object which was never created', branchExp)])
      getItem = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      getItem.setChildren([self.Cxx.getVar('PyDict_GetItem')])
      getItem.setArguments([objectsVar, addrVar])
      getAddr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      getAddr.setChildren([self.Cxx.getVar('PyInt_AsLong')])
      getAddr.setArguments([getItem])
      branchExp = ASE.Compiler.Cxx.Inequality.Inequality()
      branchExp.setChildren([getAddr, self.Cxx.getInteger(1)])
      body.addChildren([self.IOR.getErrorCheck(klass, method, None, 'Destruction of an object already destroyed', branchExp)])
      setItem = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      setItem.setChildren([self.Cxx.getVar('PyDict_SetItem')])
      setItem.setArguments([objectsVar, addrVar, twoVar])
      decrefAddr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefAddr.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefAddr.setArguments([addrVar])
      decrefTwo = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefTwo.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefTwo.setArguments([twoVar])
      decrefObjects = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefObjects.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefObjects.setArguments([objectsVar])
      body.addDeclarations([self.Cxx.getDeclaration(objectsVar, self.pyObjType), self.Cxx.getDeclaration(addrVar, self.pyObjType, addrInit), self.Cxx.getDeclaration(twoVar, self.pyObjType, twoInit)])
      body.addChildren([self.Cxx.getExpStmt(s) for s in [setItem, decrefAddr, decrefTwo, decrefObjects]])
    dataVar = self.Cxx.getVar('data')
    selfData = ASE.Compiler.Cxx.StructReference.StructReference()
    selfData.setChildren([selfVar])
    selfData.setMember('d_data')
    selfData.setPointer(1)
    dataInit = ASE.Compiler.Cxx.Initializer.Initializer()
    dataInit.setChildren([self.Cxx.castToType(selfData, self.pyObjType)])
    setData = ASE.Compiler.Cxx.Assignment.Assignment()
    setData.setChildren([selfData, self.Cxx.getNull()])
    decrefObj = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    decrefObj.setChildren([self.Cxx.getVar('Py_XDECREF')])
    decrefObj.setArguments([dataVar])
    body.addDeclarations([self.Cxx.getDeclaration(dataVar, self.pyObjType, dataInit)])
    body.addChildren([self.Cxx.getExpStmt(setData, 'If we do not set d_data to NULL, we get a SEGV on the recursive call to deleteReference() coming from self.__IORself'), self.Cxx.getExpStmt(decrefObj, 'Remove only reference to Python object')])
    return [methodSkeleton]
  
  def getInitialization(self, klass, body, imports = []):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Negation
    initVar = self.Cxx.getVar('initialized')
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    notInitVar = ASE.Compiler.Cxx.Negation.Negation()
    notInitVar.setChildren([initVar])
    setInit = ASE.Compiler.Cxx.Assignment.Assignment()
    setInit.setChildren([initVar, self.Cxx.getInteger(1)])
    initCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    initCall.setChildren([self.Cxx.getVar('SIDL_Python_Init')])
    compound.setChildren([self.Cxx.getExpStmt(setInit), self.Cxx.getExpStmt(initCall)])
    for moduleName in ['user', 'importer'] + imports:
      importCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      importCall.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
      importCall.setArguments([self.Cxx.castToString(moduleName)])
      negMod = ASE.Compiler.Cxx.Negation.Negation()
      negMod.setChildren([importCall])
      compound.addChildren([self.IOR.getErrorCheck(klass, None, None, 'Could not import ' + moduleName, negMod)])
    importCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    importCall.setChildren([self.Cxx.getVar('import_array')])
    compound.addChildren([self.Cxx.getExpStmt(importCall)])
    checkInit = ASE.Compiler.Cxx.If.If()
    checkInit.setBranch(notInitVar)
    checkInit.setChildren([compound])
    body.addDeclarations([self.Cxx.getDeclaration(initVar, self.Cxx.typeMap['int'], isStatic = 1)])
    body.addChildren([checkInit])
    return None
  
  def getSetEpvSignature(self, klass):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    epvStruct = ASE.Compiler.Cxx.Struct.Struct()
    epvStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__epv')
    epvType = ASE.Compiler.Cxx.Pointer.Pointer()
    epvType.setChildren([epvStruct])
    epvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    epvParam.setIdentifier('_epv')
    epvParam.setType(epvType)
    excType = ASE.Compiler.Cxx.Pointer.Pointer()
    excType.setChildren([self.IOR.getType(self.SIDL.getBaseException(self.SIDL.getASEException(klass)))])
    excParam = ASE.Compiler.Cxx.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setType(excType)
    methodName = ASE.Compiler.Cxx.Declarator.Declarator()
    methodName.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__set_epv')
    methodDecl = ASE.Compiler.Cxx.Function.Function()
    methodDecl.setType(self.Python.typeMap['void'])
    methodDecl.setChildren([methodName])
    methodDecl.setParameters([epvParam, excParam])
    return methodDecl
  
  def getSetEpvSkeleton(self, klass):
    import ASE.Compiler.CodePurpose
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.StructReference
    methodDecl = self.Cxx.getDecl(self.getSetEpvSignature(klass))
    methodSkeleton = self.getSetEpvSignature(klass)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    self.getInitialization(klass, body, [klass.getFullIdentifier()])
    epvVar = self.Cxx.getVar('_epv')
    methods = [self.getConstructorMethod(klass), self.getDestructorMethod(klass)]
    methods.extend([m for m in self.getTemplateMethods(klass.getImplementedMethods()) if not m.getStatic() and ASE.Compiler.CodePurpose.SKELETON in m.getPurpose()])
    for method in methods:
      funcVar = ASE.Compiler.Cxx.StructReference.StructReference()
      funcVar.setChildren([epvVar])
      funcVar.setMember(method.getIdentifier())
      funcVar.setPointer(1)
      setFunc = ASE.Compiler.Cxx.Assignment.Assignment()
      setFunc.setChildren([funcVar, self.Cxx.getVar(method.getFullIdentifier().replace('.', '_'))])
      body.addChildren([self.Cxx.getExpStmt(setFunc)])
    return [methodDecl, methodSkeleton]
  
  def getSetSepvSignature(self, klass):
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    sepvStruct = ASE.Compiler.Cxx.Struct.Struct()
    sepvStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__sepv')
    sepvType = ASE.Compiler.Cxx.Pointer.Pointer()
    sepvType.setChildren([sepvStruct])
    sepvParam = ASE.Compiler.Cxx.Parameter.Parameter()
    sepvParam.setIdentifier('_sepv')
    sepvParam.setType(sepvType)
    excType = ASE.Compiler.Cxx.Pointer.Pointer()
    excType.setChildren([self.IOR.getType(self.SIDL.getBaseException(self.SIDL.getASEException(klass)))])
    excParam = ASE.Compiler.Cxx.Parameter.Parameter()
    excParam.setIdentifier('_exception')
    excParam.setType(excType)
    methodName = ASE.Compiler.Cxx.Declarator.Declarator()
    methodName.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '__set_sepv')
    methodDecl = ASE.Compiler.Cxx.Function.Function()
    methodDecl.setType(self.Python.typeMap['void'])
    methodDecl.setChildren([methodName])
    methodDecl.setParameters([sepvParam, excParam])
    return methodDecl
  
  def getSetSepvSkeleton(self, klass):
    if not self.hasStaticMethod(klass):
      return []
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.StructReference
    methodDecl = self.Cxx.getDecl(self.getSetSepvSignature(klass))
    methodSkeleton = self.getSetSepvSignature(klass)
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    methodSkeleton.setBody(body)
    self.getInitialization(klass, body, [klass.getFullIdentifier()])
    sepvVar = self.Cxx.getVar('_sepv')
    methods = [m for m in self.getTemplateMethods(klass.getImplementedMethods()) if m.getStatic() and ASE.Compiler.CodePurpose.SKELETON in m.getPurpose()]
    for method in methods:
      funcVar = ASE.Compiler.Cxx.StructReference.StructReference()
      funcVar.setChildren([sepvVar])
      funcVar.setMember(method.getIdentifier())
      funcVar.setPointer(1)
      setFunc = ASE.Compiler.Cxx.Assignment.Assignment()
      setFunc.setChildren([funcVar, self.Cxx.getVar(method.getFullIdentifier().replace('.', '_'))])
      body.addChildren([self.Cxx.getExpStmt(setFunc)])
    return [methodDecl, methodSkeleton]
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not method.getReturnParameter() is None:
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not parameter is None:
      type = parameter.getType()
      if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not method in self.templateMethods:
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 

class Skeleton(object):
  
  def getFileBasename(self, klass):
    import os
    idChain = klass.getFullIdentifier().split('.')
    if not len(idChain[:-1]):
      return idChain[-1] + '_Skeleton'
    return os.path.join(apply(os.path.join, idChain[:-1]), idChain[-1] + '_Skeleton')
