# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.StubGenerator
import ASE.Compiler.Vertex
# Ending imports section
pass
import ASE.Compiler.Python.Python
import ASE.Compiler.Python.SIDL
# Starting static methods section
pass
# Ending static methods section
pass

class StubGenerator:
  '''
  This visitor transforms a SIDL AST into a set of Python Stub ASTs
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.source = {}

    self.sourceLanguages = ['Python', 'Cxx']
    self.Stub = Stub()
    self.Python = ASE.Compiler.Python.Python.Python()
    self.IOR = self.Python.IOR
    self.Cxx = self.Python.Cxx
    self.SIDL = self.Python.SIDL
    self.refTypeType = self.Python.getRefTypeType()
    self.pyObjType = self.Python.getObjectType()
    self.pyArrayType = self.Python.getArrayType()
    self.pyTypeType = self.Python.getTypeType()
    self.pyTypeStruct = ASE.Compiler.Cxx.Type.Type()
    self.pyTypeStruct.setIdentifier('PyTypeObject')
    self.templateMethods = {}

  
  def getSourceLanguages(self):
    '''
    Retrieve the list of the languages in which ASTs are produced
    '''
    return self.sourceLanguages
  
  def setSourceLanguages(self, outputLanguages):
    '''
    Set the list of the languages in which ASTs are produced
    '''
    self.sourceLanguages = sourceLanguages
  
  def getSource(self, language):
    '''
    Retrieve the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not language in self.source:
      self.source[language] = []
    return self.source[language]
  
  def setSource(self, language, vertices):
    '''
    Set the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not language in self.source:
      self.source[language] = []
    if len(vertices) == 0:
      self.templateMethods = {}
      self.Python.clear()
    self.source[language] = list(vertices)
  
  def addSource(self, language, vertices):
    '''
    Add to the list of source ASTs for the given language
    Throw an exception if the language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    if not language in self.source:
      self.source[language] = []
    self.source[language].extend(vertices)
  
  def removeSource(self, language, vertices):
    '''
    Remove to the list of source ASTs for the given language
    Throw an exception if a vertex or language does not exist
    '''
    if not language in self.sourceLanguages:
      e = ASE.Compiler.CompilerException()
      e.setMessage('Invalid source language: ' + str(language))
      raise e
    for vertex in vertices:
      if vertex in self.source[language]:
        self.source[language].remove(vertex)
      else:
        e = ASE.Compiler.CompilerException()
        e.setMessage('Source tree does not exist: ' + vertex.getIdentifier())
        raise e
  
  def visitPackage(self, node):
    '''
    Each package has an associated __init__.py module
    '''
    import ASE.Compiler.Python.Module
    import ASE.Compiler.Python.Stmt
    import ASE.Compiler.Python.Assign
    import ASE.Compiler.Python.AssName
    import ASE.Compiler.Python.List
    target = ASE.Compiler.Python.AssName.AssName()
    target.setIdentifier('all')
    value = ASE.Compiler.Python.List.List()
    value.setChildren([self.Python.getString(child.getIdentifier()) for child in node.getChildren()])
    assign = ASE.Compiler.Python.Assign.Assign()
    assign.setValue(value)
    assign.setChildren([target])
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    stmt.setChildren([assign])
    module = ASE.Compiler.Python.Module.Module()
    module.setIdentifier(node.getFullIdentifier() + '.__init__')
    module.setComments(node.getComments())
    module.setChildren([stmt])
    self.__IORself.visitVertex(node)
    self.addSource('Python', [module])
  
  def visitInterface(self, node):
    '''
    Each interface becomes a client module
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    objParam = ASE.Compiler.SIDL.Parameter.Parameter()
    objParam.setIdentifier('obj')
    objParam.setType(node)
    objParam.setAccessMode(ASE.Compiler.ParameterAccessMode.IN)
    objParam.setRequired(0)
    retParam = ASE.Compiler.SIDL.Parameter.Parameter()
    retParam.setIdentifier('_return_value')
    retParam.setType(node)
    retParam.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    retParam.setRequired(1)
    method = ASE.Compiler.SIDL.Method.Method()
    method.setIdentifier(node.getIdentifier())
    method.setReturnParameter(retParam)
    method.setChildren([objParam])
    method.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(node))])
    retParam.setParent(method)
    node.addChildren([method])
    constructorMethod = method
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Include
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.PythonModule
    module = ASE.Compiler.Cxx.PythonModule.PythonModule()
    module.setClass(node.getFullIdentifier())
    pyInclude = ASE.Compiler.Cxx.Include.Include()
    pyInclude.setIdentifier('<Python.h>')
    numericInclude = ASE.Compiler.Cxx.Include.Include()
    numericInclude.setIdentifier('<Numeric/arrayobject.h>')
    module.addChildren([pyInclude, numericInclude, self.IOR.getGuardedInclude(node)])
    module.addChildren([self.IOR.getGuardedInclude(self.SIDL.getLoader(node)), self.IOR.getGuardedInclude(self.SIDL.getBaseException(self.SIDL.getASEException(node)))])
    if not node.getFullIdentifier() in ['ASE.BaseInterface', 'ASE.BaseClass', 'ASE.DLL', 'ASE.Loader']:
      module.addChildren([self.IOR.getGuardedInclude(self.SIDL.getLoader(node))])
    [self.createTemplateMethod(method) for method in node.getMethods()]
    module.addChildren(self.getEPV(node))
    module.addChildren([self.Python.getRefTypeEnumeration(), self.Python.getSIDLObject(node)])
    typeForwardDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    typeForwardDecl.setIdentifier('SIDLPythonObjectType')
    typeForwardDecl.setType(self.pyTypeStruct)
    typeForwardDecl.setStatic(1)
    module.addChildren([self.Cxx.getDecl(typeForwardDecl)])
    module.addChildren([self.Python.getNew(node)])
    wrapIORFuncs = {}

    extractIORFuncs = {}

    wrapArrayFuncs = {}

    extractArrayFuncs = {}

    for method in list(node.getMethods()) + [constructorMethod]:
      for type in [parameter.getType() for parameter in method.getOutParameters() if parameter.getType().getBaseType()]:
        if type.getIdentifier() == 'opaque':
          wrapIORFuncs[type] = self.Python.getWrapOpaque(type)
      for type in [parameter.getType() for parameter in method.getOutParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Interface')]:
        wrapIORFuncs[type] = self.Python.getWrapIOR(type)
      for type in [parameter.getType() for parameter in method.getInParameters() if parameter.getType().getBaseType()]:
        if type.getIdentifier() == 'opaque':
          extractIORFuncs[type] = self.Python.getExtractOpaque(type)
      for type in [parameter.getType() for parameter in method.getInParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Interface')]:
        extractIORFuncs[type] = self.Python.getExtractIOR(type, node)
      for type in [parameter.getType() for parameter in method.getOutParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array')]:
        entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
        if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
          for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
            wrapArrayFuncs[t] = self.Python.getWrapArray(t)
        else:
          wrapArrayFuncs[entryType] = self.Python.getWrapArray(entryType)
        if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          wrapIORFuncs[entryType] = self.Python.getWrapIOR(entryType)
      for type in [parameter.getType() for parameter in method.getInParameters() if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array')]:
        entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
        if entryType.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse'):
          for t in ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(entryType).getTypes():
            extractArrayFuncs[t] = self.Python.getExtractArray(t, node, not self.isOut(parameter))
        else:
          extractArrayFuncs[entryType] = self.Python.getExtractArray(entryType, node, not self.isOut(parameter))
        if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
          extractIORFuncs[entryType] = self.Python.getExtractIOR(entryType, node)
      for exception in method.getExceptions():
        wrapIORFuncs[exception] = self.Python.getWrapIOR(exception)
    module.addChildren([self.Python.getWrapIOR(node, isWeak = 1)])
    module.addChildren(wrapIORFuncs.values())
    module.addChildren(extractIORFuncs.values())
    for (entryType, func) in wrapArrayFuncs.items():
      module.addChildren([self.IOR.getGuardedInclude(entryType), func])
    for (entryType, func) in extractArrayFuncs.items():
      module.addChildren([self.IOR.getGuardedInclude(entryType), func])
    for method in self.getTemplateMethods(node.getMethods()):
      module.addChildren(self.getMethodStub(node, method))
    weakReferenceMethod = self.getWeakReferenceMethod(node)
    weakReferenceMethod.setParent(node)
    module.addChildren(self.getWeakReference(node, weakReferenceMethod))
    null = self.Cxx.getVar('NULL')
    nullMethod = ASE.Compiler.Cxx.Initializer.Initializer()
    nullMethod.setChildren([self.Cxx.getNull(), null])
    nullMethod.setList(1)
    methodType = ASE.Compiler.Cxx.Struct.Struct()
    methodType.setIdentifier('PyMethodDef')
    module.addChildren(self.getConstructor(node, constructorMethod))
    methodTable = ASE.Compiler.Cxx.Initializer.Initializer()
    for method in self.getTemplateMethods(node.getMethods()):
      methodTable.addChildren(self.getMethodDescriptor(method))
    methodTable.addChildren(self.getMethodDescriptor(weakReferenceMethod))
    methodTable.addChildren([nullMethod])
    methodTable.setList(1)
    objectMethods = ASE.Compiler.Cxx.Array.Array()
    objectMethods.setChildren([self.Cxx.getVar('_' + node.getIdentifier() + 'ObjectMethods')])
    objectMethods.setType(methodType)
    objectMethods.setStatic(1)
    objectMethods.setInitializer(methodTable)
    module.addChildren([self.Cxx.getDecl(objectMethods)])
    methodTable = ASE.Compiler.Cxx.Initializer.Initializer()
    methodTable.addChildren([nullMethod])
    methodTable.setList(1)
    moduleMethods = ASE.Compiler.Cxx.Array.Array()
    moduleMethods.setChildren([self.Cxx.getVar('_' + node.getIdentifier() + 'ModuleMethods')])
    moduleMethods.setType(methodType)
    moduleMethods.setStatic(1)
    moduleMethods.setInitializer(methodTable)
    module.addChildren([self.Cxx.getDecl(moduleMethods)])
    module.addChildren(self.getPythonType(node))
    module.addChildren(self.getModuleInit(node))
    self.addSource('Cxx', [module])
  
  def visitClass(self, node):
    '''
    Each class becomes a client module
    '''
    self.visitInterface(node)
  
  def visitMethod(self, node):
    '''
    Generates method stub code available from getSource()
    Not called by interface or class generation
    '''
    method = node
    wrapIORFuncs = {}

    extractIORFuncs = {}

    wrapArrayFuncs = {}

    extractArrayFuncs = {}

    for type in [parameter.getType() for parameter in method.getOutParameters()]:
      if type.getIdentifier() == 'opaque':
        wrapIORFuncs[type] = self.Python.getWrapOpaque(type)
    for type in [parameter.getType() for parameter in method.getOutParameters()]:
      wrapIORFuncs[type] = self.Python.getWrapIOR(type)
    for type in [parameter.getType() for parameter in method.getInParameters()]:
      if type.getIdentifier() == 'opaque':
        extractIORFuncs[type] = self.Python.getExtractOpaque(type)
    for type in [parameter.getType() for parameter in method.getInParameters()]:
      extractIORFuncs[type] = self.Python.getExtractIOR(type, node)
    for type in [parameter.getType() for parameter in method.getOutParameters()]:
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      wrapArrayFuncs[entryType] = self.Python.getWrapArray(entryType)
      if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        wrapIORFuncs[entryType] = self.Python.getWrapIOR(entryType)
    for type in [parameter.getType() for parameter in method.getInParameters()]:
      entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
      extractArrayFuncs[entryType] = self.Python.getExtractArray(entryType, node, not self.isOut(parameter))
      if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        extractIORFuncs[entryType] = self.Python.getExtractIOR(entryType, node)
    self.addSource('Cxx', wrapIORFuncs.values())
    self.addSource('Cxx', extractIORFuncs.values())
    self.addSource('Cxx', wrapArrayFuncs.values())
    self.addSource('Cxx', extractArrayFuncs.values())
    self.addSource('Cxx', self.getMethodStub(None, method))
  
  def visitEnumeration(self, node):
    '''
    Each enumeration becomes a client module
    '''
    import ASE.Compiler.Python.Module
    import ASE.Compiler.Python.Stmt
    module = ASE.Compiler.Python.Module.Module()
    stmt = ASE.Compiler.Python.Stmt.Stmt()
    module.setIdentifier(node.getFullIdentifier())
    module.setComments(node.getComments())
    module.setChildren([stmt])
    self.enumAssigns = []
    self.__IORself.visitVertex(node)
    stmt.setChildren(self.enumAssigns)
    self.addSource('Python', [module])
  
  def visitEnumerator(self, node):
    '''
    Used only internally
    '''
    import ASE.Compiler.Python.Assign
    import ASE.Compiler.Python.AssName
    import ASE.Compiler.Python.Const
    assign = ASE.Compiler.Python.Assign.Assign()
    target = ASE.Compiler.Python.AssName.AssName()
    value = ASE.Compiler.Python.Const.Const()
    target.setIdentifier(node.getIdentifier())
    value.setValue_int(node.getValue())
    assign.setValue(value)
    assign.setChildren([target])
    self.enumAssigns.append(assign)
  # Ending class methods section
  pass
  
  def raiseException(self, message, trace = ''):
    import sys
    e = ASE.Compiler.CompilerException.CompilerException()
    e.setMessage(message)
    if isinstance(trace, list):
      for line in trace:
        e.addStackLine(line.strip())
    elif trace:
      e.addStackLine(trace)
    e.addStackEntry(__file__, sys._getframe(1).f_lineno, sys._getframe(1).f_code.co_name)
    raise ASE.Compiler.CompilerException.Exception, e
  
  def hasStaticMethod(self, type):
    '''
    We could use  if method.getStatic() and not method.getInternal():
    '''
    if type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
      for method in type.getMethods():
        if method.getStatic():
          return 1
    return 0
  
  def isOut(self, parameter):
    import ASE.Compiler.ParameterAccessMode
    mode = parameter.getAccessMode()
    return mode == ASE.Compiler.ParameterAccessMode.OUT or mode == ASE.Compiler.ParameterAccessMode.INOUT or mode == ASE.Compiler.ParameterAccessMode.RETURN
  
  def getModeName(self, mode):
    import ASE.Compiler.ParameterAccessMode
    if mode == ASE.Compiler.ParameterAccessMode.IN:
      return 'in'
    elif mode == ASE.Compiler.ParameterAccessMode.INOUT:
      return 'inout'
    elif mode == ASE.Compiler.ParameterAccessMode.OUT:
      return 'out'
    elif mode == ASE.Compiler.ParameterAccessMode.RETURN:
      return 'return'
    raise RuntimeError('Invalid parameter access mode: ' + str(mode))
  
  def getParameterString(self, parameter):
    name = parameter.getIdentifier()
    if not name:
      name = '_result'
    return self.getModeName(parameter.getAccessMode()) + ' ' + parameter.getType().getFullIdentifier() + ' ' + name
  
  def escapeComment(self, line):
    return line.replace('\\', '\\\\').replace('"', '\\"').replace('\'', '\\\'')
  
  def getObjectComment(self, obj):
    return ''.join(['\\n\\\n'.join(map(self.escapeComment, comment.getComment().split('\n'))) for comment in obj.getComments()])
  
  def getMethodComment(self, method):
    import ASE.Compiler.SIDL.Parameter
    outParams = method.getOutParameters()
    end = '\\n\\\n'
    str = method.getIdentifier() + '('
    num = len(method.getChildren())
    for (c, child) in enumerate(method.getChildren()):
      str += self.getParameterString(ASE.Compiler.SIDL.Parameter.Parameter(child))
      if c < num - 1:
        str += ', '
    str += ')' + end
    str += 'RETURNS '
    num = len(outParams)
    if num:
      str += '('
      for (p, parameter) in enumerate(outParams):
        str += self.getParameterString(parameter)
        if p < num - 1:
          str += ', '
      str += ')'
    else:
      str += 'None' + end
    str += end
    str += self.getObjectComment(method)
    return str
  
  def getEPV(self, type):
    '''
    Generate epv pointer
    '''
    epv = [self.Cxx.getDeclaration(self.Cxx.getVar('_epv'), self.IOR.getExternalType(type), isStatic = 1)]
    if self.hasStaticMethod(type):
      epv.append(self.Cxx.getDeclaration(self.Cxx.getVar('_sepv'), self.IOR.getStaticExternalType(type), isStatic = 1))
    return epv
  
  def getPythonDeallocFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.LogicalAnd
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Type
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    derefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    derefCall.setChildren([self.Cxx.getVar('PyObject_ClearWeakRefs')])
    derefCall.setArguments([self.Cxx.castToType(selfVar, self.pyObjType)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(derefCall)])
    weakRefList = ASE.Compiler.Cxx.StructReference.StructReference()
    weakRefList.setChildren([selfVar])
    weakRefList.setMember('weakRefList')
    weakRefList.setPointer(1)
    handleWeakRef = ASE.Compiler.Cxx.If.If()
    handleWeakRef.setBranch(weakRefList)
    handleWeakRef.setChildren([compound])
    objIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    objIOR.setChildren([selfVar])
    objIOR.setMember('ior')
    objIOR.setPointer(1)
    refType = ASE.Compiler.Cxx.StructReference.StructReference()
    refType.setChildren([selfVar])
    refType.setMember('refType')
    refType.setPointer(1)
    checkRef = ASE.Compiler.Cxx.Inequality.Inequality()
    checkRef.setChildren([refType, self.Cxx.getVar('WEAK_REF')])
    selfData = ASE.Compiler.Cxx.StructReference.StructReference()
    selfData.setChildren([objIOR])
    selfData.setMember('d_data')
    selfData.setPointer(1)
    checkSelf = ASE.Compiler.Cxx.LogicalAnd.LogicalAnd()
    checkSelf.setChildren([self.Cxx.getGroup(checkRef), self.Cxx.getGroup(selfData)])
    trueIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    trueIOR.setChildren([objIOR])
    trueIOR.setMember('d_object')
    trueIOR.setPointer(1)
    objEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    objEpv.setChildren([self.Cxx.getGroup(self.Cxx.castToType(objIOR, self.IOR.getType(self.SIDL.getBaseInterface(klass))))])
    objEpv.setMember('d_epv')
    objEpv.setPointer(1)
    derefFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    derefFuncPtr.setChildren([objEpv])
    derefFuncPtr.setMember('deleteReference')
    derefFuncPtr.setPointer(1)
    derefFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    derefFunc.setChildren([self.Cxx.getGroup(derefFuncPtr)])
    derefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    derefCall.setChildren([self.Cxx.getGroup(derefFunc)])
    derefCall.setArguments([trueIOR, self.Cxx.getNull()])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(derefCall)])
    handleRef = ASE.Compiler.Cxx.If.If()
    if klass.isInstanceOf('ASE.Compiler.SIDL.Class'):
      handleRef.setBranch(checkSelf)
    else:
      handleRef.setBranch(checkRef)
    handleRef.setChildren([compound])
    objType = ASE.Compiler.Cxx.StructReference.StructReference()
    objType.setChildren([selfVar])
    objType.setMember('ob_type')
    objType.setPointer(1)
    freeFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    freeFunc.setChildren([objType])
    freeFunc.setMember('tp_free')
    freeFunc.setPointer(1)
    freeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    freeCall.setChildren([freeFunc])
    freeCall.setArguments([selfVar])
    body.addChildren([handleWeakRef, handleRef, self.Cxx.getExpStmt(freeCall)])
    void = ASE.Compiler.Cxx.Type.Type()
    void.setIdentifier('void')
    void.setBaseType(1)
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    deallocName = ASE.Compiler.Cxx.Declarator.Declarator()
    deallocName.setIdentifier('sidl_dealloc')
    deallocFunc = ASE.Compiler.Cxx.Function.Function()
    deallocFunc.setChildren([deallocName])
    deallocFunc.setType(void)
    deallocFunc.setParameters([selfParam])
    deallocFunc.setBody(body)
    deallocFunc.setStatic(1)
    return deallocFunc
  
  def getPythonCmpFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Type
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    argVar = self.Cxx.getVar('arg')
    ior1Var = self.Cxx.getVar('ior1')
    objIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    objIOR.setChildren([selfVar])
    objIOR.setMember('ior')
    objIOR.setPointer(1)
    iorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    iorInit.setChildren([objIOR])
    ior1Decl = self.Cxx.getDeclaration(ior1Var, self.IOR.getType(klass), iorInit)
    ior2Var = self.Cxx.getVar('ior2')
    objIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    objIOR.setChildren([self.Cxx.getGroup(self.Cxx.castToType(argVar, self.Python.getSIDLObjectType(klass)))])
    objIOR.setMember('ior')
    objIOR.setPointer(1)
    iorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    iorInit.setChildren([self.Cxx.castToType(objIOR, self.IOR.getType(self.SIDL.getBaseInterface(klass)))])
    ior2Decl = self.Cxx.getDeclaration(ior2Var, self.IOR.getType(self.SIDL.getBaseInterface(klass)), iorInit)
    o1Var = self.Cxx.getVar('o1')
    o1Decl = self.Cxx.getDeclaration(o1Var, self.pyObjType)
    o2Var = self.Cxx.getVar('o2')
    o2Decl = self.Cxx.getDeclaration(o2Var, self.pyObjType)
    resultVar = self.Cxx.getVar('_return_value')
    resultDecl = self.Cxx.getDeclaration(resultVar, self.Cxx.typeMap['int'])
    body.addDeclarations([ior1Decl, ior2Decl, o1Decl, o2Decl, resultDecl])
    baseIORVar = ASE.Compiler.Cxx.StructReference.StructReference()
    baseIORVar.setChildren([ior1Var])
    baseIORVar.setMember('d_object')
    baseIORVar.setPointer(1)
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildCall.setArguments([self.Cxx.castToString('i'), baseIORVar])
    setO1 = ASE.Compiler.Cxx.Assignment.Assignment()
    setO1.setChildren([o1Var, buildCall])
    baseIORVar = ASE.Compiler.Cxx.StructReference.StructReference()
    baseIORVar.setChildren([ior2Var])
    baseIORVar.setMember('d_object')
    baseIORVar.setPointer(1)
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildCall.setArguments([self.Cxx.castToString('i'), baseIORVar])
    setO2 = ASE.Compiler.Cxx.Assignment.Assignment()
    setO2.setChildren([o2Var, buildCall])
    cmpCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    cmpCall.setChildren([self.Cxx.getVar('PyObject_Compare')])
    cmpCall.setArguments([o1Var, o2Var])
    setResult = ASE.Compiler.Cxx.Assignment.Assignment()
    setResult.setChildren([resultVar, cmpCall])
    derefO1 = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    derefO1.setChildren([self.Cxx.getVar('Py_DECREF')])
    derefO1.setArguments([o1Var])
    derefO2 = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    derefO2.setChildren([self.Cxx.getVar('Py_DECREF')])
    derefO2.setArguments([o2Var])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    self.getMethodSIDLObjectCheck('sidl_cmp', body, resultVar, argVar)
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_cmp()', ior1Var, resultVar), self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_cmp()', ior2Var, resultVar), self.Cxx.getExpStmt(setO1), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_cmp()', o1Var, resultVar), self.Cxx.getExpStmt(setO2), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_cmp()', o2Var, resultVar), self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(derefO1), self.Cxx.getExpStmt(derefO2), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    argParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argParam.setIdentifier(argVar.getIdentifier())
    argParam.setType(self.pyObjType)
    cmpName = ASE.Compiler.Cxx.Declarator.Declarator()
    cmpName.setIdentifier('sidl_cmp')
    cmpFunc = ASE.Compiler.Cxx.Function.Function()
    cmpFunc.setChildren([cmpName])
    cmpFunc.setType(self.Cxx.typeMap['int'])
    cmpFunc.setParameters([selfParam, argParam])
    cmpFunc.setBody(body)
    cmpFunc.setStatic(1)
    return cmpFunc
  
  def getPythonHashFunc(self, klass):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Type
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    longType = ASE.Compiler.Cxx.Type.Type()
    longType.setIdentifier('long')
    longType.setBaseType(1)
    selfVar = self.Cxx.getVar('self')
    iorVar = self.Cxx.getVar('ior')
    objIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    objIOR.setChildren([selfVar])
    objIOR.setMember('ior')
    objIOR.setPointer(1)
    iorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    iorInit.setChildren([objIOR])
    iorDecl = self.Cxx.getDeclaration(iorVar, self.IOR.getType(klass), iorInit)
    hashVar = self.Cxx.getVar('_hash_arg')
    hashArgDecl = self.Cxx.getDeclaration(hashVar, self.pyObjType)
    resultVar = self.Cxx.getVar('_return_value')
    resultDecl = self.Cxx.getDeclaration(resultVar, longType)
    body.addDeclarations([iorDecl, hashArgDecl, resultDecl])
    baseIORVar = ASE.Compiler.Cxx.StructReference.StructReference()
    baseIORVar.setChildren([iorVar])
    baseIORVar.setMember('d_object')
    baseIORVar.setPointer(1)
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildCall.setArguments([self.Cxx.castToString('i'), baseIORVar])
    setHash = ASE.Compiler.Cxx.Assignment.Assignment()
    setHash.setChildren([hashVar, buildCall])
    hashCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    hashCall.setChildren([self.Cxx.getVar('PyObject_Hash')])
    hashCall.setArguments([hashVar])
    setResult = ASE.Compiler.Cxx.Assignment.Assignment()
    setResult.setChildren([resultVar, hashCall])
    derefHash = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    derefHash.setChildren([self.Cxx.getVar('Py_DECREF')])
    derefHash.setArguments([hashVar])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_hash()', iorVar, resultVar), self.Cxx.getExpStmt(setHash), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not hash IOR pointer in sidl_hash()', hashVar, resultVar), self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(derefHash), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    hashName = ASE.Compiler.Cxx.Declarator.Declarator()
    hashName.setIdentifier('sidl_hash')
    hashFunc = ASE.Compiler.Cxx.Function.Function()
    hashFunc.setChildren([hashName])
    hashFunc.setType(longType)
    hashFunc.setParameters([selfParam])
    hashFunc.setBody(body)
    hashFunc.setStatic(1)
    return hashFunc
  
  def getPythonStrFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Type
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    iorVar = self.Cxx.getVar('ior')
    objIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    objIOR.setChildren([selfVar])
    objIOR.setMember('ior')
    objIOR.setPointer(1)
    iorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    iorInit.setChildren([objIOR])
    iorDecl = self.Cxx.getDeclaration(iorVar, self.IOR.getType(klass), iorInit)
    strVar = self.Cxx.getVar('_str')
    strDecl = self.Cxx.getDeclaration(strVar, self.IOR.typeMap['string'])
    resultVar = self.Cxx.getVar('_return_value')
    resultDecl = self.Cxx.getDeclaration(resultVar, self.pyObjType)
    body.addDeclarations([iorDecl, strDecl, resultDecl])
    iorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    iorEpv.setChildren([iorVar])
    iorEpv.setMember('d_epv')
    iorEpv.setPointer(1)
    iorMethodPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    iorMethodPtr.setChildren([iorEpv])
    iorMethodPtr.setMember('strObject')
    iorMethodPtr.setPointer(1)
    iorMethod = ASE.Compiler.Cxx.Indirection.Indirection()
    iorMethod.setChildren([self.Cxx.getGroup(iorMethodPtr)])
    strCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    strCall.setChildren([self.Cxx.getGroup(iorMethod)])
    strCall.setArguments([iorVar, self.Cxx.getNull()])
    setStr = ASE.Compiler.Cxx.Assignment.Assignment()
    setStr.setChildren([strVar, strCall])
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('PyString_FromString')])
    buildCall.setArguments([strVar])
    setResult = ASE.Compiler.Cxx.Assignment.Assignment()
    setResult.setChildren([resultVar, buildCall])
    increfResult = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfResult.setChildren([self.Cxx.getVar('Py_INCREF')])
    increfResult.setArguments([resultVar])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    body.addChildren([self.Cxx.getExpStmt(setStr), self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(increfResult), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    strName = ASE.Compiler.Cxx.Declarator.Declarator()
    strName.setIdentifier('sidl_str')
    strFunc = ASE.Compiler.Cxx.Function.Function()
    strFunc.setChildren([strName])
    strFunc.setType(self.pyObjType)
    strFunc.setParameters([selfParam])
    strFunc.setBody(body)
    strFunc.setStatic(1)
    return strFunc
  
  def getPythonRichCmpFunc(self, klass):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Switch
    import ASE.Compiler.Cxx.Type
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfVar = self.Cxx.getVar('self')
    argVar = self.Cxx.getVar('arg')
    opVar = self.Cxx.getVar('op')
    ior1Var = self.Cxx.getVar('ior1')
    objIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    objIOR.setChildren([selfVar])
    objIOR.setMember('ior')
    objIOR.setPointer(1)
    iorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    iorInit.setChildren([objIOR])
    ior1Decl = self.Cxx.getDeclaration(ior1Var, self.IOR.getType(klass), iorInit)
    ior2Var = self.Cxx.getVar('ior2')
    objIOR = ASE.Compiler.Cxx.StructReference.StructReference()
    objIOR.setChildren([self.Cxx.getGroup(self.Cxx.castToType(argVar, self.Python.getSIDLObjectType(klass)))])
    objIOR.setMember('ior')
    objIOR.setPointer(1)
    iorInit = ASE.Compiler.Cxx.Initializer.Initializer()
    iorInit.setChildren([self.Cxx.castToType(objIOR, self.IOR.getType(self.SIDL.getBaseInterface(klass)))])
    ior2Decl = self.Cxx.getDeclaration(ior2Var, self.IOR.getType(self.SIDL.getBaseInterface(klass)), iorInit)
    o1Var = self.Cxx.getVar('o1')
    o1Decl = self.Cxx.getDeclaration(o1Var, self.pyObjType)
    o2Var = self.Cxx.getVar('o2')
    o2Decl = self.Cxx.getDeclaration(o2Var, self.pyObjType)
    resultVar = self.Cxx.getVar('_return_value')
    resultDecl = self.Cxx.getDeclaration(resultVar, self.pyObjType)
    body.addDeclarations([ior1Decl, ior2Decl, o1Decl, o2Decl, resultDecl])
    switchBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    increfResult = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    increfResult.setChildren([self.Cxx.getVar('Py_INCREF')])
    increfResult.setArguments([resultVar])
    breakStmt = ASE.Compiler.Cxx.Break.Break()
    setFalse = ASE.Compiler.Cxx.Assignment.Assignment()
    setFalse.setChildren([resultVar, self.Cxx.getVar('Py_False')])
    eqCase = self.Cxx.getExpStmt(setFalse)
    eqCase.setCaseLabel(self.Cxx.getVar('Py_EQ'))
    switchBody.addChildren([eqCase, self.Cxx.getExpStmt(increfResult), breakStmt])
    setTrue = ASE.Compiler.Cxx.Assignment.Assignment()
    setTrue.setChildren([resultVar, self.Cxx.getVar('Py_True')])
    neqCase = self.Cxx.getExpStmt(setTrue)
    neqCase.setCaseLabel(self.Cxx.getVar('Py_NE'))
    switchBody.addChildren([neqCase, self.Cxx.getExpStmt(increfResult), breakStmt])
    ltCase = ASE.Compiler.Cxx.Statement.Statement()
    ltCase.setCaseLabel(self.Cxx.getVar('Py_LT'))
    leqCase = ASE.Compiler.Cxx.Statement.Statement()
    leqCase.setCaseLabel(self.Cxx.getVar('Py_LE'))
    gtCase = ASE.Compiler.Cxx.Statement.Statement()
    gtCase.setCaseLabel(self.Cxx.getVar('Py_GT'))
    geqCase = ASE.Compiler.Cxx.Statement.Statement()
    geqCase.setCaseLabel(self.Cxx.getVar('Py_GE'))
    errorCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    errorCall.setChildren([self.Cxx.getVar('PyErr_SetString')])
    errorCall.setArguments([self.Cxx.getVar('PyExc_TypeError'), self.Cxx.castToString('Could not parse arguments for sidl_richcmp')])
    switchBody.addChildren([ltCase, leqCase, gtCase, geqCase, self.Cxx.getExpStmt(errorCall)])
    switchOp = ASE.Compiler.Cxx.Switch.Switch()
    switchOp.setBranch(opVar)
    switchOp.setChildren([switchBody])
    returnResult = ASE.Compiler.Cxx.Return.Return()
    returnResult.setChildren([resultVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([switchOp, returnResult])
    self.getMethodSIDLObjectCheck('sidl_richcmp', body, resultVar, argVar, errorHandler = compound, anySIDLObject = 1)
    baseIORVar = ASE.Compiler.Cxx.StructReference.StructReference()
    baseIORVar.setChildren([ior1Var])
    baseIORVar.setMember('d_object')
    baseIORVar.setPointer(1)
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildCall.setArguments([self.Cxx.castToString('i'), baseIORVar])
    setO1 = ASE.Compiler.Cxx.Assignment.Assignment()
    setO1.setChildren([o1Var, buildCall])
    baseIORVar = ASE.Compiler.Cxx.StructReference.StructReference()
    baseIORVar.setChildren([ior2Var])
    baseIORVar.setMember('d_object')
    baseIORVar.setPointer(1)
    buildCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildCall.setArguments([self.Cxx.castToString('i'), baseIORVar])
    setO2 = ASE.Compiler.Cxx.Assignment.Assignment()
    setO2.setChildren([o2Var, buildCall])
    cmpCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    cmpCall.setChildren([self.Cxx.getVar('PyObject_RichCompare')])
    cmpCall.setArguments([o1Var, o2Var, opVar])
    setResult = ASE.Compiler.Cxx.Assignment.Assignment()
    setResult.setChildren([resultVar, cmpCall])
    derefO1 = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    derefO1.setChildren([self.Cxx.getVar('Py_DECREF')])
    derefO1.setArguments([o1Var])
    derefO2 = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    derefO2.setChildren([self.Cxx.getVar('Py_DECREF')])
    derefO2.setArguments([o2Var])
    body.addChildren([self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_richcmp()', ior1Var, resultVar), self.Python.getErrorCheck('PyExc_ValueError', 'Null IOR pointer in sidl_richcmp()', ior2Var, resultVar), self.Cxx.getExpStmt(setO1), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_richcmp()', o1Var, resultVar), self.Cxx.getExpStmt(setO2), self.Python.getErrorCheck('PyExc_RuntimeError', 'Could not build integer from IOR pointer in sidl_richcmp()', o2Var, resultVar), self.Cxx.getExpStmt(setResult), self.Cxx.getExpStmt(derefO1), self.Cxx.getExpStmt(derefO2), self.Cxx.getExpStmt(increfResult), returnResult])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier(selfVar.getIdentifier())
    selfParam.setType(self.Python.getSIDLObjectType(klass))
    argParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argParam.setIdentifier(argVar.getIdentifier())
    argParam.setType(self.pyObjType)
    opParam = ASE.Compiler.Cxx.Parameter.Parameter()
    opParam.setIdentifier(opVar.getIdentifier())
    opParam.setType(self.Cxx.typeMap['int'])
    cmpName = ASE.Compiler.Cxx.Declarator.Declarator()
    cmpName.setIdentifier('sidl_richcmp')
    cmpFunc = ASE.Compiler.Cxx.Function.Function()
    cmpFunc.setChildren([cmpName])
    cmpFunc.setType(self.pyObjType)
    cmpFunc.setParameters([selfParam, argParam, opParam])
    cmpFunc.setBody(body)
    cmpFunc.setStatic(1)
    return cmpFunc
  
  def getPythonType(self, klass):
    import ASE.Compiler.Cxx.Address
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Macro
    import ASE.Compiler.Cxx.InclusiveOr
    import ASE.Compiler.Cxx.Sizeof
    import ASE.Compiler.Cxx.Struct
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Type
    constructor = klass.getFullIdentifier().replace('.', '_') + '_' + klass.getIdentifier()
    functions = [self.getPythonDeallocFunc(klass), self.getPythonCmpFunc(klass), self.getPythonHashFunc(klass), self.getPythonStrFunc(klass), self.getPythonRichCmpFunc(klass)]
    flags = ASE.Compiler.Cxx.InclusiveOr.InclusiveOr()
    flags.setChildren([self.Cxx.getVar('Py_TPFLAGS_DEFAULT'), self.Cxx.getVar('Py_TPFLAGS_BASETYPE'), self.Cxx.getVar('Py_TPFLAGS_HAVE_WEAKREFS')])
    headInit = ASE.Compiler.Cxx.Macro.Macro()
    headInit.setIdentifier('PyObject_HEAD_INIT(NULL)')
    objStruct = ASE.Compiler.Cxx.Struct.Struct()
    objStruct.setIdentifier(klass.getFullIdentifier().replace('.', '_') + '_PythonObject')
    objDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    objDecl.setType(objStruct)
    typeSize = ASE.Compiler.Cxx.Sizeof.Sizeof()
    typeSize.setTypeName(objDecl)
    weakRefList = ASE.Compiler.Cxx.StructReference.StructReference()
    weakRefList.setChildren([self.Cxx.getGroup(self.Cxx.castToType(self.Cxx.getInteger(0), self.Python.getSIDLObjectType(klass)))])
    weakRefList.setMember('weakRefList')
    weakRefList.setPointer(1)
    weakRefListAddr = ASE.Compiler.Cxx.Address.Address()
    weakRefListAddr.setChildren([weakRefList])
    typeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    typeInit.setList(1)
    typeInit.setChildren([headInit, self.Cxx.getInteger(0), self.Cxx.castToString(klass.getFullIdentifier()), typeSize, self.Cxx.getInteger(0), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_dealloc'), 'destructor'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'printfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'getattrfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'setattrfunc'), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_cmp'), 'cmpfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'reprfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyNumberMethods', 1), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PySequenceMethods', 1), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyMappingMethods', 1), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_hash'), 'hashfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'ternaryfunc'), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_str'), 'reprfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'getattrofunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'setattrofunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyBufferProcs', 1), flags, self.Cxx.castToString('This is a Python wrapper for a SIDL object of type ' + klass.getFullIdentifier()), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'traverseproc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'inquiry'), self.Cxx.castToNamedType(self.Cxx.getVar('sidl_richcmp'), 'richcmpfunc'), self.Cxx.castToType(weakRefListAddr, self.Cxx.typeMap['int']), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'getiterfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'iternextfunc'), self.Cxx.getVar('_' + klass.getIdentifier() + 'ObjectMethods'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyMemberDef', 1, isStruct = 1), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'PyGetSetDef', 1, isStruct = 1), self.Cxx.getInteger(0), self.Cxx.getInteger(0), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'descrgetfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'descrsetfunc'), self.Cxx.getInteger(0), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'initproc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'allocfunc'), self.Cxx.castToNamedType(self.Cxx.getVar(constructor), 'newfunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'freefunc'), self.Cxx.castToNamedType(self.Cxx.getInteger(0), 'inquiry')])
    typeDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    typeDecl.setIdentifier('SIDLPythonObjectType')
    typeDecl.setType(self.pyTypeStruct)
    typeDecl.setStatic(1)
    typeDecl.setInitializer(typeInit)
    return functions + [self.Cxx.getDecl(typeDecl)]
  
  def getConstructor(self, interface, method):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.StructReference
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    objVar = self.Cxx.getVar(method.getChildren()[0].getIdentifier())
    self.getMethodPackArguments(method, body)
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      createdVar = self.Cxx.getVar('created')
      body.addDeclarations([self.Cxx.getDeclaration(createdVar, self.IOR.typeMap['bool'])])
      setCreated = ASE.Compiler.Cxx.Assignment.Assignment()
      setCreated.setChildren([createdVar, self.Cxx.getInteger(0)])
      compound1 = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound1.setChildren([self.Cxx.getExpStmt(setCreated)])
      setCreated = ASE.Compiler.Cxx.Assignment.Assignment()
      setCreated.setChildren([createdVar, self.Cxx.getInteger(1)])
      createFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      createFuncPtr.setChildren([self.Cxx.getVar('_epv')])
      createFuncPtr.setMember('createObject')
      createFuncPtr.setPointer(1)
      createFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      createFunc.setChildren([self.Cxx.getGroup(createFuncPtr)])
      createCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      createCall.setChildren([self.Cxx.getGroup(createFunc)])
      createCall.setArguments([self.Cxx.getNull()])
      setObj = ASE.Compiler.Cxx.Assignment.Assignment()
      setObj.setChildren([objVar, createCall])
      compound2 = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound2.setChildren([self.Cxx.getExpStmt(setCreated), self.Cxx.getExpStmt(setObj)])
      checkObj = ASE.Compiler.Cxx.If.If()
      checkObj.setBranch(objVar)
      checkObj.addChildren([compound1, compound2])
      body.addChildren([checkObj])
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([self.Cxx.getVar(method.getReturnParameter().getIdentifier()), objVar])
    body.addChildren([self.Cxx.getExpStmt(setReturn)])
    self.getMethodReturn(method, body)
    if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
      iorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
      iorEpv.setChildren([objVar])
      iorEpv.setMember('d_epv')
      iorEpv.setPointer(1)
      delRefFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      delRefFuncPtr.setChildren([iorEpv])
      delRefFuncPtr.setMember('deleteReference')
      delRefFuncPtr.setPointer(1)
      delRefFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      delRefFunc.setChildren([self.Cxx.getGroup(delRefFuncPtr)])
      iorObj = ASE.Compiler.Cxx.StructReference.StructReference()
      iorObj.setChildren([objVar])
      iorObj.setMember('d_object')
      iorObj.setPointer(1)
      delRefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      delRefCall.setChildren([self.Cxx.getGroup(delRefFunc)])
      delRefCall.setArguments([iorObj, self.Cxx.getNull()])
      compound1 = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
      compound1.setChildren([self.Cxx.getExpStmt(delRefCall)])
      checkRef = ASE.Compiler.Cxx.If.If()
      checkRef.setBranch(createdVar)
      checkRef.addChildren([compound1])
      children = list(body.getChildren())
      children.insert(-1, checkRef)
      body.setChildren(children)
    typeParam = ASE.Compiler.Cxx.Parameter.Parameter()
    typeParam.setIdentifier('_type')
    typeParam.setType(self.pyObjType)
    argsParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argsParam.setIdentifier('_args')
    argsParam.setType(self.pyObjType)
    kwdictParam = ASE.Compiler.Cxx.Parameter.Parameter()
    kwdictParam.setIdentifier('_kwdict')
    kwdictParam.setType(self.pyObjType)
    name = ASE.Compiler.Cxx.Declarator.Declarator()
    name.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '_' + interface.getIdentifier())
    constructor = ASE.Compiler.Cxx.Function.Function()
    constructor.setChildren([name])
    constructor.setType(self.pyObjType)
    constructor.setParameters([typeParam, argsParam, kwdictParam])
    constructor.setStatic(1)
    constructor.setBody(body)
    return [constructor]

  def getWeakReferenceMethod(self, interface):
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    retParam = ASE.Compiler.SIDL.Parameter.Parameter()
    retParam.setIdentifier('_return_value')
    retParam.setType(interface)
    retParam.setAccessMode(ASE.Compiler.ParameterAccessMode.RETURN)
    retParam.setRequired(1)
    method = ASE.Compiler.SIDL.Method.Method()
    method.setIdentifier('weakReference')
    method.setReturnParameter(retParam)
    method.setExceptions([self.SIDL.getBaseException(self.SIDL.getASEException(interface))])
    return method

  def getWeakReference(self, interface, method):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Return
    returnVar = self.Cxx.getVar('_pyReturn')
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    self.getMethodIORPointer(interface, method, body)
    self.getMethodSIDLObjectCheck(method.getIdentifier(), body, returnVar)
    body.addChildren([self.Python.getErrorCheck('PyExc_TypeError', 'Null IOR pointer in ' + method.getIdentifier(), self.Cxx.getVar('_self_ior'), returnVar)])
    self.getMethodPackArguments(method, body)
    setReturn = ASE.Compiler.Cxx.Assignment.Assignment()
    setReturn.setChildren([self.Cxx.getVar('_return_value'), self.Cxx.getVar('_self_ior')])
    body.addChildren([self.Cxx.getExpStmt(setReturn)])
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.pyObjType)])
    parameter = method.getReturnParameter()
    format = self.Python.getParseFormat(parameter)
    localArgs = self.Python.buildArgumentInterface(parameter, isArgument = 0, isWeak = 1)
    self.Python.getConvertBuildParameter(parameter, body, isArgument = 0)
    buildValueCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    buildValueCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
    buildValueCall.setArguments([self.Cxx.castToString(format)] + localArgs)
    buildValue = ASE.Compiler.Cxx.Assignment.Assignment()
    buildValue.setChildren([returnVar, buildValueCall])
    body.addChildren([self.Cxx.getExpStmt(buildValue, 'Build return values')])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    returnVal.setChildren([returnVar])
    body.addChildren([returnVal])
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    selfParam.setIdentifier('_self')
    selfParam.setType(self.pyObjType)
    argsParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argsParam.setIdentifier('_args')
    argsParam.setType(self.pyObjType)
    kwdictParam = ASE.Compiler.Cxx.Parameter.Parameter()
    kwdictParam.setIdentifier('_kwdict')
    kwdictParam.setType(self.pyObjType)
    name = ASE.Compiler.Cxx.Declarator.Declarator()
    name.setIdentifier(interface.getFullIdentifier().replace('.', '_') + '_' + method.getIdentifier())
    weakReference = ASE.Compiler.Cxx.Function.Function()
    weakReference.setChildren([name])
    weakReference.setType(self.pyObjType)
    weakReference.setStatic(1)
    weakReference.setParameters([selfParam, argsParam, kwdictParam])
    weakReference.setBody(body)
    return [weakReference]

  def getMethodIORPointer(self, interface, method, body):
    '''
    We should not need to cast here since the pointer should be cast on creation of the stub itself
    '''
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.StructReference
    selfVar = self.Cxx.getVar('_self')
    iorType = self.IOR.getType(interface)
    sidlDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    sidlDecl.setType(self.Python.getSIDLObjectType(interface))
    cast = ASE.Compiler.Cxx.Cast.Cast()
    cast.setTypeName(sidlDecl)
    cast.setChildren([selfVar])
    objVar = ASE.Compiler.Cxx.StructReference.StructReference()
    objVar.setChildren([self.Cxx.getGroup(cast)])
    objVar.setMember('ior')
    objVar.setPointer(1)
    init = ASE.Compiler.Cxx.Initializer.Initializer()
    init.setChildren([objVar])
    ior = ASE.Compiler.Cxx.Declarator.Declarator()
    ior.setIdentifier('_self_ior')
    ior.setType(iorType)
    ior.setInitializer(init)
    body.addDeclarations([self.Cxx.getDecl(ior)])
    return None
  
  def getMethodSIDLObjectCheck(self, methodName, body, returnVar, var = None, errorHandler = None, anySIDLObject = 0):
    import ASE.Compiler.Cxx.Address
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Inequality
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.StructReference
    import ASE.Compiler.Cxx.Return
    if var is None:
      var = self.Cxx.getVar('_self')
    if not anySIDLObject:
      objVar = ASE.Compiler.Cxx.StructReference.StructReference()
      objVar.setChildren([var])
      objVar.setMember('ob_type')
      objVar.setPointer(1)
      pythonType = ASE.Compiler.Cxx.Address.Address()
      pythonType.setChildren([self.Cxx.getVar('SIDLPythonObjectType')])
      branch = ASE.Compiler.Cxx.Inequality.Inequality()
      branch.setChildren([objVar, pythonType])
    else:
      hasAttr = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      hasAttr.setChildren([self.Cxx.getVar('PyObject_HasAttrString')])
      hasAttr.setArguments([var, self.Cxx.castToString('getClassName')])
      branch = ASE.Compiler.Cxx.Negation.Negation()
      branch.setChildren([hasAttr])
    exception = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    exception.setChildren([self.Cxx.getVar('PyErr_SetString')])
    exception.setArguments([self.Cxx.getVar('PyExc_TypeError'), self.Cxx.getString('Could not parse arguments for ' + methodName)])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    if not returnVar is None:
      returnVal.setChildren([returnVar])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(exception), returnVal])
    typeCheck = ASE.Compiler.Cxx.If.If()
    if not errorHandler is None:
      typeCheck.setChildren([errorHandler])
    else:
      typeCheck.setChildren([compound])
    typeCheck.setBranch(branch)
    typeCheck.setComments([self.Cxx.getComment('Check for a valid SIDL Object')])
    body.addChildren([typeCheck])
    return 
  
  def getMethodPackArguments(self, method, body):
    '''
    This method
       - Declares keyword arguments and check variables
       - Builds format and parse arguments
       - Parses Python arguments
       - Declares Python arguments and IOR arguments, and adds conversion code
       - Adds any conversion code
       - Builds Python arguments
    '''
    import ASE.Compiler.Cxx.Array
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Negation
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.Type
    entry = ASE.Compiler.Cxx.Type.Type()
    entry.setIdentifier('char')
    entry.setBaseType(1)
    entryType = ASE.Compiler.Cxx.Pointer.Pointer()
    entryType.setChildren([entry])
    kwInit = ASE.Compiler.Cxx.Initializer.Initializer()
    kwInit.setChildren([self.Cxx.castToString(a.getIdentifier()) for a in method.getInParameters()] + [self.Cxx.getVar('NULL')])
    kwInit.setList(1)
    kwDecl = ASE.Compiler.Cxx.Array.Array()
    kwDecl.setChildren([self.Cxx.getVar('_kwlist')])
    kwDecl.setType(entryType)
    kwDecl.setStatic(1)
    kwDecl.setInitializer(kwInit)
    body.addDeclarations([self.Cxx.getDecl(kwDecl)])
    optionalArgs = 0
    format = ''
    localArgs = []
    for parameter in method.getInParameters():
      if not optionalArgs:
        if not parameter.getRequired():
          format += '|'
          optionalArgs = 1
      elif parameter.getRequired():
        self.raiseException('Invalid required argument ' + parameter.getIdentifier() + ' after optional arguments')
      format += self.Python.getParseFormat(parameter)
      localArgs.extend(self.Python.getParseArguments(parameter, isArgument = 0))
    checkVar = self.Cxx.getVar('_okay')
    parseCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    parseCall.setChildren([self.Cxx.getVar('PyArg_ParseTupleAndKeywords')])
    parseCall.setArguments([self.Cxx.getVar('_args'), self.Cxx.getVar('_kwdict'), self.Cxx.castToString(format), self.Cxx.getVar('_kwlist')] + localArgs)
    parseExp = ASE.Compiler.Cxx.Assignment.Assignment()
    parseExp.setChildren([checkVar, parseCall])
    body.addDeclarations([self.Cxx.getDeclaration(checkVar, self.Python.typeMap['bool'])])
    body.addChildren([self.Cxx.getExpStmt(parseExp, 'Parse arguments')])
    for parameter in method.getInParameters():
      self.Python.getConvertParseArgument(parameter, body, isArgument = 0)
    checkExp = ASE.Compiler.Cxx.Negation.Negation()
    checkExp.setChildren([checkVar])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    returnVal.setChildren([self.Cxx.getVar('_pyReturn')])
    returnVal.setComments([self.Cxx.getComment('Let exceptions propagate')])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([returnVal])
    check = ASE.Compiler.Cxx.If.If()
    check.setBranch(checkExp)
    check.setChildren([compound])
    body.addChildren([check])
    return None
  
  def getMethodDispatch(self, interface, method, body):
    import ASE.Compiler.Cxx.Address
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    args = []
    if interface is None:
      iorCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      iorCall.setChildren([self.Cxx.getVar(method.getIdentifier() + '_impl')])
    elif method.getStatic():
      iorMethodPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      iorMethodPtr.setChildren([self.Cxx.getVar('_sepv')])
      iorMethodPtr.setMember(method.getIdentifier())
      iorMethodPtr.setPointer(1)
      iorMethod = ASE.Compiler.Cxx.Indirection.Indirection()
      iorMethod.setChildren([self.Cxx.getGroup(iorMethodPtr)])
      iorCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      iorCall.setChildren([self.Cxx.getGroup(iorMethod)])
    else:
      selfVar = self.Cxx.getVar('_self_ior')
      iorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
      iorEpv.setChildren([selfVar])
      iorEpv.setMember('d_epv')
      iorEpv.setPointer(1)
      iorMethodPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      iorMethodPtr.setChildren([iorEpv])
      iorMethodPtr.setMember(method.getIdentifier())
      iorMethodPtr.setPointer(1)
      iorMethod = ASE.Compiler.Cxx.Indirection.Indirection()
      iorMethod.setChildren([self.Cxx.getGroup(iorMethodPtr)])
      iorCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      iorCall.setChildren([self.Cxx.getGroup(iorMethod)])
      if interface.isInstanceOf('ASE.Compiler.SIDL.Class'):
        args.append(selfVar)
      else:
        trueIOR = ASE.Compiler.Cxx.StructReference.StructReference()
        trueIOR.setChildren([selfVar])
        trueIOR.setMember('d_object')
        trueIOR.setPointer(1)
        args.append(trueIOR)
    for parameter in [ASE.Compiler.SIDL.Parameter.Parameter(p) for p in method.getChildren()]:
      if self.isOut(parameter):
        arg = ASE.Compiler.Cxx.Address.Address()
        arg.setChildren([self.Cxx.getVar(parameter.getIdentifier())])
        args.append(arg)
      else:
        args.append(self.Cxx.getVar(parameter.getIdentifier()))
    if not interface is None and interface.getFullIdentifier() == 'ASE.BaseInterface':
      args.append(self.Cxx.getNull())
    else:
      excArg = ASE.Compiler.Cxx.Address.Address()
      excArg.addChildren([self.Cxx.getVar('_exception')])
      args.append(excArg)
    iorCall.setArguments(args)
    if not method.getReturnParameter() is None:
      setResult = ASE.Compiler.Cxx.Assignment.Assignment()
      setResult.setChildren([self.Cxx.getVar(method.getReturnParameter().getIdentifier()), iorCall])
      body.addChildren([self.Cxx.getExpStmt(setResult, 'Dispatch to IOR')])
    else:
      body.addChildren([self.Cxx.getExpStmt(iorCall, 'Dispatch to IOR')])
    return None
  
  def throwDefaultException(self, interface, method, excVar):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    excObj = ASE.Compiler.Cxx.StructReference.StructReference()
    excObj.setChildren([excVar])
    excObj.setMember('d_object')
    excObj.setPointer(1)
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    baseEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    baseEpv.setChildren([self.Cxx.getGroup(self.Cxx.castToType(excObj, self.IOR.getType(self.SIDL.getBaseInterface(interface))))])
    baseEpv.setMember('d_epv')
    baseEpv.setPointer(1)
    castFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
    castFuncPtr.setChildren([baseEpv])
    castFuncPtr.setMember('castObject')
    castFuncPtr.setPointer(1)
    castFunc = ASE.Compiler.Cxx.Indirection.Indirection()
    castFunc.setChildren([castFuncPtr])
    baseExcType = self.SIDL.getBaseException(method.getExceptions()[0])
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFunc)])
    castCall.setArguments([excObj, self.Cxx.getString(baseExcType.getFullIdentifier()), self.Cxx.getNull()])
    resetExc = ASE.Compiler.Cxx.Assignment.Assignment()
    resetExc.setChildren([excVar, castCall])
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([excVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    messageFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    messageFunc.setChildren([excEpv])
    messageFunc.setMember('getMessage')
    messageFunc.setPointer(1)
    getMessage = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getMessage.setChildren([messageFunc])
    getMessage.setArguments([excObj, self.Cxx.getNull()])
    setExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setExc.setChildren([self.Cxx.getVar('PyErr_SetString')])
    setExc.setArguments([self.Cxx.getVar('PyExc_RuntimeError'), getMessage])
    compound.addChildren(map(self.Cxx.getExpStmt, [resetExc, setExc]))
    return compound
  
  def throwException(self, interface, exception, excVar, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.StructReference
    moduleVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_module')
    pythonTypeVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')
    objVar = self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_obj')
    pyObjVar = self.Cxx.getVar('obj')
    moduleCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleCall.setChildren([self.Cxx.getVar('PyImport_ImportModule')])
    moduleCall.setArguments([self.Cxx.castToString(exception.getFullIdentifier())])
    moduleInit = ASE.Compiler.Cxx.Initializer.Initializer()
    moduleInit.setChildren([moduleCall])
    moduleDict = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleDict.setChildren([self.Cxx.getVar('PyModule_GetDict')])
    moduleDict.setArguments([moduleVar])
    typeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    typeCall.setChildren([self.Cxx.getVar('PyMapping_GetItemString')])
    typeCall.setArguments([moduleDict, self.Cxx.castToString('Exception')])
    typeInit = ASE.Compiler.Cxx.Initializer.Initializer()
    typeInit.setChildren([typeCall])
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([excVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([excEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    castFuncInd.setChildren([self.Cxx.getGroup(castFunc)])
    excObj = ASE.Compiler.Cxx.StructReference.StructReference()
    excObj.setChildren([excVar])
    excObj.setMember('d_object')
    excObj.setPointer(1)
    castCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castCall.setChildren([self.Cxx.getGroup(castFuncInd)])
    castCall.setArguments([excObj, self.Cxx.getString(exception.getFullIdentifier()), self.Cxx.getNull()])
    objInit = ASE.Compiler.Cxx.Initializer.Initializer()
    objInit.setChildren([castCall])
    body.addDeclarations([self.Cxx.getDeclaration(moduleVar, self.pyObjType, moduleInit), self.Cxx.getDeclaration(pythonTypeVar, self.pyObjType, typeInit), self.Cxx.getDeclaration(objVar, self.IOR.getType(exception), objInit)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    createObj = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    createObj.setChildren([self.Python.getWrapIOR(exception).getChildren()[0]])
    createObj.setArguments([objVar])
    pyObjInit = ASE.Compiler.Cxx.Initializer.Initializer()
    pyObjInit.setChildren([createObj])
    compound.addDeclarations([self.Cxx.getDeclaration(pyObjVar, self.pyObjType, pyObjInit)])
    setExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setExc.setChildren([self.Cxx.getVar('PyErr_SetObject')])
    setExc.setArguments([pythonTypeVar, pyObjVar])
    compound.addChildren([self.Cxx.getExpStmt(setExc)])
    checkExcType = ASE.Compiler.Cxx.If.If()
    checkExcType.setBranch(objVar)
    checkExcType.setChildren([compound])
    return checkExcType
  
  def getMethodHandleExceptions(self, interface, method, body):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Return
    if interface is None:
      return None
    excVar = self.Cxx.getVar('_exception')
    body.addDeclarations([self.Cxx.getDeclaration(excVar, self.IOR.getType(self.SIDL.getBaseException(method.getExceptions()[0])))])
    excBody = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    checkExcType = None
    for exception in method.getExceptions():
      check = self.throwException(interface, exception, excVar, excBody)
      if checkExcType is None:
        checkExcType = check
        fullExcCheck = checkExcType
      else:
        checkExcType.addChildren([check])
        checkExcType = check
    checkExcType.addChildren([self.throwDefaultException(interface, method, excVar)])
    excBody.addChildren([fullExcCheck])
    for exception in method.getExceptions():
      decrefModule = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefModule.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefModule.setArguments([self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_module')])
      decrefType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      decrefType.setChildren([self.Cxx.getVar('Py_DECREF')])
      decrefType.setArguments([self.Cxx.getVar(exception.getFullIdentifier().replace('.', '_') + '_type')])
      excBody.addChildren(map(self.Cxx.getExpStmt, [decrefModule, decrefType]))
    returnExc = ASE.Compiler.Cxx.Return.Return()
    returnExc.setChildren([self.Cxx.getVar('_pyReturn')])
    excBody.addChildren([returnExc])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([excBody])
    body.addChildren([checkExc])
    return None
  
  def getMethodReturn(self, method, body):
    '''
    This method:
       - Declares the Python and IOR return values
       - Declares local arguments, builds format, and adds any conversion code
       - Builds the Python return value
       - Cleans up arguments
    '''
    import ASE.Compiler.ParameterAccessMode
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.Return
    import ASE.Compiler.Cxx.StructReference
    returnVar = self.Cxx.getVar('_pyReturn')
    body.addDeclarations([self.Cxx.getDeclaration(returnVar, self.pyObjType)])
    format = ''
    localArgs = []
    for parameter in method.getOutParameters():
      format += self.Python.getParseFormat(parameter)
      localArgs.extend(self.Python.getBuildArguments(parameter, isArgument = 0))
    for parameter in method.getOutParameters():
      self.Python.getConvertBuildParameter(parameter, body, isArgument = 0)
    if len(method.getOutParameters()):
      if len(method.getOutParameters()) > 1:
        format = '(' + format + ')'
      buildValueCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      buildValueCall.setChildren([self.Cxx.getVar('Py_BuildValue')])
      buildValueCall.setArguments([self.Cxx.castToString(format)] + localArgs)
      buildValue = ASE.Compiler.Cxx.Assignment.Assignment()
      buildValue.setChildren([returnVar, buildValueCall])
      body.addChildren([self.Cxx.getExpStmt(buildValue, 'Build return values')])
    else:
      buildValue = ASE.Compiler.Cxx.Assignment.Assignment()
      buildValue.setChildren([returnVar, self.Cxx.getVar('Py_None')])
      buildValue.setComments([self.Cxx.getComment('Build return values')])
      incRefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      incRefCall.setChildren([self.Cxx.getVar('Py_INCREF')])
      incRefCall.setArguments([returnVar])
      body.addChildren([self.Cxx.getExpStmt(buildValue), self.Cxx.getExpStmt(incRefCall)])
    for parameter in method.getInParameters():
      self.Python.getCleanupBuildParameter(parameter, body, isArgument = 0)
    parameters = list(method.getChildren())
    if not method.getReturnParameter() is None:
      parameters.append(method.getReturnParameter())
    for child in parameters:
      parameter = ASE.Compiler.SIDL.Parameter.Parameter(child)
      type = parameter.getType()
      var = self.Cxx.getVar(parameter.getIdentifier())
      if self.isOut(parameter) and type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        entryType = ASE.Compiler.SIDL.Array.Array(type).getType()
        if entryType.isInstanceOf('ASE.Compiler.SIDL.Interface') and not entryType.getFullIdentifier() in ['ASE.BaseInterface', 'ASE.BaseClass', 'ASE.DLL', 'ASE.Loader']:
          loaderExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          loaderExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
          staticEPV = ASE.Compiler.Cxx.StructReference.StructReference()
          staticEPV.setChildren([loaderExtCall])
          staticEPV.setMember('getStaticEPV')
          staticEPV.setPointer(1)
          staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          staticEPVCall.setChildren([staticEPV])
          staticEPVCall.setArguments([self.Cxx.getNull()])
          objExternals = ASE.Compiler.Cxx.StructReference.StructReference()
          objExternals.setChildren([staticEPVCall])
          objExternals.setMember('getClassMethodTable')
          objExternals.setPointer(1)
          objExternalsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          objExternalsCall.setChildren([objExternals])
          objExternalsCall.setArguments([self.Cxx.getString(entryType.getFullIdentifier()), self.Cxx.getNull()])
          externalsFuncType = ASE.Compiler.Cxx.Function.Function()
          externalsFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
          externalsFuncType.setType(self.IOR.getExternalType(entryType))
          externalsCast = ASE.Compiler.Cxx.Cast.Cast()
          externalsCast.setChildren([objExternalsCall])
          externalsCast.setTypeName(externalsFuncType)
          objExtFunc = ASE.Compiler.Cxx.Indirection.Indirection()
          objExtFunc.setChildren([self.Cxx.getGroup(externalsCast)])
          objExt = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          objExt.setChildren([self.Cxx.getGroup(objExtFunc)])
        else:
          objExt = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
          objExt.setChildren([self.Cxx.getVar(entryType.getFullIdentifier().replace('.', '_') + '__externals')])
        destroyFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
        destroyFuncPtr.setChildren([objExt])
        destroyFuncPtr.setMember('destroyArray')
        destroyFuncPtr.setPointer(1)
        destroyFunc = ASE.Compiler.Cxx.Indirection.Indirection()
        destroyFunc.setChildren([self.Cxx.getGroup(destroyFuncPtr)])
        deRefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        deRefCall.setChildren([self.Cxx.getGroup(destroyFunc)])
        deRefCall.setArguments([var, self.Cxx.getNull()])
        body.addChildren([self.Cxx.getExpStmt(deRefCall)])
      elif self.isOut(parameter) and type.isInstanceOf('ASE.Compiler.SIDL.Interface'):
        import ASE.Compiler.Cxx.CompoundStatement
        import ASE.Compiler.Cxx.If
        objVar = self.Cxx.castToType(var, self.IOR.getType(self.SIDL.getBaseInterface(type)))
        iorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
        iorEpv.setChildren([self.Cxx.getGroup(objVar)])
        iorEpv.setMember('d_epv')
        iorEpv.setPointer(1)
        addRefFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
        addRefFuncPtr.setChildren([iorEpv])
        addRefFuncPtr.setMember('addReference')
        addRefFuncPtr.setPointer(1)
        addRefFunc = ASE.Compiler.Cxx.Indirection.Indirection()
        addRefFunc.setChildren([self.Cxx.getGroup(addRefFuncPtr)])
        iorObj = ASE.Compiler.Cxx.StructReference.StructReference()
        iorObj.setChildren([self.Cxx.getGroup(objVar)])
        iorObj.setMember('d_object')
        iorObj.setPointer(1)
        addRefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
        addRefCall.setChildren([self.Cxx.getGroup(addRefFunc)])
        addRefCall.setArguments([iorObj, self.Cxx.getNull()])
        compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
        compound.setChildren([self.Cxx.getExpStmt(addRefCall)])
        checkVar = ASE.Compiler.Cxx.If.If()
        checkVar.setBranch(var)
        checkVar.setChildren([compound])
        body.addChildren([checkVar])
    returnVal = ASE.Compiler.Cxx.Return.Return()
    returnVal.setChildren([returnVar])
    body.addChildren([returnVal])
    return None
  
  def getMethodStub(self, interface, method):
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.Parameter
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.Type
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    selfParam = ASE.Compiler.Cxx.Parameter.Parameter()
    if method.getStatic():
      selfParam.setIdentifier('_ignored')
    else:
      selfParam.setIdentifier('_self')
    selfParam.setType(self.pyObjType)
    argsParam = ASE.Compiler.Cxx.Parameter.Parameter()
    argsParam.setIdentifier('_args')
    argsParam.setType(self.pyObjType)
    kwdictParam = ASE.Compiler.Cxx.Parameter.Parameter()
    kwdictParam.setIdentifier('_kwdict')
    kwdictParam.setType(self.pyObjType)
    returnVar = self.Cxx.getVar('_pyReturn')
    if not method.getStatic():
      self.getMethodIORPointer(interface, method, body)
      self.getMethodSIDLObjectCheck(method.getIdentifier(), body, returnVar)
      body.addChildren([self.Python.getErrorCheck('PyExc_TypeError', 'Null IOR pointer in ' + method.getIdentifier(), self.Cxx.getVar('_self_ior'), returnVar)])
    self.getMethodPackArguments(method, body)
    self.getMethodDispatch(interface, method, body)
    self.getMethodHandleExceptions(interface, method, body)
    self.getMethodReturn(method, body)
    if method.getFullIdentifier() == 'ASE.Loader.createClass':
      import ASE.Compiler.Cxx.Indirection
      import ASE.Compiler.Cxx.FunctionCall
      import ASE.Compiler.Cxx.StructReference
      parameter = method.getReturnParameter()
      var = self.Cxx.getVar(parameter.getIdentifier())
      type = parameter.getType()
      objVar = self.Cxx.castToType(var, self.IOR.getType(self.SIDL.getBaseInterface(type)))
      iorEpv = ASE.Compiler.Cxx.StructReference.StructReference()
      iorEpv.setChildren([self.Cxx.getGroup(objVar)])
      iorEpv.setMember('d_epv')
      iorEpv.setPointer(1)
      delRefFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      delRefFuncPtr.setChildren([iorEpv])
      delRefFuncPtr.setMember('deleteReference')
      delRefFuncPtr.setPointer(1)
      delRefFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      delRefFunc.setChildren([self.Cxx.getGroup(delRefFuncPtr)])
      iorObj = ASE.Compiler.Cxx.StructReference.StructReference()
      iorObj.setChildren([self.Cxx.getGroup(objVar)])
      iorObj.setMember('d_object')
      iorObj.setPointer(1)
      delRefCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      delRefCall.setChildren([self.Cxx.getGroup(delRefFunc)])
      delRefCall.setArguments([iorObj, self.Cxx.getNull()])
      children = list(body.getChildren())
      children.insert(-1, self.Cxx.getExpStmt(delRefCall))
      body.setChildren(children)
    name = ASE.Compiler.Cxx.Declarator.Declarator()
    name.setIdentifier(method.getFullIdentifier().replace('.', '_'))
    methodStub = ASE.Compiler.Cxx.Function.Function()
    methodStub.setChildren([name])
    methodStub.setType(self.pyObjType)
    methodStub.setStatic(1)
    methodStub.setParameters([selfParam, argsParam, kwdictParam])
    methodStub.setBody(body)
    return [methodStub]
  
  def getMethodDescriptor(self, method):
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.InclusiveOr
    import ASE.Compiler.Cxx.Initializer
    import ASE.Compiler.Cxx.Type
    funcType = ASE.Compiler.Cxx.Type.Type()
    funcType.setIdentifier('PyCFunction')
    funcDecl = ASE.Compiler.Cxx.Declarator.Declarator()
    funcDecl.setType(funcType)
    cast = ASE.Compiler.Cxx.Cast.Cast()
    cast.setChildren([self.Cxx.getVar(method.getFullIdentifier().replace('.', '_'))])
    cast.setTypeName(funcDecl)
    orExp = ASE.Compiler.Cxx.InclusiveOr.InclusiveOr()
    orExp.setChildren([self.Cxx.getVar('METH_VARARGS'), self.Cxx.getVar('METH_KEYWORDS')])
    if method.getStatic():
      orExp2 = ASE.Compiler.Cxx.InclusiveOr.InclusiveOr()
      orExp2.setChildren([orExp, self.Cxx.getVar('METH_STATIC')])
      orExp = orExp2
    methodDesc = ASE.Compiler.Cxx.Initializer.Initializer()
    methodDesc.setChildren([self.Cxx.castToString(method.getIdentifier()), cast, self.Cxx.getGroup(orExp), self.Cxx.castToString(self.getMethodComment(method))])
    methodDesc.setList(1)
    return [methodDesc]
  
  def setupPythonType(self, body):
    import ASE.Compiler.Cxx.Address
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.LessThan
    import ASE.Compiler.Cxx.Return
    pythonType = ASE.Compiler.Cxx.Address.Address()
    pythonType.setChildren([self.Cxx.getVar('SIDLPythonObjectType')])
    readyTypeCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    readyTypeCall.setChildren([self.Cxx.getVar('PyType_Ready')])
    readyTypeCall.setArguments([pythonType])
    readyTypeCheck = ASE.Compiler.Cxx.LessThan.LessThan()
    readyTypeCheck.setChildren([readyTypeCall, self.Cxx.getInteger(0)])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([ASE.Compiler.Cxx.Return.Return()])
    readyType = ASE.Compiler.Cxx.If.If()
    readyType.setBranch(readyTypeCheck)
    readyType.setChildren([compound])
    readyType.setComments([self.Cxx.getComment('Setup the python type')])
    body.addChildren([readyType])
    return pythonType
  
  def initializeModule(self, klass, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    modVar = self.Cxx.getVar('module')
    modDecl = self.Cxx.getDeclaration(modVar, self.pyObjType)
    body.addDeclarations([modDecl])
    moduleInit = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    moduleInit.setArguments([self.Cxx.castToString(klass.getIdentifier()), self.Cxx.getVar('_' + klass.getIdentifier() + 'ModuleMethods'), self.Cxx.castToString(self.getObjectComment(klass))])
    moduleInit.setChildren([self.Cxx.getVar('Py_InitModule3')])
    setModule = ASE.Compiler.Cxx.Assignment.Assignment()
    setModule.setChildren([modVar, moduleInit])
    body.addChildren([self.Cxx.getExpStmt(setModule, 'Initialize the module'), self.Python.getErrorCheck('PyExc_ImportError', 'Could not initialize module ' + klass.getFullIdentifier())])
    return None
  
  def getIORExternals(self, klass, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.Cast
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Pointer
    import ASE.Compiler.Cxx.StructReference
    excVar = self.Cxx.getVar('_exception')
    extFuncVar = self.Cxx.getVar('_extFunc')
    extFuncPtr = ASE.Compiler.Cxx.Declarator.Declarator()
    extFuncPtr.setIdentifier(extFuncVar.getIdentifier())
    extFuncPtr.setType(ASE.Compiler.Cxx.Pointer.Pointer())
    extFuncDecl = ASE.Compiler.Cxx.Function.Function()
    extFuncDecl.setChildren([self.Cxx.getDeclGroup(extFuncPtr)])
    extFuncDecl.setType(self.IOR.getExternalType(klass))
    body.addDeclarations([self.Cxx.getDecl(extFuncDecl)])
    extFuncType = ASE.Compiler.Cxx.Function.Function()
    extFuncType.setChildren([self.Cxx.getDeclGroup(ASE.Compiler.Cxx.Pointer.Pointer())])
    extFuncType.setType(self.IOR.getExternalType(klass))
    sidlExtCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    sidlExtCall.setChildren([self.Cxx.getVar('ASE_Loader__externals')])
    staticEPV = ASE.Compiler.Cxx.StructReference.StructReference()
    staticEPV.setChildren([sidlExtCall])
    staticEPV.setMember('getStaticEPV')
    staticEPV.setPointer(1)
    staticEPVCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    staticEPVCall.setChildren([staticEPV])
    staticEPVCall.setArguments([self.Cxx.getAddress(excVar)])
    classMethods = ASE.Compiler.Cxx.StructReference.StructReference()
    classMethods.setChildren([staticEPVCall])
    classMethods.setMember('getClassMethodTable')
    classMethods.setPointer(1)
    classMethodsCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    classMethodsCall.setChildren([classMethods])
    classMethodsCall.setArguments([self.Cxx.getString(klass.getFullIdentifier()), self.Cxx.getAddress(excVar)])
    extFunc = ASE.Compiler.Cxx.Cast.Cast()
    extFunc.setTypeName(extFuncType)
    extFunc.setChildren([classMethodsCall])
    setExternals = ASE.Compiler.Cxx.Assignment.Assignment()
    setExternals.setChildren([extFuncVar, extFunc])
    body.addDeclarations([self.Cxx.getDeclaration(excVar, self.IOR.getType(self.SIDL.getBaseException(self.SIDL.getASEException(klass))))])
    body.addChildren([self.Cxx.getExpStmt(setExternals, 'Set externals function')])
    return None
  
  def checkException(self, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.StructReference
    excVar = self.Cxx.getVar('_exception')
    excEpv = ASE.Compiler.Cxx.StructReference.StructReference()
    excEpv.setChildren([excVar])
    excEpv.setMember('d_epv')
    excEpv.setPointer(1)
    castFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    castFunc.setChildren([excEpv])
    castFunc.setMember('castObject')
    castFunc.setPointer(1)
    castExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    castExc.setChildren([castFunc])
    castExc.setArguments([excVar, self.Cxx.getString('ASE.BaseException'), self.Cxx.getNull()])
    resetExc = ASE.Compiler.Cxx.Assignment.Assignment()
    resetExc.setChildren([excVar, castExc])
    messageFunc = ASE.Compiler.Cxx.StructReference.StructReference()
    messageFunc.setChildren([excEpv])
    messageFunc.setMember('getMessage')
    messageFunc.setPointer(1)
    excObj = ASE.Compiler.Cxx.StructReference.StructReference()
    excObj.setChildren([excVar])
    excObj.setMember('d_object')
    excObj.setPointer(1)
    getMessage = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    getMessage.setChildren([messageFunc])
    getMessage.setArguments([excObj, self.Cxx.getNull()])
    setExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    setExc.setChildren([self.Cxx.getVar('PyErr_SetString')])
    setExc.setArguments([self.Cxx.getVar('PyExc_RuntimeError'), getMessage])
    compound = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    compound.setChildren([self.Cxx.getExpStmt(resetExc), self.Cxx.getExpStmt(setExc), ASE.Compiler.Cxx.Return.Return()])
    checkExc = ASE.Compiler.Cxx.If.If()
    checkExc.setBranch(excVar)
    checkExc.setChildren([compound])
    checkExc.setComments([self.Cxx.getComment('Check exception')])
    body.addChildren([checkExc])
    return None
  
  def setEpvPointer(self, klass, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.If
    import ASE.Compiler.Cxx.Indirection
    epvVar = self.Cxx.getVar('_epv')
    extFuncVar = self.Cxx.getVar('_extFunc')
    extFuncInd = ASE.Compiler.Cxx.Indirection.Indirection()
    extFuncInd.setChildren([extFuncVar])
    extFuncCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    extFuncCall.setChildren([self.Cxx.getGroup(extFuncInd)])
    assignEpv = ASE.Compiler.Cxx.Assignment.Assignment()
    assignEpv.setChildren([epvVar, extFuncCall])
    assignEpvSt = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    assignEpvSt.setChildren([self.Cxx.getExpStmt(assignEpv)])
    setEpv = ASE.Compiler.Cxx.If.If()
    setEpv.setBranch(extFuncVar)
    setEpv.setChildren([assignEpvSt])
    setEpv.setComments([self.Cxx.getComment('Set epv pointer')])
    body.addChildren([setEpv, self.Python.getErrorCheck('PyExc_ImportError', 'Cannot load SIDL implementation for ' + klass.getFullIdentifier(), epvVar)])
    return None
  
  def setSepvPointer(self, klass, body):
    import ASE.Compiler.Cxx.Assignment
    import ASE.Compiler.Cxx.FunctionCall
    import ASE.Compiler.Cxx.Indirection
    import ASE.Compiler.Cxx.StructReference
    if self.hasStaticMethod(klass):
      epvVar = self.Cxx.getVar('_epv')
      sepvVar = self.Cxx.getVar('_sepv')
      sepvFuncPtr = ASE.Compiler.Cxx.StructReference.StructReference()
      sepvFuncPtr.setChildren([epvVar])
      sepvFuncPtr.setMember('getStaticEPV')
      sepvFuncPtr.setPointer(1)
      sepvFunc = ASE.Compiler.Cxx.Indirection.Indirection()
      sepvFunc.setChildren([self.Cxx.getGroup(sepvFuncPtr)])
      sepvCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      sepvCall.setChildren([self.Cxx.getGroup(sepvFunc)])
      sepvCall.setArguments([self.Cxx.getNull()])
      setSepv = ASE.Compiler.Cxx.Assignment.Assignment()
      setSepv.setChildren([sepvVar, sepvCall])
      body.addChildren([self.Cxx.getExpStmt(setSepv, 'Set sepv pointer'), self.Python.getErrorCheck('PyExc_ImportError', 'Cannot load static methods for ' + klass.getFullIdentifier(), sepvVar)])
    return None
  
  def getModuleInit(self, klass):
    '''
    Generate module initializer
    '''
    import ASE.Compiler.Cxx.CompoundStatement
    import ASE.Compiler.Cxx.Declarator
    import ASE.Compiler.Cxx.Function
    import ASE.Compiler.Cxx.FunctionCall
    modVar = self.Cxx.getVar('module')
    body = ASE.Compiler.Cxx.CompoundStatement.CompoundStatement()
    pythonType = self.setupPythonType(body)
    self.initializeModule(klass, body)
    incRef = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    incRef.setChildren([self.Cxx.getVar('Py_INCREF')])
    incRef.setArguments([pythonType])
    addType = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    addType.setChildren([self.Cxx.getVar('PyModule_AddObject')])
    addType.setArguments([modVar, self.Cxx.castToString(klass.getIdentifier()), self.Cxx.castToType(pythonType, self.pyObjType)])
    body.addChildren([self.Cxx.getExpStmt(incRef, 'Add the type to the module and increment the ref count'), self.Cxx.getExpStmt(addType)])
    self.getIORExternals(klass, body)
    self.checkException(body)
    self.setEpvPointer(klass, body)
    self.setSepvPointer(klass, body)
    if not self.SIDL.getBaseException(klass) is None:
      newExcCall = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      newExcCall.setChildren([self.Cxx.getVar('PyErr_NewException')])
      newExcCall.setArguments([self.Cxx.castToString(klass.getFullIdentifier() + '.Exception'), self.Cxx.getNull(), self.Cxx.getNull()])
      setExc = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
      setExc.setChildren([self.Cxx.getVar('PyModule_AddObject')])
      setExc.setArguments([modVar, self.Cxx.castToString('Exception'), self.Cxx.castToType(newExcCall, self.pyObjType)])
      body.addChildren([self.Cxx.getExpStmt(setExc, 'Create an Exception class')])
    importNumeric = ASE.Compiler.Cxx.FunctionCall.FunctionCall()
    importNumeric.setChildren([self.Cxx.getVar('import_array')])
    body.addChildren([self.Cxx.getExpStmt(importNumeric, 'Import Numeric'), self.Python.getErrorCheck('PyExc_ImportError', 'Cannot import Numeric Python array module')])
    initName = ASE.Compiler.Cxx.Declarator.Declarator()
    initName.setIdentifier('init' + klass.getIdentifier())
    initFunction = ASE.Compiler.Cxx.Function.Function()
    initFunction.setChildren([initName])
    initFunction.setType(self.Python.typeMap['void'])
    initFunction.setBody(body)
    return [self.IOR.getGuardedInclude(self.SIDL.getLoader(klass)), initFunction]
  
  def copyType(self, type):
    if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      newType = ASE.Compiler.SIDL.Array.Array()
      newType.setIdentifier(type.getIdentifier())
      newType.setAttributes(type.getAttributes())
      newType.setLineNumber(type.getLineNumber())
      newType.setComments(type.getComments())
      newType.setChildren([child for child in type.getChildren()])
      newType.setBaseType(type.getBaseType())
      newType.setType(self.copyType(ASE.Compiler.SIDL.Array.Array(type).getType()))
      return newType
    return type
  
  def copyParameter(self, parameter):
    '''
    This will be replaced by parameter.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Parameter
    newParameter = ASE.Compiler.SIDL.Parameter.Parameter()
    newParameter.setIdentifier(parameter.getIdentifier())
    newParameter.setAttributes(parameter.getAttributes())
    newParameter.setParent(parameter.getParent())
    newParameter.setChildren(parameter.getChildren())
    newParameter.setLineNumber(parameter.getLineNumber())
    newParameter.setComments(parameter.getComments())
    newParameter.setDependencies(parameter.getDependencies())
    newParameter.setType(self.copyType(parameter.getType()))
    newParameter.setAccessMode(parameter.getAccessMode())
    newParameter.setRequired(parameter.getRequired())
    return newParameter
  
  def copyMethod(self, method):
    '''
    This will be replaced by method.copy() when I implement it
       - There is a complication, in that I do not want a deep copy of resolved types
    '''
    import ASE.Compiler.SIDL.Method
    import ASE.Compiler.SIDL.Parameter
    newMethod = ASE.Compiler.SIDL.Method.Method()
    newMethod.setIdentifier(method.getIdentifier())
    newMethod.setAttributes(method.getAttributes())
    newMethod.setParent(method.getParent())
    newMethod.setChildren([self.copyParameter(ASE.Compiler.SIDL.Parameter.Parameter(child)) for child in method.getChildren()])
    newMethod.setLineNumber(method.getLineNumber())
    newMethod.setComments(method.getComments())
    newMethod.setDependencies(method.getDependencies())
    newMethod.setTemplateIdentifiers(method.getTemplateIdentifiers())
    if not method.getReturnParameter() is None:
      newMethod.setReturnParameter(self.copyParameter(method.getReturnParameter()))
    newMethod.setExceptions(method.getExceptions())
    newMethod.setStatic(method.getStatic())
    newMethod.setInternal(method.getInternal())
    newMethod.setVirtual(method.getVirtual())
    newMethod.setPurpose(method.getPurpose())
    return newMethod
  
  def checkTemplateParameter(self, name, parameter):
    if not parameter is None:
      type = parameter.getType()
      if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
        return True
      elif type.isInstanceOf('ASE.Compiler.SIDL.Array'):
        import ASE.Compiler.SIDL.Array
        type = ASE.Compiler.SIDL.Array.Array(parameter.getType()).getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.TypeUniverse') and type.getIdentifier() == name:
          return True
    return False
  
  def insertType(self, parameter, type):
    if parameter.getType().isInstanceOf('ASE.Compiler.SIDL.Array'):
      import ASE.Compiler.SIDL.Array
      self.insertType(ASE.Compiler.SIDL.Array.Array(parameter.getType()), type)
    else:
      parameter.setType(type)
    return 
  
  def insertTypes(self, method, universe, type):
    import ASE.Compiler.SIDL.Parameter
    name = universe.getIdentifier()
    newMethod = self.copyMethod(method)
    newMethod.setIdentifier(newMethod.getIdentifier() + '_' + type.getFullIdentifier())
    if self.checkTemplateParameter(name, method.getReturnParameter()):
      self.insertType(newMethod.getReturnParameter(), type)
    for (child, newChild) in zip(method.getChildren(), newMethod.getChildren()):
      if self.checkTemplateParameter(name, ASE.Compiler.SIDL.Parameter.Parameter(child)):
        self.insertType(ASE.Compiler.SIDL.Parameter.Parameter(newChild), type)
    return newMethod
  
  def template(self, method, name):
    import ASE.Compiler.SIDL.Parameter
    import ASE.Compiler.SIDL.TypeUniverse
    universe = None
    for parameter in [method.getReturnParameter()] + [ASE.Compiler.SIDL.Parameter.Parameter(child) for child in method.getChildren()]:
      if self.checkTemplateParameter(name, parameter):
        type = parameter.getType()
        if type.isInstanceOf('ASE.Compiler.SIDL.Array'):
          import ASE.Compiler.SIDL.Array
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(ASE.Compiler.SIDL.Array.Array(type).getType())
        else:
          universe = ASE.Compiler.SIDL.TypeUniverse.TypeUniverse(type)
        break
    if universe is None:
      raise RuntimeError('Could not find type universe for template ' + name + ' in method ' + method.getFullIdentifier())
    self.templateMethods[method] = [self.insertTypes(method, universe, type) for type in universe.getTypes()]
    return None
  
  def createTemplateMethod(self, method):
    if len(method.getTemplateIdentifiers()):
      if len(method.getTemplateIdentifiers()) > 1:
        raise RuntimeError('Can currently only handle a single template identifier')
      if not method in self.templateMethods:
        self.template(method, method.getTemplateIdentifiers()[0])
    return None
  
  def getTemplateMethods(self, methods):
    for method in methods:
      if method in self.templateMethods:
        for m in self.templateMethods[method]:
          yield m
      else:
        yield method
    return 

class Stub(object):
  
  def __init__(self):
    return None
