# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.Attribute
import ASE.Compiler.Comment
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Templated
import ASE.Compiler.SIDL.Vertex
import ASE.Compiler.SIDL.Visitor
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class Class:
  '''
  A SIDL class
  The children of this node are methods defined in the class
  The parents are now superclasses
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.interfaces = []
    self.implementedMethods = []
    self.templateIds = []
  
  def getTemplateIdentifiers(self):
    '''
    Get the identifiers which are quantified over types
    '''
    return self.templateIds
  
  def setTemplateIdentifiers(self, templateIds):
    '''
    Set the identifiers which are quantified over types
    '''
    self.templateIds = templateIds
  
  def accept(self, visitor):
    '''
    Overridden to dispatch correctly
    '''
    visitor.visitClass(self.__IORself)
  
  def getInterfaces(self):
    '''
    Retrieve the interfaces implemented by this class
    '''
    return self.interfaces
  
  def setInterfaces(self, interfaces):
    '''
    Set the interfaces implemented by this class
    '''
    self.interfaces = list(interfaces)
  
  def addInterfaces(self, interfaces):
    '''
    Add to the interfaces implemented by this class
    '''
    self.interfaces.extend(interfaces)
  
  def removeInterfaces(self, interfaces):
    '''
    Remove from the interfaces implemented by this class
    Throws an exception if an interface does not exist
    '''
    for interface in interfaces:
      if interface in self.interfaces:
        self.interfaces.remove(interface)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Interface does not exist: ' + interface.getIdentifier())
        raise e
  
  def getImplementedMethods(self):
    '''
    Retrieve the set of methods implemented by this class
    '''
    return self.implementedMethods
  
  def setImplementedMethods(self, methods):
    '''
    Set the set of methods implemented by this class
    '''
    self.implementedMethods = methods
  
  def addImplementedMethods(self, methods):
    '''
    Add to the set of methods implemented by this class
    Method uniqueness is decided only the the method name, keeping the oldest one
    '''
    for method in methods:
      found = 0
      for m in self.implementedMethods:
        if m.getIdentifier() == method.getIdentifier():
          found = 1
          break
      if not found:
        self.implementedMethods.append(method)
  
  def removeImplementedMethods(self, methods):
    '''
    Remove from the set of methods implemented by this class
    Throws an exception if a method does not exist
    '''
    for implementedMethod in methods:
      if implementedMethod in self.implementedMethods:
        self.implementedMethods.remove(implementedMethod)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Implemented method does not exist: ' + implementedMethod.getIdentifier())
        raise e
  # Ending class methods section
  pass
