# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.Attribute
import ASE.Compiler.Comment
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Templated
import ASE.Compiler.SIDL.Vertex
import ASE.Compiler.SIDL.Visitor
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class Method:
  '''
  A SIDL method
  The children of this node are the method parameters
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
    self.returnParameter = None
    self.exceptions = []
    self.isStatic = 0
    self.isInternal = 0
    self.templateIds = []
    self.isVirtual = 0
    self.purpose = []
  
  def getTemplateIdentifiers(self):
    '''
    Get the identifiers which are quantified over types
    '''
    return self.templateIds
  
  def setTemplateIdentifiers(self, templateIds):
    '''
    Set the identifiers which are quantified over types
    '''
    self.templateIds = templateIds
  
  def accept(self, visitor):
    '''
    Overridden to dispatch correctly
    '''
    visitor.visitMethod(self.__IORself)
  
  def getFullIdentifier(self):
    '''
    Need to override this to calculate the fully qualified method name
    '''
    id = self.__IORself.getIdentifier()
    parent = self.__IORself.getParent()
    if not parent is None:
      return ASE.Compiler.SIDL.Vertex.Vertex(parent).getFullIdentifier() + '.' + id
    return id
  
  def getReturnParameter(self):
    '''
    Retrieve the return parameter
    '''
    return self.returnParameter
  
  def setReturnParameter(self, parameter):
    '''
    Set the return parameter
    '''
    self.returnParameter = parameter
  
  def getExceptions(self):
    '''
    Retrieve the exceptions thrown by the method
    '''
    return self.exceptions
  
  def setExceptions(self, exceptions):
    '''
    Set the exceptions thrown by the method
    '''
    self.exceptions = [exception.weakReference() for exception in exceptions]
  
  def addExceptions(self, exceptions):
    '''
    Add to the exceptions thrown by the method
    '''
    self.exceptions.extend(exceptions)
  
  def removeExceptions(self, exceptions):
    '''
    Remove from the exceptions thrown by the method
    Throws an exception if an exception does not exist
    '''
    for exception in exceptions:
      if exception in self.exceptions:
        self.exceptions.remove(exception)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Exception does not exist: ' + exception.getIdentifier())
        raise e
  
  def getStatic(self):
    '''
    Retrieve the flag indicating whether the method is static
    '''
    return self.isStatic
  
  def setStatic(self, isStatic):
    '''
    Set the flag indicating whether the method is static
    '''
    self.isStatic = isStatic
  
  def getInternal(self):
    '''
    Retrieve the flag indicating whether the method is not present in the SIDL interface
    '''
    return self.isInternal
  
  def setInternal(self, isInternal):
    '''
    Set the flag indicating whether the method is not present in the SIDL interface
    '''
    self.isInternal = isInternal
  
  def getVirtual(self):
    '''
    Retrieve the flag indicating whether this method may be inherited, or must be implemented by every class
    '''
    return self.isVirtual
  
  def setVirtual(self, isVirtual):
    '''
    Set the flag indicating whether this method may be inherited, or must be implemented by every class
    '''
    self.isVirtual = isVirtual
  
  def getPurpose(self):
    '''
    NOTICE: These should be ASE.Compiler.CodePurpose when this gets fixed
    Retrieve the list of purposes for which this method is defined
    '''
    return self.purpose
  
  def setPurpose(self, purpose):
    '''
    Set the list of purposes for which this method is defined
    '''
    self.purpose = list(purpose)
  
  def addPurpose(self, purpose):
    '''
    Add purposes in which the method is defined
    '''
    self.purpose.extend(purpose)
  
  def removePurpose(self, purpose):
    '''
    Remove purposes in which the method is defined
    Throws an exception if a purpose does not exist
    '''
    for p in purpose:
      if p in self.purpose:
        self.purpose.remove(p)
      else:
        e = ASE.Compiler.CompilerException.CompilerException()
        e.setMessage('Purpose does not exist: ' + p.getIdentifier())
        raise e
  
  def getInParameters(self):
    '''
    Retrieve the parameters whose values will be read on input
    '''
    import ASE.Compiler.ParameterAccessMode
    inParameters = []
    for child in self.__IORself.getChildren():
      parameter = ASE.Compiler.SIDL.Parameter.Parameter(child)
      mode = parameter.getAccessMode()
      if mode == ASE.Compiler.ParameterAccessMode.IN or mode == ASE.Compiler.ParameterAccessMode.INOUT:
        inParameters.append(parameter)
    return inParameters
  
  def getOutParameters(self):
    '''
    Retrieve the parameters which will return values, including the return value
    '''
    import ASE.Compiler.ParameterAccessMode
    outParameters = []
    if not self.__IORself.getReturnParameter() is None:
      outParameters.append(self.__IORself.getReturnParameter())
    for child in self.__IORself.getChildren():
      parameter = ASE.Compiler.SIDL.Parameter.Parameter(child)
      mode = parameter.getAccessMode()
      if mode == ASE.Compiler.ParameterAccessMode.OUT or mode == ASE.Compiler.ParameterAccessMode.INOUT:
        outParameters.append(parameter)
    return outParameters
  # Ending class methods section
  pass
