# Starting imports section
pass
import ASE.BaseException
import ASE.BaseInterface
import ASE.Compiler.CompilerException
import ASE.Compiler.SIDL.Array
import ASE.Compiler.SIDL.Class
import ASE.Compiler.SIDL.DepthFirstVisitor
import ASE.Compiler.SIDL.Enumeration
import ASE.Compiler.SIDL.Enumerator
import ASE.Compiler.SIDL.Interface
import ASE.Compiler.SIDL.Method
import ASE.Compiler.SIDL.Package
import ASE.Compiler.SIDL.Parameter
import ASE.Compiler.SIDL.Specification
import ASE.Compiler.SIDL.Type
import ASE.Compiler.SIDL.TypeUniverse
import ASE.Compiler.Vertex
# Ending imports section
pass
# Starting static methods section
pass
# Ending static methods section
pass

class MethodProcessor:
  '''
  Processing of methods for interfaces and classes, including:
  Determining full method lists
  Determining implemented methods for classes
  Adding special methods
  '''
  # Starting class methods section
  pass
  
  def __init__(self, IORself):
    self.__IORself = IORself
  
  def visitInterface(self, node):
    '''
    Determine the full method list based upon parent interfaces
    '''
    if len(node.getMethods()):
      return None
    for interface in node.getParents():
      interface.accept(self.__IORself)
      node.addMethods(interface.getMethods())
    node.addMethods(node.getChildren())
  
  def visitClass(self, node):
    '''
    Determine the full method list based upon parent classes and implemented interfaces
    Determine the implemented methods
    '''
    if len(node.getImplementedMethods()):
      return None
    from ASE.Compiler.CodePurpose import IMPLEMENTATION, SKELETON
    implMethodNames = [m.getIdentifier() for m in node.getChildren()]
    for klass in node.getParents():
      klass.accept(self.__IORself)
      implMethodNames.extend([m.getIdentifier() for m in klass.getMethods()])
    for interface in node.getInterfaces():
      interface.accept(self.__IORself)
      node.addMethods(interface.getMethods())
      node.addImplementedMethods([m for m in interface.getMethods() if not m.getIdentifier() in implMethodNames])
    for klass in node.getParents():
      node.addMethods(klass.getMethods())
    node.addMethods(node.getChildren())
    node.addImplementedMethods(node.getChildren())
    self.setMethodPurpose(node)
  # Ending class methods section
  pass
  
  def setMethodPurpose(self, klass):
    '''
    Determine where methods are implemented
       - Right now, this is hard coded but we would like to incorporate this into SIDL
    '''
    from ASE.Compiler.CodePurpose import IMPLEMENTATION, SKELETON
    for method in klass.getImplementedMethods():
      method.addPurpose([SKELETON, IMPLEMENTATION])
    if klass.getFullIdentifier() == 'ASE.BaseClass':
      for method in klass.getImplementedMethods():
        if method.getIdentifier() in ['isSame', 'isInstanceOf', 'getSuper']:
          method.removePurpose([SKELETON, IMPLEMENTATION])
        elif method.getIdentifier() in ['getClassName']:
          method.removePurpose([SKELETON, IMPLEMENTATION])
          method.setVirtual(0)
    elif klass.getFullIdentifier() == 'ASE.Loader':
      for method in klass.getImplementedMethods():
        if method.getIdentifier() in ['createClass']:
          method.removePurpose([SKELETON, IMPLEMENTATION])
