#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_Python_Visitor
#include "ASE/Compiler/Python/Visitor_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

#ifndef included_ASE_BaseException
#include "ASE/BaseException_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

static const struct ASE_Compiler_Python_Visitor__external *_epv = NULL;

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_Python_Visitor_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_Python_Visitor__object *ior;
  enum ReferenceType refType;
};

static PyTypeObject SIDLPythonObjectType;

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_Python_Visitor_PythonObject *self = (struct ASE_Compiler_Python_Visitor_PythonObject *) type->tp_alloc(type, 0);
    
    if (self)
    {
      struct ASE_BaseInterface__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseInterface", NULL);
      
      self->ior = (*(ior->d_epv->castObject))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      if (refType == NEW_REF)
      {
        (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_Compiler_Vertex__wrap_ior(struct ASE_Compiler_Vertex__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Vertex");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vertex");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Vertex");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Vertex");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_BaseInterface__wrap_ior(struct ASE_BaseInterface__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseInterface");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseInterface");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseInterface");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseInterface");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_CompilerException__wrap_ior(struct ASE_Compiler_CompilerException__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "CompilerException");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.CompilerException");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.CompilerException");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_BaseException__wrap_ior(struct ASE_BaseException__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseException");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseException");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseException");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseException");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Visitor__wrap_ior(struct ASE_Compiler_Python_Visitor__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Visitor");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Visitor");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Visitor");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Visitor");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_Compiler_Python_Mod__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Mod__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Mod", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Mod");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Return__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Return__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Return", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Return");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Raise__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Raise__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Raise", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Raise");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Sub__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Sub__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Sub", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Sub");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Printnl__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Printnl__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Printnl", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Printnl");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Add__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Add__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Add", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Add");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_ListCompFor__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_ListCompFor__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.ListCompFor", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.ListCompFor");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Dict__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Dict__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Dict", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Dict");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Pass__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Pass__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Pass", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Pass");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_TryExcept__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_TryExcept__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.TryExcept", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.TryExcept");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_GetAttr__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_GetAttr__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.GetAttr", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.GetAttr");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_AssName__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_AssName__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.AssName", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.AssName");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_AssAttr__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_AssAttr__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.AssAttr", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.AssAttr");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_If__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_If__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.If", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.If");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Or__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Or__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Or", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Or");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Function__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Function__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Function", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Function");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Not__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Not__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Not", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Not");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Break__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Break__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Break", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Break");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Bitxor__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Bitxor__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Bitxor", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Bitxor");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Ellipsis__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Ellipsis__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Ellipsis", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Ellipsis");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Bitor__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Bitor__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Bitor", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Bitor");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Bitand__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Bitand__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Bitand", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Bitand");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Subscript__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Subscript__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Subscript", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Subscript");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Compare__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Compare__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Compare", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Compare");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_CallFunc__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_CallFunc__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.CallFunc", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.CallFunc");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_For__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_For__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.For", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.For");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Div__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Div__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Div", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Div");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_AugAssign__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_AugAssign__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.AugAssign", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.AugAssign");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Vertex__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Vertex");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Assign__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Assign__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Assign", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Assign");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Invert__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Invert__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Invert", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Invert");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_UnarySub__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_UnarySub__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.UnarySub", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.UnarySub");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Module__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Module__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Module", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Module");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Power__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Power__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Power", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Power");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_RightShift__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_RightShift__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.RightShift", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.RightShift");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_TryFinally__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_TryFinally__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.TryFinally", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.TryFinally");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Yield__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Yield__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Yield", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Yield");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Visitor__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Visitor__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Visitor", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Visitor");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_While__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_While__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.While", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.While");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Continue__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Continue__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Continue", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Continue");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Discard__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Discard__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Discard", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Discard");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_ListComp__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_ListComp__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.ListComp", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.ListComp");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_PyNone__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_PyNone__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.PyNone", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.PyNone");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Tuple__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Tuple__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Tuple", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Tuple");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Stmt__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Stmt__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Stmt", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Stmt");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_ListCompIf__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_ListCompIf__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.ListCompIf", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.ListCompIf");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_UnaryAdd__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_UnaryAdd__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.UnaryAdd", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.UnaryAdd");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_And__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_And__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.And", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.And");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Const__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Const__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Const", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Const");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Global__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Global__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Global", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Global");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Name__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Name__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Name", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Name");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_From__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_From__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.From", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.From");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Print__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Print__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Print", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Print");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Lambda__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Lambda__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Lambda", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Lambda");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_List__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_List__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.List", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.List");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_AssList__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_AssList__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.AssList", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.AssList");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_AssTuple__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_AssTuple__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.AssTuple", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.AssTuple");
    return 0;
  }
  return 1;
}

static long ASE_BaseInterface__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_BaseInterface__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.BaseInterface", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.BaseInterface");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Slice__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Slice__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Slice", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Slice");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_LeftShift__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_LeftShift__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.LeftShift", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.LeftShift");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Keyword__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Keyword__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Keyword", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Keyword");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Import__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Import__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Import", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Import");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Mul__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Mul__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Mul", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Mul");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Python_Class__extract_ior(struct ASE_Compiler_Python_Visitor_PythonObject *obj, struct ASE_Compiler_Python_Class__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Python.Class", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Python.Class");
    return 0;
  }
  return 1;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static PyObject *ASE_Compiler_Vertex__wrap_array(struct ASE_Compiler_Vertex__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    long dimen = (*_extFunc->getDimen)(array, NULL);
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (array)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Vertex__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Vertex__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static long ASE_Compiler_Vertex__extract_array(PyObject *obj, struct ASE_Compiler_Vertex__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Vertex__object *elem;
      
      if (!ASE_Compiler_Vertex__extract_ior((struct ASE_Compiler_Python_Visitor_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

static PyObject *ASE_BaseInterface_getReferenceCount(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getReferenceCount");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getReferenceCount");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getReferenceCount))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Convert int32_t to long */
  _local__return_value = _return_value;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "l", _local__return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_addReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for addReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in addReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->addReference))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_deleteReference(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for deleteReference");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in deleteReference");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->deleteReference))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isSame(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  int _okay = 0;
  struct ASE_BaseInterface__object *obj = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isSame");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isSame");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_BaseInterface__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isSame))(_self_ior->d_object, obj, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getClassName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getClassName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getClassName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getClassName))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  const char *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getName))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "z", _return_value);
  /* Freeing IOR memory since Python has made a copy of the string */
  free((void *) _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_setName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "name",
    NULL};
  int _okay = 0;
  const char *name = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &name);
  /* Copying the Python string to IOR controlled memory */
  name = name ? strcpy(malloc(strlen(name) + 1), name) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setName))(_self_ior->d_object, name, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_isInstanceOf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "typeName",
    NULL};
  int _okay = 0;
  const char *typeName = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for isInstanceOf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in isInstanceOf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "z", _kwlist, &typeName);
  /* Copying the Python string to IOR controlled memory */
  typeName = typeName ? strcpy(malloc(strlen(typeName) + 1), typeName) : NULL;
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->isInstanceOf))(_self_ior->d_object, typeName, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "i", _return_value);
  return _pyReturn;
}

static PyObject *ASE_BaseInterface_getSuper(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_BaseInterface__object *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getSuper");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getSuper");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getSuper))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_BaseInterface__wrap_ior, _return_value);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Visitor_getRepository(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {NULL};
  int _okay = 0;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_Compiler_Vertex__array *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for getRepository");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in getRepository");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "", _kwlist);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  _return_value = (*(_self_ior->d_epv->getRepository))(_self_ior->d_object, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Vertex__wrap_array, _return_value);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(_return_value, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Visitor_setRepository(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "repository",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Vertex__array *repository = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for setRepository");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in setRepository");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_array, &repository);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->setRepository))(_self_ior->d_object, repository, &_exception);
  if (_exception)
  {
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  (*((*((const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL)))()->destroyArray))(repository, NULL);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Visitor_visitVertex(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "vertex",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Vertex__object *vertex = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitVertex");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitVertex");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Vertex__extract_ior, &vertex);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitVertex))(_self_ior->d_object, vertex, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAdd(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Add__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAdd");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAdd");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Add__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAdd))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAnd(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_And__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAnd");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAnd");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_And__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAnd))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAssAttr(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_AssAttr__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAssAttr");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAssAttr");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_AssAttr__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAssAttr))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAssList(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_AssList__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAssList");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAssList");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_AssList__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAssList))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAssName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_AssName__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAssName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAssName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_AssName__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAssName))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAssTuple(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_AssTuple__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAssTuple");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAssTuple");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_AssTuple__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAssTuple))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAssign(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Assign__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAssign");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAssign");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Assign__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAssign))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitAugAssign(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_AugAssign__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitAugAssign");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitAugAssign");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_AugAssign__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitAugAssign))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitBitand(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Bitand__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitBitand");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitBitand");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Bitand__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitBitand))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitBitor(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Bitor__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitBitor");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitBitor");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Bitor__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitBitor))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitBitxor(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Bitxor__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitBitxor");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitBitxor");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Bitxor__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitBitxor))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitBreak(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Break__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitBreak");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitBreak");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Break__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitBreak))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitCallFunc(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_CallFunc__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitCallFunc");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitCallFunc");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_CallFunc__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitCallFunc))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitClass(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Class__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitClass");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitClass");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Class__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitClass))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitCompare(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Compare__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitCompare");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitCompare");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Compare__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitCompare))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitConst(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Const__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitConst");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitConst");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Const__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitConst))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitPyNone(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_PyNone__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitPyNone");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitPyNone");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_PyNone__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitPyNone))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitContinue(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Continue__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitContinue");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitContinue");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Continue__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitContinue))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitDict(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Dict__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitDict");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitDict");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Dict__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitDict))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitDiscard(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Discard__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitDiscard");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitDiscard");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Discard__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitDiscard))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitDiv(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Div__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitDiv");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitDiv");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Div__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitDiv))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitEllipsis(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Ellipsis__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitEllipsis");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitEllipsis");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Ellipsis__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitEllipsis))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitFor(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_For__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitFor");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitFor");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_For__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitFor))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitListCompFor(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_ListCompFor__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitListCompFor");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitListCompFor");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_ListCompFor__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitListCompFor))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitFunction(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Function__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitFunction");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitFunction");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Function__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitFunction))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitLambda(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Lambda__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitLambda");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitLambda");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Lambda__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitLambda))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitGetAttr(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_GetAttr__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitGetAttr");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitGetAttr");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_GetAttr__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitGetAttr))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitGlobal(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Global__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitGlobal");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitGlobal");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Global__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitGlobal))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitIf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_If__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitIf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitIf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_If__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitIf))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitListCompIf(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_ListCompIf__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitListCompIf");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitListCompIf");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_ListCompIf__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitListCompIf))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitImport(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Import__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitImport");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitImport");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Import__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitImport))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitFrom(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_From__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitFrom");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitFrom");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_From__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitFrom))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitInvert(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Invert__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitInvert");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitInvert");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Invert__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitInvert))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitKeyword(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Keyword__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitKeyword");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitKeyword");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Keyword__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitKeyword))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitLeftShift(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_LeftShift__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitLeftShift");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitLeftShift");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_LeftShift__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitLeftShift))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitList(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_List__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitList");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitList");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_List__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitList))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitListComp(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_ListComp__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitListComp");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitListComp");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_ListComp__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitListComp))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitMod(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Mod__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitMod");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitMod");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Mod__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitMod))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitModule(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Module__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitModule");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitModule");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Module__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitModule))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitMul(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Mul__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitMul");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitMul");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Mul__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitMul))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitName(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Name__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitName");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitName");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Name__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitName))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitNot(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Not__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitNot");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitNot");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Not__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitNot))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitOr(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Or__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitOr");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitOr");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Or__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitOr))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitPass(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Pass__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitPass");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitPass");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Pass__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitPass))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitPower(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Power__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitPower");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitPower");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Power__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitPower))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitPrint(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Print__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitPrint");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitPrint");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Print__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitPrint))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitPrintnl(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Printnl__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitPrintnl");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitPrintnl");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Printnl__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitPrintnl))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitRaise(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Raise__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitRaise");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitRaise");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Raise__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitRaise))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitReturn(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Return__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitReturn");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitReturn");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Return__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitReturn))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitRightShift(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_RightShift__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitRightShift");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitRightShift");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_RightShift__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitRightShift))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitSlice(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Slice__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitSlice");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitSlice");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Slice__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitSlice))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitStmt(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Stmt__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitStmt");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitStmt");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Stmt__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitStmt))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitSub(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Sub__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitSub");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitSub");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Sub__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitSub))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitSubscript(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Subscript__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitSubscript");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitSubscript");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Subscript__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitSubscript))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitTryExcept(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_TryExcept__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitTryExcept");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitTryExcept");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_TryExcept__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitTryExcept))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitTryFinally(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_TryFinally__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitTryFinally");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitTryFinally");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_TryFinally__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitTryFinally))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitTuple(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Tuple__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitTuple");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitTuple");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Tuple__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitTuple))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitUnaryAdd(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_UnaryAdd__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitUnaryAdd");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitUnaryAdd");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_UnaryAdd__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitUnaryAdd))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitUnarySub(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_UnarySub__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitUnarySub");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitUnarySub");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_UnarySub__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitUnarySub))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitWhile(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_While__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitWhile");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitWhile");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_While__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitWhile))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_visitYield(PyObject *_self, PyObject *_args, PyObject *_kwdict)
{
  struct ASE_Compiler_Python_Visitor__object *_self_ior = ((struct ASE_Compiler_Python_Visitor_PythonObject *) _self)->ior;
  static char *_kwlist[] = {
    (char *) "node",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Yield__object *node = NULL;
  struct ASE_BaseException__object *_exception = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Check for a valid SIDL Object */
  if (_self->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for visitYield");
    return _pyReturn;
  }
  if (!_self_ior)
  {
    PyErr_SetString(PyExc_TypeError, "Null IOR pointer in visitYield");
    return _pyReturn;
  }
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "O&", _kwlist, (void *) ASE_Compiler_Python_Yield__extract_ior, &node);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  /* Dispatch to IOR */
  (*(_self_ior->d_epv->visitYield))(_self_ior->d_object, node, &_exception);
  if (_exception)
  {
    PyObject *ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    PyObject *ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    struct ASE_Compiler_CompilerException__object *ASE_Compiler_CompilerException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.Compiler.CompilerException", NULL);
    PyObject *ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    PyObject *ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    struct ASE_BaseException__object *ASE_BaseException_obj = (*(_exception->d_epv->castObject))(_exception->d_object, "ASE.BaseException", NULL);
    
    if (ASE_Compiler_CompilerException_obj)
    {
      PyObject *obj = ASE_Compiler_CompilerException__wrap_ior(ASE_Compiler_CompilerException_obj);
      
      PyErr_SetObject(ASE_Compiler_CompilerException_type, obj);
      Py_DECREF(obj);
    }
    else
    if (ASE_BaseException_obj)
    {
      PyObject *obj = ASE_BaseException__wrap_ior(ASE_BaseException_obj);
      
      PyErr_SetObject(ASE_BaseException_type, obj);
      Py_DECREF(obj);
    }
    else
    {
      _exception = (*((struct ASE_BaseInterface__object *) _exception->d_object)->d_epv->castObject)(_exception->d_object, "ASE.BaseException", NULL);
      PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    }
    Py_DECREF(ASE_Compiler_CompilerException_module);
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_module);
    Py_DECREF(ASE_BaseException_type);
    return _pyReturn;
  }
  _pyReturn = Py_None;
  Py_INCREF(_pyReturn);
  return _pyReturn;
}

static PyObject *ASE_Compiler_Python_Visitor_Visitor(PyObject *_type, PyObject *_args, PyObject *_kwdict)
{
  static char *_kwlist[] = {
    (char *) "obj",
    NULL};
  int _okay = 0;
  struct ASE_Compiler_Python_Visitor__object *obj = NULL;
  PyObject *_pyReturn = NULL;
  struct ASE_Compiler_Python_Visitor__object *_return_value = NULL;
  
  /* Parse arguments */
  _okay = PyArg_ParseTupleAndKeywords(_args, _kwdict, (char *) "|O&", _kwlist, (void *) ASE_Compiler_Python_Visitor__extract_ior, &obj);
  if (!_okay)
  {
    /* Let exceptions propagate */
    return _pyReturn;
  }
  if (obj)
  {
    (*(obj->d_epv->addReference))(obj->d_object, NULL);
  }
  _return_value = obj;
  /* Build return values */
  _pyReturn = Py_BuildValue((char *) "O&", (void *) ASE_Compiler_Python_Visitor__wrap_ior, _return_value);
  return _pyReturn;
}

static struct PyMethodDef _VisitorObjectMethods[] = {
  {(char *) "getReferenceCount", (PyCFunction) ASE_BaseInterface_getReferenceCount, (METH_VARARGS | METH_KEYWORDS), (char *) "getReferenceCount()\n\
RETURNS (return int _return_value)\n\
 Retrieve the intrinsic reference count of the underlying object"},
  {(char *) "addReference", (PyCFunction) ASE_BaseInterface_addReference, (METH_VARARGS | METH_KEYWORDS), (char *) "addReference()\n\
RETURNS None\n\
\n\
 Add one to the intrinsic reference count of the underlying object - Objects continue to exist as long as the reference count is positive - Clients should call this method whenever they create another ongoing reference to an object or interface"},
  {(char *) "deleteReference", (PyCFunction) ASE_BaseInterface_deleteReference, (METH_VARARGS | METH_KEYWORDS), (char *) "deleteReference()\n\
RETURNS None\n\
\n\
 Decrease by one the intrinsic reference count of the underlying object - Delete the object if the reference count is non-positive - Clients should call this method whenever they remove a reference to an object or interface"},
  {(char *) "isSame", (PyCFunction) ASE_BaseInterface_isSame, (METH_VARARGS | METH_KEYWORDS), (char *) "isSame(in ASE.BaseInterface obj)\n\
RETURNS (return bool _return_value)\n\
 Return true if and only if <code>obj</code> refers to the same underlying object as this object"},
  {(char *) "getClassName", (PyCFunction) ASE_BaseInterface_getClassName, (METH_VARARGS | METH_KEYWORDS), (char *) "getClassName()\n\
RETURNS (return string _return_value)\n\
 Return the type name of the underlying class object"},
  {(char *) "getName", (PyCFunction) ASE_BaseInterface_getName, (METH_VARARGS | METH_KEYWORDS), (char *) "getName()\n\
RETURNS (return string _return_value)\n\
 Retrieve the object name"},
  {(char *) "setName", (PyCFunction) ASE_BaseInterface_setName, (METH_VARARGS | METH_KEYWORDS), (char *) "setName(in string name)\n\
RETURNS None\n\
\n\
 Set the object name"},
  {(char *) "isInstanceOf", (PyCFunction) ASE_BaseInterface_isInstanceOf, (METH_VARARGS | METH_KEYWORDS), (char *) "isInstanceOf(in string typeName)\n\
RETURNS (return bool _return_value)\n\
 Return <code>true</code> if this object is an instance of the specified type - This routine will return <code>true</code> if and only if a cast to the specified type would succeed"},
  {(char *) "getSuper", (PyCFunction) ASE_BaseInterface_getSuper, (METH_VARARGS | METH_KEYWORDS), (char *) "getSuper()\n\
RETURNS (return ASE.BaseInterface _return_value)\n\
 Retrieve an object with the parent interface IMPL: Obviously, this must be implemented in the IOR of each interface/class"},
  {(char *) "getRepository", (PyCFunction) ASE_Compiler_Visitor_getRepository, (METH_VARARGS | METH_KEYWORDS), (char *) "getRepository()\n\
RETURNS (return array _return_value)\n\
 Retrieve auxilliary ASTs"},
  {(char *) "setRepository", (PyCFunction) ASE_Compiler_Visitor_setRepository, (METH_VARARGS | METH_KEYWORDS), (char *) "setRepository(in array repository)\n\
RETURNS None\n\
\n\
 Set auxilliary ASTs"},
  {(char *) "visitVertex", (PyCFunction) ASE_Compiler_Visitor_visitVertex, (METH_VARARGS | METH_KEYWORDS), (char *) "visitVertex(in ASE.Compiler.Vertex vertex)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAdd", (PyCFunction) ASE_Compiler_Python_Visitor_visitAdd, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAdd(in ASE.Compiler.Python.Add node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAnd", (PyCFunction) ASE_Compiler_Python_Visitor_visitAnd, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAnd(in ASE.Compiler.Python.And node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAssAttr", (PyCFunction) ASE_Compiler_Python_Visitor_visitAssAttr, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAssAttr(in ASE.Compiler.Python.AssAttr node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAssList", (PyCFunction) ASE_Compiler_Python_Visitor_visitAssList, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAssList(in ASE.Compiler.Python.AssList node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAssName", (PyCFunction) ASE_Compiler_Python_Visitor_visitAssName, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAssName(in ASE.Compiler.Python.AssName node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAssTuple", (PyCFunction) ASE_Compiler_Python_Visitor_visitAssTuple, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAssTuple(in ASE.Compiler.Python.AssTuple node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAssign", (PyCFunction) ASE_Compiler_Python_Visitor_visitAssign, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAssign(in ASE.Compiler.Python.Assign node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitAugAssign", (PyCFunction) ASE_Compiler_Python_Visitor_visitAugAssign, (METH_VARARGS | METH_KEYWORDS), (char *) "visitAugAssign(in ASE.Compiler.Python.AugAssign node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitBitand", (PyCFunction) ASE_Compiler_Python_Visitor_visitBitand, (METH_VARARGS | METH_KEYWORDS), (char *) "visitBitand(in ASE.Compiler.Python.Bitand node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitBitor", (PyCFunction) ASE_Compiler_Python_Visitor_visitBitor, (METH_VARARGS | METH_KEYWORDS), (char *) "visitBitor(in ASE.Compiler.Python.Bitor node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitBitxor", (PyCFunction) ASE_Compiler_Python_Visitor_visitBitxor, (METH_VARARGS | METH_KEYWORDS), (char *) "visitBitxor(in ASE.Compiler.Python.Bitxor node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitBreak", (PyCFunction) ASE_Compiler_Python_Visitor_visitBreak, (METH_VARARGS | METH_KEYWORDS), (char *) "visitBreak(in ASE.Compiler.Python.Break node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitCallFunc", (PyCFunction) ASE_Compiler_Python_Visitor_visitCallFunc, (METH_VARARGS | METH_KEYWORDS), (char *) "visitCallFunc(in ASE.Compiler.Python.CallFunc node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitClass", (PyCFunction) ASE_Compiler_Python_Visitor_visitClass, (METH_VARARGS | METH_KEYWORDS), (char *) "visitClass(in ASE.Compiler.Python.Class node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitCompare", (PyCFunction) ASE_Compiler_Python_Visitor_visitCompare, (METH_VARARGS | METH_KEYWORDS), (char *) "visitCompare(in ASE.Compiler.Python.Compare node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitConst", (PyCFunction) ASE_Compiler_Python_Visitor_visitConst, (METH_VARARGS | METH_KEYWORDS), (char *) "visitConst(in ASE.Compiler.Python.Const node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitPyNone", (PyCFunction) ASE_Compiler_Python_Visitor_visitPyNone, (METH_VARARGS | METH_KEYWORDS), (char *) "visitPyNone(in ASE.Compiler.Python.PyNone node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitContinue", (PyCFunction) ASE_Compiler_Python_Visitor_visitContinue, (METH_VARARGS | METH_KEYWORDS), (char *) "visitContinue(in ASE.Compiler.Python.Continue node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitDict", (PyCFunction) ASE_Compiler_Python_Visitor_visitDict, (METH_VARARGS | METH_KEYWORDS), (char *) "visitDict(in ASE.Compiler.Python.Dict node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitDiscard", (PyCFunction) ASE_Compiler_Python_Visitor_visitDiscard, (METH_VARARGS | METH_KEYWORDS), (char *) "visitDiscard(in ASE.Compiler.Python.Discard node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitDiv", (PyCFunction) ASE_Compiler_Python_Visitor_visitDiv, (METH_VARARGS | METH_KEYWORDS), (char *) "visitDiv(in ASE.Compiler.Python.Div node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitEllipsis", (PyCFunction) ASE_Compiler_Python_Visitor_visitEllipsis, (METH_VARARGS | METH_KEYWORDS), (char *) "visitEllipsis(in ASE.Compiler.Python.Ellipsis node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitFor", (PyCFunction) ASE_Compiler_Python_Visitor_visitFor, (METH_VARARGS | METH_KEYWORDS), (char *) "visitFor(in ASE.Compiler.Python.For node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitListCompFor", (PyCFunction) ASE_Compiler_Python_Visitor_visitListCompFor, (METH_VARARGS | METH_KEYWORDS), (char *) "visitListCompFor(in ASE.Compiler.Python.ListCompFor node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitFunction", (PyCFunction) ASE_Compiler_Python_Visitor_visitFunction, (METH_VARARGS | METH_KEYWORDS), (char *) "visitFunction(in ASE.Compiler.Python.Function node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitLambda", (PyCFunction) ASE_Compiler_Python_Visitor_visitLambda, (METH_VARARGS | METH_KEYWORDS), (char *) "visitLambda(in ASE.Compiler.Python.Lambda node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitGetAttr", (PyCFunction) ASE_Compiler_Python_Visitor_visitGetAttr, (METH_VARARGS | METH_KEYWORDS), (char *) "visitGetAttr(in ASE.Compiler.Python.GetAttr node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitGlobal", (PyCFunction) ASE_Compiler_Python_Visitor_visitGlobal, (METH_VARARGS | METH_KEYWORDS), (char *) "visitGlobal(in ASE.Compiler.Python.Global node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitIf", (PyCFunction) ASE_Compiler_Python_Visitor_visitIf, (METH_VARARGS | METH_KEYWORDS), (char *) "visitIf(in ASE.Compiler.Python.If node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitListCompIf", (PyCFunction) ASE_Compiler_Python_Visitor_visitListCompIf, (METH_VARARGS | METH_KEYWORDS), (char *) "visitListCompIf(in ASE.Compiler.Python.ListCompIf node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitImport", (PyCFunction) ASE_Compiler_Python_Visitor_visitImport, (METH_VARARGS | METH_KEYWORDS), (char *) "visitImport(in ASE.Compiler.Python.Import node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitFrom", (PyCFunction) ASE_Compiler_Python_Visitor_visitFrom, (METH_VARARGS | METH_KEYWORDS), (char *) "visitFrom(in ASE.Compiler.Python.From node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitInvert", (PyCFunction) ASE_Compiler_Python_Visitor_visitInvert, (METH_VARARGS | METH_KEYWORDS), (char *) "visitInvert(in ASE.Compiler.Python.Invert node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitKeyword", (PyCFunction) ASE_Compiler_Python_Visitor_visitKeyword, (METH_VARARGS | METH_KEYWORDS), (char *) "visitKeyword(in ASE.Compiler.Python.Keyword node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitLeftShift", (PyCFunction) ASE_Compiler_Python_Visitor_visitLeftShift, (METH_VARARGS | METH_KEYWORDS), (char *) "visitLeftShift(in ASE.Compiler.Python.LeftShift node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitList", (PyCFunction) ASE_Compiler_Python_Visitor_visitList, (METH_VARARGS | METH_KEYWORDS), (char *) "visitList(in ASE.Compiler.Python.List node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitListComp", (PyCFunction) ASE_Compiler_Python_Visitor_visitListComp, (METH_VARARGS | METH_KEYWORDS), (char *) "visitListComp(in ASE.Compiler.Python.ListComp node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitMod", (PyCFunction) ASE_Compiler_Python_Visitor_visitMod, (METH_VARARGS | METH_KEYWORDS), (char *) "visitMod(in ASE.Compiler.Python.Mod node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitModule", (PyCFunction) ASE_Compiler_Python_Visitor_visitModule, (METH_VARARGS | METH_KEYWORDS), (char *) "visitModule(in ASE.Compiler.Python.Module node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitMul", (PyCFunction) ASE_Compiler_Python_Visitor_visitMul, (METH_VARARGS | METH_KEYWORDS), (char *) "visitMul(in ASE.Compiler.Python.Mul node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitName", (PyCFunction) ASE_Compiler_Python_Visitor_visitName, (METH_VARARGS | METH_KEYWORDS), (char *) "visitName(in ASE.Compiler.Python.Name node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitNot", (PyCFunction) ASE_Compiler_Python_Visitor_visitNot, (METH_VARARGS | METH_KEYWORDS), (char *) "visitNot(in ASE.Compiler.Python.Not node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitOr", (PyCFunction) ASE_Compiler_Python_Visitor_visitOr, (METH_VARARGS | METH_KEYWORDS), (char *) "visitOr(in ASE.Compiler.Python.Or node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitPass", (PyCFunction) ASE_Compiler_Python_Visitor_visitPass, (METH_VARARGS | METH_KEYWORDS), (char *) "visitPass(in ASE.Compiler.Python.Pass node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitPower", (PyCFunction) ASE_Compiler_Python_Visitor_visitPower, (METH_VARARGS | METH_KEYWORDS), (char *) "visitPower(in ASE.Compiler.Python.Power node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitPrint", (PyCFunction) ASE_Compiler_Python_Visitor_visitPrint, (METH_VARARGS | METH_KEYWORDS), (char *) "visitPrint(in ASE.Compiler.Python.Print node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitPrintnl", (PyCFunction) ASE_Compiler_Python_Visitor_visitPrintnl, (METH_VARARGS | METH_KEYWORDS), (char *) "visitPrintnl(in ASE.Compiler.Python.Printnl node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitRaise", (PyCFunction) ASE_Compiler_Python_Visitor_visitRaise, (METH_VARARGS | METH_KEYWORDS), (char *) "visitRaise(in ASE.Compiler.Python.Raise node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitReturn", (PyCFunction) ASE_Compiler_Python_Visitor_visitReturn, (METH_VARARGS | METH_KEYWORDS), (char *) "visitReturn(in ASE.Compiler.Python.Return node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitRightShift", (PyCFunction) ASE_Compiler_Python_Visitor_visitRightShift, (METH_VARARGS | METH_KEYWORDS), (char *) "visitRightShift(in ASE.Compiler.Python.RightShift node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitSlice", (PyCFunction) ASE_Compiler_Python_Visitor_visitSlice, (METH_VARARGS | METH_KEYWORDS), (char *) "visitSlice(in ASE.Compiler.Python.Slice node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitStmt", (PyCFunction) ASE_Compiler_Python_Visitor_visitStmt, (METH_VARARGS | METH_KEYWORDS), (char *) "visitStmt(in ASE.Compiler.Python.Stmt node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitSub", (PyCFunction) ASE_Compiler_Python_Visitor_visitSub, (METH_VARARGS | METH_KEYWORDS), (char *) "visitSub(in ASE.Compiler.Python.Sub node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitSubscript", (PyCFunction) ASE_Compiler_Python_Visitor_visitSubscript, (METH_VARARGS | METH_KEYWORDS), (char *) "visitSubscript(in ASE.Compiler.Python.Subscript node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitTryExcept", (PyCFunction) ASE_Compiler_Python_Visitor_visitTryExcept, (METH_VARARGS | METH_KEYWORDS), (char *) "visitTryExcept(in ASE.Compiler.Python.TryExcept node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitTryFinally", (PyCFunction) ASE_Compiler_Python_Visitor_visitTryFinally, (METH_VARARGS | METH_KEYWORDS), (char *) "visitTryFinally(in ASE.Compiler.Python.TryFinally node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitTuple", (PyCFunction) ASE_Compiler_Python_Visitor_visitTuple, (METH_VARARGS | METH_KEYWORDS), (char *) "visitTuple(in ASE.Compiler.Python.Tuple node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitUnaryAdd", (PyCFunction) ASE_Compiler_Python_Visitor_visitUnaryAdd, (METH_VARARGS | METH_KEYWORDS), (char *) "visitUnaryAdd(in ASE.Compiler.Python.UnaryAdd node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitUnarySub", (PyCFunction) ASE_Compiler_Python_Visitor_visitUnarySub, (METH_VARARGS | METH_KEYWORDS), (char *) "visitUnarySub(in ASE.Compiler.Python.UnarySub node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitWhile", (PyCFunction) ASE_Compiler_Python_Visitor_visitWhile, (METH_VARARGS | METH_KEYWORDS), (char *) "visitWhile(in ASE.Compiler.Python.While node)\n\
RETURNS None\n\
\n\
"},
  {(char *) "visitYield", (PyCFunction) ASE_Compiler_Python_Visitor_visitYield, (METH_VARARGS | METH_KEYWORDS), (char *) "visitYield(in ASE.Compiler.Python.Yield node)\n\
RETURNS None\n\
\n\
"},
  {NULL, NULL}};

static struct PyMethodDef _VisitorModuleMethods[] = {{NULL, NULL}};

static void sidl_dealloc(struct ASE_Compiler_Python_Visitor_PythonObject *self)
{
  if (self->refType != WEAK_REF)
  {
    (*(((struct ASE_BaseInterface__object *) self->ior)->d_epv->deleteReference))(self->ior->d_object, NULL);
  }
  self->ob_type->tp_free(self);
}

static int sidl_cmp(struct ASE_Compiler_Python_Visitor_PythonObject *self, PyObject *arg)
{
  struct ASE_Compiler_Python_Visitor__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_Python_Visitor_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  int _return_value = 0;
  
  /* Check for a valid SIDL Object */
  if (arg->ob_type != &SIDLPythonObjectType)
  {
    PyErr_SetString(PyExc_TypeError, "Could not parse arguments for sidl_cmp");
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_cmp()");
    return _return_value;
  }
  _return_value = PyObject_Compare(o1, o2);
  Py_DECREF(o1);
  Py_DECREF(o2);
  return _return_value;
}

static long sidl_hash(struct ASE_Compiler_Python_Visitor_PythonObject *self)
{
  struct ASE_Compiler_Python_Visitor__object *ior = self->ior;
  PyObject *_hash_arg = NULL;
  long _return_value = 0;
  
  if (!ior)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_hash()");
    return _return_value;
  }
  _hash_arg = Py_BuildValue((char *) "i", ior->d_object);
  if (!_hash_arg)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not hash IOR pointer in sidl_hash()");
    return _return_value;
  }
  _return_value = PyObject_Hash(_hash_arg);
  Py_DECREF(_hash_arg);
  return _return_value;
}

static PyObject *sidl_str(struct ASE_Compiler_Python_Visitor_PythonObject *self)
{
  struct ASE_Compiler_Python_Visitor__object *ior = self->ior;
  const char *_str = NULL;
  PyObject *_return_value = NULL;
  
  _str = (*(ior->d_epv->strObject))(ior, NULL);
  _return_value = PyString_FromString(_str);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyObject *sidl_richcmp(struct ASE_Compiler_Python_Visitor_PythonObject *self, PyObject *arg, int op)
{
  struct ASE_Compiler_Python_Visitor__object *ior1 = self->ior;
  struct ASE_BaseInterface__object *ior2 = (struct ASE_BaseInterface__object *) ((struct ASE_Compiler_Python_Visitor_PythonObject *) arg)->ior;
  PyObject *o1 = NULL;
  PyObject *o2 = NULL;
  PyObject *_return_value = NULL;
  
  /* Check for a valid SIDL Object */
  if (!PyObject_HasAttrString(arg, (char *) "getClassName"))
  {
    switch(op) {
      case Py_EQ:
        _return_value = Py_False;
        Py_INCREF(_return_value);
        break;
      case Py_NE:
        _return_value = Py_True;
        Py_INCREF(_return_value);
        break;
      case Py_LT:
      case Py_LE:
      case Py_GT:
      case Py_GE:
        PyErr_SetString(PyExc_TypeError, (char *) "Could not parse arguments for sidl_richcmp");
    }
    return _return_value;
  }
  if (!ior1)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  if (!ior2)
  {
    PyErr_SetString(PyExc_ValueError, "Null IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o1 = Py_BuildValue((char *) "i", ior1->d_object);
  if (!o1)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  o2 = Py_BuildValue((char *) "i", ior2->d_object);
  if (!o2)
  {
    PyErr_SetString(PyExc_RuntimeError, "Could not build integer from IOR pointer in sidl_richcmp()");
    return _return_value;
  }
  _return_value = PyObject_RichCompare(o1, o2, op);
  Py_DECREF(o1);
  Py_DECREF(o2);
  Py_INCREF(_return_value);
  return _return_value;
}

static PyTypeObject SIDLPythonObjectType = {
  PyObject_HEAD_INIT(NULL)
  0,
  (char *) "ASE.Compiler.Python.Visitor",
  sizeof(struct ASE_Compiler_Python_Visitor_PythonObject),
  0,
  (destructor) sidl_dealloc,
  (printfunc) 0,
  (getattrfunc) 0,
  (setattrfunc) 0,
  (cmpfunc) sidl_cmp,
  (reprfunc) 0,
  (PyNumberMethods *) 0,
  (PySequenceMethods *) 0,
  (PyMappingMethods *) 0,
  (hashfunc) sidl_hash,
  (ternaryfunc) 0,
  (reprfunc) sidl_str,
  (getattrofunc) 0,
  (setattrofunc) 0,
  (PyBufferProcs *) 0,
  Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE,
  (char *) "This is a Python wrapper for a SIDL object of type ASE.Compiler.Python.Visitor",
  (traverseproc) 0,
  (inquiry) 0,
  (richcmpfunc) sidl_richcmp,
  0,
  (getiterfunc) 0,
  (iternextfunc) 0,
  _VisitorObjectMethods,
  (struct PyMemberDef *) 0,
  (struct PyGetSetDef *) 0,
  0,
  0,
  (descrgetfunc) 0,
  (descrsetfunc) 0,
  0,
  (initproc) 0,
  (allocfunc) 0,
  (newfunc) ASE_Compiler_Python_Visitor_Visitor,
  (freefunc) 0,
  (inquiry) 0};

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

void initVisitor(void)
{
  PyObject *module = NULL;
  const struct ASE_Compiler_Python_Visitor__external *(*_extFunc)(void);
  struct ASE_BaseException__object *_exception = NULL;
  
  /* Setup the python type */
  if (PyType_Ready(&SIDLPythonObjectType) < 0)
  {
    return;
  }
  /* Initialize the module */
  module = Py_InitModule3((char *) "Visitor", _VisitorModuleMethods, (char *) " Interface for interacting with the AST");
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Could not initialize module ASE.Compiler.Python.Visitor");
    return;
  }
  /* Add the type to the module and increment the ref count */
  Py_INCREF(&SIDLPythonObjectType);
  PyModule_AddObject(module, (char *) "Visitor", (PyObject *) &SIDLPythonObjectType);
  /* Set externals function */
  _extFunc = (const struct ASE_Compiler_Python_Visitor__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(&_exception)->getClassMethodTable("ASE.Compiler.Python.Visitor", &_exception);
  /* Check exception */
  if (_exception)
  {
    _exception = _exception->d_epv->castObject(_exception, "ASE.BaseException", NULL);
    PyErr_SetString(PyExc_RuntimeError, _exception->d_epv->getMessage(_exception->d_object, NULL));
    return;
  }
  /* Set epv pointer */
  if (_extFunc)
  {
    _epv = (*_extFunc)();
  }
  if (!_epv)
  {
    PyErr_SetString(PyExc_ImportError, "Cannot load SIDL implementation for ASE.Compiler.Python.Visitor");
    return;
  }
  /* Import Numeric */
  import_array();
  if (PyErr_Occurred())
  {
    PyErr_SetString(PyExc_ImportError, "Cannot import Numeric Python array module");
    return;
  }
}
