#include <Python.h>

#include <Numeric/arrayobject.h>

#ifndef included_ASE_Compiler_Python_Printer
#include "ASE/Compiler/Python/Printer_IOR.h"
#endif

#ifndef included_ASE_Loader
#include "ASE/Loader_IOR.h"
#endif

#ifndef included_ASE_ASEException
#include "ASE/ASEException_IOR.h"
#endif

#ifndef included_ASE_Compiler_CompilerException
#include "ASE/Compiler/CompilerException_IOR.h"
#endif

#ifndef included_ASE_BaseException
#include "ASE/BaseException_IOR.h"
#endif

enum ReferenceType {STOLEN_REF, WEAK_REF, NEW_REF};

struct ASE_Compiler_Python_Printer_PythonObject {
  PyObject_HEAD;
  struct ASE_Compiler_Python_Printer__object *ior;
  enum ReferenceType refType;
};

static PyObject *sidl_new(PyTypeObject *type, struct ASE_BaseInterface__object *ior, enum ReferenceType refType)
{
  PyObject *return_value = NULL;
  
  if (ior)
  {
    struct ASE_Compiler_Python_Printer_PythonObject *self = (struct ASE_Compiler_Python_Printer_PythonObject *) type->tp_alloc(type, 0);
    
    if (self)
    {
      struct ASE_BaseClass__object *baseIor = (*(ior->d_epv->castObject))(ior->d_object, "ASE.BaseClass", NULL);
      
      self->ior = (*(ior->d_epv->castObject))(ior->d_object, type->tp_name, NULL);
      self->refType = refType;
      /* Allow objects without an attached implementation (used in the skeleton constructor) */
      if (baseIor->d_data)
      {
        if (refType == NEW_REF)
        {
          (*(baseIor->d_epv->addReference))(baseIor->d_object, NULL);
        }
      }
      return_value = (PyObject *) self;
    }
    else
    {
      PyErr_SetString(PyExc_MemoryError, (char *) "Unable to allocate interface object");
    }
  }
  else
  {
    return_value = Py_None;
    Py_INCREF(return_value);
  }
  return return_value;
}

static PyObject *ASE_Compiler_Python_Import__wrap_ior(struct ASE_Compiler_Python_Import__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Import");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Import");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Import");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Import");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_For__wrap_ior(struct ASE_Compiler_Python_For__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.For");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "For");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.For");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.For");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Return__wrap_ior(struct ASE_Compiler_Python_Return__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Return");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Return");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Return");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Return");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Discard__wrap_ior(struct ASE_Compiler_Python_Discard__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Discard");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Discard");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Discard");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Discard");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Printnl__wrap_ior(struct ASE_Compiler_Python_Printnl__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Printnl");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Printnl");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Printnl");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Printnl");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_AssName__wrap_ior(struct ASE_Compiler_Python_AssName__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.AssName");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "AssName");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.AssName");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.AssName");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_ListCompFor__wrap_ior(struct ASE_Compiler_Python_ListCompFor__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.ListCompFor");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "ListCompFor");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.ListCompFor");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.ListCompFor");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Compare__wrap_ior(struct ASE_Compiler_Python_Compare__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Compare");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Compare");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Compare");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Compare");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Pass__wrap_ior(struct ASE_Compiler_Python_Pass__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Pass");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Pass");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Pass");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Pass");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_AssAttr__wrap_ior(struct ASE_Compiler_Python_AssAttr__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.AssAttr");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "AssAttr");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.AssAttr");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.AssAttr");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_GetAttr__wrap_ior(struct ASE_Compiler_Python_GetAttr__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.GetAttr");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "GetAttr");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.GetAttr");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.GetAttr");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_From__wrap_ior(struct ASE_Compiler_Python_From__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.From");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "From");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.From");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.From");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Or__wrap_ior(struct ASE_Compiler_Python_Or__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Or");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Or");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Or");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Or");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_If__wrap_ior(struct ASE_Compiler_Python_If__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.If");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "If");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.If");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.If");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Printer__wrap_ior(struct ASE_Compiler_Python_Printer__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Printer");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Printer");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Printer");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Printer");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Not__wrap_ior(struct ASE_Compiler_Python_Not__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Not");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Not");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Not");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Not");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Power__wrap_ior(struct ASE_Compiler_Python_Power__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Power");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Power");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Power");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Power");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_UnarySub__wrap_ior(struct ASE_Compiler_Python_UnarySub__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.UnarySub");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "UnarySub");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.UnarySub");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.UnarySub");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Dict__wrap_ior(struct ASE_Compiler_Python_Dict__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Dict");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Dict");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Dict");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Dict");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Bitand__wrap_ior(struct ASE_Compiler_Python_Bitand__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Bitand");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Bitand");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Bitand");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Bitand");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_AugAssign__wrap_ior(struct ASE_Compiler_Python_AugAssign__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.AugAssign");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "AugAssign");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.AugAssign");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.AugAssign");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Mod__wrap_ior(struct ASE_Compiler_Python_Mod__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Mod");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Mod");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Mod");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Mod");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Add__wrap_ior(struct ASE_Compiler_Python_Add__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Add");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Add");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Add");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Add");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_LeftShift__wrap_ior(struct ASE_Compiler_Python_LeftShift__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.LeftShift");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "LeftShift");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.LeftShift");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.LeftShift");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Subscript__wrap_ior(struct ASE_Compiler_Python_Subscript__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Subscript");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Subscript");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Subscript");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Subscript");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Yield__wrap_ior(struct ASE_Compiler_Python_Yield__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Yield");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Yield");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Yield");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Yield");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Keyword__wrap_ior(struct ASE_Compiler_Python_Keyword__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Keyword");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Keyword");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Keyword");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Keyword");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Vertex__wrap_ior(struct ASE_Compiler_Vertex__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Vertex");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vertex");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Vertex");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Vertex");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Assign__wrap_ior(struct ASE_Compiler_Python_Assign__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Assign");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Assign");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Assign");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Assign");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Raise__wrap_ior(struct ASE_Compiler_Python_Raise__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Raise");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Raise");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Raise");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Raise");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Class__wrap_ior(struct ASE_Compiler_Python_Class__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Class");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Class");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Class");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Class");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_RightShift__wrap_ior(struct ASE_Compiler_Python_RightShift__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.RightShift");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "RightShift");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.RightShift");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.RightShift");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_TryFinally__wrap_ior(struct ASE_Compiler_Python_TryFinally__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.TryFinally");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "TryFinally");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.TryFinally");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.TryFinally");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Module__wrap_ior(struct ASE_Compiler_Python_Module__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Module");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Module");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Module");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Module");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_And__wrap_ior(struct ASE_Compiler_Python_And__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.And");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "And");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.And");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.And");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_CallFunc__wrap_ior(struct ASE_Compiler_Python_CallFunc__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.CallFunc");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "CallFunc");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.CallFunc");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.CallFunc");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_While__wrap_ior(struct ASE_Compiler_Python_While__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.While");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "While");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.While");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.While");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Sub__wrap_ior(struct ASE_Compiler_Python_Sub__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Sub");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Sub");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Sub");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Sub");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Bitor__wrap_ior(struct ASE_Compiler_Python_Bitor__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Bitor");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Bitor");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Bitor");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Bitor");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Continue__wrap_ior(struct ASE_Compiler_Python_Continue__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Continue");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Continue");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Continue");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Continue");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Function__wrap_ior(struct ASE_Compiler_Python_Function__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Function");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Function");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Function");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Function");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_ListComp__wrap_ior(struct ASE_Compiler_Python_ListComp__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.ListComp");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "ListComp");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.ListComp");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.ListComp");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Ellipsis__wrap_ior(struct ASE_Compiler_Python_Ellipsis__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Ellipsis");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Ellipsis");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Ellipsis");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Ellipsis");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_PyNone__wrap_ior(struct ASE_Compiler_Python_PyNone__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.PyNone");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "PyNone");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.PyNone");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.PyNone");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Tuple__wrap_ior(struct ASE_Compiler_Python_Tuple__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Tuple");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Tuple");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Tuple");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Tuple");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_ListCompIf__wrap_ior(struct ASE_Compiler_Python_ListCompIf__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.ListCompIf");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "ListCompIf");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.ListCompIf");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.ListCompIf");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Const__wrap_ior(struct ASE_Compiler_Python_Const__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Const");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Const");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Const");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Const");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Global__wrap_ior(struct ASE_Compiler_Python_Global__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Global");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Global");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Global");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Global");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Break__wrap_ior(struct ASE_Compiler_Python_Break__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Break");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Break");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Break");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Break");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Name__wrap_ior(struct ASE_Compiler_Python_Name__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Name");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Name");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Name");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Name");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_UnaryAdd__wrap_ior(struct ASE_Compiler_Python_UnaryAdd__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.UnaryAdd");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "UnaryAdd");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.UnaryAdd");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.UnaryAdd");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Stmt__wrap_ior(struct ASE_Compiler_Python_Stmt__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Stmt");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Stmt");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Stmt");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Stmt");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_TryExcept__wrap_ior(struct ASE_Compiler_Python_TryExcept__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.TryExcept");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "TryExcept");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.TryExcept");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.TryExcept");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Bitxor__wrap_ior(struct ASE_Compiler_Python_Bitxor__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Bitxor");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Bitxor");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Bitxor");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Bitxor");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Lambda__wrap_ior(struct ASE_Compiler_Python_Lambda__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Lambda");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Lambda");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Lambda");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Lambda");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_AssList__wrap_ior(struct ASE_Compiler_Python_AssList__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.AssList");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "AssList");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.AssList");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.AssList");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_List__wrap_ior(struct ASE_Compiler_Python_List__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.List");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "List");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.List");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.List");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_AssTuple__wrap_ior(struct ASE_Compiler_Python_AssTuple__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.AssTuple");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "AssTuple");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.AssTuple");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.AssTuple");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Mul__wrap_ior(struct ASE_Compiler_Python_Mul__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Mul");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Mul");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Mul");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Mul");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Div__wrap_ior(struct ASE_Compiler_Python_Div__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Div");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Div");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Div");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Div");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_BaseInterface__wrap_ior(struct ASE_BaseInterface__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.BaseInterface");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "BaseInterface");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.BaseInterface");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.BaseInterface");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Vertex__wrap_ior(struct ASE_Compiler_Python_Vertex__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Vertex");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Vertex");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Vertex");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Vertex");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Print__wrap_ior(struct ASE_Compiler_Python_Print__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Print");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Print");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Print");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Print");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Slice__wrap_ior(struct ASE_Compiler_Python_Slice__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Slice");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Slice");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Slice");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Slice");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static PyObject *ASE_Compiler_Python_Invert__wrap_ior(struct ASE_Compiler_Python_Invert__object *ior)
{
  PyObject *module = PyImport_ImportModule((char *) "ASE.Compiler.Python.Invert");
  PyTypeObject *type = (PyTypeObject *) PyObject_GetAttrString(module, (char *) "Invert");
  
  if (!module)
  {
    PyErr_SetString(PyExc_ImportError, "Could not import module ASE.Compiler.Python.Invert");
    return 0;
  }
  if (!type)
  {
    PyErr_SetString(PyExc_TypeError, "Could not find type ASE.Compiler.Python.Invert");
    return 0;
  }
  return sidl_new(type, (struct ASE_BaseInterface__object *) ior, STOLEN_REF);
}

static long ASE_BaseInterface__extract_ior(struct ASE_Compiler_Python_Printer_PythonObject *obj, struct ASE_BaseInterface__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.BaseInterface", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.BaseInterface");
    return 0;
  }
  return 1;
}

static long ASE_Compiler_Vertex__extract_ior(struct ASE_Compiler_Python_Printer_PythonObject *obj, struct ASE_Compiler_Vertex__object **ior)
{
  struct ASE_BaseInterface__object *objIor = (struct ASE_BaseInterface__object *) obj->ior;
  
  if ((PyObject *) obj == Py_None)
  {
    *ior = NULL;
    return 1;
  }
  *ior = (*(objIor->d_epv->castObject))(objIor->d_object, "ASE.Compiler.Vertex", NULL);
  if (!(*ior))
  {
    PyErr_SetString(PyExc_TypeError, "Argument is not of type ASE.Compiler.Vertex");
    return 0;
  }
  return 1;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static PyObject *ASE_Compiler_Vertex__wrap_array(struct ASE_Compiler_Vertex__array *array)
{
  PyArrayObject *_return_value = NULL;
  
  if (array)
  {
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {0};
    int32_t indStride[1] = {0};
    long dimen = (*_extFunc->getDimen)(array, NULL);
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array numElem = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    long i = 0;
    
    for(i = 0; i < dimen; i++)
    {
      lower.d_firstElement[i] = (*_extFunc->getLower)(array, i, NULL);
      upper.d_firstElement[i] = (*_extFunc->getUpper)(array, i, NULL);
      numElem.d_firstElement[i] = upper.d_firstElement[i] - lower.d_firstElement[i] + 1;
    }
    _return_value = (PyArrayObject *) PyArray_FromDims(dimen, numElem.d_firstElement, PyArray_OBJECT);
    if (array)
    {
      long size = 1;
      char *data = _return_value->data;
      
      for(i = 0; i < dimen; i++)
      {
        size *= numElem.d_firstElement[i];
      }
      while(size--)
      {
        struct ASE_Compiler_Vertex__object *elem = (*_extFunc->getElement)(array, &lower, NULL);
        
        (*(elem->d_epv->addReference))(elem->d_object, NULL);
        *(PyObject **) data = ASE_Compiler_Vertex__wrap_ior(elem);
        if (!(*(PyObject **) data))
        {
          Py_DECREF(_return_value);
          _return_value = NULL;
          goto error;
        }
        for(i = 0; i < dimen; i++)
        {
          data += _return_value->strides[i];
          if (++(lower.d_firstElement[i]) <= upper.d_firstElement[i])
          {
            break;
          }
          else
          {
            data -= _return_value->strides[i] * numElem.d_firstElement[i];
            lower.d_firstElement[i] -= numElem.d_firstElement[i];
          }
        }
      }
    }
    error: free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(numElem.d_firstElement);
  }
  return (PyObject *) _return_value;
}

#ifndef included_ASE_Compiler_Vertex
#include "ASE/Compiler/Vertex_IOR.h"
#endif

static long ASE_Compiler_Vertex__extract_array(PyObject *obj, struct ASE_Compiler_Vertex__array **array)
{
  /* This will create a new Numeric array from a Python sequence */
  PyArrayObject *pyArray = (PyArrayObject *) PyArray_FromObject(obj, PyArray_OBJECT, 0, 0);
  long _return_value = 0;
  
  *array = NULL;
  if (pyArray)
  {
    int32_t dimen = pyArray->nd;
    int32_t indLower[1] = {0};
    int32_t indUpper[1] = {dimen - 1};
    int32_t indStride[1] = {1};
    struct int__array lower = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array upper = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    struct int__array stride = {
      malloc(sizeof(int32_t) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    int32_t d = 0;
    const struct ASE_Compiler_Vertex__external *(*_externals)(void) = (const struct ASE_Compiler_Vertex__external *(*)(void)) ASE_Loader__externals()->getStaticEPV(NULL)->getClassMethodTable("ASE.Compiler.Vertex", NULL);
    const struct ASE_Compiler_Vertex__external *_extFunc = (*_externals)();
    long size = PyArray_Size((PyObject *) pyArray);
    char *data = pyArray->data;
    long i;
    struct int__array indices = {
      malloc(sizeof(long) * (dimen ? dimen : 1)),
      indLower,
      indUpper,
      indStride,
      0,
      1};
    
    for(d = 0; d < dimen; d++)
    {
      lower.d_firstElement[d] = 0;
      upper.d_firstElement[d] = pyArray->dimensions[d] - 1;
      stride.d_firstElement[d] = pyArray->strides[d] / pyArray->descr->elsize;
      if (pyArray->strides[d] % pyArray->descr->elsize)
      {
        free(lower.d_firstElement);
        free(upper.d_firstElement);
        free(stride.d_firstElement);
        return 0;
      }
    }
    *array = (*_extFunc->createArray)(dimen, &lower, &upper, NULL);
    free(lower.d_firstElement);
    free(upper.d_firstElement);
    free(stride.d_firstElement);
    memset(indices.d_firstElement, 0, sizeof(long) * (dimen ? dimen : 1));
    for(i = 0; i < size; i++)
    {
      struct ASE_Compiler_Vertex__object *elem;
      
      if (!ASE_Compiler_Vertex__extract_ior((struct ASE_Compiler_Python_Printer_PythonObject *) *(PyObject **) data, &elem))
      {
        (*_extFunc->destroyArray)(*array, NULL);
        *array = NULL;
        goto error;
      }
      (*_extFunc->setElement)(*array, &indices, elem, NULL);
      for(d = 0; d < dimen; d++)
      {
        data += pyArray->strides[d];
        if (++(indices.d_firstElement[d]) < pyArray->dimensions[d])
        {
          break;
        }
        else
        {
          indices.d_firstElement[d] = 0;
          data -= pyArray->dimensions[d] * pyArray->strides[d];
        }
      }
    }
    _return_value = 1;
    error: free(indices.d_firstElement);
    Py_DECREF(pyArray);
  }
  return _return_value;
}

static const char *ASE_Compiler_Printer_getTab(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  const char *_return_value = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.getTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "getTab");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.getTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.getTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "z", &_return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.Python.Printer.getTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  /* Copying the Python string to IOR controlled memory */
  _return_value = _return_value ? strcpy(malloc(strlen(_return_value) + 1), _return_value) : NULL;
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static void ASE_Compiler_Printer_setTab(struct ASE_Compiler_Python_Printer__object *_self, const char *tab, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.setTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "setTab");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.setTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(z)", tab);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.setTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.setTab", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static int32_t ASE_Compiler_Printer_getIndentSize(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  PyObject *returnTuple = NULL;
  int32_t _return_value = 0;
  long _local__return_value = 0;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.getIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "getIndentSize");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.getIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.getIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return _return_value;
  }
  returnTuple = PyTuple_New(1);
  PyTuple_SetItem(returnTuple, 0, _pyReturn);
  /* Parse arguments */
  _okay = PyArg_ParseTuple(returnTuple, (char *) "l", &_local__return_value);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_okay)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not parse return value for ASE.Compiler.Python.Printer.getIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return _return_value;
  }
  /* Cleanup local return value */
  Py_DECREF(returnTuple);
  _return_value = _local__return_value;
  /* Cleanup arguments */
  Py_DECREF(_args);
  return _return_value;
}

static void ASE_Compiler_Printer_setIndentSize(struct ASE_Compiler_Python_Printer__object *_self, int32_t indentSize, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  long _local_indentSize = 0;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.setIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "setIndentSize");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.setIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Convert int32_t to long */
  _local_indentSize = indentSize;
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(l)", _local_indentSize);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.setIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.setIndentSize", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Printer_increaseIndent(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.increaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "increaseIndent");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.increaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.increaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.increaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Printer_decreaseIndent(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.decreaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "decreaseIndent");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.decreaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.decreaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.decreaseIndent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Printer_indent(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.indent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "indent");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.indent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "()");
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.indent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.indent", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Printer_write(struct ASE_Compiler_Python_Printer__object *_self, const char *s, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.write", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "write");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.write", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(z)", s);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.write", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.write", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Printer_println(struct ASE_Compiler_Python_Printer__object *_self, const char *s, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.println", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "println");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.println", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(z)", s);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.println", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.println", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAdd(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Add__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAdd");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Add__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAnd(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_And__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAnd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAnd");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAnd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_And__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAnd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAnd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAssAttr(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_AssAttr__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAssAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAssAttr");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAssAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_AssAttr__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAssAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAssAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAssList(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_AssList__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAssList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAssList");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAssList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_AssList__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAssList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAssList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAssName(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_AssName__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAssName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAssName");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAssName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_AssName__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAssName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAssName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAssTuple(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_AssTuple__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAssTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAssTuple");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAssTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_AssTuple__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAssTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAssTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAssign(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Assign__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAssign");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Assign__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitAugAssign(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_AugAssign__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitAugAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitAugAssign");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitAugAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_AugAssign__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitAugAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitAugAssign", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitBitand(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Bitand__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitBitand", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitBitand");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitBitand", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Bitand__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitBitand", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitBitand", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitBitor(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Bitor__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitBitor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitBitor");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitBitor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Bitor__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitBitor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitBitor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitBitxor(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Bitxor__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitBitxor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitBitxor");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitBitxor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Bitxor__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitBitxor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitBitxor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitBreak(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Break__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitBreak", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitBreak");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitBreak", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Break__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitBreak", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitBreak", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitCallFunc(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_CallFunc__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitCallFunc", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitCallFunc");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitCallFunc", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_CallFunc__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitCallFunc", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitCallFunc", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitClass(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Class__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitClass");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Class__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitClass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitCompare(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Compare__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitCompare", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitCompare");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitCompare", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Compare__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitCompare", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitCompare", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitConst(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Const__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitConst", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitConst");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitConst", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Const__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitConst", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitConst", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitPyNone(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_PyNone__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitPyNone", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitPyNone");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitPyNone", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_PyNone__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitPyNone", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitPyNone", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitContinue(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Continue__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitContinue", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitContinue");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitContinue", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Continue__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitContinue", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitContinue", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitDict(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Dict__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitDict", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitDict");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitDict", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Dict__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitDict", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitDict", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitDiscard(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Discard__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitDiscard", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitDiscard");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitDiscard", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Discard__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitDiscard", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitDiscard", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitDiv(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Div__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitDiv", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitDiv");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitDiv", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Div__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitDiv", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitDiv", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitEllipsis(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Ellipsis__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitEllipsis", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitEllipsis");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitEllipsis", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Ellipsis__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitEllipsis", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitEllipsis", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitFor(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_For__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitFor");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_For__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitListCompFor(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_ListCompFor__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitListCompFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitListCompFor");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitListCompFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_ListCompFor__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitListCompFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitListCompFor", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitFunction(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Function__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitFunction", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitFunction");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitFunction", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Function__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitFunction", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitFunction", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitLambda(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Lambda__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitLambda", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitLambda");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitLambda", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Lambda__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitLambda", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitLambda", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitGetAttr(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_GetAttr__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitGetAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitGetAttr");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitGetAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_GetAttr__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitGetAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitGetAttr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitGlobal(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Global__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitGlobal", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitGlobal");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitGlobal", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Global__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitGlobal", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitGlobal", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitIf(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_If__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitIf");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_If__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitListCompIf(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_ListCompIf__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitListCompIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitListCompIf");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitListCompIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_ListCompIf__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitListCompIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitListCompIf", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitImport(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Import__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitImport", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitImport");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitImport", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Import__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitImport", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitImport", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitFrom(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_From__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitFrom", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitFrom");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitFrom", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_From__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitFrom", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitFrom", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitInvert(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Invert__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitInvert", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitInvert");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitInvert", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Invert__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitInvert", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitInvert", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitKeyword(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Keyword__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitKeyword", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitKeyword");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitKeyword", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Keyword__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitKeyword", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitKeyword", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitLeftShift(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_LeftShift__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitLeftShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitLeftShift");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitLeftShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_LeftShift__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitLeftShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitLeftShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitList(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_List__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitList");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_List__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitList", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitListComp(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_ListComp__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitListComp", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitListComp");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitListComp", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_ListComp__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitListComp", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitListComp", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitMod(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Mod__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitMod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitMod");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitMod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Mod__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitMod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitMod", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitModule(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Module__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitModule", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitModule");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitModule", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Module__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitModule", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitModule", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitMul(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Mul__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitMul", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitMul");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitMul", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Mul__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitMul", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitMul", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitName(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Name__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitName");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Name__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitName", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitNot(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Not__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitNot", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitNot");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitNot", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Not__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitNot", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitNot", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitOr(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Or__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitOr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitOr");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitOr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Or__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitOr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitOr", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitPass(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Pass__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitPass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitPass");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitPass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Pass__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitPass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitPass", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitPower(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Power__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitPower", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitPower");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitPower", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Power__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitPower", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitPower", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitPrint(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Print__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitPrint", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitPrint");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitPrint", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Print__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitPrint", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitPrint", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitPrintnl(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Printnl__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitPrintnl", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitPrintnl");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitPrintnl", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Printnl__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitPrintnl", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitPrintnl", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitRaise(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Raise__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitRaise", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitRaise");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitRaise", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Raise__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitRaise", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitRaise", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitReturn(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Return__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitReturn", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitReturn");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitReturn", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Return__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitReturn", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitReturn", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitRightShift(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_RightShift__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitRightShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitRightShift");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitRightShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_RightShift__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitRightShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitRightShift", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitSlice(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Slice__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitSlice", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitSlice");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitSlice", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Slice__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitSlice", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitSlice", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitStmt(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Stmt__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitStmt", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitStmt");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitStmt", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Stmt__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitStmt", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitStmt", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitSub(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Sub__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitSub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitSub");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitSub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Sub__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitSub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitSub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitSubscript(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Subscript__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitSubscript", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitSubscript");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitSubscript", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Subscript__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitSubscript", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitSubscript", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitTryExcept(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_TryExcept__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitTryExcept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitTryExcept");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitTryExcept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_TryExcept__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitTryExcept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitTryExcept", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitTryFinally(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_TryFinally__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitTryFinally", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitTryFinally");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitTryFinally", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_TryFinally__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitTryFinally", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitTryFinally", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitTuple(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Tuple__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitTuple");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Tuple__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitTuple", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitUnaryAdd(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_UnaryAdd__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitUnaryAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitUnaryAdd");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitUnaryAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_UnaryAdd__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitUnaryAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitUnaryAdd", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitUnarySub(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_UnarySub__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitUnarySub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitUnarySub");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitUnarySub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_UnarySub__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitUnarySub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitUnarySub", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitWhile(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_While__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitWhile", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitWhile");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitWhile", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_While__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitWhile", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitWhile", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_visitYield(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Yield__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.visitYield", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "visitYield");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.visitYield", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Yield__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.visitYield", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_Compiler_CompilerException_module = NULL;
    PyObject *ASE_Compiler_CompilerException_type = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_Compiler_CompilerException_module = PyImport_ImportModule((char *) "ASE.Compiler.CompilerException");
    ASE_Compiler_CompilerException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_Compiler_CompilerException_module), (char *) "Exception");
    Py_DECREF(ASE_Compiler_CompilerException_module);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_Compiler_CompilerException_type))
    {
      struct ASE_Compiler_CompilerException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_Compiler_CompilerException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_Compiler_CompilerException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.Compiler.CompilerException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_Compiler_CompilerException_type);
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.visitYield", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_printDocComment(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_Compiler_Python_Vertex__object *node, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = _self ? _self->d_data : NULL;
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.printDocComment", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "printDocComment");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.printDocComment", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  (*(((struct ASE_BaseInterface__object *) node)->d_epv->addReference))(((struct ASE_BaseInterface__object *) node)->d_object, NULL);
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Vertex__wrap_ior, node);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.printDocComment", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Dispatch to implementation */
  _pyReturn = PyObject_CallObject(_method, _args);
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.printDocComment", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_Printer(struct ASE_Compiler_Python_Printer__object *obj, struct ASE_BaseException__object **_exception)
{
  PyObject *_context = NULL;
  PyObject *_method = NULL;
  PyObject *_args = NULL;
  int _okay = 0;
  PyObject *_return_value = NULL;
  PyObject *_pyReturn = NULL;
  
  /* Get implementation object */
  _context = PyImport_ImportModule((char *) "ASE.Compiler.Python.Printer_impl");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_context)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation object for ASE.Compiler.Python.Printer.Printer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Get method object */
  _method = PyObject_GetAttrString(_context, (char *) "Printer");
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_method)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Null implementation method for ASE.Compiler.Python.Printer.Printer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Pack arguments for dispatch to implementation */
  _args = Py_BuildValue((char *) "(O&)", (void *) ASE_Compiler_Python_Printer__wrap_ior, obj);
  if (!_args)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Unable to parse arguments for ASE.Compiler.Python.Printer.Printer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Create a new instance */
  _return_value = PyInstance_New(_method, _args, NULL);
  _pyReturn = Py_None;
  Py_DECREF(_method);
  if (PyErr_Occurred())
  {
    PyErr_Clear();
  }
  if (!_return_value)
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Could not create instance of ASE.Compiler.Python.Printer_impl.Printer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  obj->d_data = _return_value;
  if (PyErr_Occurred())
  {
    PyObject *_excType = NULL;
    PyObject *_excValue = NULL;
    PyObject *_excTraceback = NULL;
    PyObject *ASE_BaseException_module = NULL;
    PyObject *ASE_BaseException_type = NULL;
    
    PyErr_Fetch(&_excType, &_excValue, &_excTraceback);
    ASE_BaseException_module = PyImport_ImportModule((char *) "ASE.BaseException");
    ASE_BaseException_type = PyMapping_GetItemString(PyModule_GetDict(ASE_BaseException_module), (char *) "Exception");
    Py_DECREF(ASE_BaseException_module);
    if (PyErr_GivenExceptionMatches(_excType, ASE_BaseException_type))
    {
      struct ASE_BaseException__object *_local_exception = NULL;
      struct ASE_Compiler_Python_Printer_PythonObject *_sidlExc = NULL;
      int isClass = PyObject_IsInstance(_excValue, ASE_BaseException_type);
      
      if (isClass)
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) PySequence_GetItem(PyObject_GetAttrString(_excValue, (char *) "args"), 0);
      }
      else
      {
        _sidlExc = (struct ASE_Compiler_Python_Printer_PythonObject *) _excValue;
      }
      PyObject_CallMethod((PyObject *) _sidlExc, (char *) "addReference", (char *) "");
      _local_exception = (struct ASE_BaseException__object *) _sidlExc->ior;
      _local_exception = (*(_local_exception->d_epv->castObject))(_local_exception->d_object, "ASE.BaseException", NULL);
      if (isClass)
      {
        Py_DECREF(_sidlExc);
      }
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    else
    {
      struct ASE_ASEException__object *_local_exception = NULL;
      
      _local_exception = ASE_ASEException__externals()->createObject(NULL);
      _local_exception->d_epv->setMessage(_local_exception, PyString_AsString(PyObject_Str(_excValue)), NULL);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
    }
    if (_exception)
    {
      while(_excTraceback && (_excTraceback != Py_None))
      {
        PyObject *code = PyObject_GetAttrString(PyObject_GetAttrString(_excTraceback, "tb_frame"), "f_code");
        
        (*_exception)->d_epv->addStackEntry((*_exception)->d_object, PyString_AsString(PyObject_GetAttrString(code, "co_filename")), PyInt_AsLong(PyObject_GetAttrString(_excTraceback, "tb_lineno")), PyString_AsString(PyObject_GetAttrString(code, "co_name")), NULL);
        _excTraceback = PyObject_GetAttrString(_excTraceback, "tb_next");
      }
    }
    Py_DECREF(ASE_BaseException_type);
    Py_DECREF(_excType);
    Py_DECREF(_excValue);
    Py_XDECREF(_excTraceback);
    return;
  }
  if (!(_pyReturn == Py_None))
  {
    struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
    
    _local_exception->d_epv->setMessage(_local_exception, "Invalid return from void method ASE.Compiler.Python.Printer.Printer", _exception);
    if (_exception)
    {
      *_exception = (struct ASE_BaseException__object *) _local_exception;
    }
    return;
  }
  /* Cleanup arguments */
  Py_DECREF(_args);
  return;
}

static void ASE_Compiler_Python_Printer_Printer_(struct ASE_Compiler_Python_Printer__object *_self, struct ASE_BaseException__object **_exception)
{
  PyObject *data = (PyObject *) _self->d_data;
  
  /* If we do not set d_data to NULL, we get a SEGV on the recursive call to deleteReference() coming from self.__IORself */
  _self->d_data = NULL;
  /* Remove only reference to Python object */
  Py_XDECREF(data);
}

void ASE_Compiler_Python_Printer__set_epv(struct ASE_Compiler_Python_Printer__epv *_epv, struct ASE_BaseException__object **_exception);

void ASE_Compiler_Python_Printer__set_epv(struct ASE_Compiler_Python_Printer__epv *_epv, struct ASE_BaseException__object **_exception)
{
  static int initialized = 0;
  
  if (!initialized)
  {
    initialized = 1;
    SIDL_Python_Init();
    if (!PyImport_ImportModule((char *) "user"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import user", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "importer"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import importer", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    if (!PyImport_ImportModule((char *) "ASE.Compiler.Python.Printer"))
    {
      struct ASE_ASEException__object *_local_exception = ASE_ASEException__externals()->createObject(NULL);
      
      _local_exception->d_epv->setMessage(_local_exception, "Could not import ASE.Compiler.Python.Printer", _exception);
      if (_exception)
      {
        *_exception = (struct ASE_BaseException__object *) _local_exception;
      }
      return;
    }
    import_array();
  }
  _epv->Printer = ASE_Compiler_Python_Printer_Printer;
  _epv->Printer_ = ASE_Compiler_Python_Printer_Printer_;
  _epv->getTab = ASE_Compiler_Printer_getTab;
  _epv->setTab = ASE_Compiler_Printer_setTab;
  _epv->getIndentSize = ASE_Compiler_Printer_getIndentSize;
  _epv->setIndentSize = ASE_Compiler_Printer_setIndentSize;
  _epv->increaseIndent = ASE_Compiler_Printer_increaseIndent;
  _epv->decreaseIndent = ASE_Compiler_Printer_decreaseIndent;
  _epv->indent = ASE_Compiler_Printer_indent;
  _epv->write = ASE_Compiler_Printer_write;
  _epv->println = ASE_Compiler_Printer_println;
  _epv->visitAdd = ASE_Compiler_Python_Printer_visitAdd;
  _epv->visitAnd = ASE_Compiler_Python_Printer_visitAnd;
  _epv->visitAssAttr = ASE_Compiler_Python_Printer_visitAssAttr;
  _epv->visitAssList = ASE_Compiler_Python_Printer_visitAssList;
  _epv->visitAssName = ASE_Compiler_Python_Printer_visitAssName;
  _epv->visitAssTuple = ASE_Compiler_Python_Printer_visitAssTuple;
  _epv->visitAssign = ASE_Compiler_Python_Printer_visitAssign;
  _epv->visitAugAssign = ASE_Compiler_Python_Printer_visitAugAssign;
  _epv->visitBitand = ASE_Compiler_Python_Printer_visitBitand;
  _epv->visitBitor = ASE_Compiler_Python_Printer_visitBitor;
  _epv->visitBitxor = ASE_Compiler_Python_Printer_visitBitxor;
  _epv->visitBreak = ASE_Compiler_Python_Printer_visitBreak;
  _epv->visitCallFunc = ASE_Compiler_Python_Printer_visitCallFunc;
  _epv->visitClass = ASE_Compiler_Python_Printer_visitClass;
  _epv->visitCompare = ASE_Compiler_Python_Printer_visitCompare;
  _epv->visitConst = ASE_Compiler_Python_Printer_visitConst;
  _epv->visitPyNone = ASE_Compiler_Python_Printer_visitPyNone;
  _epv->visitContinue = ASE_Compiler_Python_Printer_visitContinue;
  _epv->visitDict = ASE_Compiler_Python_Printer_visitDict;
  _epv->visitDiscard = ASE_Compiler_Python_Printer_visitDiscard;
  _epv->visitDiv = ASE_Compiler_Python_Printer_visitDiv;
  _epv->visitEllipsis = ASE_Compiler_Python_Printer_visitEllipsis;
  _epv->visitFor = ASE_Compiler_Python_Printer_visitFor;
  _epv->visitListCompFor = ASE_Compiler_Python_Printer_visitListCompFor;
  _epv->visitFunction = ASE_Compiler_Python_Printer_visitFunction;
  _epv->visitLambda = ASE_Compiler_Python_Printer_visitLambda;
  _epv->visitGetAttr = ASE_Compiler_Python_Printer_visitGetAttr;
  _epv->visitGlobal = ASE_Compiler_Python_Printer_visitGlobal;
  _epv->visitIf = ASE_Compiler_Python_Printer_visitIf;
  _epv->visitListCompIf = ASE_Compiler_Python_Printer_visitListCompIf;
  _epv->visitImport = ASE_Compiler_Python_Printer_visitImport;
  _epv->visitFrom = ASE_Compiler_Python_Printer_visitFrom;
  _epv->visitInvert = ASE_Compiler_Python_Printer_visitInvert;
  _epv->visitKeyword = ASE_Compiler_Python_Printer_visitKeyword;
  _epv->visitLeftShift = ASE_Compiler_Python_Printer_visitLeftShift;
  _epv->visitList = ASE_Compiler_Python_Printer_visitList;
  _epv->visitListComp = ASE_Compiler_Python_Printer_visitListComp;
  _epv->visitMod = ASE_Compiler_Python_Printer_visitMod;
  _epv->visitModule = ASE_Compiler_Python_Printer_visitModule;
  _epv->visitMul = ASE_Compiler_Python_Printer_visitMul;
  _epv->visitName = ASE_Compiler_Python_Printer_visitName;
  _epv->visitNot = ASE_Compiler_Python_Printer_visitNot;
  _epv->visitOr = ASE_Compiler_Python_Printer_visitOr;
  _epv->visitPass = ASE_Compiler_Python_Printer_visitPass;
  _epv->visitPower = ASE_Compiler_Python_Printer_visitPower;
  _epv->visitPrint = ASE_Compiler_Python_Printer_visitPrint;
  _epv->visitPrintnl = ASE_Compiler_Python_Printer_visitPrintnl;
  _epv->visitRaise = ASE_Compiler_Python_Printer_visitRaise;
  _epv->visitReturn = ASE_Compiler_Python_Printer_visitReturn;
  _epv->visitRightShift = ASE_Compiler_Python_Printer_visitRightShift;
  _epv->visitSlice = ASE_Compiler_Python_Printer_visitSlice;
  _epv->visitStmt = ASE_Compiler_Python_Printer_visitStmt;
  _epv->visitSub = ASE_Compiler_Python_Printer_visitSub;
  _epv->visitSubscript = ASE_Compiler_Python_Printer_visitSubscript;
  _epv->visitTryExcept = ASE_Compiler_Python_Printer_visitTryExcept;
  _epv->visitTryFinally = ASE_Compiler_Python_Printer_visitTryFinally;
  _epv->visitTuple = ASE_Compiler_Python_Printer_visitTuple;
  _epv->visitUnaryAdd = ASE_Compiler_Python_Printer_visitUnaryAdd;
  _epv->visitUnarySub = ASE_Compiler_Python_Printer_visitUnarySub;
  _epv->visitWhile = ASE_Compiler_Python_Printer_visitWhile;
  _epv->visitYield = ASE_Compiler_Python_Printer_visitYield;
  _epv->printDocComment = ASE_Compiler_Python_Printer_printDocComment;
}
